# Generated by Django 4.2.14 on 2024-07-23 07:41

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('eveonline', '0017_alliance_and_corp_names_are_not_unique'),
        ('aa_contacts', '0003_corporationcontact_is_watched_corporationtoken'),
    ]

    operations = [
        migrations.CreateModel(
            name='StandingFilter',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=500)),
                ('description', models.CharField(max_length=500)),
                ('comparison', models.CharField(choices=[('>', 'Greater Than'), ('>=', 'Greater Or Equal'), ('<', 'Less Than'), ('<=', 'Less Or Equal'), ('=', 'Equal')], max_length=2)),
                ('standing', models.FloatField()),
                ('alliances', models.ManyToManyField(blank=True, help_text='The alliances that have set the standings', related_name='alliance_standing_filters', to='eveonline.eveallianceinfo')),
                ('corporations', models.ManyToManyField(blank=True, help_text='The corporations that have set the standings', related_name='corp_standing_filters', to='eveonline.evecorporationinfo')),
            ],
            options={
                'verbose_name': 'Smart Filter: User Standings',
                'verbose_name_plural': 'Smart Filter: User Standings',
                'default_permissions': (),
            },
        ),
    ]
