import{r as a0,a as l0,b as jl,j as Cv,g as i0}from"./@bootstrap-libs-BWoQ1j8F.js";var Os={exports:{}},zu={},zs={exports:{}},Ls={};/**
 * @license React
 * scheduler.production.js
 *
 * Copyright (c) Meta Platforms, Inc. and affiliates.
 *
 * This source code is licensed under the MIT license found in the
 * LICENSE file in the root directory of this source tree.
 */var Av;function r0(){return Av||(Av=1,(function(a){function i(I,ue){var me=I.length;I.push(ue);e:for(;0<me;){var qe=me-1>>>1,lt=I[qe];if(0<d(lt,ue))I[qe]=ue,I[me]=lt,me=qe;else break e}}function r(I){return I.length===0?null:I[0]}function f(I){if(I.length===0)return null;var ue=I[0],me=I.pop();if(me!==ue){I[0]=me;e:for(var qe=0,lt=I.length,Rt=lt>>>1;qe<Rt;){var ot=2*(qe+1)-1,Qe=I[ot],bt=ot+1,wn=I[bt];if(0>d(Qe,me))bt<lt&&0>d(wn,Qe)?(I[qe]=wn,I[bt]=me,qe=bt):(I[qe]=Qe,I[ot]=me,qe=ot);else if(bt<lt&&0>d(wn,me))I[qe]=wn,I[bt]=me,qe=bt;else break e}}return ue}function d(I,ue){var me=I.sortIndex-ue.sortIndex;return me!==0?me:I.id-ue.id}if(a.unstable_now=void 0,typeof performance=="object"&&typeof performance.now=="function"){var h=performance;a.unstable_now=function(){return h.now()}}else{var v=Date,g=v.now();a.unstable_now=function(){return v.now()-g}}var x=[],T=[],A=1,N=null,j=3,X=!1,U=!1,B=!1,Y=!1,F=typeof setTimeout=="function"?setTimeout:null,re=typeof clearTimeout=="function"?clearTimeout:null,Se=typeof setImmediate<"u"?setImmediate:null;function Te(I){for(var ue=r(T);ue!==null;){if(ue.callback===null)f(T);else if(ue.startTime<=I)f(T),ue.sortIndex=ue.expirationTime,i(x,ue);else break;ue=r(T)}}function Oe(I){if(B=!1,Te(I),!U)if(r(x)!==null)U=!0,ge||(ge=!0,Xe());else{var ue=r(T);ue!==null&&$e(Oe,ue.startTime-I)}}var ge=!1,y=-1,Ze=5,Ce=-1;function Tt(){return Y?!0:!(a.unstable_now()-Ce<Ze)}function vt(){if(Y=!1,ge){var I=a.unstable_now();Ce=I;var ue=!0;try{e:{U=!1,B&&(B=!1,re(y),y=-1),X=!0;var me=j;try{t:{for(Te(I),N=r(x);N!==null&&!(N.expirationTime>I&&Tt());){var qe=N.callback;if(typeof qe=="function"){N.callback=null,j=N.priorityLevel;var lt=qe(N.expirationTime<=I);if(I=a.unstable_now(),typeof lt=="function"){N.callback=lt,Te(I),ue=!0;break t}N===r(x)&&f(x),Te(I)}else f(x);N=r(x)}if(N!==null)ue=!0;else{var Rt=r(T);Rt!==null&&$e(Oe,Rt.startTime-I),ue=!1}}break e}finally{N=null,j=me,X=!1}ue=void 0}}finally{ue?Xe():ge=!1}}}var Xe;if(typeof Se=="function")Xe=function(){Se(vt)};else if(typeof MessageChannel<"u"){var Me=new MessageChannel,$t=Me.port2;Me.port1.onmessage=vt,Xe=function(){$t.postMessage(null)}}else Xe=function(){F(vt,0)};function $e(I,ue){y=F(function(){I(a.unstable_now())},ue)}a.unstable_IdlePriority=5,a.unstable_ImmediatePriority=1,a.unstable_LowPriority=4,a.unstable_NormalPriority=3,a.unstable_Profiling=null,a.unstable_UserBlockingPriority=2,a.unstable_cancelCallback=function(I){I.callback=null},a.unstable_forceFrameRate=function(I){0>I||125<I?console.error("forceFrameRate takes a positive int between 0 and 125, forcing frame rates higher than 125 fps is not supported"):Ze=0<I?Math.floor(1e3/I):5},a.unstable_getCurrentPriorityLevel=function(){return j},a.unstable_next=function(I){switch(j){case 1:case 2:case 3:var ue=3;break;default:ue=j}var me=j;j=ue;try{return I()}finally{j=me}},a.unstable_requestPaint=function(){Y=!0},a.unstable_runWithPriority=function(I,ue){switch(I){case 1:case 2:case 3:case 4:case 5:break;default:I=3}var me=j;j=I;try{return ue()}finally{j=me}},a.unstable_scheduleCallback=function(I,ue,me){var qe=a.unstable_now();switch(typeof me=="object"&&me!==null?(me=me.delay,me=typeof me=="number"&&0<me?qe+me:qe):me=qe,I){case 1:var lt=-1;break;case 2:lt=250;break;case 5:lt=1073741823;break;case 4:lt=1e4;break;default:lt=5e3}return lt=me+lt,I={id:A++,callback:ue,priorityLevel:I,startTime:me,expirationTime:lt,sortIndex:-1},me>qe?(I.sortIndex=me,i(T,I),r(x)===null&&I===r(T)&&(B?(re(y),y=-1):B=!0,$e(Oe,me-qe))):(I.sortIndex=lt,i(x,I),U||X||(U=!0,ge||(ge=!0,Xe()))),I},a.unstable_shouldYield=Tt,a.unstable_wrapCallback=function(I){var ue=j;return function(){var me=j;j=ue;try{return I.apply(this,arguments)}finally{j=me}}}})(Ls)),Ls}var Ev;function u0(){return Ev||(Ev=1,zs.exports=r0()),zs.exports}/**
 * @license React
 * react-dom-client.production.js
 *
 * Copyright (c) Meta Platforms, Inc. and affiliates.
 *
 * This source code is licensed under the MIT license found in the
 * LICENSE file in the root directory of this source tree.
 */var Nv;function f0(){if(Nv)return zu;Nv=1;var a=u0(),i=a0(),r=l0();function f(e){var t="https://react.dev/errors/"+e;if(1<arguments.length){t+="?args[]="+encodeURIComponent(arguments[1]);for(var n=2;n<arguments.length;n++)t+="&args[]="+encodeURIComponent(arguments[n])}return"Minified React error #"+e+"; visit "+t+" for the full message or use the non-minified dev environment for full errors and additional helpful warnings."}function d(e){return!(!e||e.nodeType!==1&&e.nodeType!==9&&e.nodeType!==11)}function h(e){var t=e,n=e;if(e.alternate)for(;t.return;)t=t.return;else{e=t;do t=e,(t.flags&4098)!==0&&(n=t.return),e=t.return;while(e)}return t.tag===3?n:null}function v(e){if(e.tag===13){var t=e.memoizedState;if(t===null&&(e=e.alternate,e!==null&&(t=e.memoizedState)),t!==null)return t.dehydrated}return null}function g(e){if(e.tag===31){var t=e.memoizedState;if(t===null&&(e=e.alternate,e!==null&&(t=e.memoizedState)),t!==null)return t.dehydrated}return null}function x(e){if(h(e)!==e)throw Error(f(188))}function T(e){var t=e.alternate;if(!t){if(t=h(e),t===null)throw Error(f(188));return t!==e?null:e}for(var n=e,l=t;;){var o=n.return;if(o===null)break;var s=o.alternate;if(s===null){if(l=o.return,l!==null){n=l;continue}break}if(o.child===s.child){for(s=o.child;s;){if(s===n)return x(o),e;if(s===l)return x(o),t;s=s.sibling}throw Error(f(188))}if(n.return!==l.return)n=o,l=s;else{for(var b=!1,C=o.child;C;){if(C===n){b=!0,n=o,l=s;break}if(C===l){b=!0,l=o,n=s;break}C=C.sibling}if(!b){for(C=s.child;C;){if(C===n){b=!0,n=s,l=o;break}if(C===l){b=!0,l=s,n=o;break}C=C.sibling}if(!b)throw Error(f(189))}}if(n.alternate!==l)throw Error(f(190))}if(n.tag!==3)throw Error(f(188));return n.stateNode.current===n?e:t}function A(e){var t=e.tag;if(t===5||t===26||t===27||t===6)return e;for(e=e.child;e!==null;){if(t=A(e),t!==null)return t;e=e.sibling}return null}var N=Object.assign,j=Symbol.for("react.element"),X=Symbol.for("react.transitional.element"),U=Symbol.for("react.portal"),B=Symbol.for("react.fragment"),Y=Symbol.for("react.strict_mode"),F=Symbol.for("react.profiler"),re=Symbol.for("react.consumer"),Se=Symbol.for("react.context"),Te=Symbol.for("react.forward_ref"),Oe=Symbol.for("react.suspense"),ge=Symbol.for("react.suspense_list"),y=Symbol.for("react.memo"),Ze=Symbol.for("react.lazy"),Ce=Symbol.for("react.activity"),Tt=Symbol.for("react.memo_cache_sentinel"),vt=Symbol.iterator;function Xe(e){return e===null||typeof e!="object"?null:(e=vt&&e[vt]||e["@@iterator"],typeof e=="function"?e:null)}var Me=Symbol.for("react.client.reference");function $t(e){if(e==null)return null;if(typeof e=="function")return e.$$typeof===Me?null:e.displayName||e.name||null;if(typeof e=="string")return e;switch(e){case B:return"Fragment";case F:return"Profiler";case Y:return"StrictMode";case Oe:return"Suspense";case ge:return"SuspenseList";case Ce:return"Activity"}if(typeof e=="object")switch(e.$$typeof){case U:return"Portal";case Se:return e.displayName||"Context";case re:return(e._context.displayName||"Context")+".Consumer";case Te:var t=e.render;return e=e.displayName,e||(e=t.displayName||t.name||"",e=e!==""?"ForwardRef("+e+")":"ForwardRef"),e;case y:return t=e.displayName||null,t!==null?t:$t(e.type)||"Memo";case Ze:t=e._payload,e=e._init;try{return $t(e(t))}catch{}}return null}var $e=Array.isArray,I=i.__CLIENT_INTERNALS_DO_NOT_USE_OR_WARN_USERS_THEY_CANNOT_UPGRADE,ue=r.__DOM_INTERNALS_DO_NOT_USE_OR_WARN_USERS_THEY_CANNOT_UPGRADE,me={pending:!1,data:null,method:null,action:null},qe=[],lt=-1;function Rt(e){return{current:e}}function ot(e){0>lt||(e.current=qe[lt],qe[lt]=null,lt--)}function Qe(e,t){lt++,qe[lt]=e.current,e.current=t}var bt=Rt(null),wn=Rt(null),oa=Rt(null),Ai=Rt(null);function Ei(e,t){switch(Qe(oa,t),Qe(wn,e),Qe(bt,null),t.nodeType){case 9:case 11:e=(e=t.documentElement)&&(e=e.namespaceURI)?kp(e):0;break;default:if(e=t.tagName,t=t.namespaceURI)t=kp(t),e=Fp(t,e);else switch(e){case"svg":e=1;break;case"math":e=2;break;default:e=0}}ot(bt),Qe(bt,e)}function Da(){ot(bt),ot(wn),ot(oa)}function dn(e){e.memoizedState!==null&&Qe(Ai,e);var t=bt.current,n=Fp(t,e.type);t!==n&&(Qe(wn,e),Qe(bt,n))}function Ni(e){wn.current===e&&(ot(bt),ot(wn)),Ai.current===e&&(ot(Ai),Nu._currentValue=me)}var ca,Xl;function sa(e){if(ca===void 0)try{throw Error()}catch(n){var t=n.stack.trim().match(/\n( *(at )?)/);ca=t&&t[1]||"",Xl=-1<n.stack.indexOf(`
    at`)?" (<anonymous>)":-1<n.stack.indexOf("@")?"@unknown:0:0":""}return`
`+ca+e+Xl}var wr=!1;function Gl(e,t){if(!e||wr)return"";wr=!0;var n=Error.prepareStackTrace;Error.prepareStackTrace=void 0;try{var l={DetermineComponentFrameRoot:function(){try{if(t){var ae=function(){throw Error()};if(Object.defineProperty(ae.prototype,"props",{set:function(){throw Error()}}),typeof Reflect=="object"&&Reflect.construct){try{Reflect.construct(ae,[])}catch(k){var K=k}Reflect.construct(e,[],ae)}else{try{ae.call()}catch(k){K=k}e.call(ae.prototype)}}else{try{throw Error()}catch(k){K=k}(ae=e())&&typeof ae.catch=="function"&&ae.catch(function(){})}}catch(k){if(k&&K&&typeof k.stack=="string")return[k.stack,K.stack]}return[null,null]}};l.DetermineComponentFrameRoot.displayName="DetermineComponentFrameRoot";var o=Object.getOwnPropertyDescriptor(l.DetermineComponentFrameRoot,"name");o&&o.configurable&&Object.defineProperty(l.DetermineComponentFrameRoot,"name",{value:"DetermineComponentFrameRoot"});var s=l.DetermineComponentFrameRoot(),b=s[0],C=s[1];if(b&&C){var z=b.split(`
`),Q=C.split(`
`);for(o=l=0;l<z.length&&!z[l].includes("DetermineComponentFrameRoot");)l++;for(;o<Q.length&&!Q[o].includes("DetermineComponentFrameRoot");)o++;if(l===z.length||o===Q.length)for(l=z.length-1,o=Q.length-1;1<=l&&0<=o&&z[l]!==Q[o];)o--;for(;1<=l&&0<=o;l--,o--)if(z[l]!==Q[o]){if(l!==1||o!==1)do if(l--,o--,0>o||z[l]!==Q[o]){var ee=`
`+z[l].replace(" at new "," at ");return e.displayName&&ee.includes("<anonymous>")&&(ee=ee.replace("<anonymous>",e.displayName)),ee}while(1<=l&&0<=o);break}}}finally{wr=!1,Error.prepareStackTrace=n}return(n=e?e.displayName||e.name:"")?sa(n):""}function Mi(e,t){switch(e.tag){case 26:case 27:case 5:return sa(e.type);case 16:return sa("Lazy");case 13:return e.child!==t&&t!==null?sa("Suspense Fallback"):sa("Suspense");case 19:return sa("SuspenseList");case 0:case 15:return Gl(e.type,!1);case 11:return Gl(e.type.render,!1);case 1:return Gl(e.type,!0);case 31:return sa("Activity");default:return""}}function On(e){try{var t="",n=null;do t+=Mi(e,n),n=e,e=e.return;while(e);return t}catch(l){return`
Error generating stack: `+l.message+`
`+l.stack}}var Or=Object.prototype.hasOwnProperty,zr=a.unstable_scheduleCallback,Lr=a.unstable_cancelCallback,zn=a.unstable_shouldYield,Vl=a.unstable_requestPaint,wt=a.unstable_now,ve=a.unstable_getCurrentPriorityLevel,Gt=a.unstable_ImmediatePriority,Vu=a.unstable_UserBlockingPriority,wi=a.unstable_NormalPriority,Ho=a.unstable_LowPriority,Hr=a.unstable_IdlePriority,Qu=a.log,Ql=a.unstable_setDisableYieldValue,nn=null,gt=null;function an(e){if(typeof Qu=="function"&&Ql(e),gt&&typeof gt.setStrictMode=="function")try{gt.setStrictMode(nn,e)}catch{}}var ln=Math.clz32?Math.clz32:Ku,Oi=Math.log,Zu=Math.LN2;function Ku(e){return e>>>=0,e===0?32:31-(Oi(e)/Zu|0)|0}var zi=256,In=262144,da=4194304;function ha(e){var t=e&42;if(t!==0)return t;switch(e&-e){case 1:return 1;case 2:return 2;case 4:return 4;case 8:return 8;case 16:return 16;case 32:return 32;case 64:return 64;case 128:return 128;case 256:case 512:case 1024:case 2048:case 4096:case 8192:case 16384:case 32768:case 65536:case 131072:return e&261888;case 262144:case 524288:case 1048576:case 2097152:return e&3932160;case 4194304:case 8388608:case 16777216:case 33554432:return e&62914560;case 67108864:return 67108864;case 134217728:return 134217728;case 268435456:return 268435456;case 536870912:return 536870912;case 1073741824:return 0;default:return e}}function Zl(e,t,n){var l=e.pendingLanes;if(l===0)return 0;var o=0,s=e.suspendedLanes,b=e.pingedLanes;e=e.warmLanes;var C=l&134217727;return C!==0?(l=C&~s,l!==0?o=ha(l):(b&=C,b!==0?o=ha(b):n||(n=C&~e,n!==0&&(o=ha(n))))):(C=l&~s,C!==0?o=ha(C):b!==0?o=ha(b):n||(n=l&~e,n!==0&&(o=ha(n)))),o===0?0:t!==0&&t!==o&&(t&s)===0&&(s=o&-o,n=t&-t,s>=n||s===32&&(n&4194048)!==0)?t:o}function qt(e,t){return(e.pendingLanes&~(e.suspendedLanes&~e.pingedLanes)&t)===0}function It(e,t){switch(e){case 1:case 2:case 4:case 8:case 64:return t+250;case 16:case 32:case 128:case 256:case 512:case 1024:case 2048:case 4096:case 8192:case 16384:case 32768:case 65536:case 131072:case 262144:case 524288:case 1048576:case 2097152:return t+5e3;case 4194304:case 8388608:case 16777216:case 33554432:return-1;case 67108864:case 134217728:case 268435456:case 536870912:case 1073741824:return-1;default:return-1}}function Li(){var e=da;return da<<=1,(da&62914560)===0&&(da=4194304),e}function jr(e){for(var t=[],n=0;31>n;n++)t.push(e);return t}function el(e,t){e.pendingLanes|=t,t!==268435456&&(e.suspendedLanes=0,e.pingedLanes=0,e.warmLanes=0)}function Ju(e,t,n,l,o,s){var b=e.pendingLanes;e.pendingLanes=n,e.suspendedLanes=0,e.pingedLanes=0,e.warmLanes=0,e.expiredLanes&=n,e.entangledLanes&=n,e.errorRecoveryDisabledLanes&=n,e.shellSuspendCounter=0;var C=e.entanglements,z=e.expirationTimes,Q=e.hiddenUpdates;for(n=b&~n;0<n;){var ee=31-ln(n),ae=1<<ee;C[ee]=0,z[ee]=-1;var K=Q[ee];if(K!==null)for(Q[ee]=null,ee=0;ee<K.length;ee++){var k=K[ee];k!==null&&(k.lane&=-536870913)}n&=~ae}l!==0&&Ta(e,l,0),s!==0&&o===0&&e.tag!==0&&(e.suspendedLanes|=s&~(b&~t))}function Ta(e,t,n){e.pendingLanes|=t,e.suspendedLanes&=~t;var l=31-ln(t);e.entangledLanes|=t,e.entanglements[l]=e.entanglements[l]|1073741824|n&261930}function Ca(e,t){var n=e.entangledLanes|=t;for(e=e.entanglements;n;){var l=31-ln(n),o=1<<l;o&t|e[l]&t&&(e[l]|=t),n&=~o}}function Hi(e,t){var n=t&-t;return n=(n&42)!==0?1:tl(n),(n&(e.suspendedLanes|t))!==0?0:n}function tl(e){switch(e){case 2:e=1;break;case 8:e=4;break;case 32:e=16;break;case 256:case 512:case 1024:case 2048:case 4096:case 8192:case 16384:case 32768:case 65536:case 131072:case 262144:case 524288:case 1048576:case 2097152:case 4194304:case 8388608:case 16777216:case 33554432:e=128;break;case 268435456:e=134217728;break;default:e=0}return e}function Rr(e){return e&=-e,2<e?8<e?(e&134217727)!==0?32:268435456:8:2}function ku(){var e=ue.p;return e!==0?e:(e=window.event,e===void 0?32:bv(e.type))}function Fu(e,t){var n=ue.p;try{return ue.p=e,t()}finally{ue.p=n}}var Pn=Math.random().toString(36).slice(2),Ot="__reactFiber$"+Pn,Pt="__reactProps$"+Pn,Aa="__reactContainer$"+Pn,qr="__reactEvents$"+Pn,nl="__reactListeners$"+Pn,Wu="__reactHandles$"+Pn,ji="__reactResources$"+Pn,Ea="__reactMarker$"+Pn;function al(e){delete e[Ot],delete e[Pt],delete e[qr],delete e[nl],delete e[Wu]}function Na(e){var t=e[Ot];if(t)return t;for(var n=e.parentNode;n;){if(t=n[Aa]||n[Ot]){if(n=t.alternate,t.child!==null||n!==null&&n.child!==null)for(e=nv(e);e!==null;){if(n=e[Ot])return n;e=nv(e)}return t}e=n,n=e.parentNode}return null}function ll(e){if(e=e[Ot]||e[Aa]){var t=e.tag;if(t===5||t===6||t===13||t===31||t===26||t===27||t===3)return e}return null}function ea(e){var t=e.tag;if(t===5||t===26||t===27||t===6)return e.stateNode;throw Error(f(33))}function Ma(e){var t=e[ji];return t||(t=e[ji]={hoistableStyles:new Map,hoistableScripts:new Map}),t}function St(e){e[Ea]=!0}var Ur=new Set,Br={};function wa(e,t){pa(e,t),pa(e+"Capture",t)}function pa(e,t){for(Br[e]=t,e=0;e<t.length;e++)Ur.add(t[e])}var jo=RegExp("^[:A-Z_a-z\\u00C0-\\u00D6\\u00D8-\\u00F6\\u00F8-\\u02FF\\u0370-\\u037D\\u037F-\\u1FFF\\u200C-\\u200D\\u2070-\\u218F\\u2C00-\\u2FEF\\u3001-\\uD7FF\\uF900-\\uFDCF\\uFDF0-\\uFFFD][:A-Z_a-z\\u00C0-\\u00D6\\u00D8-\\u00F6\\u00F8-\\u02FF\\u0370-\\u037D\\u037F-\\u1FFF\\u200C-\\u200D\\u2070-\\u218F\\u2C00-\\u2FEF\\u3001-\\uD7FF\\uF900-\\uFDCF\\uFDF0-\\uFFFD\\-.0-9\\u00B7\\u0300-\\u036F\\u203F-\\u2040]*$"),$u={},Yr={};function Iu(e){return Or.call(Yr,e)?!0:Or.call($u,e)?!1:jo.test(e)?Yr[e]=!0:($u[e]=!0,!1)}function il(e,t,n){if(Iu(t))if(n===null)e.removeAttribute(t);else{switch(typeof n){case"undefined":case"function":case"symbol":e.removeAttribute(t);return;case"boolean":var l=t.toLowerCase().slice(0,5);if(l!=="data-"&&l!=="aria-"){e.removeAttribute(t);return}}e.setAttribute(t,""+n)}}function Kl(e,t,n){if(n===null)e.removeAttribute(t);else{switch(typeof n){case"undefined":case"function":case"symbol":case"boolean":e.removeAttribute(t);return}e.setAttribute(t,""+n)}}function dt(e,t,n,l){if(l===null)e.removeAttribute(n);else{switch(typeof l){case"undefined":case"function":case"symbol":case"boolean":e.removeAttribute(n);return}e.setAttributeNS(t,n,""+l)}}function zt(e){switch(typeof e){case"bigint":case"boolean":case"number":case"string":case"undefined":return e;case"object":return e;default:return""}}function Jl(e){var t=e.type;return(e=e.nodeName)&&e.toLowerCase()==="input"&&(t==="checkbox"||t==="radio")}function Ro(e,t,n){var l=Object.getOwnPropertyDescriptor(e.constructor.prototype,t);if(!e.hasOwnProperty(t)&&typeof l<"u"&&typeof l.get=="function"&&typeof l.set=="function"){var o=l.get,s=l.set;return Object.defineProperty(e,t,{configurable:!0,get:function(){return o.call(this)},set:function(b){n=""+b,s.call(this,b)}}),Object.defineProperty(e,t,{enumerable:l.enumerable}),{getValue:function(){return n},setValue:function(b){n=""+b},stopTracking:function(){e._valueTracker=null,delete e[t]}}}}function Xr(e){if(!e._valueTracker){var t=Jl(e)?"checked":"value";e._valueTracker=Ro(e,t,""+e[t])}}function Ri(e){if(!e)return!1;var t=e._valueTracker;if(!t)return!0;var n=t.getValue(),l="";return e&&(l=Jl(e)?e.checked?"true":"false":e.value),e=l,e!==n?(t.setValue(e),!0):!1}function kl(e){if(e=e||(typeof document<"u"?document:void 0),typeof e>"u")return null;try{return e.activeElement||e.body}catch{return e.body}}var qi=/[\n"\\]/g;function rn(e){return e.replace(qi,function(t){return"\\"+t.charCodeAt(0).toString(16)+" "})}function Gr(e,t,n,l,o,s,b,C){e.name="",b!=null&&typeof b!="function"&&typeof b!="symbol"&&typeof b!="boolean"?e.type=b:e.removeAttribute("type"),t!=null?b==="number"?(t===0&&e.value===""||e.value!=t)&&(e.value=""+zt(t)):e.value!==""+zt(t)&&(e.value=""+zt(t)):b!=="submit"&&b!=="reset"||e.removeAttribute("value"),t!=null?un(e,b,zt(t)):n!=null?un(e,b,zt(n)):l!=null&&e.removeAttribute("value"),o==null&&s!=null&&(e.defaultChecked=!!s),o!=null&&(e.checked=o&&typeof o!="function"&&typeof o!="symbol"),C!=null&&typeof C!="function"&&typeof C!="symbol"&&typeof C!="boolean"?e.name=""+zt(C):e.removeAttribute("name")}function Pu(e,t,n,l,o,s,b,C){if(s!=null&&typeof s!="function"&&typeof s!="symbol"&&typeof s!="boolean"&&(e.type=s),t!=null||n!=null){if(!(s!=="submit"&&s!=="reset"||t!=null)){Xr(e);return}n=n!=null?""+zt(n):"",t=t!=null?""+zt(t):n,C||t===e.value||(e.value=t),e.defaultValue=t}l=l??o,l=typeof l!="function"&&typeof l!="symbol"&&!!l,e.checked=C?e.checked:!!l,e.defaultChecked=!!l,b!=null&&typeof b!="function"&&typeof b!="symbol"&&typeof b!="boolean"&&(e.name=b),Xr(e)}function un(e,t,n){t==="number"&&kl(e.ownerDocument)===e||e.defaultValue===""+n||(e.defaultValue=""+n)}function Oa(e,t,n,l){if(e=e.options,t){t={};for(var o=0;o<n.length;o++)t["$"+n[o]]=!0;for(n=0;n<e.length;n++)o=t.hasOwnProperty("$"+e[n].value),e[n].selected!==o&&(e[n].selected=o),o&&l&&(e[n].defaultSelected=!0)}else{for(n=""+zt(n),t=null,o=0;o<e.length;o++){if(e[o].value===n){e[o].selected=!0,l&&(e[o].defaultSelected=!0);return}t!==null||e[o].disabled||(t=e[o])}t!==null&&(t.selected=!0)}}function rl(e,t,n){if(t!=null&&(t=""+zt(t),t!==e.value&&(e.value=t),n==null)){e.defaultValue!==t&&(e.defaultValue=t);return}e.defaultValue=n!=null?""+zt(n):""}function ef(e,t,n,l){if(t==null){if(l!=null){if(n!=null)throw Error(f(92));if($e(l)){if(1<l.length)throw Error(f(93));l=l[0]}n=l}n==null&&(n=""),t=n}n=zt(t),e.defaultValue=n,l=e.textContent,l===n&&l!==""&&l!==null&&(e.value=l),Xr(e)}function ul(e,t){if(t){var n=e.firstChild;if(n&&n===e.lastChild&&n.nodeType===3){n.nodeValue=t;return}}e.textContent=t}var za=new Set("animationIterationCount aspectRatio borderImageOutset borderImageSlice borderImageWidth boxFlex boxFlexGroup boxOrdinalGroup columnCount columns flex flexGrow flexPositive flexShrink flexNegative flexOrder gridArea gridRow gridRowEnd gridRowSpan gridRowStart gridColumn gridColumnEnd gridColumnSpan gridColumnStart fontWeight lineClamp lineHeight opacity order orphans scale tabSize widows zIndex zoom fillOpacity floodOpacity stopOpacity strokeDasharray strokeDashoffset strokeMiterlimit strokeOpacity strokeWidth MozAnimationIterationCount MozBoxFlex MozBoxFlexGroup MozLineClamp msAnimationIterationCount msFlex msZoom msFlexGrow msFlexNegative msFlexOrder msFlexPositive msFlexShrink msGridColumn msGridColumnSpan msGridRow msGridRowSpan WebkitAnimationIterationCount WebkitBoxFlex WebKitBoxFlexGroup WebkitBoxOrdinalGroup WebkitColumnCount WebkitColumns WebkitFlex WebkitFlexGrow WebkitFlexPositive WebkitFlexShrink WebkitLineClamp".split(" "));function va(e,t,n){var l=t.indexOf("--")===0;n==null||typeof n=="boolean"||n===""?l?e.setProperty(t,""):t==="float"?e.cssFloat="":e[t]="":l?e.setProperty(t,n):typeof n!="number"||n===0||za.has(t)?t==="float"?e.cssFloat=n:e[t]=(""+n).trim():e[t]=n+"px"}function Ui(e,t,n){if(t!=null&&typeof t!="object")throw Error(f(62));if(e=e.style,n!=null){for(var l in n)!n.hasOwnProperty(l)||t!=null&&t.hasOwnProperty(l)||(l.indexOf("--")===0?e.setProperty(l,""):l==="float"?e.cssFloat="":e[l]="");for(var o in t)l=t[o],t.hasOwnProperty(o)&&n[o]!==l&&va(e,o,l)}else for(var s in t)t.hasOwnProperty(s)&&va(e,s,t[s])}function Vr(e){if(e.indexOf("-")===-1)return!1;switch(e){case"annotation-xml":case"color-profile":case"font-face":case"font-face-src":case"font-face-uri":case"font-face-format":case"font-face-name":case"missing-glyph":return!1;default:return!0}}var Fl=new Map([["acceptCharset","accept-charset"],["htmlFor","for"],["httpEquiv","http-equiv"],["crossOrigin","crossorigin"],["accentHeight","accent-height"],["alignmentBaseline","alignment-baseline"],["arabicForm","arabic-form"],["baselineShift","baseline-shift"],["capHeight","cap-height"],["clipPath","clip-path"],["clipRule","clip-rule"],["colorInterpolation","color-interpolation"],["colorInterpolationFilters","color-interpolation-filters"],["colorProfile","color-profile"],["colorRendering","color-rendering"],["dominantBaseline","dominant-baseline"],["enableBackground","enable-background"],["fillOpacity","fill-opacity"],["fillRule","fill-rule"],["floodColor","flood-color"],["floodOpacity","flood-opacity"],["fontFamily","font-family"],["fontSize","font-size"],["fontSizeAdjust","font-size-adjust"],["fontStretch","font-stretch"],["fontStyle","font-style"],["fontVariant","font-variant"],["fontWeight","font-weight"],["glyphName","glyph-name"],["glyphOrientationHorizontal","glyph-orientation-horizontal"],["glyphOrientationVertical","glyph-orientation-vertical"],["horizAdvX","horiz-adv-x"],["horizOriginX","horiz-origin-x"],["imageRendering","image-rendering"],["letterSpacing","letter-spacing"],["lightingColor","lighting-color"],["markerEnd","marker-end"],["markerMid","marker-mid"],["markerStart","marker-start"],["overlinePosition","overline-position"],["overlineThickness","overline-thickness"],["paintOrder","paint-order"],["panose-1","panose-1"],["pointerEvents","pointer-events"],["renderingIntent","rendering-intent"],["shapeRendering","shape-rendering"],["stopColor","stop-color"],["stopOpacity","stop-opacity"],["strikethroughPosition","strikethrough-position"],["strikethroughThickness","strikethrough-thickness"],["strokeDasharray","stroke-dasharray"],["strokeDashoffset","stroke-dashoffset"],["strokeLinecap","stroke-linecap"],["strokeLinejoin","stroke-linejoin"],["strokeMiterlimit","stroke-miterlimit"],["strokeOpacity","stroke-opacity"],["strokeWidth","stroke-width"],["textAnchor","text-anchor"],["textDecoration","text-decoration"],["textRendering","text-rendering"],["transformOrigin","transform-origin"],["underlinePosition","underline-position"],["underlineThickness","underline-thickness"],["unicodeBidi","unicode-bidi"],["unicodeRange","unicode-range"],["unitsPerEm","units-per-em"],["vAlphabetic","v-alphabetic"],["vHanging","v-hanging"],["vIdeographic","v-ideographic"],["vMathematical","v-mathematical"],["vectorEffect","vector-effect"],["vertAdvY","vert-adv-y"],["vertOriginX","vert-origin-x"],["vertOriginY","vert-origin-y"],["wordSpacing","word-spacing"],["writingMode","writing-mode"],["xmlnsXlink","xmlns:xlink"],["xHeight","x-height"]]),tf=/^[\u0000-\u001F ]*j[\r\n\t]*a[\r\n\t]*v[\r\n\t]*a[\r\n\t]*s[\r\n\t]*c[\r\n\t]*r[\r\n\t]*i[\r\n\t]*p[\r\n\t]*t[\r\n\t]*:/i;function fl(e){return tf.test(""+e)?"javascript:throw new Error('React has blocked a javascript: URL as a security precaution.')":e}function Ln(){}var Bi=null;function Qr(e){return e=e.target||e.srcElement||window,e.correspondingUseElement&&(e=e.correspondingUseElement),e.nodeType===3?e.parentNode:e}var La=null,ol=null;function nf(e){var t=ll(e);if(t&&(e=t.stateNode)){var n=e[Pt]||null;e:switch(e=t.stateNode,t.type){case"input":if(Gr(e,n.value,n.defaultValue,n.defaultValue,n.checked,n.defaultChecked,n.type,n.name),t=n.name,n.type==="radio"&&t!=null){for(n=e;n.parentNode;)n=n.parentNode;for(n=n.querySelectorAll('input[name="'+rn(""+t)+'"][type="radio"]'),t=0;t<n.length;t++){var l=n[t];if(l!==e&&l.form===e.form){var o=l[Pt]||null;if(!o)throw Error(f(90));Gr(l,o.value,o.defaultValue,o.defaultValue,o.checked,o.defaultChecked,o.type,o.name)}}for(t=0;t<n.length;t++)l=n[t],l.form===e.form&&Ri(l)}break e;case"textarea":rl(e,n.value,n.defaultValue);break e;case"select":t=n.value,t!=null&&Oa(e,!!n.multiple,t,!1)}}}var Wl=!1;function af(e,t,n){if(Wl)return e(t,n);Wl=!0;try{var l=e(t);return l}finally{if(Wl=!1,(La!==null||ol!==null)&&(Yf(),La&&(t=La,e=ol,ol=La=null,nf(t),e)))for(t=0;t<e.length;t++)nf(e[t])}}function $l(e,t){var n=e.stateNode;if(n===null)return null;var l=n[Pt]||null;if(l===null)return null;n=l[t];e:switch(t){case"onClick":case"onClickCapture":case"onDoubleClick":case"onDoubleClickCapture":case"onMouseDown":case"onMouseDownCapture":case"onMouseMove":case"onMouseMoveCapture":case"onMouseUp":case"onMouseUpCapture":case"onMouseEnter":(l=!l.disabled)||(e=e.type,l=!(e==="button"||e==="input"||e==="select"||e==="textarea")),e=!l;break e;default:e=!1}if(e)return null;if(n&&typeof n!="function")throw Error(f(231,t,typeof n));return n}var ta=!(typeof window>"u"||typeof window.document>"u"||typeof window.document.createElement>"u"),Zr=!1;if(ta)try{var Il={};Object.defineProperty(Il,"passive",{get:function(){Zr=!0}}),window.addEventListener("test",Il,Il),window.removeEventListener("test",Il,Il)}catch{Zr=!1}var ma=null,Kr=null,Pl=null;function Yi(){if(Pl)return Pl;var e,t=Kr,n=t.length,l,o="value"in ma?ma.value:ma.textContent,s=o.length;for(e=0;e<n&&t[e]===o[e];e++);var b=n-e;for(l=1;l<=b&&t[n-l]===o[s-l];l++);return Pl=o.slice(e,1<l?1-l:void 0)}function ei(e){var t=e.keyCode;return"charCode"in e?(e=e.charCode,e===0&&t===13&&(e=13)):e=t,e===10&&(e=13),32<=e||e===13?e:0}function cl(){return!0}function Jr(){return!1}function Vt(e){function t(n,l,o,s,b){this._reactName=n,this._targetInst=o,this.type=l,this.nativeEvent=s,this.target=b,this.currentTarget=null;for(var C in e)e.hasOwnProperty(C)&&(n=e[C],this[C]=n?n(s):s[C]);return this.isDefaultPrevented=(s.defaultPrevented!=null?s.defaultPrevented:s.returnValue===!1)?cl:Jr,this.isPropagationStopped=Jr,this}return N(t.prototype,{preventDefault:function(){this.defaultPrevented=!0;var n=this.nativeEvent;n&&(n.preventDefault?n.preventDefault():typeof n.returnValue!="unknown"&&(n.returnValue=!1),this.isDefaultPrevented=cl)},stopPropagation:function(){var n=this.nativeEvent;n&&(n.stopPropagation?n.stopPropagation():typeof n.cancelBubble!="unknown"&&(n.cancelBubble=!0),this.isPropagationStopped=cl)},persist:function(){},isPersistent:cl}),t}var na={eventPhase:0,bubbles:0,cancelable:0,timeStamp:function(e){return e.timeStamp||Date.now()},defaultPrevented:0,isTrusted:0},Xi=Vt(na),ti=N({},na,{view:0,detail:0}),qo=Vt(ti),Ha,Gi,ja,Vi=N({},ti,{screenX:0,screenY:0,clientX:0,clientY:0,pageX:0,pageY:0,ctrlKey:0,shiftKey:0,altKey:0,metaKey:0,getModifierState:le,button:0,buttons:0,relatedTarget:function(e){return e.relatedTarget===void 0?e.fromElement===e.srcElement?e.toElement:e.fromElement:e.relatedTarget},movementX:function(e){return"movementX"in e?e.movementX:(e!==ja&&(ja&&e.type==="mousemove"?(Ha=e.screenX-ja.screenX,Gi=e.screenY-ja.screenY):Gi=Ha=0,ja=e),Ha)},movementY:function(e){return"movementY"in e?e.movementY:Gi}}),lf=Vt(Vi),Uo=N({},Vi,{dataTransfer:0}),u=Vt(Uo),c=N({},ti,{relatedTarget:0}),p=Vt(c),m=N({},na,{animationName:0,elapsedTime:0,pseudoElement:0}),S=Vt(m),_=N({},na,{clipboardData:function(e){return"clipboardData"in e?e.clipboardData:window.clipboardData}}),D=Vt(_),O=N({},na,{data:0}),w=Vt(O),R={Esc:"Escape",Spacebar:" ",Left:"ArrowLeft",Up:"ArrowUp",Right:"ArrowRight",Down:"ArrowDown",Del:"Delete",Win:"OS",Menu:"ContextMenu",Apps:"ContextMenu",Scroll:"ScrollLock",MozPrintableKey:"Unidentified"},W={8:"Backspace",9:"Tab",12:"Clear",13:"Enter",16:"Shift",17:"Control",18:"Alt",19:"Pause",20:"CapsLock",27:"Escape",32:" ",33:"PageUp",34:"PageDown",35:"End",36:"Home",37:"ArrowLeft",38:"ArrowUp",39:"ArrowRight",40:"ArrowDown",45:"Insert",46:"Delete",112:"F1",113:"F2",114:"F3",115:"F4",116:"F5",117:"F6",118:"F7",119:"F8",120:"F9",121:"F10",122:"F11",123:"F12",144:"NumLock",145:"ScrollLock",224:"Meta"},te={Alt:"altKey",Control:"ctrlKey",Meta:"metaKey",Shift:"shiftKey"};function Z(e){var t=this.nativeEvent;return t.getModifierState?t.getModifierState(e):(e=te[e])?!!t[e]:!1}function le(){return Z}var De=N({},ti,{key:function(e){if(e.key){var t=R[e.key]||e.key;if(t!=="Unidentified")return t}return e.type==="keypress"?(e=ei(e),e===13?"Enter":String.fromCharCode(e)):e.type==="keydown"||e.type==="keyup"?W[e.keyCode]||"Unidentified":""},code:0,location:0,ctrlKey:0,shiftKey:0,altKey:0,metaKey:0,repeat:0,locale:0,getModifierState:le,charCode:function(e){return e.type==="keypress"?ei(e):0},keyCode:function(e){return e.type==="keydown"||e.type==="keyup"?e.keyCode:0},which:function(e){return e.type==="keypress"?ei(e):e.type==="keydown"||e.type==="keyup"?e.keyCode:0}}),Ue=Vt(De),Ee=N({},Vi,{pointerId:0,width:0,height:0,pressure:0,tangentialPressure:0,tiltX:0,tiltY:0,twist:0,pointerType:0,isPrimary:0}),Ct=Vt(Ee),_t=N({},ti,{touches:0,targetTouches:0,changedTouches:0,altKey:0,metaKey:0,ctrlKey:0,shiftKey:0,getModifierState:le}),Hn=Vt(_t),jn=N({},na,{propertyName:0,elapsedTime:0,pseudoElement:0}),Le=Vt(jn),Ra=N({},Vi,{deltaX:function(e){return"deltaX"in e?e.deltaX:"wheelDeltaX"in e?-e.wheelDeltaX:0},deltaY:function(e){return"deltaY"in e?e.deltaY:"wheelDeltaY"in e?-e.wheelDeltaY:"wheelDelta"in e?-e.wheelDelta:0},deltaZ:0,deltaMode:0}),Ge=Vt(Ra),ht=N({},na,{newState:0,oldState:0}),ni=Vt(ht),kr=[9,13,27,32],hn=ta&&"CompositionEvent"in window,aa=null;ta&&"documentMode"in document&&(aa=document.documentMode);var Rn=ta&&"TextEvent"in window&&!aa,la=ta&&(!hn||aa&&8<aa&&11>=aa),ia=" ",fn=!1;function Qi(e,t){switch(e){case"keyup":return kr.indexOf(t.keyCode)!==-1;case"keydown":return t.keyCode!==229;case"keypress":case"mousedown":case"focusout":return!0;default:return!1}}function qn(e){return e=e.detail,typeof e=="object"&&"data"in e?e.data:null}var en=!1;function Bo(e,t){switch(e){case"compositionend":return qn(t);case"keypress":return t.which!==32?null:(fn=!0,ia);case"textInput":return e=t.data,e===ia&&fn?null:e;default:return null}}function Yo(e,t){if(en)return e==="compositionend"||!hn&&Qi(e,t)?(e=Yi(),Pl=Kr=ma=null,en=!1,e):null;switch(e){case"paste":return null;case"keypress":if(!(t.ctrlKey||t.altKey||t.metaKey)||t.ctrlKey&&t.altKey){if(t.char&&1<t.char.length)return t.char;if(t.which)return String.fromCharCode(t.which)}return null;case"compositionend":return la&&t.locale!=="ko"?null:t.data;default:return null}}var Xo={color:!0,date:!0,datetime:!0,"datetime-local":!0,email:!0,month:!0,number:!0,password:!0,range:!0,search:!0,tel:!0,text:!0,time:!0,url:!0,week:!0};function Je(e){var t=e&&e.nodeName&&e.nodeName.toLowerCase();return t==="input"?!!Xo[e.type]:t==="textarea"}function ai(e,t,n,l){La?ol?ol.push(l):ol=[l]:La=l,t=Jf(t,"onChange"),0<t.length&&(n=new Xi("onChange","change",null,n,l),e.push({event:n,listeners:t}))}var Qt=null,Un=null;function Go(e){Gp(e,0)}function Zi(e){var t=ea(e);if(Ri(t))return e}function Fr(e,t){if(e==="change")return t}var ya=!1;if(ta){var li;if(ta){var Bn="oninput"in document;if(!Bn){var Wr=document.createElement("div");Wr.setAttribute("oninput","return;"),Bn=typeof Wr.oninput=="function"}li=Bn}else li=!1;ya=li&&(!document.documentMode||9<document.documentMode)}function sl(){Qt&&(Qt.detachEvent("onpropertychange",ii),Un=Qt=null)}function ii(e){if(e.propertyName==="value"&&Zi(Un)){var t=[];ai(t,Un,e,Qr(e)),af(Go,t)}}function Ki(e,t,n){e==="focusin"?(sl(),Qt=t,Un=n,Qt.attachEvent("onpropertychange",ii)):e==="focusout"&&sl()}function $r(e){if(e==="selectionchange"||e==="keyup"||e==="keydown")return Zi(Un)}function Vo(e,t){if(e==="click")return Zi(t)}function Ji(e,t){if(e==="input"||e==="change")return Zi(t)}function Ir(e,t){return e===t&&(e!==0||1/e===1/t)||e!==e&&t!==t}var Zt=typeof Object.is=="function"?Object.is:Ir;function ri(e,t){if(Zt(e,t))return!0;if(typeof e!="object"||e===null||typeof t!="object"||t===null)return!1;var n=Object.keys(e),l=Object.keys(t);if(n.length!==l.length)return!1;for(l=0;l<n.length;l++){var o=n[l];if(!Or.call(t,o)||!Zt(e[o],t[o]))return!1}return!0}function ki(e){for(;e&&e.firstChild;)e=e.firstChild;return e}function Pr(e,t){var n=ki(e);e=0;for(var l;n;){if(n.nodeType===3){if(l=e+n.textContent.length,e<=t&&l>=t)return{node:n,offset:t-e};e=l}e:{for(;n;){if(n.nextSibling){n=n.nextSibling;break e}n=n.parentNode}n=void 0}n=ki(n)}}function E(e,t){return e&&t?e===t?!0:e&&e.nodeType===3?!1:t&&t.nodeType===3?E(e,t.parentNode):"contains"in e?e.contains(t):e.compareDocumentPosition?!!(e.compareDocumentPosition(t)&16):!1:!1}function L(e){e=e!=null&&e.ownerDocument!=null&&e.ownerDocument.defaultView!=null?e.ownerDocument.defaultView:window;for(var t=kl(e.document);t instanceof e.HTMLIFrameElement;){try{var n=typeof t.contentWindow.location.href=="string"}catch{n=!1}if(n)e=t.contentWindow;else break;t=kl(e.document)}return t}function G(e){var t=e&&e.nodeName&&e.nodeName.toLowerCase();return t&&(t==="input"&&(e.type==="text"||e.type==="search"||e.type==="tel"||e.type==="url"||e.type==="password")||t==="textarea"||e.contentEditable==="true")}var J=ta&&"documentMode"in document&&11>=document.documentMode,P=null,ie=null,fe=null,se=!1;function oe(e,t,n){var l=n.window===n?n.document:n.nodeType===9?n:n.ownerDocument;se||P==null||P!==kl(l)||(l=P,"selectionStart"in l&&G(l)?l={start:l.selectionStart,end:l.selectionEnd}:(l=(l.ownerDocument&&l.ownerDocument.defaultView||window).getSelection(),l={anchorNode:l.anchorNode,anchorOffset:l.anchorOffset,focusNode:l.focusNode,focusOffset:l.focusOffset}),fe&&ri(fe,l)||(fe=l,l=Jf(ie,"onSelect"),0<l.length&&(t=new Xi("onSelect","select",null,t,n),e.push({event:t,listeners:l}),t.target=P)))}function Ae(e,t){var n={};return n[e.toLowerCase()]=t.toLowerCase(),n["Webkit"+e]="webkit"+t,n["Moz"+e]="moz"+t,n}var ye={animationend:Ae("Animation","AnimationEnd"),animationiteration:Ae("Animation","AnimationIteration"),animationstart:Ae("Animation","AnimationStart"),transitionrun:Ae("Transition","TransitionRun"),transitionstart:Ae("Transition","TransitionStart"),transitioncancel:Ae("Transition","TransitionCancel"),transitionend:Ae("Transition","TransitionEnd")},xe={},ze={};ta&&(ze=document.createElement("div").style,"AnimationEvent"in window||(delete ye.animationend.animation,delete ye.animationiteration.animation,delete ye.animationstart.animation),"TransitionEvent"in window||delete ye.transitionend.transition);function pe(e){if(xe[e])return xe[e];if(!ye[e])return e;var t=ye[e],n;for(n in t)if(t.hasOwnProperty(n)&&n in ze)return xe[e]=t[n];return e}var Pe=pe("animationend"),Kt=pe("animationiteration"),Lt=pe("animationstart"),pn=pe("transitionrun"),Yn=pe("transitionstart"),dl=pe("transitioncancel"),Ut=pe("transitionend"),xd=new Map,Qo="abort auxClick beforeToggle cancel canPlay canPlayThrough click close contextMenu copy cut drag dragEnd dragEnter dragExit dragLeave dragOver dragStart drop durationChange emptied encrypted ended error gotPointerCapture input invalid keyDown keyPress keyUp load loadedData loadedMetadata loadStart lostPointerCapture mouseDown mouseMove mouseOut mouseOver mouseUp paste pause play playing pointerCancel pointerDown pointerMove pointerOut pointerOver pointerUp progress rateChange reset resize seeked seeking stalled submit suspend timeUpdate touchCancel touchEnd touchStart volumeChange scroll toggle touchMove waiting wheel".split(" ");Qo.push("scrollEnd");function ra(e,t){xd.set(e,t),wa(t,[e])}var rf=typeof reportError=="function"?reportError:function(e){if(typeof window=="object"&&typeof window.ErrorEvent=="function"){var t=new window.ErrorEvent("error",{bubbles:!0,cancelable:!0,message:typeof e=="object"&&e!==null&&typeof e.message=="string"?String(e.message):String(e),error:e});if(!window.dispatchEvent(t))return}else if(typeof process=="object"&&typeof process.emit=="function"){process.emit("uncaughtException",e);return}console.error(e)},Xn=[],Fi=0,Zo=0;function uf(){for(var e=Fi,t=Zo=Fi=0;t<e;){var n=Xn[t];Xn[t++]=null;var l=Xn[t];Xn[t++]=null;var o=Xn[t];Xn[t++]=null;var s=Xn[t];if(Xn[t++]=null,l!==null&&o!==null){var b=l.pending;b===null?o.next=o:(o.next=b.next,b.next=o),l.pending=o}s!==0&&Dd(n,o,s)}}function ff(e,t,n,l){Xn[Fi++]=e,Xn[Fi++]=t,Xn[Fi++]=n,Xn[Fi++]=l,Zo|=l,e.lanes|=l,e=e.alternate,e!==null&&(e.lanes|=l)}function Ko(e,t,n,l){return ff(e,t,n,l),of(e)}function ui(e,t){return ff(e,null,null,t),of(e)}function Dd(e,t,n){e.lanes|=n;var l=e.alternate;l!==null&&(l.lanes|=n);for(var o=!1,s=e.return;s!==null;)s.childLanes|=n,l=s.alternate,l!==null&&(l.childLanes|=n),s.tag===22&&(e=s.stateNode,e===null||e._visibility&1||(o=!0)),e=s,s=s.return;return e.tag===3?(s=e.stateNode,o&&t!==null&&(o=31-ln(n),e=s.hiddenUpdates,l=e[o],l===null?e[o]=[t]:l.push(t),t.lane=n|536870912),s):null}function of(e){if(50<_u)throw _u=0,ts=null,Error(f(185));for(var t=e.return;t!==null;)e=t,t=e.return;return e.tag===3?e.stateNode:null}var Wi={};function Ym(e,t,n,l){this.tag=e,this.key=n,this.sibling=this.child=this.return=this.stateNode=this.type=this.elementType=null,this.index=0,this.refCleanup=this.ref=null,this.pendingProps=t,this.dependencies=this.memoizedState=this.updateQueue=this.memoizedProps=null,this.mode=l,this.subtreeFlags=this.flags=0,this.deletions=null,this.childLanes=this.lanes=0,this.alternate=null}function _n(e,t,n,l){return new Ym(e,t,n,l)}function Jo(e){return e=e.prototype,!(!e||!e.isReactComponent)}function qa(e,t){var n=e.alternate;return n===null?(n=_n(e.tag,t,e.key,e.mode),n.elementType=e.elementType,n.type=e.type,n.stateNode=e.stateNode,n.alternate=e,e.alternate=n):(n.pendingProps=t,n.type=e.type,n.flags=0,n.subtreeFlags=0,n.deletions=null),n.flags=e.flags&65011712,n.childLanes=e.childLanes,n.lanes=e.lanes,n.child=e.child,n.memoizedProps=e.memoizedProps,n.memoizedState=e.memoizedState,n.updateQueue=e.updateQueue,t=e.dependencies,n.dependencies=t===null?null:{lanes:t.lanes,firstContext:t.firstContext},n.sibling=e.sibling,n.index=e.index,n.ref=e.ref,n.refCleanup=e.refCleanup,n}function Td(e,t){e.flags&=65011714;var n=e.alternate;return n===null?(e.childLanes=0,e.lanes=t,e.child=null,e.subtreeFlags=0,e.memoizedProps=null,e.memoizedState=null,e.updateQueue=null,e.dependencies=null,e.stateNode=null):(e.childLanes=n.childLanes,e.lanes=n.lanes,e.child=n.child,e.subtreeFlags=0,e.deletions=null,e.memoizedProps=n.memoizedProps,e.memoizedState=n.memoizedState,e.updateQueue=n.updateQueue,e.type=n.type,t=n.dependencies,e.dependencies=t===null?null:{lanes:t.lanes,firstContext:t.firstContext}),e}function cf(e,t,n,l,o,s){var b=0;if(l=e,typeof e=="function")Jo(e)&&(b=1);else if(typeof e=="string")b=Zy(e,n,bt.current)?26:e==="html"||e==="head"||e==="body"?27:5;else e:switch(e){case Ce:return e=_n(31,n,t,o),e.elementType=Ce,e.lanes=s,e;case B:return fi(n.children,o,s,t);case Y:b=8,o|=24;break;case F:return e=_n(12,n,t,o|2),e.elementType=F,e.lanes=s,e;case Oe:return e=_n(13,n,t,o),e.elementType=Oe,e.lanes=s,e;case ge:return e=_n(19,n,t,o),e.elementType=ge,e.lanes=s,e;default:if(typeof e=="object"&&e!==null)switch(e.$$typeof){case Se:b=10;break e;case re:b=9;break e;case Te:b=11;break e;case y:b=14;break e;case Ze:b=16,l=null;break e}b=29,n=Error(f(130,e===null?"null":typeof e,"")),l=null}return t=_n(b,n,t,o),t.elementType=e,t.type=l,t.lanes=s,t}function fi(e,t,n,l){return e=_n(7,e,l,t),e.lanes=n,e}function ko(e,t,n){return e=_n(6,e,null,t),e.lanes=n,e}function Cd(e){var t=_n(18,null,null,0);return t.stateNode=e,t}function Fo(e,t,n){return t=_n(4,e.children!==null?e.children:[],e.key,t),t.lanes=n,t.stateNode={containerInfo:e.containerInfo,pendingChildren:null,implementation:e.implementation},t}var Ad=new WeakMap;function Gn(e,t){if(typeof e=="object"&&e!==null){var n=Ad.get(e);return n!==void 0?n:(t={value:e,source:t,stack:On(t)},Ad.set(e,t),t)}return{value:e,source:t,stack:On(t)}}var $i=[],Ii=0,sf=null,eu=0,Vn=[],Qn=0,hl=null,ba=1,ga="";function Ua(e,t){$i[Ii++]=eu,$i[Ii++]=sf,sf=e,eu=t}function Ed(e,t,n){Vn[Qn++]=ba,Vn[Qn++]=ga,Vn[Qn++]=hl,hl=e;var l=ba;e=ga;var o=32-ln(l)-1;l&=~(1<<o),n+=1;var s=32-ln(t)+o;if(30<s){var b=o-o%5;s=(l&(1<<b)-1).toString(32),l>>=b,o-=b,ba=1<<32-ln(t)+o|n<<o|l,ga=s+e}else ba=1<<s|n<<o|l,ga=e}function Wo(e){e.return!==null&&(Ua(e,1),Ed(e,1,0))}function $o(e){for(;e===sf;)sf=$i[--Ii],$i[Ii]=null,eu=$i[--Ii],$i[Ii]=null;for(;e===hl;)hl=Vn[--Qn],Vn[Qn]=null,ga=Vn[--Qn],Vn[Qn]=null,ba=Vn[--Qn],Vn[Qn]=null}function Nd(e,t){Vn[Qn++]=ba,Vn[Qn++]=ga,Vn[Qn++]=hl,ba=t.id,ga=t.overflow,hl=e}var Jt=null,ct=null,Ve=!1,pl=null,Zn=!1,Io=Error(f(519));function vl(e){var t=Error(f(418,1<arguments.length&&arguments[1]!==void 0&&arguments[1]?"text":"HTML",""));throw tu(Gn(t,e)),Io}function Md(e){var t=e.stateNode,n=e.type,l=e.memoizedProps;switch(t[Ot]=e,t[Pt]=l,n){case"dialog":je("cancel",t),je("close",t);break;case"iframe":case"object":case"embed":je("load",t);break;case"video":case"audio":for(n=0;n<Du.length;n++)je(Du[n],t);break;case"source":je("error",t);break;case"img":case"image":case"link":je("error",t),je("load",t);break;case"details":je("toggle",t);break;case"input":je("invalid",t),Pu(t,l.value,l.defaultValue,l.checked,l.defaultChecked,l.type,l.name,!0);break;case"select":je("invalid",t);break;case"textarea":je("invalid",t),ef(t,l.value,l.defaultValue,l.children)}n=l.children,typeof n!="string"&&typeof n!="number"&&typeof n!="bigint"||t.textContent===""+n||l.suppressHydrationWarning===!0||Kp(t.textContent,n)?(l.popover!=null&&(je("beforetoggle",t),je("toggle",t)),l.onScroll!=null&&je("scroll",t),l.onScrollEnd!=null&&je("scrollend",t),l.onClick!=null&&(t.onclick=Ln),t=!0):t=!1,t||vl(e,!0)}function wd(e){for(Jt=e.return;Jt;)switch(Jt.tag){case 5:case 31:case 13:Zn=!1;return;case 27:case 3:Zn=!0;return;default:Jt=Jt.return}}function Pi(e){if(e!==Jt)return!1;if(!Ve)return wd(e),Ve=!0,!1;var t=e.tag,n;if((n=t!==3&&t!==27)&&((n=t===5)&&(n=e.type,n=!(n!=="form"&&n!=="button")||ms(e.type,e.memoizedProps)),n=!n),n&&ct&&vl(e),wd(e),t===13){if(e=e.memoizedState,e=e!==null?e.dehydrated:null,!e)throw Error(f(317));ct=tv(e)}else if(t===31){if(e=e.memoizedState,e=e!==null?e.dehydrated:null,!e)throw Error(f(317));ct=tv(e)}else t===27?(t=ct,Ml(e.type)?(e=_s,_s=null,ct=e):ct=t):ct=Jt?Jn(e.stateNode.nextSibling):null;return!0}function oi(){ct=Jt=null,Ve=!1}function Po(){var e=pl;return e!==null&&(bn===null?bn=e:bn.push.apply(bn,e),pl=null),e}function tu(e){pl===null?pl=[e]:pl.push(e)}var ec=Rt(null),ci=null,Ba=null;function ml(e,t,n){Qe(ec,t._currentValue),t._currentValue=n}function Ya(e){e._currentValue=ec.current,ot(ec)}function tc(e,t,n){for(;e!==null;){var l=e.alternate;if((e.childLanes&t)!==t?(e.childLanes|=t,l!==null&&(l.childLanes|=t)):l!==null&&(l.childLanes&t)!==t&&(l.childLanes|=t),e===n)break;e=e.return}}function nc(e,t,n,l){var o=e.child;for(o!==null&&(o.return=e);o!==null;){var s=o.dependencies;if(s!==null){var b=o.child;s=s.firstContext;e:for(;s!==null;){var C=s;s=o;for(var z=0;z<t.length;z++)if(C.context===t[z]){s.lanes|=n,C=s.alternate,C!==null&&(C.lanes|=n),tc(s.return,n,e),l||(b=null);break e}s=C.next}}else if(o.tag===18){if(b=o.return,b===null)throw Error(f(341));b.lanes|=n,s=b.alternate,s!==null&&(s.lanes|=n),tc(b,n,e),b=null}else b=o.child;if(b!==null)b.return=o;else for(b=o;b!==null;){if(b===e){b=null;break}if(o=b.sibling,o!==null){o.return=b.return,b=o;break}b=b.return}o=b}}function er(e,t,n,l){e=null;for(var o=t,s=!1;o!==null;){if(!s){if((o.flags&524288)!==0)s=!0;else if((o.flags&262144)!==0)break}if(o.tag===10){var b=o.alternate;if(b===null)throw Error(f(387));if(b=b.memoizedProps,b!==null){var C=o.type;Zt(o.pendingProps.value,b.value)||(e!==null?e.push(C):e=[C])}}else if(o===Ai.current){if(b=o.alternate,b===null)throw Error(f(387));b.memoizedState.memoizedState!==o.memoizedState.memoizedState&&(e!==null?e.push(Nu):e=[Nu])}o=o.return}e!==null&&nc(t,e,n,l),t.flags|=262144}function df(e){for(e=e.firstContext;e!==null;){if(!Zt(e.context._currentValue,e.memoizedValue))return!0;e=e.next}return!1}function si(e){ci=e,Ba=null,e=e.dependencies,e!==null&&(e.firstContext=null)}function kt(e){return Od(ci,e)}function hf(e,t){return ci===null&&si(e),Od(e,t)}function Od(e,t){var n=t._currentValue;if(t={context:t,memoizedValue:n,next:null},Ba===null){if(e===null)throw Error(f(308));Ba=t,e.dependencies={lanes:0,firstContext:t},e.flags|=524288}else Ba=Ba.next=t;return n}var Xm=typeof AbortController<"u"?AbortController:function(){var e=[],t=this.signal={aborted:!1,addEventListener:function(n,l){e.push(l)}};this.abort=function(){t.aborted=!0,e.forEach(function(n){return n()})}},Gm=a.unstable_scheduleCallback,Vm=a.unstable_NormalPriority,At={$$typeof:Se,Consumer:null,Provider:null,_currentValue:null,_currentValue2:null,_threadCount:0};function ac(){return{controller:new Xm,data:new Map,refCount:0}}function nu(e){e.refCount--,e.refCount===0&&Gm(Vm,function(){e.controller.abort()})}var au=null,lc=0,tr=0,nr=null;function Qm(e,t){if(au===null){var n=au=[];lc=0,tr=us(),nr={status:"pending",value:void 0,then:function(l){n.push(l)}}}return lc++,t.then(zd,zd),t}function zd(){if(--lc===0&&au!==null){nr!==null&&(nr.status="fulfilled");var e=au;au=null,tr=0,nr=null;for(var t=0;t<e.length;t++)(0,e[t])()}}function Zm(e,t){var n=[],l={status:"pending",value:null,reason:null,then:function(o){n.push(o)}};return e.then(function(){l.status="fulfilled",l.value=t;for(var o=0;o<n.length;o++)(0,n[o])(t)},function(o){for(l.status="rejected",l.reason=o,o=0;o<n.length;o++)(0,n[o])(void 0)}),l}var Ld=I.S;I.S=function(e,t){mp=wt(),typeof t=="object"&&t!==null&&typeof t.then=="function"&&Qm(e,t),Ld!==null&&Ld(e,t)};var di=Rt(null);function ic(){var e=di.current;return e!==null?e:ft.pooledCache}function pf(e,t){t===null?Qe(di,di.current):Qe(di,t.pool)}function Hd(){var e=ic();return e===null?null:{parent:At._currentValue,pool:e}}var ar=Error(f(460)),rc=Error(f(474)),vf=Error(f(542)),mf={then:function(){}};function jd(e){return e=e.status,e==="fulfilled"||e==="rejected"}function Rd(e,t,n){switch(n=e[n],n===void 0?e.push(t):n!==t&&(t.then(Ln,Ln),t=n),t.status){case"fulfilled":return t.value;case"rejected":throw e=t.reason,Ud(e),e;default:if(typeof t.status=="string")t.then(Ln,Ln);else{if(e=ft,e!==null&&100<e.shellSuspendCounter)throw Error(f(482));e=t,e.status="pending",e.then(function(l){if(t.status==="pending"){var o=t;o.status="fulfilled",o.value=l}},function(l){if(t.status==="pending"){var o=t;o.status="rejected",o.reason=l}})}switch(t.status){case"fulfilled":return t.value;case"rejected":throw e=t.reason,Ud(e),e}throw pi=t,ar}}function hi(e){try{var t=e._init;return t(e._payload)}catch(n){throw n!==null&&typeof n=="object"&&typeof n.then=="function"?(pi=n,ar):n}}var pi=null;function qd(){if(pi===null)throw Error(f(459));var e=pi;return pi=null,e}function Ud(e){if(e===ar||e===vf)throw Error(f(483))}var lr=null,lu=0;function yf(e){var t=lu;return lu+=1,lr===null&&(lr=[]),Rd(lr,e,t)}function iu(e,t){t=t.props.ref,e.ref=t!==void 0?t:null}function bf(e,t){throw t.$$typeof===j?Error(f(525)):(e=Object.prototype.toString.call(t),Error(f(31,e==="[object Object]"?"object with keys {"+Object.keys(t).join(", ")+"}":e)))}function Bd(e){function t(q,H){if(e){var V=q.deletions;V===null?(q.deletions=[H],q.flags|=16):V.push(H)}}function n(q,H){if(!e)return null;for(;H!==null;)t(q,H),H=H.sibling;return null}function l(q){for(var H=new Map;q!==null;)q.key!==null?H.set(q.key,q):H.set(q.index,q),q=q.sibling;return H}function o(q,H){return q=qa(q,H),q.index=0,q.sibling=null,q}function s(q,H,V){return q.index=V,e?(V=q.alternate,V!==null?(V=V.index,V<H?(q.flags|=67108866,H):V):(q.flags|=67108866,H)):(q.flags|=1048576,H)}function b(q){return e&&q.alternate===null&&(q.flags|=67108866),q}function C(q,H,V,ne){return H===null||H.tag!==6?(H=ko(V,q.mode,ne),H.return=q,H):(H=o(H,V),H.return=q,H)}function z(q,H,V,ne){var be=V.type;return be===B?ee(q,H,V.props.children,ne,V.key):H!==null&&(H.elementType===be||typeof be=="object"&&be!==null&&be.$$typeof===Ze&&hi(be)===H.type)?(H=o(H,V.props),iu(H,V),H.return=q,H):(H=cf(V.type,V.key,V.props,null,q.mode,ne),iu(H,V),H.return=q,H)}function Q(q,H,V,ne){return H===null||H.tag!==4||H.stateNode.containerInfo!==V.containerInfo||H.stateNode.implementation!==V.implementation?(H=Fo(V,q.mode,ne),H.return=q,H):(H=o(H,V.children||[]),H.return=q,H)}function ee(q,H,V,ne,be){return H===null||H.tag!==7?(H=fi(V,q.mode,ne,be),H.return=q,H):(H=o(H,V),H.return=q,H)}function ae(q,H,V){if(typeof H=="string"&&H!==""||typeof H=="number"||typeof H=="bigint")return H=ko(""+H,q.mode,V),H.return=q,H;if(typeof H=="object"&&H!==null){switch(H.$$typeof){case X:return V=cf(H.type,H.key,H.props,null,q.mode,V),iu(V,H),V.return=q,V;case U:return H=Fo(H,q.mode,V),H.return=q,H;case Ze:return H=hi(H),ae(q,H,V)}if($e(H)||Xe(H))return H=fi(H,q.mode,V,null),H.return=q,H;if(typeof H.then=="function")return ae(q,yf(H),V);if(H.$$typeof===Se)return ae(q,hf(q,H),V);bf(q,H)}return null}function K(q,H,V,ne){var be=H!==null?H.key:null;if(typeof V=="string"&&V!==""||typeof V=="number"||typeof V=="bigint")return be!==null?null:C(q,H,""+V,ne);if(typeof V=="object"&&V!==null){switch(V.$$typeof){case X:return V.key===be?z(q,H,V,ne):null;case U:return V.key===be?Q(q,H,V,ne):null;case Ze:return V=hi(V),K(q,H,V,ne)}if($e(V)||Xe(V))return be!==null?null:ee(q,H,V,ne,null);if(typeof V.then=="function")return K(q,H,yf(V),ne);if(V.$$typeof===Se)return K(q,H,hf(q,V),ne);bf(q,V)}return null}function k(q,H,V,ne,be){if(typeof ne=="string"&&ne!==""||typeof ne=="number"||typeof ne=="bigint")return q=q.get(V)||null,C(H,q,""+ne,be);if(typeof ne=="object"&&ne!==null){switch(ne.$$typeof){case X:return q=q.get(ne.key===null?V:ne.key)||null,z(H,q,ne,be);case U:return q=q.get(ne.key===null?V:ne.key)||null,Q(H,q,ne,be);case Ze:return ne=hi(ne),k(q,H,V,ne,be)}if($e(ne)||Xe(ne))return q=q.get(V)||null,ee(H,q,ne,be,null);if(typeof ne.then=="function")return k(q,H,V,yf(ne),be);if(ne.$$typeof===Se)return k(q,H,V,hf(H,ne),be);bf(H,ne)}return null}function ce(q,H,V,ne){for(var be=null,ke=null,de=H,we=H=0,Ye=null;de!==null&&we<V.length;we++){de.index>we?(Ye=de,de=null):Ye=de.sibling;var Fe=K(q,de,V[we],ne);if(Fe===null){de===null&&(de=Ye);break}e&&de&&Fe.alternate===null&&t(q,de),H=s(Fe,H,we),ke===null?be=Fe:ke.sibling=Fe,ke=Fe,de=Ye}if(we===V.length)return n(q,de),Ve&&Ua(q,we),be;if(de===null){for(;we<V.length;we++)de=ae(q,V[we],ne),de!==null&&(H=s(de,H,we),ke===null?be=de:ke.sibling=de,ke=de);return Ve&&Ua(q,we),be}for(de=l(de);we<V.length;we++)Ye=k(de,q,we,V[we],ne),Ye!==null&&(e&&Ye.alternate!==null&&de.delete(Ye.key===null?we:Ye.key),H=s(Ye,H,we),ke===null?be=Ye:ke.sibling=Ye,ke=Ye);return e&&de.forEach(function(Hl){return t(q,Hl)}),Ve&&Ua(q,we),be}function _e(q,H,V,ne){if(V==null)throw Error(f(151));for(var be=null,ke=null,de=H,we=H=0,Ye=null,Fe=V.next();de!==null&&!Fe.done;we++,Fe=V.next()){de.index>we?(Ye=de,de=null):Ye=de.sibling;var Hl=K(q,de,Fe.value,ne);if(Hl===null){de===null&&(de=Ye);break}e&&de&&Hl.alternate===null&&t(q,de),H=s(Hl,H,we),ke===null?be=Hl:ke.sibling=Hl,ke=Hl,de=Ye}if(Fe.done)return n(q,de),Ve&&Ua(q,we),be;if(de===null){for(;!Fe.done;we++,Fe=V.next())Fe=ae(q,Fe.value,ne),Fe!==null&&(H=s(Fe,H,we),ke===null?be=Fe:ke.sibling=Fe,ke=Fe);return Ve&&Ua(q,we),be}for(de=l(de);!Fe.done;we++,Fe=V.next())Fe=k(de,q,we,Fe.value,ne),Fe!==null&&(e&&Fe.alternate!==null&&de.delete(Fe.key===null?we:Fe.key),H=s(Fe,H,we),ke===null?be=Fe:ke.sibling=Fe,ke=Fe);return e&&de.forEach(function(n0){return t(q,n0)}),Ve&&Ua(q,we),be}function ut(q,H,V,ne){if(typeof V=="object"&&V!==null&&V.type===B&&V.key===null&&(V=V.props.children),typeof V=="object"&&V!==null){switch(V.$$typeof){case X:e:{for(var be=V.key;H!==null;){if(H.key===be){if(be=V.type,be===B){if(H.tag===7){n(q,H.sibling),ne=o(H,V.props.children),ne.return=q,q=ne;break e}}else if(H.elementType===be||typeof be=="object"&&be!==null&&be.$$typeof===Ze&&hi(be)===H.type){n(q,H.sibling),ne=o(H,V.props),iu(ne,V),ne.return=q,q=ne;break e}n(q,H);break}else t(q,H);H=H.sibling}V.type===B?(ne=fi(V.props.children,q.mode,ne,V.key),ne.return=q,q=ne):(ne=cf(V.type,V.key,V.props,null,q.mode,ne),iu(ne,V),ne.return=q,q=ne)}return b(q);case U:e:{for(be=V.key;H!==null;){if(H.key===be)if(H.tag===4&&H.stateNode.containerInfo===V.containerInfo&&H.stateNode.implementation===V.implementation){n(q,H.sibling),ne=o(H,V.children||[]),ne.return=q,q=ne;break e}else{n(q,H);break}else t(q,H);H=H.sibling}ne=Fo(V,q.mode,ne),ne.return=q,q=ne}return b(q);case Ze:return V=hi(V),ut(q,H,V,ne)}if($e(V))return ce(q,H,V,ne);if(Xe(V)){if(be=Xe(V),typeof be!="function")throw Error(f(150));return V=be.call(V),_e(q,H,V,ne)}if(typeof V.then=="function")return ut(q,H,yf(V),ne);if(V.$$typeof===Se)return ut(q,H,hf(q,V),ne);bf(q,V)}return typeof V=="string"&&V!==""||typeof V=="number"||typeof V=="bigint"?(V=""+V,H!==null&&H.tag===6?(n(q,H.sibling),ne=o(H,V),ne.return=q,q=ne):(n(q,H),ne=ko(V,q.mode,ne),ne.return=q,q=ne),b(q)):n(q,H)}return function(q,H,V,ne){try{lu=0;var be=ut(q,H,V,ne);return lr=null,be}catch(de){if(de===ar||de===vf)throw de;var ke=_n(29,de,null,q.mode);return ke.lanes=ne,ke.return=q,ke}finally{}}}var vi=Bd(!0),Yd=Bd(!1),yl=!1;function uc(e){e.updateQueue={baseState:e.memoizedState,firstBaseUpdate:null,lastBaseUpdate:null,shared:{pending:null,lanes:0,hiddenCallbacks:null},callbacks:null}}function fc(e,t){e=e.updateQueue,t.updateQueue===e&&(t.updateQueue={baseState:e.baseState,firstBaseUpdate:e.firstBaseUpdate,lastBaseUpdate:e.lastBaseUpdate,shared:e.shared,callbacks:null})}function bl(e){return{lane:e,tag:0,payload:null,callback:null,next:null}}function gl(e,t,n){var l=e.updateQueue;if(l===null)return null;if(l=l.shared,(Ie&2)!==0){var o=l.pending;return o===null?t.next=t:(t.next=o.next,o.next=t),l.pending=t,t=of(e),Dd(e,null,n),t}return ff(e,l,t,n),of(e)}function ru(e,t,n){if(t=t.updateQueue,t!==null&&(t=t.shared,(n&4194048)!==0)){var l=t.lanes;l&=e.pendingLanes,n|=l,t.lanes=n,Ca(e,n)}}function oc(e,t){var n=e.updateQueue,l=e.alternate;if(l!==null&&(l=l.updateQueue,n===l)){var o=null,s=null;if(n=n.firstBaseUpdate,n!==null){do{var b={lane:n.lane,tag:n.tag,payload:n.payload,callback:null,next:null};s===null?o=s=b:s=s.next=b,n=n.next}while(n!==null);s===null?o=s=t:s=s.next=t}else o=s=t;n={baseState:l.baseState,firstBaseUpdate:o,lastBaseUpdate:s,shared:l.shared,callbacks:l.callbacks},e.updateQueue=n;return}e=n.lastBaseUpdate,e===null?n.firstBaseUpdate=t:e.next=t,n.lastBaseUpdate=t}var cc=!1;function uu(){if(cc){var e=nr;if(e!==null)throw e}}function fu(e,t,n,l){cc=!1;var o=e.updateQueue;yl=!1;var s=o.firstBaseUpdate,b=o.lastBaseUpdate,C=o.shared.pending;if(C!==null){o.shared.pending=null;var z=C,Q=z.next;z.next=null,b===null?s=Q:b.next=Q,b=z;var ee=e.alternate;ee!==null&&(ee=ee.updateQueue,C=ee.lastBaseUpdate,C!==b&&(C===null?ee.firstBaseUpdate=Q:C.next=Q,ee.lastBaseUpdate=z))}if(s!==null){var ae=o.baseState;b=0,ee=Q=z=null,C=s;do{var K=C.lane&-536870913,k=K!==C.lane;if(k?(Be&K)===K:(l&K)===K){K!==0&&K===tr&&(cc=!0),ee!==null&&(ee=ee.next={lane:0,tag:C.tag,payload:C.payload,callback:null,next:null});e:{var ce=e,_e=C;K=t;var ut=n;switch(_e.tag){case 1:if(ce=_e.payload,typeof ce=="function"){ae=ce.call(ut,ae,K);break e}ae=ce;break e;case 3:ce.flags=ce.flags&-65537|128;case 0:if(ce=_e.payload,K=typeof ce=="function"?ce.call(ut,ae,K):ce,K==null)break e;ae=N({},ae,K);break e;case 2:yl=!0}}K=C.callback,K!==null&&(e.flags|=64,k&&(e.flags|=8192),k=o.callbacks,k===null?o.callbacks=[K]:k.push(K))}else k={lane:K,tag:C.tag,payload:C.payload,callback:C.callback,next:null},ee===null?(Q=ee=k,z=ae):ee=ee.next=k,b|=K;if(C=C.next,C===null){if(C=o.shared.pending,C===null)break;k=C,C=k.next,k.next=null,o.lastBaseUpdate=k,o.shared.pending=null}}while(!0);ee===null&&(z=ae),o.baseState=z,o.firstBaseUpdate=Q,o.lastBaseUpdate=ee,s===null&&(o.shared.lanes=0),Tl|=b,e.lanes=b,e.memoizedState=ae}}function Xd(e,t){if(typeof e!="function")throw Error(f(191,e));e.call(t)}function Gd(e,t){var n=e.callbacks;if(n!==null)for(e.callbacks=null,e=0;e<n.length;e++)Xd(n[e],t)}var ir=Rt(null),gf=Rt(0);function Vd(e,t){e=Fa,Qe(gf,e),Qe(ir,t),Fa=e|t.baseLanes}function sc(){Qe(gf,Fa),Qe(ir,ir.current)}function dc(){Fa=gf.current,ot(ir),ot(gf)}var xn=Rt(null),Kn=null;function Sl(e){var t=e.alternate;Qe(xt,xt.current&1),Qe(xn,e),Kn===null&&(t===null||ir.current!==null||t.memoizedState!==null)&&(Kn=e)}function hc(e){Qe(xt,xt.current),Qe(xn,e),Kn===null&&(Kn=e)}function Qd(e){e.tag===22?(Qe(xt,xt.current),Qe(xn,e),Kn===null&&(Kn=e)):_l()}function _l(){Qe(xt,xt.current),Qe(xn,xn.current)}function Dn(e){ot(xn),Kn===e&&(Kn=null),ot(xt)}var xt=Rt(0);function Sf(e){for(var t=e;t!==null;){if(t.tag===13){var n=t.memoizedState;if(n!==null&&(n=n.dehydrated,n===null||gs(n)||Ss(n)))return t}else if(t.tag===19&&(t.memoizedProps.revealOrder==="forwards"||t.memoizedProps.revealOrder==="backwards"||t.memoizedProps.revealOrder==="unstable_legacy-backwards"||t.memoizedProps.revealOrder==="together")){if((t.flags&128)!==0)return t}else if(t.child!==null){t.child.return=t,t=t.child;continue}if(t===e)break;for(;t.sibling===null;){if(t.return===null||t.return===e)return null;t=t.return}t.sibling.return=t.return,t=t.sibling}return null}var Xa=0,Ne=null,it=null,Et=null,_f=!1,rr=!1,mi=!1,xf=0,ou=0,ur=null,Km=0;function mt(){throw Error(f(321))}function pc(e,t){if(t===null)return!1;for(var n=0;n<t.length&&n<e.length;n++)if(!Zt(e[n],t[n]))return!1;return!0}function vc(e,t,n,l,o,s){return Xa=s,Ne=t,t.memoizedState=null,t.updateQueue=null,t.lanes=0,I.H=e===null||e.memoizedState===null?Eh:wc,mi=!1,s=n(l,o),mi=!1,rr&&(s=Kd(t,n,l,o)),Zd(e),s}function Zd(e){I.H=du;var t=it!==null&&it.next!==null;if(Xa=0,Et=it=Ne=null,_f=!1,ou=0,ur=null,t)throw Error(f(300));e===null||Nt||(e=e.dependencies,e!==null&&df(e)&&(Nt=!0))}function Kd(e,t,n,l){Ne=e;var o=0;do{if(rr&&(ur=null),ou=0,rr=!1,25<=o)throw Error(f(301));if(o+=1,Et=it=null,e.updateQueue!=null){var s=e.updateQueue;s.lastEffect=null,s.events=null,s.stores=null,s.memoCache!=null&&(s.memoCache.index=0)}I.H=Nh,s=t(n,l)}while(rr);return s}function Jm(){var e=I.H,t=e.useState()[0];return t=typeof t.then=="function"?cu(t):t,e=e.useState()[0],(it!==null?it.memoizedState:null)!==e&&(Ne.flags|=1024),t}function mc(){var e=xf!==0;return xf=0,e}function yc(e,t,n){t.updateQueue=e.updateQueue,t.flags&=-2053,e.lanes&=~n}function bc(e){if(_f){for(e=e.memoizedState;e!==null;){var t=e.queue;t!==null&&(t.pending=null),e=e.next}_f=!1}Xa=0,Et=it=Ne=null,rr=!1,ou=xf=0,ur=null}function on(){var e={memoizedState:null,baseState:null,baseQueue:null,queue:null,next:null};return Et===null?Ne.memoizedState=Et=e:Et=Et.next=e,Et}function Dt(){if(it===null){var e=Ne.alternate;e=e!==null?e.memoizedState:null}else e=it.next;var t=Et===null?Ne.memoizedState:Et.next;if(t!==null)Et=t,it=e;else{if(e===null)throw Ne.alternate===null?Error(f(467)):Error(f(310));it=e,e={memoizedState:it.memoizedState,baseState:it.baseState,baseQueue:it.baseQueue,queue:it.queue,next:null},Et===null?Ne.memoizedState=Et=e:Et=Et.next=e}return Et}function Df(){return{lastEffect:null,events:null,stores:null,memoCache:null}}function cu(e){var t=ou;return ou+=1,ur===null&&(ur=[]),e=Rd(ur,e,t),t=Ne,(Et===null?t.memoizedState:Et.next)===null&&(t=t.alternate,I.H=t===null||t.memoizedState===null?Eh:wc),e}function Tf(e){if(e!==null&&typeof e=="object"){if(typeof e.then=="function")return cu(e);if(e.$$typeof===Se)return kt(e)}throw Error(f(438,String(e)))}function gc(e){var t=null,n=Ne.updateQueue;if(n!==null&&(t=n.memoCache),t==null){var l=Ne.alternate;l!==null&&(l=l.updateQueue,l!==null&&(l=l.memoCache,l!=null&&(t={data:l.data.map(function(o){return o.slice()}),index:0})))}if(t==null&&(t={data:[],index:0}),n===null&&(n=Df(),Ne.updateQueue=n),n.memoCache=t,n=t.data[t.index],n===void 0)for(n=t.data[t.index]=Array(e),l=0;l<e;l++)n[l]=Tt;return t.index++,n}function Ga(e,t){return typeof t=="function"?t(e):t}function Cf(e){var t=Dt();return Sc(t,it,e)}function Sc(e,t,n){var l=e.queue;if(l===null)throw Error(f(311));l.lastRenderedReducer=n;var o=e.baseQueue,s=l.pending;if(s!==null){if(o!==null){var b=o.next;o.next=s.next,s.next=b}t.baseQueue=o=s,l.pending=null}if(s=e.baseState,o===null)e.memoizedState=s;else{t=o.next;var C=b=null,z=null,Q=t,ee=!1;do{var ae=Q.lane&-536870913;if(ae!==Q.lane?(Be&ae)===ae:(Xa&ae)===ae){var K=Q.revertLane;if(K===0)z!==null&&(z=z.next={lane:0,revertLane:0,gesture:null,action:Q.action,hasEagerState:Q.hasEagerState,eagerState:Q.eagerState,next:null}),ae===tr&&(ee=!0);else if((Xa&K)===K){Q=Q.next,K===tr&&(ee=!0);continue}else ae={lane:0,revertLane:Q.revertLane,gesture:null,action:Q.action,hasEagerState:Q.hasEagerState,eagerState:Q.eagerState,next:null},z===null?(C=z=ae,b=s):z=z.next=ae,Ne.lanes|=K,Tl|=K;ae=Q.action,mi&&n(s,ae),s=Q.hasEagerState?Q.eagerState:n(s,ae)}else K={lane:ae,revertLane:Q.revertLane,gesture:Q.gesture,action:Q.action,hasEagerState:Q.hasEagerState,eagerState:Q.eagerState,next:null},z===null?(C=z=K,b=s):z=z.next=K,Ne.lanes|=ae,Tl|=ae;Q=Q.next}while(Q!==null&&Q!==t);if(z===null?b=s:z.next=C,!Zt(s,e.memoizedState)&&(Nt=!0,ee&&(n=nr,n!==null)))throw n;e.memoizedState=s,e.baseState=b,e.baseQueue=z,l.lastRenderedState=s}return o===null&&(l.lanes=0),[e.memoizedState,l.dispatch]}function _c(e){var t=Dt(),n=t.queue;if(n===null)throw Error(f(311));n.lastRenderedReducer=e;var l=n.dispatch,o=n.pending,s=t.memoizedState;if(o!==null){n.pending=null;var b=o=o.next;do s=e(s,b.action),b=b.next;while(b!==o);Zt(s,t.memoizedState)||(Nt=!0),t.memoizedState=s,t.baseQueue===null&&(t.baseState=s),n.lastRenderedState=s}return[s,l]}function Jd(e,t,n){var l=Ne,o=Dt(),s=Ve;if(s){if(n===void 0)throw Error(f(407));n=n()}else n=t();var b=!Zt((it||o).memoizedState,n);if(b&&(o.memoizedState=n,Nt=!0),o=o.queue,Tc(Wd.bind(null,l,o,e),[e]),o.getSnapshot!==t||b||Et!==null&&Et.memoizedState.tag&1){if(l.flags|=2048,fr(9,{destroy:void 0},Fd.bind(null,l,o,n,t),null),ft===null)throw Error(f(349));s||(Xa&127)!==0||kd(l,t,n)}return n}function kd(e,t,n){e.flags|=16384,e={getSnapshot:t,value:n},t=Ne.updateQueue,t===null?(t=Df(),Ne.updateQueue=t,t.stores=[e]):(n=t.stores,n===null?t.stores=[e]:n.push(e))}function Fd(e,t,n,l){t.value=n,t.getSnapshot=l,$d(t)&&Id(e)}function Wd(e,t,n){return n(function(){$d(t)&&Id(e)})}function $d(e){var t=e.getSnapshot;e=e.value;try{var n=t();return!Zt(e,n)}catch{return!0}}function Id(e){var t=ui(e,2);t!==null&&gn(t,e,2)}function xc(e){var t=on();if(typeof e=="function"){var n=e;if(e=n(),mi){an(!0);try{n()}finally{an(!1)}}}return t.memoizedState=t.baseState=e,t.queue={pending:null,lanes:0,dispatch:null,lastRenderedReducer:Ga,lastRenderedState:e},t}function Pd(e,t,n,l){return e.baseState=n,Sc(e,it,typeof l=="function"?l:Ga)}function km(e,t,n,l,o){if(Nf(e))throw Error(f(485));if(e=t.action,e!==null){var s={payload:o,action:e,next:null,isTransition:!0,status:"pending",value:null,reason:null,listeners:[],then:function(b){s.listeners.push(b)}};I.T!==null?n(!0):s.isTransition=!1,l(s),n=t.pending,n===null?(s.next=t.pending=s,eh(t,s)):(s.next=n.next,t.pending=n.next=s)}}function eh(e,t){var n=t.action,l=t.payload,o=e.state;if(t.isTransition){var s=I.T,b={};I.T=b;try{var C=n(o,l),z=I.S;z!==null&&z(b,C),th(e,t,C)}catch(Q){Dc(e,t,Q)}finally{s!==null&&b.types!==null&&(s.types=b.types),I.T=s}}else try{s=n(o,l),th(e,t,s)}catch(Q){Dc(e,t,Q)}}function th(e,t,n){n!==null&&typeof n=="object"&&typeof n.then=="function"?n.then(function(l){nh(e,t,l)},function(l){return Dc(e,t,l)}):nh(e,t,n)}function nh(e,t,n){t.status="fulfilled",t.value=n,ah(t),e.state=n,t=e.pending,t!==null&&(n=t.next,n===t?e.pending=null:(n=n.next,t.next=n,eh(e,n)))}function Dc(e,t,n){var l=e.pending;if(e.pending=null,l!==null){l=l.next;do t.status="rejected",t.reason=n,ah(t),t=t.next;while(t!==l)}e.action=null}function ah(e){e=e.listeners;for(var t=0;t<e.length;t++)(0,e[t])()}function lh(e,t){return t}function ih(e,t){if(Ve){var n=ft.formState;if(n!==null){e:{var l=Ne;if(Ve){if(ct){t:{for(var o=ct,s=Zn;o.nodeType!==8;){if(!s){o=null;break t}if(o=Jn(o.nextSibling),o===null){o=null;break t}}s=o.data,o=s==="F!"||s==="F"?o:null}if(o){ct=Jn(o.nextSibling),l=o.data==="F!";break e}}vl(l)}l=!1}l&&(t=n[0])}}return n=on(),n.memoizedState=n.baseState=t,l={pending:null,lanes:0,dispatch:null,lastRenderedReducer:lh,lastRenderedState:t},n.queue=l,n=Th.bind(null,Ne,l),l.dispatch=n,l=xc(!1),s=Mc.bind(null,Ne,!1,l.queue),l=on(),o={state:t,dispatch:null,action:e,pending:null},l.queue=o,n=km.bind(null,Ne,o,s,n),o.dispatch=n,l.memoizedState=e,[t,n,!1]}function rh(e){var t=Dt();return uh(t,it,e)}function uh(e,t,n){if(t=Sc(e,t,lh)[0],e=Cf(Ga)[0],typeof t=="object"&&t!==null&&typeof t.then=="function")try{var l=cu(t)}catch(b){throw b===ar?vf:b}else l=t;t=Dt();var o=t.queue,s=o.dispatch;return n!==t.memoizedState&&(Ne.flags|=2048,fr(9,{destroy:void 0},Fm.bind(null,o,n),null)),[l,s,e]}function Fm(e,t){e.action=t}function fh(e){var t=Dt(),n=it;if(n!==null)return uh(t,n,e);Dt(),t=t.memoizedState,n=Dt();var l=n.queue.dispatch;return n.memoizedState=e,[t,l,!1]}function fr(e,t,n,l){return e={tag:e,create:n,deps:l,inst:t,next:null},t=Ne.updateQueue,t===null&&(t=Df(),Ne.updateQueue=t),n=t.lastEffect,n===null?t.lastEffect=e.next=e:(l=n.next,n.next=e,e.next=l,t.lastEffect=e),e}function oh(){return Dt().memoizedState}function Af(e,t,n,l){var o=on();Ne.flags|=e,o.memoizedState=fr(1|t,{destroy:void 0},n,l===void 0?null:l)}function Ef(e,t,n,l){var o=Dt();l=l===void 0?null:l;var s=o.memoizedState.inst;it!==null&&l!==null&&pc(l,it.memoizedState.deps)?o.memoizedState=fr(t,s,n,l):(Ne.flags|=e,o.memoizedState=fr(1|t,s,n,l))}function ch(e,t){Af(8390656,8,e,t)}function Tc(e,t){Ef(2048,8,e,t)}function Wm(e){Ne.flags|=4;var t=Ne.updateQueue;if(t===null)t=Df(),Ne.updateQueue=t,t.events=[e];else{var n=t.events;n===null?t.events=[e]:n.push(e)}}function sh(e){var t=Dt().memoizedState;return Wm({ref:t,nextImpl:e}),function(){if((Ie&2)!==0)throw Error(f(440));return t.impl.apply(void 0,arguments)}}function dh(e,t){return Ef(4,2,e,t)}function hh(e,t){return Ef(4,4,e,t)}function ph(e,t){if(typeof t=="function"){e=e();var n=t(e);return function(){typeof n=="function"?n():t(null)}}if(t!=null)return e=e(),t.current=e,function(){t.current=null}}function vh(e,t,n){n=n!=null?n.concat([e]):null,Ef(4,4,ph.bind(null,t,e),n)}function Cc(){}function mh(e,t){var n=Dt();t=t===void 0?null:t;var l=n.memoizedState;return t!==null&&pc(t,l[1])?l[0]:(n.memoizedState=[e,t],e)}function yh(e,t){var n=Dt();t=t===void 0?null:t;var l=n.memoizedState;if(t!==null&&pc(t,l[1]))return l[0];if(l=e(),mi){an(!0);try{e()}finally{an(!1)}}return n.memoizedState=[l,t],l}function Ac(e,t,n){return n===void 0||(Xa&1073741824)!==0&&(Be&261930)===0?e.memoizedState=t:(e.memoizedState=n,e=bp(),Ne.lanes|=e,Tl|=e,n)}function bh(e,t,n,l){return Zt(n,t)?n:ir.current!==null?(e=Ac(e,n,l),Zt(e,t)||(Nt=!0),e):(Xa&42)===0||(Xa&1073741824)!==0&&(Be&261930)===0?(Nt=!0,e.memoizedState=n):(e=bp(),Ne.lanes|=e,Tl|=e,t)}function gh(e,t,n,l,o){var s=ue.p;ue.p=s!==0&&8>s?s:8;var b=I.T,C={};I.T=C,Mc(e,!1,t,n);try{var z=o(),Q=I.S;if(Q!==null&&Q(C,z),z!==null&&typeof z=="object"&&typeof z.then=="function"){var ee=Zm(z,l);su(e,t,ee,An(e))}else su(e,t,l,An(e))}catch(ae){su(e,t,{then:function(){},status:"rejected",reason:ae},An())}finally{ue.p=s,b!==null&&C.types!==null&&(b.types=C.types),I.T=b}}function $m(){}function Ec(e,t,n,l){if(e.tag!==5)throw Error(f(476));var o=Sh(e).queue;gh(e,o,t,me,n===null?$m:function(){return _h(e),n(l)})}function Sh(e){var t=e.memoizedState;if(t!==null)return t;t={memoizedState:me,baseState:me,baseQueue:null,queue:{pending:null,lanes:0,dispatch:null,lastRenderedReducer:Ga,lastRenderedState:me},next:null};var n={};return t.next={memoizedState:n,baseState:n,baseQueue:null,queue:{pending:null,lanes:0,dispatch:null,lastRenderedReducer:Ga,lastRenderedState:n},next:null},e.memoizedState=t,e=e.alternate,e!==null&&(e.memoizedState=t),t}function _h(e){var t=Sh(e);t.next===null&&(t=e.alternate.memoizedState),su(e,t.next.queue,{},An())}function Nc(){return kt(Nu)}function xh(){return Dt().memoizedState}function Dh(){return Dt().memoizedState}function Im(e){for(var t=e.return;t!==null;){switch(t.tag){case 24:case 3:var n=An();e=bl(n);var l=gl(t,e,n);l!==null&&(gn(l,t,n),ru(l,t,n)),t={cache:ac()},e.payload=t;return}t=t.return}}function Pm(e,t,n){var l=An();n={lane:l,revertLane:0,gesture:null,action:n,hasEagerState:!1,eagerState:null,next:null},Nf(e)?Ch(t,n):(n=Ko(e,t,n,l),n!==null&&(gn(n,e,l),Ah(n,t,l)))}function Th(e,t,n){var l=An();su(e,t,n,l)}function su(e,t,n,l){var o={lane:l,revertLane:0,gesture:null,action:n,hasEagerState:!1,eagerState:null,next:null};if(Nf(e))Ch(t,o);else{var s=e.alternate;if(e.lanes===0&&(s===null||s.lanes===0)&&(s=t.lastRenderedReducer,s!==null))try{var b=t.lastRenderedState,C=s(b,n);if(o.hasEagerState=!0,o.eagerState=C,Zt(C,b))return ff(e,t,o,0),ft===null&&uf(),!1}catch{}finally{}if(n=Ko(e,t,o,l),n!==null)return gn(n,e,l),Ah(n,t,l),!0}return!1}function Mc(e,t,n,l){if(l={lane:2,revertLane:us(),gesture:null,action:l,hasEagerState:!1,eagerState:null,next:null},Nf(e)){if(t)throw Error(f(479))}else t=Ko(e,n,l,2),t!==null&&gn(t,e,2)}function Nf(e){var t=e.alternate;return e===Ne||t!==null&&t===Ne}function Ch(e,t){rr=_f=!0;var n=e.pending;n===null?t.next=t:(t.next=n.next,n.next=t),e.pending=t}function Ah(e,t,n){if((n&4194048)!==0){var l=t.lanes;l&=e.pendingLanes,n|=l,t.lanes=n,Ca(e,n)}}var du={readContext:kt,use:Tf,useCallback:mt,useContext:mt,useEffect:mt,useImperativeHandle:mt,useLayoutEffect:mt,useInsertionEffect:mt,useMemo:mt,useReducer:mt,useRef:mt,useState:mt,useDebugValue:mt,useDeferredValue:mt,useTransition:mt,useSyncExternalStore:mt,useId:mt,useHostTransitionStatus:mt,useFormState:mt,useActionState:mt,useOptimistic:mt,useMemoCache:mt,useCacheRefresh:mt};du.useEffectEvent=mt;var Eh={readContext:kt,use:Tf,useCallback:function(e,t){return on().memoizedState=[e,t===void 0?null:t],e},useContext:kt,useEffect:ch,useImperativeHandle:function(e,t,n){n=n!=null?n.concat([e]):null,Af(4194308,4,ph.bind(null,t,e),n)},useLayoutEffect:function(e,t){return Af(4194308,4,e,t)},useInsertionEffect:function(e,t){Af(4,2,e,t)},useMemo:function(e,t){var n=on();t=t===void 0?null:t;var l=e();if(mi){an(!0);try{e()}finally{an(!1)}}return n.memoizedState=[l,t],l},useReducer:function(e,t,n){var l=on();if(n!==void 0){var o=n(t);if(mi){an(!0);try{n(t)}finally{an(!1)}}}else o=t;return l.memoizedState=l.baseState=o,e={pending:null,lanes:0,dispatch:null,lastRenderedReducer:e,lastRenderedState:o},l.queue=e,e=e.dispatch=Pm.bind(null,Ne,e),[l.memoizedState,e]},useRef:function(e){var t=on();return e={current:e},t.memoizedState=e},useState:function(e){e=xc(e);var t=e.queue,n=Th.bind(null,Ne,t);return t.dispatch=n,[e.memoizedState,n]},useDebugValue:Cc,useDeferredValue:function(e,t){var n=on();return Ac(n,e,t)},useTransition:function(){var e=xc(!1);return e=gh.bind(null,Ne,e.queue,!0,!1),on().memoizedState=e,[!1,e]},useSyncExternalStore:function(e,t,n){var l=Ne,o=on();if(Ve){if(n===void 0)throw Error(f(407));n=n()}else{if(n=t(),ft===null)throw Error(f(349));(Be&127)!==0||kd(l,t,n)}o.memoizedState=n;var s={value:n,getSnapshot:t};return o.queue=s,ch(Wd.bind(null,l,s,e),[e]),l.flags|=2048,fr(9,{destroy:void 0},Fd.bind(null,l,s,n,t),null),n},useId:function(){var e=on(),t=ft.identifierPrefix;if(Ve){var n=ga,l=ba;n=(l&~(1<<32-ln(l)-1)).toString(32)+n,t="_"+t+"R_"+n,n=xf++,0<n&&(t+="H"+n.toString(32)),t+="_"}else n=Km++,t="_"+t+"r_"+n.toString(32)+"_";return e.memoizedState=t},useHostTransitionStatus:Nc,useFormState:ih,useActionState:ih,useOptimistic:function(e){var t=on();t.memoizedState=t.baseState=e;var n={pending:null,lanes:0,dispatch:null,lastRenderedReducer:null,lastRenderedState:null};return t.queue=n,t=Mc.bind(null,Ne,!0,n),n.dispatch=t,[e,t]},useMemoCache:gc,useCacheRefresh:function(){return on().memoizedState=Im.bind(null,Ne)},useEffectEvent:function(e){var t=on(),n={impl:e};return t.memoizedState=n,function(){if((Ie&2)!==0)throw Error(f(440));return n.impl.apply(void 0,arguments)}}},wc={readContext:kt,use:Tf,useCallback:mh,useContext:kt,useEffect:Tc,useImperativeHandle:vh,useInsertionEffect:dh,useLayoutEffect:hh,useMemo:yh,useReducer:Cf,useRef:oh,useState:function(){return Cf(Ga)},useDebugValue:Cc,useDeferredValue:function(e,t){var n=Dt();return bh(n,it.memoizedState,e,t)},useTransition:function(){var e=Cf(Ga)[0],t=Dt().memoizedState;return[typeof e=="boolean"?e:cu(e),t]},useSyncExternalStore:Jd,useId:xh,useHostTransitionStatus:Nc,useFormState:rh,useActionState:rh,useOptimistic:function(e,t){var n=Dt();return Pd(n,it,e,t)},useMemoCache:gc,useCacheRefresh:Dh};wc.useEffectEvent=sh;var Nh={readContext:kt,use:Tf,useCallback:mh,useContext:kt,useEffect:Tc,useImperativeHandle:vh,useInsertionEffect:dh,useLayoutEffect:hh,useMemo:yh,useReducer:_c,useRef:oh,useState:function(){return _c(Ga)},useDebugValue:Cc,useDeferredValue:function(e,t){var n=Dt();return it===null?Ac(n,e,t):bh(n,it.memoizedState,e,t)},useTransition:function(){var e=_c(Ga)[0],t=Dt().memoizedState;return[typeof e=="boolean"?e:cu(e),t]},useSyncExternalStore:Jd,useId:xh,useHostTransitionStatus:Nc,useFormState:fh,useActionState:fh,useOptimistic:function(e,t){var n=Dt();return it!==null?Pd(n,it,e,t):(n.baseState=e,[e,n.queue.dispatch])},useMemoCache:gc,useCacheRefresh:Dh};Nh.useEffectEvent=sh;function Oc(e,t,n,l){t=e.memoizedState,n=n(l,t),n=n==null?t:N({},t,n),e.memoizedState=n,e.lanes===0&&(e.updateQueue.baseState=n)}var zc={enqueueSetState:function(e,t,n){e=e._reactInternals;var l=An(),o=bl(l);o.payload=t,n!=null&&(o.callback=n),t=gl(e,o,l),t!==null&&(gn(t,e,l),ru(t,e,l))},enqueueReplaceState:function(e,t,n){e=e._reactInternals;var l=An(),o=bl(l);o.tag=1,o.payload=t,n!=null&&(o.callback=n),t=gl(e,o,l),t!==null&&(gn(t,e,l),ru(t,e,l))},enqueueForceUpdate:function(e,t){e=e._reactInternals;var n=An(),l=bl(n);l.tag=2,t!=null&&(l.callback=t),t=gl(e,l,n),t!==null&&(gn(t,e,n),ru(t,e,n))}};function Mh(e,t,n,l,o,s,b){return e=e.stateNode,typeof e.shouldComponentUpdate=="function"?e.shouldComponentUpdate(l,s,b):t.prototype&&t.prototype.isPureReactComponent?!ri(n,l)||!ri(o,s):!0}function wh(e,t,n,l){e=t.state,typeof t.componentWillReceiveProps=="function"&&t.componentWillReceiveProps(n,l),typeof t.UNSAFE_componentWillReceiveProps=="function"&&t.UNSAFE_componentWillReceiveProps(n,l),t.state!==e&&zc.enqueueReplaceState(t,t.state,null)}function yi(e,t){var n=t;if("ref"in t){n={};for(var l in t)l!=="ref"&&(n[l]=t[l])}if(e=e.defaultProps){n===t&&(n=N({},n));for(var o in e)n[o]===void 0&&(n[o]=e[o])}return n}function Oh(e){rf(e)}function zh(e){console.error(e)}function Lh(e){rf(e)}function Mf(e,t){try{var n=e.onUncaughtError;n(t.value,{componentStack:t.stack})}catch(l){setTimeout(function(){throw l})}}function Hh(e,t,n){try{var l=e.onCaughtError;l(n.value,{componentStack:n.stack,errorBoundary:t.tag===1?t.stateNode:null})}catch(o){setTimeout(function(){throw o})}}function Lc(e,t,n){return n=bl(n),n.tag=3,n.payload={element:null},n.callback=function(){Mf(e,t)},n}function jh(e){return e=bl(e),e.tag=3,e}function Rh(e,t,n,l){var o=n.type.getDerivedStateFromError;if(typeof o=="function"){var s=l.value;e.payload=function(){return o(s)},e.callback=function(){Hh(t,n,l)}}var b=n.stateNode;b!==null&&typeof b.componentDidCatch=="function"&&(e.callback=function(){Hh(t,n,l),typeof o!="function"&&(Cl===null?Cl=new Set([this]):Cl.add(this));var C=l.stack;this.componentDidCatch(l.value,{componentStack:C!==null?C:""})})}function ey(e,t,n,l,o){if(n.flags|=32768,l!==null&&typeof l=="object"&&typeof l.then=="function"){if(t=n.alternate,t!==null&&er(t,n,o,!0),n=xn.current,n!==null){switch(n.tag){case 31:case 13:return Kn===null?Xf():n.alternate===null&&yt===0&&(yt=3),n.flags&=-257,n.flags|=65536,n.lanes=o,l===mf?n.flags|=16384:(t=n.updateQueue,t===null?n.updateQueue=new Set([l]):t.add(l),ls(e,l,o)),!1;case 22:return n.flags|=65536,l===mf?n.flags|=16384:(t=n.updateQueue,t===null?(t={transitions:null,markerInstances:null,retryQueue:new Set([l])},n.updateQueue=t):(n=t.retryQueue,n===null?t.retryQueue=new Set([l]):n.add(l)),ls(e,l,o)),!1}throw Error(f(435,n.tag))}return ls(e,l,o),Xf(),!1}if(Ve)return t=xn.current,t!==null?((t.flags&65536)===0&&(t.flags|=256),t.flags|=65536,t.lanes=o,l!==Io&&(e=Error(f(422),{cause:l}),tu(Gn(e,n)))):(l!==Io&&(t=Error(f(423),{cause:l}),tu(Gn(t,n))),e=e.current.alternate,e.flags|=65536,o&=-o,e.lanes|=o,l=Gn(l,n),o=Lc(e.stateNode,l,o),oc(e,o),yt!==4&&(yt=2)),!1;var s=Error(f(520),{cause:l});if(s=Gn(s,n),Su===null?Su=[s]:Su.push(s),yt!==4&&(yt=2),t===null)return!0;l=Gn(l,n),n=t;do{switch(n.tag){case 3:return n.flags|=65536,e=o&-o,n.lanes|=e,e=Lc(n.stateNode,l,e),oc(n,e),!1;case 1:if(t=n.type,s=n.stateNode,(n.flags&128)===0&&(typeof t.getDerivedStateFromError=="function"||s!==null&&typeof s.componentDidCatch=="function"&&(Cl===null||!Cl.has(s))))return n.flags|=65536,o&=-o,n.lanes|=o,o=jh(o),Rh(o,e,n,l),oc(n,o),!1}n=n.return}while(n!==null);return!1}var Hc=Error(f(461)),Nt=!1;function Ft(e,t,n,l){t.child=e===null?Yd(t,null,n,l):vi(t,e.child,n,l)}function qh(e,t,n,l,o){n=n.render;var s=t.ref;if("ref"in l){var b={};for(var C in l)C!=="ref"&&(b[C]=l[C])}else b=l;return si(t),l=vc(e,t,n,b,s,o),C=mc(),e!==null&&!Nt?(yc(e,t,o),Va(e,t,o)):(Ve&&C&&Wo(t),t.flags|=1,Ft(e,t,l,o),t.child)}function Uh(e,t,n,l,o){if(e===null){var s=n.type;return typeof s=="function"&&!Jo(s)&&s.defaultProps===void 0&&n.compare===null?(t.tag=15,t.type=s,Bh(e,t,s,l,o)):(e=cf(n.type,null,l,t,t.mode,o),e.ref=t.ref,e.return=t,t.child=e)}if(s=e.child,!Gc(e,o)){var b=s.memoizedProps;if(n=n.compare,n=n!==null?n:ri,n(b,l)&&e.ref===t.ref)return Va(e,t,o)}return t.flags|=1,e=qa(s,l),e.ref=t.ref,e.return=t,t.child=e}function Bh(e,t,n,l,o){if(e!==null){var s=e.memoizedProps;if(ri(s,l)&&e.ref===t.ref)if(Nt=!1,t.pendingProps=l=s,Gc(e,o))(e.flags&131072)!==0&&(Nt=!0);else return t.lanes=e.lanes,Va(e,t,o)}return jc(e,t,n,l,o)}function Yh(e,t,n,l){var o=l.children,s=e!==null?e.memoizedState:null;if(e===null&&t.stateNode===null&&(t.stateNode={_visibility:1,_pendingMarkers:null,_retryCache:null,_transitions:null}),l.mode==="hidden"){if((t.flags&128)!==0){if(s=s!==null?s.baseLanes|n:n,e!==null){for(l=t.child=e.child,o=0;l!==null;)o=o|l.lanes|l.childLanes,l=l.sibling;l=o&~s}else l=0,t.child=null;return Xh(e,t,s,n,l)}if((n&536870912)!==0)t.memoizedState={baseLanes:0,cachePool:null},e!==null&&pf(t,s!==null?s.cachePool:null),s!==null?Vd(t,s):sc(),Qd(t);else return l=t.lanes=536870912,Xh(e,t,s!==null?s.baseLanes|n:n,n,l)}else s!==null?(pf(t,s.cachePool),Vd(t,s),_l(),t.memoizedState=null):(e!==null&&pf(t,null),sc(),_l());return Ft(e,t,o,n),t.child}function hu(e,t){return e!==null&&e.tag===22||t.stateNode!==null||(t.stateNode={_visibility:1,_pendingMarkers:null,_retryCache:null,_transitions:null}),t.sibling}function Xh(e,t,n,l,o){var s=ic();return s=s===null?null:{parent:At._currentValue,pool:s},t.memoizedState={baseLanes:n,cachePool:s},e!==null&&pf(t,null),sc(),Qd(t),e!==null&&er(e,t,l,!0),t.childLanes=o,null}function wf(e,t){return t=zf({mode:t.mode,children:t.children},e.mode),t.ref=e.ref,e.child=t,t.return=e,t}function Gh(e,t,n){return vi(t,e.child,null,n),e=wf(t,t.pendingProps),e.flags|=2,Dn(t),t.memoizedState=null,e}function ty(e,t,n){var l=t.pendingProps,o=(t.flags&128)!==0;if(t.flags&=-129,e===null){if(Ve){if(l.mode==="hidden")return e=wf(t,l),t.lanes=536870912,hu(null,e);if(hc(t),(e=ct)?(e=ev(e,Zn),e=e!==null&&e.data==="&"?e:null,e!==null&&(t.memoizedState={dehydrated:e,treeContext:hl!==null?{id:ba,overflow:ga}:null,retryLane:536870912,hydrationErrors:null},n=Cd(e),n.return=t,t.child=n,Jt=t,ct=null)):e=null,e===null)throw vl(t);return t.lanes=536870912,null}return wf(t,l)}var s=e.memoizedState;if(s!==null){var b=s.dehydrated;if(hc(t),o)if(t.flags&256)t.flags&=-257,t=Gh(e,t,n);else if(t.memoizedState!==null)t.child=e.child,t.flags|=128,t=null;else throw Error(f(558));else if(Nt||er(e,t,n,!1),o=(n&e.childLanes)!==0,Nt||o){if(l=ft,l!==null&&(b=Hi(l,n),b!==0&&b!==s.retryLane))throw s.retryLane=b,ui(e,b),gn(l,e,b),Hc;Xf(),t=Gh(e,t,n)}else e=s.treeContext,ct=Jn(b.nextSibling),Jt=t,Ve=!0,pl=null,Zn=!1,e!==null&&Nd(t,e),t=wf(t,l),t.flags|=4096;return t}return e=qa(e.child,{mode:l.mode,children:l.children}),e.ref=t.ref,t.child=e,e.return=t,e}function Of(e,t){var n=t.ref;if(n===null)e!==null&&e.ref!==null&&(t.flags|=4194816);else{if(typeof n!="function"&&typeof n!="object")throw Error(f(284));(e===null||e.ref!==n)&&(t.flags|=4194816)}}function jc(e,t,n,l,o){return si(t),n=vc(e,t,n,l,void 0,o),l=mc(),e!==null&&!Nt?(yc(e,t,o),Va(e,t,o)):(Ve&&l&&Wo(t),t.flags|=1,Ft(e,t,n,o),t.child)}function Vh(e,t,n,l,o,s){return si(t),t.updateQueue=null,n=Kd(t,l,n,o),Zd(e),l=mc(),e!==null&&!Nt?(yc(e,t,s),Va(e,t,s)):(Ve&&l&&Wo(t),t.flags|=1,Ft(e,t,n,s),t.child)}function Qh(e,t,n,l,o){if(si(t),t.stateNode===null){var s=Wi,b=n.contextType;typeof b=="object"&&b!==null&&(s=kt(b)),s=new n(l,s),t.memoizedState=s.state!==null&&s.state!==void 0?s.state:null,s.updater=zc,t.stateNode=s,s._reactInternals=t,s=t.stateNode,s.props=l,s.state=t.memoizedState,s.refs={},uc(t),b=n.contextType,s.context=typeof b=="object"&&b!==null?kt(b):Wi,s.state=t.memoizedState,b=n.getDerivedStateFromProps,typeof b=="function"&&(Oc(t,n,b,l),s.state=t.memoizedState),typeof n.getDerivedStateFromProps=="function"||typeof s.getSnapshotBeforeUpdate=="function"||typeof s.UNSAFE_componentWillMount!="function"&&typeof s.componentWillMount!="function"||(b=s.state,typeof s.componentWillMount=="function"&&s.componentWillMount(),typeof s.UNSAFE_componentWillMount=="function"&&s.UNSAFE_componentWillMount(),b!==s.state&&zc.enqueueReplaceState(s,s.state,null),fu(t,l,s,o),uu(),s.state=t.memoizedState),typeof s.componentDidMount=="function"&&(t.flags|=4194308),l=!0}else if(e===null){s=t.stateNode;var C=t.memoizedProps,z=yi(n,C);s.props=z;var Q=s.context,ee=n.contextType;b=Wi,typeof ee=="object"&&ee!==null&&(b=kt(ee));var ae=n.getDerivedStateFromProps;ee=typeof ae=="function"||typeof s.getSnapshotBeforeUpdate=="function",C=t.pendingProps!==C,ee||typeof s.UNSAFE_componentWillReceiveProps!="function"&&typeof s.componentWillReceiveProps!="function"||(C||Q!==b)&&wh(t,s,l,b),yl=!1;var K=t.memoizedState;s.state=K,fu(t,l,s,o),uu(),Q=t.memoizedState,C||K!==Q||yl?(typeof ae=="function"&&(Oc(t,n,ae,l),Q=t.memoizedState),(z=yl||Mh(t,n,z,l,K,Q,b))?(ee||typeof s.UNSAFE_componentWillMount!="function"&&typeof s.componentWillMount!="function"||(typeof s.componentWillMount=="function"&&s.componentWillMount(),typeof s.UNSAFE_componentWillMount=="function"&&s.UNSAFE_componentWillMount()),typeof s.componentDidMount=="function"&&(t.flags|=4194308)):(typeof s.componentDidMount=="function"&&(t.flags|=4194308),t.memoizedProps=l,t.memoizedState=Q),s.props=l,s.state=Q,s.context=b,l=z):(typeof s.componentDidMount=="function"&&(t.flags|=4194308),l=!1)}else{s=t.stateNode,fc(e,t),b=t.memoizedProps,ee=yi(n,b),s.props=ee,ae=t.pendingProps,K=s.context,Q=n.contextType,z=Wi,typeof Q=="object"&&Q!==null&&(z=kt(Q)),C=n.getDerivedStateFromProps,(Q=typeof C=="function"||typeof s.getSnapshotBeforeUpdate=="function")||typeof s.UNSAFE_componentWillReceiveProps!="function"&&typeof s.componentWillReceiveProps!="function"||(b!==ae||K!==z)&&wh(t,s,l,z),yl=!1,K=t.memoizedState,s.state=K,fu(t,l,s,o),uu();var k=t.memoizedState;b!==ae||K!==k||yl||e!==null&&e.dependencies!==null&&df(e.dependencies)?(typeof C=="function"&&(Oc(t,n,C,l),k=t.memoizedState),(ee=yl||Mh(t,n,ee,l,K,k,z)||e!==null&&e.dependencies!==null&&df(e.dependencies))?(Q||typeof s.UNSAFE_componentWillUpdate!="function"&&typeof s.componentWillUpdate!="function"||(typeof s.componentWillUpdate=="function"&&s.componentWillUpdate(l,k,z),typeof s.UNSAFE_componentWillUpdate=="function"&&s.UNSAFE_componentWillUpdate(l,k,z)),typeof s.componentDidUpdate=="function"&&(t.flags|=4),typeof s.getSnapshotBeforeUpdate=="function"&&(t.flags|=1024)):(typeof s.componentDidUpdate!="function"||b===e.memoizedProps&&K===e.memoizedState||(t.flags|=4),typeof s.getSnapshotBeforeUpdate!="function"||b===e.memoizedProps&&K===e.memoizedState||(t.flags|=1024),t.memoizedProps=l,t.memoizedState=k),s.props=l,s.state=k,s.context=z,l=ee):(typeof s.componentDidUpdate!="function"||b===e.memoizedProps&&K===e.memoizedState||(t.flags|=4),typeof s.getSnapshotBeforeUpdate!="function"||b===e.memoizedProps&&K===e.memoizedState||(t.flags|=1024),l=!1)}return s=l,Of(e,t),l=(t.flags&128)!==0,s||l?(s=t.stateNode,n=l&&typeof n.getDerivedStateFromError!="function"?null:s.render(),t.flags|=1,e!==null&&l?(t.child=vi(t,e.child,null,o),t.child=vi(t,null,n,o)):Ft(e,t,n,o),t.memoizedState=s.state,e=t.child):e=Va(e,t,o),e}function Zh(e,t,n,l){return oi(),t.flags|=256,Ft(e,t,n,l),t.child}var Rc={dehydrated:null,treeContext:null,retryLane:0,hydrationErrors:null};function qc(e){return{baseLanes:e,cachePool:Hd()}}function Uc(e,t,n){return e=e!==null?e.childLanes&~n:0,t&&(e|=Cn),e}function Kh(e,t,n){var l=t.pendingProps,o=!1,s=(t.flags&128)!==0,b;if((b=s)||(b=e!==null&&e.memoizedState===null?!1:(xt.current&2)!==0),b&&(o=!0,t.flags&=-129),b=(t.flags&32)!==0,t.flags&=-33,e===null){if(Ve){if(o?Sl(t):_l(),(e=ct)?(e=ev(e,Zn),e=e!==null&&e.data!=="&"?e:null,e!==null&&(t.memoizedState={dehydrated:e,treeContext:hl!==null?{id:ba,overflow:ga}:null,retryLane:536870912,hydrationErrors:null},n=Cd(e),n.return=t,t.child=n,Jt=t,ct=null)):e=null,e===null)throw vl(t);return Ss(e)?t.lanes=32:t.lanes=536870912,null}var C=l.children;return l=l.fallback,o?(_l(),o=t.mode,C=zf({mode:"hidden",children:C},o),l=fi(l,o,n,null),C.return=t,l.return=t,C.sibling=l,t.child=C,l=t.child,l.memoizedState=qc(n),l.childLanes=Uc(e,b,n),t.memoizedState=Rc,hu(null,l)):(Sl(t),Bc(t,C))}var z=e.memoizedState;if(z!==null&&(C=z.dehydrated,C!==null)){if(s)t.flags&256?(Sl(t),t.flags&=-257,t=Yc(e,t,n)):t.memoizedState!==null?(_l(),t.child=e.child,t.flags|=128,t=null):(_l(),C=l.fallback,o=t.mode,l=zf({mode:"visible",children:l.children},o),C=fi(C,o,n,null),C.flags|=2,l.return=t,C.return=t,l.sibling=C,t.child=l,vi(t,e.child,null,n),l=t.child,l.memoizedState=qc(n),l.childLanes=Uc(e,b,n),t.memoizedState=Rc,t=hu(null,l));else if(Sl(t),Ss(C)){if(b=C.nextSibling&&C.nextSibling.dataset,b)var Q=b.dgst;b=Q,l=Error(f(419)),l.stack="",l.digest=b,tu({value:l,source:null,stack:null}),t=Yc(e,t,n)}else if(Nt||er(e,t,n,!1),b=(n&e.childLanes)!==0,Nt||b){if(b=ft,b!==null&&(l=Hi(b,n),l!==0&&l!==z.retryLane))throw z.retryLane=l,ui(e,l),gn(b,e,l),Hc;gs(C)||Xf(),t=Yc(e,t,n)}else gs(C)?(t.flags|=192,t.child=e.child,t=null):(e=z.treeContext,ct=Jn(C.nextSibling),Jt=t,Ve=!0,pl=null,Zn=!1,e!==null&&Nd(t,e),t=Bc(t,l.children),t.flags|=4096);return t}return o?(_l(),C=l.fallback,o=t.mode,z=e.child,Q=z.sibling,l=qa(z,{mode:"hidden",children:l.children}),l.subtreeFlags=z.subtreeFlags&65011712,Q!==null?C=qa(Q,C):(C=fi(C,o,n,null),C.flags|=2),C.return=t,l.return=t,l.sibling=C,t.child=l,hu(null,l),l=t.child,C=e.child.memoizedState,C===null?C=qc(n):(o=C.cachePool,o!==null?(z=At._currentValue,o=o.parent!==z?{parent:z,pool:z}:o):o=Hd(),C={baseLanes:C.baseLanes|n,cachePool:o}),l.memoizedState=C,l.childLanes=Uc(e,b,n),t.memoizedState=Rc,hu(e.child,l)):(Sl(t),n=e.child,e=n.sibling,n=qa(n,{mode:"visible",children:l.children}),n.return=t,n.sibling=null,e!==null&&(b=t.deletions,b===null?(t.deletions=[e],t.flags|=16):b.push(e)),t.child=n,t.memoizedState=null,n)}function Bc(e,t){return t=zf({mode:"visible",children:t},e.mode),t.return=e,e.child=t}function zf(e,t){return e=_n(22,e,null,t),e.lanes=0,e}function Yc(e,t,n){return vi(t,e.child,null,n),e=Bc(t,t.pendingProps.children),e.flags|=2,t.memoizedState=null,e}function Jh(e,t,n){e.lanes|=t;var l=e.alternate;l!==null&&(l.lanes|=t),tc(e.return,t,n)}function Xc(e,t,n,l,o,s){var b=e.memoizedState;b===null?e.memoizedState={isBackwards:t,rendering:null,renderingStartTime:0,last:l,tail:n,tailMode:o,treeForkCount:s}:(b.isBackwards=t,b.rendering=null,b.renderingStartTime=0,b.last=l,b.tail=n,b.tailMode=o,b.treeForkCount=s)}function kh(e,t,n){var l=t.pendingProps,o=l.revealOrder,s=l.tail;l=l.children;var b=xt.current,C=(b&2)!==0;if(C?(b=b&1|2,t.flags|=128):b&=1,Qe(xt,b),Ft(e,t,l,n),l=Ve?eu:0,!C&&e!==null&&(e.flags&128)!==0)e:for(e=t.child;e!==null;){if(e.tag===13)e.memoizedState!==null&&Jh(e,n,t);else if(e.tag===19)Jh(e,n,t);else if(e.child!==null){e.child.return=e,e=e.child;continue}if(e===t)break e;for(;e.sibling===null;){if(e.return===null||e.return===t)break e;e=e.return}e.sibling.return=e.return,e=e.sibling}switch(o){case"forwards":for(n=t.child,o=null;n!==null;)e=n.alternate,e!==null&&Sf(e)===null&&(o=n),n=n.sibling;n=o,n===null?(o=t.child,t.child=null):(o=n.sibling,n.sibling=null),Xc(t,!1,o,n,s,l);break;case"backwards":case"unstable_legacy-backwards":for(n=null,o=t.child,t.child=null;o!==null;){if(e=o.alternate,e!==null&&Sf(e)===null){t.child=o;break}e=o.sibling,o.sibling=n,n=o,o=e}Xc(t,!0,n,null,s,l);break;case"together":Xc(t,!1,null,null,void 0,l);break;default:t.memoizedState=null}return t.child}function Va(e,t,n){if(e!==null&&(t.dependencies=e.dependencies),Tl|=t.lanes,(n&t.childLanes)===0)if(e!==null){if(er(e,t,n,!1),(n&t.childLanes)===0)return null}else return null;if(e!==null&&t.child!==e.child)throw Error(f(153));if(t.child!==null){for(e=t.child,n=qa(e,e.pendingProps),t.child=n,n.return=t;e.sibling!==null;)e=e.sibling,n=n.sibling=qa(e,e.pendingProps),n.return=t;n.sibling=null}return t.child}function Gc(e,t){return(e.lanes&t)!==0?!0:(e=e.dependencies,!!(e!==null&&df(e)))}function ny(e,t,n){switch(t.tag){case 3:Ei(t,t.stateNode.containerInfo),ml(t,At,e.memoizedState.cache),oi();break;case 27:case 5:dn(t);break;case 4:Ei(t,t.stateNode.containerInfo);break;case 10:ml(t,t.type,t.memoizedProps.value);break;case 31:if(t.memoizedState!==null)return t.flags|=128,hc(t),null;break;case 13:var l=t.memoizedState;if(l!==null)return l.dehydrated!==null?(Sl(t),t.flags|=128,null):(n&t.child.childLanes)!==0?Kh(e,t,n):(Sl(t),e=Va(e,t,n),e!==null?e.sibling:null);Sl(t);break;case 19:var o=(e.flags&128)!==0;if(l=(n&t.childLanes)!==0,l||(er(e,t,n,!1),l=(n&t.childLanes)!==0),o){if(l)return kh(e,t,n);t.flags|=128}if(o=t.memoizedState,o!==null&&(o.rendering=null,o.tail=null,o.lastEffect=null),Qe(xt,xt.current),l)break;return null;case 22:return t.lanes=0,Yh(e,t,n,t.pendingProps);case 24:ml(t,At,e.memoizedState.cache)}return Va(e,t,n)}function Fh(e,t,n){if(e!==null)if(e.memoizedProps!==t.pendingProps)Nt=!0;else{if(!Gc(e,n)&&(t.flags&128)===0)return Nt=!1,ny(e,t,n);Nt=(e.flags&131072)!==0}else Nt=!1,Ve&&(t.flags&1048576)!==0&&Ed(t,eu,t.index);switch(t.lanes=0,t.tag){case 16:e:{var l=t.pendingProps;if(e=hi(t.elementType),t.type=e,typeof e=="function")Jo(e)?(l=yi(e,l),t.tag=1,t=Qh(null,t,e,l,n)):(t.tag=0,t=jc(null,t,e,l,n));else{if(e!=null){var o=e.$$typeof;if(o===Te){t.tag=11,t=qh(null,t,e,l,n);break e}else if(o===y){t.tag=14,t=Uh(null,t,e,l,n);break e}}throw t=$t(e)||e,Error(f(306,t,""))}}return t;case 0:return jc(e,t,t.type,t.pendingProps,n);case 1:return l=t.type,o=yi(l,t.pendingProps),Qh(e,t,l,o,n);case 3:e:{if(Ei(t,t.stateNode.containerInfo),e===null)throw Error(f(387));l=t.pendingProps;var s=t.memoizedState;o=s.element,fc(e,t),fu(t,l,null,n);var b=t.memoizedState;if(l=b.cache,ml(t,At,l),l!==s.cache&&nc(t,[At],n,!0),uu(),l=b.element,s.isDehydrated)if(s={element:l,isDehydrated:!1,cache:b.cache},t.updateQueue.baseState=s,t.memoizedState=s,t.flags&256){t=Zh(e,t,l,n);break e}else if(l!==o){o=Gn(Error(f(424)),t),tu(o),t=Zh(e,t,l,n);break e}else{switch(e=t.stateNode.containerInfo,e.nodeType){case 9:e=e.body;break;default:e=e.nodeName==="HTML"?e.ownerDocument.body:e}for(ct=Jn(e.firstChild),Jt=t,Ve=!0,pl=null,Zn=!0,n=Yd(t,null,l,n),t.child=n;n;)n.flags=n.flags&-3|4096,n=n.sibling}else{if(oi(),l===o){t=Va(e,t,n);break e}Ft(e,t,l,n)}t=t.child}return t;case 26:return Of(e,t),e===null?(n=rv(t.type,null,t.pendingProps,null))?t.memoizedState=n:Ve||(n=t.type,e=t.pendingProps,l=kf(oa.current).createElement(n),l[Ot]=t,l[Pt]=e,Wt(l,n,e),St(l),t.stateNode=l):t.memoizedState=rv(t.type,e.memoizedProps,t.pendingProps,e.memoizedState),null;case 27:return dn(t),e===null&&Ve&&(l=t.stateNode=av(t.type,t.pendingProps,oa.current),Jt=t,Zn=!0,o=ct,Ml(t.type)?(_s=o,ct=Jn(l.firstChild)):ct=o),Ft(e,t,t.pendingProps.children,n),Of(e,t),e===null&&(t.flags|=4194304),t.child;case 5:return e===null&&Ve&&((o=l=ct)&&(l=zy(l,t.type,t.pendingProps,Zn),l!==null?(t.stateNode=l,Jt=t,ct=Jn(l.firstChild),Zn=!1,o=!0):o=!1),o||vl(t)),dn(t),o=t.type,s=t.pendingProps,b=e!==null?e.memoizedProps:null,l=s.children,ms(o,s)?l=null:b!==null&&ms(o,b)&&(t.flags|=32),t.memoizedState!==null&&(o=vc(e,t,Jm,null,null,n),Nu._currentValue=o),Of(e,t),Ft(e,t,l,n),t.child;case 6:return e===null&&Ve&&((e=n=ct)&&(n=Ly(n,t.pendingProps,Zn),n!==null?(t.stateNode=n,Jt=t,ct=null,e=!0):e=!1),e||vl(t)),null;case 13:return Kh(e,t,n);case 4:return Ei(t,t.stateNode.containerInfo),l=t.pendingProps,e===null?t.child=vi(t,null,l,n):Ft(e,t,l,n),t.child;case 11:return qh(e,t,t.type,t.pendingProps,n);case 7:return Ft(e,t,t.pendingProps,n),t.child;case 8:return Ft(e,t,t.pendingProps.children,n),t.child;case 12:return Ft(e,t,t.pendingProps.children,n),t.child;case 10:return l=t.pendingProps,ml(t,t.type,l.value),Ft(e,t,l.children,n),t.child;case 9:return o=t.type._context,l=t.pendingProps.children,si(t),o=kt(o),l=l(o),t.flags|=1,Ft(e,t,l,n),t.child;case 14:return Uh(e,t,t.type,t.pendingProps,n);case 15:return Bh(e,t,t.type,t.pendingProps,n);case 19:return kh(e,t,n);case 31:return ty(e,t,n);case 22:return Yh(e,t,n,t.pendingProps);case 24:return si(t),l=kt(At),e===null?(o=ic(),o===null&&(o=ft,s=ac(),o.pooledCache=s,s.refCount++,s!==null&&(o.pooledCacheLanes|=n),o=s),t.memoizedState={parent:l,cache:o},uc(t),ml(t,At,o)):((e.lanes&n)!==0&&(fc(e,t),fu(t,null,null,n),uu()),o=e.memoizedState,s=t.memoizedState,o.parent!==l?(o={parent:l,cache:l},t.memoizedState=o,t.lanes===0&&(t.memoizedState=t.updateQueue.baseState=o),ml(t,At,l)):(l=s.cache,ml(t,At,l),l!==o.cache&&nc(t,[At],n,!0))),Ft(e,t,t.pendingProps.children,n),t.child;case 29:throw t.pendingProps}throw Error(f(156,t.tag))}function Qa(e){e.flags|=4}function Vc(e,t,n,l,o){if((t=(e.mode&32)!==0)&&(t=!1),t){if(e.flags|=16777216,(o&335544128)===o)if(e.stateNode.complete)e.flags|=8192;else if(xp())e.flags|=8192;else throw pi=mf,rc}else e.flags&=-16777217}function Wh(e,t){if(t.type!=="stylesheet"||(t.state.loading&4)!==0)e.flags&=-16777217;else if(e.flags|=16777216,!sv(t))if(xp())e.flags|=8192;else throw pi=mf,rc}function Lf(e,t){t!==null&&(e.flags|=4),e.flags&16384&&(t=e.tag!==22?Li():536870912,e.lanes|=t,dr|=t)}function pu(e,t){if(!Ve)switch(e.tailMode){case"hidden":t=e.tail;for(var n=null;t!==null;)t.alternate!==null&&(n=t),t=t.sibling;n===null?e.tail=null:n.sibling=null;break;case"collapsed":n=e.tail;for(var l=null;n!==null;)n.alternate!==null&&(l=n),n=n.sibling;l===null?t||e.tail===null?e.tail=null:e.tail.sibling=null:l.sibling=null}}function st(e){var t=e.alternate!==null&&e.alternate.child===e.child,n=0,l=0;if(t)for(var o=e.child;o!==null;)n|=o.lanes|o.childLanes,l|=o.subtreeFlags&65011712,l|=o.flags&65011712,o.return=e,o=o.sibling;else for(o=e.child;o!==null;)n|=o.lanes|o.childLanes,l|=o.subtreeFlags,l|=o.flags,o.return=e,o=o.sibling;return e.subtreeFlags|=l,e.childLanes=n,t}function ay(e,t,n){var l=t.pendingProps;switch($o(t),t.tag){case 16:case 15:case 0:case 11:case 7:case 8:case 12:case 9:case 14:return st(t),null;case 1:return st(t),null;case 3:return n=t.stateNode,l=null,e!==null&&(l=e.memoizedState.cache),t.memoizedState.cache!==l&&(t.flags|=2048),Ya(At),Da(),n.pendingContext&&(n.context=n.pendingContext,n.pendingContext=null),(e===null||e.child===null)&&(Pi(t)?Qa(t):e===null||e.memoizedState.isDehydrated&&(t.flags&256)===0||(t.flags|=1024,Po())),st(t),null;case 26:var o=t.type,s=t.memoizedState;return e===null?(Qa(t),s!==null?(st(t),Wh(t,s)):(st(t),Vc(t,o,null,l,n))):s?s!==e.memoizedState?(Qa(t),st(t),Wh(t,s)):(st(t),t.flags&=-16777217):(e=e.memoizedProps,e!==l&&Qa(t),st(t),Vc(t,o,e,l,n)),null;case 27:if(Ni(t),n=oa.current,o=t.type,e!==null&&t.stateNode!=null)e.memoizedProps!==l&&Qa(t);else{if(!l){if(t.stateNode===null)throw Error(f(166));return st(t),null}e=bt.current,Pi(t)?Md(t):(e=av(o,l,n),t.stateNode=e,Qa(t))}return st(t),null;case 5:if(Ni(t),o=t.type,e!==null&&t.stateNode!=null)e.memoizedProps!==l&&Qa(t);else{if(!l){if(t.stateNode===null)throw Error(f(166));return st(t),null}if(s=bt.current,Pi(t))Md(t);else{var b=kf(oa.current);switch(s){case 1:s=b.createElementNS("http://www.w3.org/2000/svg",o);break;case 2:s=b.createElementNS("http://www.w3.org/1998/Math/MathML",o);break;default:switch(o){case"svg":s=b.createElementNS("http://www.w3.org/2000/svg",o);break;case"math":s=b.createElementNS("http://www.w3.org/1998/Math/MathML",o);break;case"script":s=b.createElement("div"),s.innerHTML="<script><\/script>",s=s.removeChild(s.firstChild);break;case"select":s=typeof l.is=="string"?b.createElement("select",{is:l.is}):b.createElement("select"),l.multiple?s.multiple=!0:l.size&&(s.size=l.size);break;default:s=typeof l.is=="string"?b.createElement(o,{is:l.is}):b.createElement(o)}}s[Ot]=t,s[Pt]=l;e:for(b=t.child;b!==null;){if(b.tag===5||b.tag===6)s.appendChild(b.stateNode);else if(b.tag!==4&&b.tag!==27&&b.child!==null){b.child.return=b,b=b.child;continue}if(b===t)break e;for(;b.sibling===null;){if(b.return===null||b.return===t)break e;b=b.return}b.sibling.return=b.return,b=b.sibling}t.stateNode=s;e:switch(Wt(s,o,l),o){case"button":case"input":case"select":case"textarea":l=!!l.autoFocus;break e;case"img":l=!0;break e;default:l=!1}l&&Qa(t)}}return st(t),Vc(t,t.type,e===null?null:e.memoizedProps,t.pendingProps,n),null;case 6:if(e&&t.stateNode!=null)e.memoizedProps!==l&&Qa(t);else{if(typeof l!="string"&&t.stateNode===null)throw Error(f(166));if(e=oa.current,Pi(t)){if(e=t.stateNode,n=t.memoizedProps,l=null,o=Jt,o!==null)switch(o.tag){case 27:case 5:l=o.memoizedProps}e[Ot]=t,e=!!(e.nodeValue===n||l!==null&&l.suppressHydrationWarning===!0||Kp(e.nodeValue,n)),e||vl(t,!0)}else e=kf(e).createTextNode(l),e[Ot]=t,t.stateNode=e}return st(t),null;case 31:if(n=t.memoizedState,e===null||e.memoizedState!==null){if(l=Pi(t),n!==null){if(e===null){if(!l)throw Error(f(318));if(e=t.memoizedState,e=e!==null?e.dehydrated:null,!e)throw Error(f(557));e[Ot]=t}else oi(),(t.flags&128)===0&&(t.memoizedState=null),t.flags|=4;st(t),e=!1}else n=Po(),e!==null&&e.memoizedState!==null&&(e.memoizedState.hydrationErrors=n),e=!0;if(!e)return t.flags&256?(Dn(t),t):(Dn(t),null);if((t.flags&128)!==0)throw Error(f(558))}return st(t),null;case 13:if(l=t.memoizedState,e===null||e.memoizedState!==null&&e.memoizedState.dehydrated!==null){if(o=Pi(t),l!==null&&l.dehydrated!==null){if(e===null){if(!o)throw Error(f(318));if(o=t.memoizedState,o=o!==null?o.dehydrated:null,!o)throw Error(f(317));o[Ot]=t}else oi(),(t.flags&128)===0&&(t.memoizedState=null),t.flags|=4;st(t),o=!1}else o=Po(),e!==null&&e.memoizedState!==null&&(e.memoizedState.hydrationErrors=o),o=!0;if(!o)return t.flags&256?(Dn(t),t):(Dn(t),null)}return Dn(t),(t.flags&128)!==0?(t.lanes=n,t):(n=l!==null,e=e!==null&&e.memoizedState!==null,n&&(l=t.child,o=null,l.alternate!==null&&l.alternate.memoizedState!==null&&l.alternate.memoizedState.cachePool!==null&&(o=l.alternate.memoizedState.cachePool.pool),s=null,l.memoizedState!==null&&l.memoizedState.cachePool!==null&&(s=l.memoizedState.cachePool.pool),s!==o&&(l.flags|=2048)),n!==e&&n&&(t.child.flags|=8192),Lf(t,t.updateQueue),st(t),null);case 4:return Da(),e===null&&ss(t.stateNode.containerInfo),st(t),null;case 10:return Ya(t.type),st(t),null;case 19:if(ot(xt),l=t.memoizedState,l===null)return st(t),null;if(o=(t.flags&128)!==0,s=l.rendering,s===null)if(o)pu(l,!1);else{if(yt!==0||e!==null&&(e.flags&128)!==0)for(e=t.child;e!==null;){if(s=Sf(e),s!==null){for(t.flags|=128,pu(l,!1),e=s.updateQueue,t.updateQueue=e,Lf(t,e),t.subtreeFlags=0,e=n,n=t.child;n!==null;)Td(n,e),n=n.sibling;return Qe(xt,xt.current&1|2),Ve&&Ua(t,l.treeForkCount),t.child}e=e.sibling}l.tail!==null&&wt()>Uf&&(t.flags|=128,o=!0,pu(l,!1),t.lanes=4194304)}else{if(!o)if(e=Sf(s),e!==null){if(t.flags|=128,o=!0,e=e.updateQueue,t.updateQueue=e,Lf(t,e),pu(l,!0),l.tail===null&&l.tailMode==="hidden"&&!s.alternate&&!Ve)return st(t),null}else 2*wt()-l.renderingStartTime>Uf&&n!==536870912&&(t.flags|=128,o=!0,pu(l,!1),t.lanes=4194304);l.isBackwards?(s.sibling=t.child,t.child=s):(e=l.last,e!==null?e.sibling=s:t.child=s,l.last=s)}return l.tail!==null?(e=l.tail,l.rendering=e,l.tail=e.sibling,l.renderingStartTime=wt(),e.sibling=null,n=xt.current,Qe(xt,o?n&1|2:n&1),Ve&&Ua(t,l.treeForkCount),e):(st(t),null);case 22:case 23:return Dn(t),dc(),l=t.memoizedState!==null,e!==null?e.memoizedState!==null!==l&&(t.flags|=8192):l&&(t.flags|=8192),l?(n&536870912)!==0&&(t.flags&128)===0&&(st(t),t.subtreeFlags&6&&(t.flags|=8192)):st(t),n=t.updateQueue,n!==null&&Lf(t,n.retryQueue),n=null,e!==null&&e.memoizedState!==null&&e.memoizedState.cachePool!==null&&(n=e.memoizedState.cachePool.pool),l=null,t.memoizedState!==null&&t.memoizedState.cachePool!==null&&(l=t.memoizedState.cachePool.pool),l!==n&&(t.flags|=2048),e!==null&&ot(di),null;case 24:return n=null,e!==null&&(n=e.memoizedState.cache),t.memoizedState.cache!==n&&(t.flags|=2048),Ya(At),st(t),null;case 25:return null;case 30:return null}throw Error(f(156,t.tag))}function ly(e,t){switch($o(t),t.tag){case 1:return e=t.flags,e&65536?(t.flags=e&-65537|128,t):null;case 3:return Ya(At),Da(),e=t.flags,(e&65536)!==0&&(e&128)===0?(t.flags=e&-65537|128,t):null;case 26:case 27:case 5:return Ni(t),null;case 31:if(t.memoizedState!==null){if(Dn(t),t.alternate===null)throw Error(f(340));oi()}return e=t.flags,e&65536?(t.flags=e&-65537|128,t):null;case 13:if(Dn(t),e=t.memoizedState,e!==null&&e.dehydrated!==null){if(t.alternate===null)throw Error(f(340));oi()}return e=t.flags,e&65536?(t.flags=e&-65537|128,t):null;case 19:return ot(xt),null;case 4:return Da(),null;case 10:return Ya(t.type),null;case 22:case 23:return Dn(t),dc(),e!==null&&ot(di),e=t.flags,e&65536?(t.flags=e&-65537|128,t):null;case 24:return Ya(At),null;case 25:return null;default:return null}}function $h(e,t){switch($o(t),t.tag){case 3:Ya(At),Da();break;case 26:case 27:case 5:Ni(t);break;case 4:Da();break;case 31:t.memoizedState!==null&&Dn(t);break;case 13:Dn(t);break;case 19:ot(xt);break;case 10:Ya(t.type);break;case 22:case 23:Dn(t),dc(),e!==null&&ot(di);break;case 24:Ya(At)}}function vu(e,t){try{var n=t.updateQueue,l=n!==null?n.lastEffect:null;if(l!==null){var o=l.next;n=o;do{if((n.tag&e)===e){l=void 0;var s=n.create,b=n.inst;l=s(),b.destroy=l}n=n.next}while(n!==o)}}catch(C){tt(t,t.return,C)}}function xl(e,t,n){try{var l=t.updateQueue,o=l!==null?l.lastEffect:null;if(o!==null){var s=o.next;l=s;do{if((l.tag&e)===e){var b=l.inst,C=b.destroy;if(C!==void 0){b.destroy=void 0,o=t;var z=n,Q=C;try{Q()}catch(ee){tt(o,z,ee)}}}l=l.next}while(l!==s)}}catch(ee){tt(t,t.return,ee)}}function Ih(e){var t=e.updateQueue;if(t!==null){var n=e.stateNode;try{Gd(t,n)}catch(l){tt(e,e.return,l)}}}function Ph(e,t,n){n.props=yi(e.type,e.memoizedProps),n.state=e.memoizedState;try{n.componentWillUnmount()}catch(l){tt(e,t,l)}}function mu(e,t){try{var n=e.ref;if(n!==null){switch(e.tag){case 26:case 27:case 5:var l=e.stateNode;break;case 30:l=e.stateNode;break;default:l=e.stateNode}typeof n=="function"?e.refCleanup=n(l):n.current=l}}catch(o){tt(e,t,o)}}function Sa(e,t){var n=e.ref,l=e.refCleanup;if(n!==null)if(typeof l=="function")try{l()}catch(o){tt(e,t,o)}finally{e.refCleanup=null,e=e.alternate,e!=null&&(e.refCleanup=null)}else if(typeof n=="function")try{n(null)}catch(o){tt(e,t,o)}else n.current=null}function ep(e){var t=e.type,n=e.memoizedProps,l=e.stateNode;try{e:switch(t){case"button":case"input":case"select":case"textarea":n.autoFocus&&l.focus();break e;case"img":n.src?l.src=n.src:n.srcSet&&(l.srcset=n.srcSet)}}catch(o){tt(e,e.return,o)}}function Qc(e,t,n){try{var l=e.stateNode;Ay(l,e.type,n,t),l[Pt]=t}catch(o){tt(e,e.return,o)}}function tp(e){return e.tag===5||e.tag===3||e.tag===26||e.tag===27&&Ml(e.type)||e.tag===4}function Zc(e){e:for(;;){for(;e.sibling===null;){if(e.return===null||tp(e.return))return null;e=e.return}for(e.sibling.return=e.return,e=e.sibling;e.tag!==5&&e.tag!==6&&e.tag!==18;){if(e.tag===27&&Ml(e.type)||e.flags&2||e.child===null||e.tag===4)continue e;e.child.return=e,e=e.child}if(!(e.flags&2))return e.stateNode}}function Kc(e,t,n){var l=e.tag;if(l===5||l===6)e=e.stateNode,t?(n.nodeType===9?n.body:n.nodeName==="HTML"?n.ownerDocument.body:n).insertBefore(e,t):(t=n.nodeType===9?n.body:n.nodeName==="HTML"?n.ownerDocument.body:n,t.appendChild(e),n=n._reactRootContainer,n!=null||t.onclick!==null||(t.onclick=Ln));else if(l!==4&&(l===27&&Ml(e.type)&&(n=e.stateNode,t=null),e=e.child,e!==null))for(Kc(e,t,n),e=e.sibling;e!==null;)Kc(e,t,n),e=e.sibling}function Hf(e,t,n){var l=e.tag;if(l===5||l===6)e=e.stateNode,t?n.insertBefore(e,t):n.appendChild(e);else if(l!==4&&(l===27&&Ml(e.type)&&(n=e.stateNode),e=e.child,e!==null))for(Hf(e,t,n),e=e.sibling;e!==null;)Hf(e,t,n),e=e.sibling}function np(e){var t=e.stateNode,n=e.memoizedProps;try{for(var l=e.type,o=t.attributes;o.length;)t.removeAttributeNode(o[0]);Wt(t,l,n),t[Ot]=e,t[Pt]=n}catch(s){tt(e,e.return,s)}}var Za=!1,Mt=!1,Jc=!1,ap=typeof WeakSet=="function"?WeakSet:Set,Bt=null;function iy(e,t){if(e=e.containerInfo,ps=to,e=L(e),G(e)){if("selectionStart"in e)var n={start:e.selectionStart,end:e.selectionEnd};else e:{n=(n=e.ownerDocument)&&n.defaultView||window;var l=n.getSelection&&n.getSelection();if(l&&l.rangeCount!==0){n=l.anchorNode;var o=l.anchorOffset,s=l.focusNode;l=l.focusOffset;try{n.nodeType,s.nodeType}catch{n=null;break e}var b=0,C=-1,z=-1,Q=0,ee=0,ae=e,K=null;t:for(;;){for(var k;ae!==n||o!==0&&ae.nodeType!==3||(C=b+o),ae!==s||l!==0&&ae.nodeType!==3||(z=b+l),ae.nodeType===3&&(b+=ae.nodeValue.length),(k=ae.firstChild)!==null;)K=ae,ae=k;for(;;){if(ae===e)break t;if(K===n&&++Q===o&&(C=b),K===s&&++ee===l&&(z=b),(k=ae.nextSibling)!==null)break;ae=K,K=ae.parentNode}ae=k}n=C===-1||z===-1?null:{start:C,end:z}}else n=null}n=n||{start:0,end:0}}else n=null;for(vs={focusedElem:e,selectionRange:n},to=!1,Bt=t;Bt!==null;)if(t=Bt,e=t.child,(t.subtreeFlags&1028)!==0&&e!==null)e.return=t,Bt=e;else for(;Bt!==null;){switch(t=Bt,s=t.alternate,e=t.flags,t.tag){case 0:if((e&4)!==0&&(e=t.updateQueue,e=e!==null?e.events:null,e!==null))for(n=0;n<e.length;n++)o=e[n],o.ref.impl=o.nextImpl;break;case 11:case 15:break;case 1:if((e&1024)!==0&&s!==null){e=void 0,n=t,o=s.memoizedProps,s=s.memoizedState,l=n.stateNode;try{var ce=yi(n.type,o);e=l.getSnapshotBeforeUpdate(ce,s),l.__reactInternalSnapshotBeforeUpdate=e}catch(_e){tt(n,n.return,_e)}}break;case 3:if((e&1024)!==0){if(e=t.stateNode.containerInfo,n=e.nodeType,n===9)bs(e);else if(n===1)switch(e.nodeName){case"HEAD":case"HTML":case"BODY":bs(e);break;default:e.textContent=""}}break;case 5:case 26:case 27:case 6:case 4:case 17:break;default:if((e&1024)!==0)throw Error(f(163))}if(e=t.sibling,e!==null){e.return=t.return,Bt=e;break}Bt=t.return}}function lp(e,t,n){var l=n.flags;switch(n.tag){case 0:case 11:case 15:Ja(e,n),l&4&&vu(5,n);break;case 1:if(Ja(e,n),l&4)if(e=n.stateNode,t===null)try{e.componentDidMount()}catch(b){tt(n,n.return,b)}else{var o=yi(n.type,t.memoizedProps);t=t.memoizedState;try{e.componentDidUpdate(o,t,e.__reactInternalSnapshotBeforeUpdate)}catch(b){tt(n,n.return,b)}}l&64&&Ih(n),l&512&&mu(n,n.return);break;case 3:if(Ja(e,n),l&64&&(e=n.updateQueue,e!==null)){if(t=null,n.child!==null)switch(n.child.tag){case 27:case 5:t=n.child.stateNode;break;case 1:t=n.child.stateNode}try{Gd(e,t)}catch(b){tt(n,n.return,b)}}break;case 27:t===null&&l&4&&np(n);case 26:case 5:Ja(e,n),t===null&&l&4&&ep(n),l&512&&mu(n,n.return);break;case 12:Ja(e,n);break;case 31:Ja(e,n),l&4&&up(e,n);break;case 13:Ja(e,n),l&4&&fp(e,n),l&64&&(e=n.memoizedState,e!==null&&(e=e.dehydrated,e!==null&&(n=py.bind(null,n),Hy(e,n))));break;case 22:if(l=n.memoizedState!==null||Za,!l){t=t!==null&&t.memoizedState!==null||Mt,o=Za;var s=Mt;Za=l,(Mt=t)&&!s?ka(e,n,(n.subtreeFlags&8772)!==0):Ja(e,n),Za=o,Mt=s}break;case 30:break;default:Ja(e,n)}}function ip(e){var t=e.alternate;t!==null&&(e.alternate=null,ip(t)),e.child=null,e.deletions=null,e.sibling=null,e.tag===5&&(t=e.stateNode,t!==null&&al(t)),e.stateNode=null,e.return=null,e.dependencies=null,e.memoizedProps=null,e.memoizedState=null,e.pendingProps=null,e.stateNode=null,e.updateQueue=null}var pt=null,vn=!1;function Ka(e,t,n){for(n=n.child;n!==null;)rp(e,t,n),n=n.sibling}function rp(e,t,n){if(gt&&typeof gt.onCommitFiberUnmount=="function")try{gt.onCommitFiberUnmount(nn,n)}catch{}switch(n.tag){case 26:Mt||Sa(n,t),Ka(e,t,n),n.memoizedState?n.memoizedState.count--:n.stateNode&&(n=n.stateNode,n.parentNode.removeChild(n));break;case 27:Mt||Sa(n,t);var l=pt,o=vn;Ml(n.type)&&(pt=n.stateNode,vn=!1),Ka(e,t,n),Cu(n.stateNode),pt=l,vn=o;break;case 5:Mt||Sa(n,t);case 6:if(l=pt,o=vn,pt=null,Ka(e,t,n),pt=l,vn=o,pt!==null)if(vn)try{(pt.nodeType===9?pt.body:pt.nodeName==="HTML"?pt.ownerDocument.body:pt).removeChild(n.stateNode)}catch(s){tt(n,t,s)}else try{pt.removeChild(n.stateNode)}catch(s){tt(n,t,s)}break;case 18:pt!==null&&(vn?(e=pt,Ip(e.nodeType===9?e.body:e.nodeName==="HTML"?e.ownerDocument.body:e,n.stateNode),Sr(e)):Ip(pt,n.stateNode));break;case 4:l=pt,o=vn,pt=n.stateNode.containerInfo,vn=!0,Ka(e,t,n),pt=l,vn=o;break;case 0:case 11:case 14:case 15:xl(2,n,t),Mt||xl(4,n,t),Ka(e,t,n);break;case 1:Mt||(Sa(n,t),l=n.stateNode,typeof l.componentWillUnmount=="function"&&Ph(n,t,l)),Ka(e,t,n);break;case 21:Ka(e,t,n);break;case 22:Mt=(l=Mt)||n.memoizedState!==null,Ka(e,t,n),Mt=l;break;default:Ka(e,t,n)}}function up(e,t){if(t.memoizedState===null&&(e=t.alternate,e!==null&&(e=e.memoizedState,e!==null))){e=e.dehydrated;try{Sr(e)}catch(n){tt(t,t.return,n)}}}function fp(e,t){if(t.memoizedState===null&&(e=t.alternate,e!==null&&(e=e.memoizedState,e!==null&&(e=e.dehydrated,e!==null))))try{Sr(e)}catch(n){tt(t,t.return,n)}}function ry(e){switch(e.tag){case 31:case 13:case 19:var t=e.stateNode;return t===null&&(t=e.stateNode=new ap),t;case 22:return e=e.stateNode,t=e._retryCache,t===null&&(t=e._retryCache=new ap),t;default:throw Error(f(435,e.tag))}}function jf(e,t){var n=ry(e);t.forEach(function(l){if(!n.has(l)){n.add(l);var o=vy.bind(null,e,l);l.then(o,o)}})}function mn(e,t){var n=t.deletions;if(n!==null)for(var l=0;l<n.length;l++){var o=n[l],s=e,b=t,C=b;e:for(;C!==null;){switch(C.tag){case 27:if(Ml(C.type)){pt=C.stateNode,vn=!1;break e}break;case 5:pt=C.stateNode,vn=!1;break e;case 3:case 4:pt=C.stateNode.containerInfo,vn=!0;break e}C=C.return}if(pt===null)throw Error(f(160));rp(s,b,o),pt=null,vn=!1,s=o.alternate,s!==null&&(s.return=null),o.return=null}if(t.subtreeFlags&13886)for(t=t.child;t!==null;)op(t,e),t=t.sibling}var ua=null;function op(e,t){var n=e.alternate,l=e.flags;switch(e.tag){case 0:case 11:case 14:case 15:mn(t,e),yn(e),l&4&&(xl(3,e,e.return),vu(3,e),xl(5,e,e.return));break;case 1:mn(t,e),yn(e),l&512&&(Mt||n===null||Sa(n,n.return)),l&64&&Za&&(e=e.updateQueue,e!==null&&(l=e.callbacks,l!==null&&(n=e.shared.hiddenCallbacks,e.shared.hiddenCallbacks=n===null?l:n.concat(l))));break;case 26:var o=ua;if(mn(t,e),yn(e),l&512&&(Mt||n===null||Sa(n,n.return)),l&4){var s=n!==null?n.memoizedState:null;if(l=e.memoizedState,n===null)if(l===null)if(e.stateNode===null){e:{l=e.type,n=e.memoizedProps,o=o.ownerDocument||o;t:switch(l){case"title":s=o.getElementsByTagName("title")[0],(!s||s[Ea]||s[Ot]||s.namespaceURI==="http://www.w3.org/2000/svg"||s.hasAttribute("itemprop"))&&(s=o.createElement(l),o.head.insertBefore(s,o.querySelector("head > title"))),Wt(s,l,n),s[Ot]=e,St(s),l=s;break e;case"link":var b=ov("link","href",o).get(l+(n.href||""));if(b){for(var C=0;C<b.length;C++)if(s=b[C],s.getAttribute("href")===(n.href==null||n.href===""?null:n.href)&&s.getAttribute("rel")===(n.rel==null?null:n.rel)&&s.getAttribute("title")===(n.title==null?null:n.title)&&s.getAttribute("crossorigin")===(n.crossOrigin==null?null:n.crossOrigin)){b.splice(C,1);break t}}s=o.createElement(l),Wt(s,l,n),o.head.appendChild(s);break;case"meta":if(b=ov("meta","content",o).get(l+(n.content||""))){for(C=0;C<b.length;C++)if(s=b[C],s.getAttribute("content")===(n.content==null?null:""+n.content)&&s.getAttribute("name")===(n.name==null?null:n.name)&&s.getAttribute("property")===(n.property==null?null:n.property)&&s.getAttribute("http-equiv")===(n.httpEquiv==null?null:n.httpEquiv)&&s.getAttribute("charset")===(n.charSet==null?null:n.charSet)){b.splice(C,1);break t}}s=o.createElement(l),Wt(s,l,n),o.head.appendChild(s);break;default:throw Error(f(468,l))}s[Ot]=e,St(s),l=s}e.stateNode=l}else cv(o,e.type,e.stateNode);else e.stateNode=fv(o,l,e.memoizedProps);else s!==l?(s===null?n.stateNode!==null&&(n=n.stateNode,n.parentNode.removeChild(n)):s.count--,l===null?cv(o,e.type,e.stateNode):fv(o,l,e.memoizedProps)):l===null&&e.stateNode!==null&&Qc(e,e.memoizedProps,n.memoizedProps)}break;case 27:mn(t,e),yn(e),l&512&&(Mt||n===null||Sa(n,n.return)),n!==null&&l&4&&Qc(e,e.memoizedProps,n.memoizedProps);break;case 5:if(mn(t,e),yn(e),l&512&&(Mt||n===null||Sa(n,n.return)),e.flags&32){o=e.stateNode;try{ul(o,"")}catch(ce){tt(e,e.return,ce)}}l&4&&e.stateNode!=null&&(o=e.memoizedProps,Qc(e,o,n!==null?n.memoizedProps:o)),l&1024&&(Jc=!0);break;case 6:if(mn(t,e),yn(e),l&4){if(e.stateNode===null)throw Error(f(162));l=e.memoizedProps,n=e.stateNode;try{n.nodeValue=l}catch(ce){tt(e,e.return,ce)}}break;case 3:if($f=null,o=ua,ua=Ff(t.containerInfo),mn(t,e),ua=o,yn(e),l&4&&n!==null&&n.memoizedState.isDehydrated)try{Sr(t.containerInfo)}catch(ce){tt(e,e.return,ce)}Jc&&(Jc=!1,cp(e));break;case 4:l=ua,ua=Ff(e.stateNode.containerInfo),mn(t,e),yn(e),ua=l;break;case 12:mn(t,e),yn(e);break;case 31:mn(t,e),yn(e),l&4&&(l=e.updateQueue,l!==null&&(e.updateQueue=null,jf(e,l)));break;case 13:mn(t,e),yn(e),e.child.flags&8192&&e.memoizedState!==null!=(n!==null&&n.memoizedState!==null)&&(qf=wt()),l&4&&(l=e.updateQueue,l!==null&&(e.updateQueue=null,jf(e,l)));break;case 22:o=e.memoizedState!==null;var z=n!==null&&n.memoizedState!==null,Q=Za,ee=Mt;if(Za=Q||o,Mt=ee||z,mn(t,e),Mt=ee,Za=Q,yn(e),l&8192)e:for(t=e.stateNode,t._visibility=o?t._visibility&-2:t._visibility|1,o&&(n===null||z||Za||Mt||bi(e)),n=null,t=e;;){if(t.tag===5||t.tag===26){if(n===null){z=n=t;try{if(s=z.stateNode,o)b=s.style,typeof b.setProperty=="function"?b.setProperty("display","none","important"):b.display="none";else{C=z.stateNode;var ae=z.memoizedProps.style,K=ae!=null&&ae.hasOwnProperty("display")?ae.display:null;C.style.display=K==null||typeof K=="boolean"?"":(""+K).trim()}}catch(ce){tt(z,z.return,ce)}}}else if(t.tag===6){if(n===null){z=t;try{z.stateNode.nodeValue=o?"":z.memoizedProps}catch(ce){tt(z,z.return,ce)}}}else if(t.tag===18){if(n===null){z=t;try{var k=z.stateNode;o?Pp(k,!0):Pp(z.stateNode,!1)}catch(ce){tt(z,z.return,ce)}}}else if((t.tag!==22&&t.tag!==23||t.memoizedState===null||t===e)&&t.child!==null){t.child.return=t,t=t.child;continue}if(t===e)break e;for(;t.sibling===null;){if(t.return===null||t.return===e)break e;n===t&&(n=null),t=t.return}n===t&&(n=null),t.sibling.return=t.return,t=t.sibling}l&4&&(l=e.updateQueue,l!==null&&(n=l.retryQueue,n!==null&&(l.retryQueue=null,jf(e,n))));break;case 19:mn(t,e),yn(e),l&4&&(l=e.updateQueue,l!==null&&(e.updateQueue=null,jf(e,l)));break;case 30:break;case 21:break;default:mn(t,e),yn(e)}}function yn(e){var t=e.flags;if(t&2){try{for(var n,l=e.return;l!==null;){if(tp(l)){n=l;break}l=l.return}if(n==null)throw Error(f(160));switch(n.tag){case 27:var o=n.stateNode,s=Zc(e);Hf(e,s,o);break;case 5:var b=n.stateNode;n.flags&32&&(ul(b,""),n.flags&=-33);var C=Zc(e);Hf(e,C,b);break;case 3:case 4:var z=n.stateNode.containerInfo,Q=Zc(e);Kc(e,Q,z);break;default:throw Error(f(161))}}catch(ee){tt(e,e.return,ee)}e.flags&=-3}t&4096&&(e.flags&=-4097)}function cp(e){if(e.subtreeFlags&1024)for(e=e.child;e!==null;){var t=e;cp(t),t.tag===5&&t.flags&1024&&t.stateNode.reset(),e=e.sibling}}function Ja(e,t){if(t.subtreeFlags&8772)for(t=t.child;t!==null;)lp(e,t.alternate,t),t=t.sibling}function bi(e){for(e=e.child;e!==null;){var t=e;switch(t.tag){case 0:case 11:case 14:case 15:xl(4,t,t.return),bi(t);break;case 1:Sa(t,t.return);var n=t.stateNode;typeof n.componentWillUnmount=="function"&&Ph(t,t.return,n),bi(t);break;case 27:Cu(t.stateNode);case 26:case 5:Sa(t,t.return),bi(t);break;case 22:t.memoizedState===null&&bi(t);break;case 30:bi(t);break;default:bi(t)}e=e.sibling}}function ka(e,t,n){for(n=n&&(t.subtreeFlags&8772)!==0,t=t.child;t!==null;){var l=t.alternate,o=e,s=t,b=s.flags;switch(s.tag){case 0:case 11:case 15:ka(o,s,n),vu(4,s);break;case 1:if(ka(o,s,n),l=s,o=l.stateNode,typeof o.componentDidMount=="function")try{o.componentDidMount()}catch(Q){tt(l,l.return,Q)}if(l=s,o=l.updateQueue,o!==null){var C=l.stateNode;try{var z=o.shared.hiddenCallbacks;if(z!==null)for(o.shared.hiddenCallbacks=null,o=0;o<z.length;o++)Xd(z[o],C)}catch(Q){tt(l,l.return,Q)}}n&&b&64&&Ih(s),mu(s,s.return);break;case 27:np(s);case 26:case 5:ka(o,s,n),n&&l===null&&b&4&&ep(s),mu(s,s.return);break;case 12:ka(o,s,n);break;case 31:ka(o,s,n),n&&b&4&&up(o,s);break;case 13:ka(o,s,n),n&&b&4&&fp(o,s);break;case 22:s.memoizedState===null&&ka(o,s,n),mu(s,s.return);break;case 30:break;default:ka(o,s,n)}t=t.sibling}}function kc(e,t){var n=null;e!==null&&e.memoizedState!==null&&e.memoizedState.cachePool!==null&&(n=e.memoizedState.cachePool.pool),e=null,t.memoizedState!==null&&t.memoizedState.cachePool!==null&&(e=t.memoizedState.cachePool.pool),e!==n&&(e!=null&&e.refCount++,n!=null&&nu(n))}function Fc(e,t){e=null,t.alternate!==null&&(e=t.alternate.memoizedState.cache),t=t.memoizedState.cache,t!==e&&(t.refCount++,e!=null&&nu(e))}function fa(e,t,n,l){if(t.subtreeFlags&10256)for(t=t.child;t!==null;)sp(e,t,n,l),t=t.sibling}function sp(e,t,n,l){var o=t.flags;switch(t.tag){case 0:case 11:case 15:fa(e,t,n,l),o&2048&&vu(9,t);break;case 1:fa(e,t,n,l);break;case 3:fa(e,t,n,l),o&2048&&(e=null,t.alternate!==null&&(e=t.alternate.memoizedState.cache),t=t.memoizedState.cache,t!==e&&(t.refCount++,e!=null&&nu(e)));break;case 12:if(o&2048){fa(e,t,n,l),e=t.stateNode;try{var s=t.memoizedProps,b=s.id,C=s.onPostCommit;typeof C=="function"&&C(b,t.alternate===null?"mount":"update",e.passiveEffectDuration,-0)}catch(z){tt(t,t.return,z)}}else fa(e,t,n,l);break;case 31:fa(e,t,n,l);break;case 13:fa(e,t,n,l);break;case 23:break;case 22:s=t.stateNode,b=t.alternate,t.memoizedState!==null?s._visibility&2?fa(e,t,n,l):yu(e,t):s._visibility&2?fa(e,t,n,l):(s._visibility|=2,or(e,t,n,l,(t.subtreeFlags&10256)!==0||!1)),o&2048&&kc(b,t);break;case 24:fa(e,t,n,l),o&2048&&Fc(t.alternate,t);break;default:fa(e,t,n,l)}}function or(e,t,n,l,o){for(o=o&&((t.subtreeFlags&10256)!==0||!1),t=t.child;t!==null;){var s=e,b=t,C=n,z=l,Q=b.flags;switch(b.tag){case 0:case 11:case 15:or(s,b,C,z,o),vu(8,b);break;case 23:break;case 22:var ee=b.stateNode;b.memoizedState!==null?ee._visibility&2?or(s,b,C,z,o):yu(s,b):(ee._visibility|=2,or(s,b,C,z,o)),o&&Q&2048&&kc(b.alternate,b);break;case 24:or(s,b,C,z,o),o&&Q&2048&&Fc(b.alternate,b);break;default:or(s,b,C,z,o)}t=t.sibling}}function yu(e,t){if(t.subtreeFlags&10256)for(t=t.child;t!==null;){var n=e,l=t,o=l.flags;switch(l.tag){case 22:yu(n,l),o&2048&&kc(l.alternate,l);break;case 24:yu(n,l),o&2048&&Fc(l.alternate,l);break;default:yu(n,l)}t=t.sibling}}var bu=8192;function cr(e,t,n){if(e.subtreeFlags&bu)for(e=e.child;e!==null;)dp(e,t,n),e=e.sibling}function dp(e,t,n){switch(e.tag){case 26:cr(e,t,n),e.flags&bu&&e.memoizedState!==null&&Ky(n,ua,e.memoizedState,e.memoizedProps);break;case 5:cr(e,t,n);break;case 3:case 4:var l=ua;ua=Ff(e.stateNode.containerInfo),cr(e,t,n),ua=l;break;case 22:e.memoizedState===null&&(l=e.alternate,l!==null&&l.memoizedState!==null?(l=bu,bu=16777216,cr(e,t,n),bu=l):cr(e,t,n));break;default:cr(e,t,n)}}function hp(e){var t=e.alternate;if(t!==null&&(e=t.child,e!==null)){t.child=null;do t=e.sibling,e.sibling=null,e=t;while(e!==null)}}function gu(e){var t=e.deletions;if((e.flags&16)!==0){if(t!==null)for(var n=0;n<t.length;n++){var l=t[n];Bt=l,vp(l,e)}hp(e)}if(e.subtreeFlags&10256)for(e=e.child;e!==null;)pp(e),e=e.sibling}function pp(e){switch(e.tag){case 0:case 11:case 15:gu(e),e.flags&2048&&xl(9,e,e.return);break;case 3:gu(e);break;case 12:gu(e);break;case 22:var t=e.stateNode;e.memoizedState!==null&&t._visibility&2&&(e.return===null||e.return.tag!==13)?(t._visibility&=-3,Rf(e)):gu(e);break;default:gu(e)}}function Rf(e){var t=e.deletions;if((e.flags&16)!==0){if(t!==null)for(var n=0;n<t.length;n++){var l=t[n];Bt=l,vp(l,e)}hp(e)}for(e=e.child;e!==null;){switch(t=e,t.tag){case 0:case 11:case 15:xl(8,t,t.return),Rf(t);break;case 22:n=t.stateNode,n._visibility&2&&(n._visibility&=-3,Rf(t));break;default:Rf(t)}e=e.sibling}}function vp(e,t){for(;Bt!==null;){var n=Bt;switch(n.tag){case 0:case 11:case 15:xl(8,n,t);break;case 23:case 22:if(n.memoizedState!==null&&n.memoizedState.cachePool!==null){var l=n.memoizedState.cachePool.pool;l!=null&&l.refCount++}break;case 24:nu(n.memoizedState.cache)}if(l=n.child,l!==null)l.return=n,Bt=l;else e:for(n=e;Bt!==null;){l=Bt;var o=l.sibling,s=l.return;if(ip(l),l===n){Bt=null;break e}if(o!==null){o.return=s,Bt=o;break e}Bt=s}}}var uy={getCacheForType:function(e){var t=kt(At),n=t.data.get(e);return n===void 0&&(n=e(),t.data.set(e,n)),n},cacheSignal:function(){return kt(At).controller.signal}},fy=typeof WeakMap=="function"?WeakMap:Map,Ie=0,ft=null,He=null,Be=0,et=0,Tn=null,Dl=!1,sr=!1,Wc=!1,Fa=0,yt=0,Tl=0,gi=0,$c=0,Cn=0,dr=0,Su=null,bn=null,Ic=!1,qf=0,mp=0,Uf=1/0,Bf=null,Cl=null,Ht=0,Al=null,hr=null,Wa=0,Pc=0,es=null,yp=null,_u=0,ts=null;function An(){return(Ie&2)!==0&&Be!==0?Be&-Be:I.T!==null?us():ku()}function bp(){if(Cn===0)if((Be&536870912)===0||Ve){var e=In;In<<=1,(In&3932160)===0&&(In=262144),Cn=e}else Cn=536870912;return e=xn.current,e!==null&&(e.flags|=32),Cn}function gn(e,t,n){(e===ft&&(et===2||et===9)||e.cancelPendingCommit!==null)&&(pr(e,0),El(e,Be,Cn,!1)),el(e,n),((Ie&2)===0||e!==ft)&&(e===ft&&((Ie&2)===0&&(gi|=n),yt===4&&El(e,Be,Cn,!1)),_a(e))}function gp(e,t,n){if((Ie&6)!==0)throw Error(f(327));var l=!n&&(t&127)===0&&(t&e.expiredLanes)===0||qt(e,t),o=l?sy(e,t):as(e,t,!0),s=l;do{if(o===0){sr&&!l&&El(e,t,0,!1);break}else{if(n=e.current.alternate,s&&!oy(n)){o=as(e,t,!1),s=!1;continue}if(o===2){if(s=t,e.errorRecoveryDisabledLanes&s)var b=0;else b=e.pendingLanes&-536870913,b=b!==0?b:b&536870912?536870912:0;if(b!==0){t=b;e:{var C=e;o=Su;var z=C.current.memoizedState.isDehydrated;if(z&&(pr(C,b).flags|=256),b=as(C,b,!1),b!==2){if(Wc&&!z){C.errorRecoveryDisabledLanes|=s,gi|=s,o=4;break e}s=bn,bn=o,s!==null&&(bn===null?bn=s:bn.push.apply(bn,s))}o=b}if(s=!1,o!==2)continue}}if(o===1){pr(e,0),El(e,t,0,!0);break}e:{switch(l=e,s=o,s){case 0:case 1:throw Error(f(345));case 4:if((t&4194048)!==t)break;case 6:El(l,t,Cn,!Dl);break e;case 2:bn=null;break;case 3:case 5:break;default:throw Error(f(329))}if((t&62914560)===t&&(o=qf+300-wt(),10<o)){if(El(l,t,Cn,!Dl),Zl(l,0,!0)!==0)break e;Wa=t,l.timeoutHandle=Wp(Sp.bind(null,l,n,bn,Bf,Ic,t,Cn,gi,dr,Dl,s,"Throttled",-0,0),o);break e}Sp(l,n,bn,Bf,Ic,t,Cn,gi,dr,Dl,s,null,-0,0)}}break}while(!0);_a(e)}function Sp(e,t,n,l,o,s,b,C,z,Q,ee,ae,K,k){if(e.timeoutHandle=-1,ae=t.subtreeFlags,ae&8192||(ae&16785408)===16785408){ae={stylesheets:null,count:0,imgCount:0,imgBytes:0,suspenseyImages:[],waitingForImages:!0,waitingForViewTransition:!1,unsuspend:Ln},dp(t,s,ae);var ce=(s&62914560)===s?qf-wt():(s&4194048)===s?mp-wt():0;if(ce=Jy(ae,ce),ce!==null){Wa=s,e.cancelPendingCommit=ce(Np.bind(null,e,t,s,n,l,o,b,C,z,ee,ae,null,K,k)),El(e,s,b,!Q);return}}Np(e,t,s,n,l,o,b,C,z)}function oy(e){for(var t=e;;){var n=t.tag;if((n===0||n===11||n===15)&&t.flags&16384&&(n=t.updateQueue,n!==null&&(n=n.stores,n!==null)))for(var l=0;l<n.length;l++){var o=n[l],s=o.getSnapshot;o=o.value;try{if(!Zt(s(),o))return!1}catch{return!1}}if(n=t.child,t.subtreeFlags&16384&&n!==null)n.return=t,t=n;else{if(t===e)break;for(;t.sibling===null;){if(t.return===null||t.return===e)return!0;t=t.return}t.sibling.return=t.return,t=t.sibling}}return!0}function El(e,t,n,l){t&=~$c,t&=~gi,e.suspendedLanes|=t,e.pingedLanes&=~t,l&&(e.warmLanes|=t),l=e.expirationTimes;for(var o=t;0<o;){var s=31-ln(o),b=1<<s;l[s]=-1,o&=~b}n!==0&&Ta(e,n,t)}function Yf(){return(Ie&6)===0?(xu(0),!1):!0}function ns(){if(He!==null){if(et===0)var e=He.return;else e=He,Ba=ci=null,bc(e),lr=null,lu=0,e=He;for(;e!==null;)$h(e.alternate,e),e=e.return;He=null}}function pr(e,t){var n=e.timeoutHandle;n!==-1&&(e.timeoutHandle=-1,My(n)),n=e.cancelPendingCommit,n!==null&&(e.cancelPendingCommit=null,n()),Wa=0,ns(),ft=e,He=n=qa(e.current,null),Be=t,et=0,Tn=null,Dl=!1,sr=qt(e,t),Wc=!1,dr=Cn=$c=gi=Tl=yt=0,bn=Su=null,Ic=!1,(t&8)!==0&&(t|=t&32);var l=e.entangledLanes;if(l!==0)for(e=e.entanglements,l&=t;0<l;){var o=31-ln(l),s=1<<o;t|=e[o],l&=~s}return Fa=t,uf(),n}function _p(e,t){Ne=null,I.H=du,t===ar||t===vf?(t=qd(),et=3):t===rc?(t=qd(),et=4):et=t===Hc?8:t!==null&&typeof t=="object"&&typeof t.then=="function"?6:1,Tn=t,He===null&&(yt=1,Mf(e,Gn(t,e.current)))}function xp(){var e=xn.current;return e===null?!0:(Be&4194048)===Be?Kn===null:(Be&62914560)===Be||(Be&536870912)!==0?e===Kn:!1}function Dp(){var e=I.H;return I.H=du,e===null?du:e}function Tp(){var e=I.A;return I.A=uy,e}function Xf(){yt=4,Dl||(Be&4194048)!==Be&&xn.current!==null||(sr=!0),(Tl&134217727)===0&&(gi&134217727)===0||ft===null||El(ft,Be,Cn,!1)}function as(e,t,n){var l=Ie;Ie|=2;var o=Dp(),s=Tp();(ft!==e||Be!==t)&&(Bf=null,pr(e,t)),t=!1;var b=yt;e:do try{if(et!==0&&He!==null){var C=He,z=Tn;switch(et){case 8:ns(),b=6;break e;case 3:case 2:case 9:case 6:xn.current===null&&(t=!0);var Q=et;if(et=0,Tn=null,vr(e,C,z,Q),n&&sr){b=0;break e}break;default:Q=et,et=0,Tn=null,vr(e,C,z,Q)}}cy(),b=yt;break}catch(ee){_p(e,ee)}while(!0);return t&&e.shellSuspendCounter++,Ba=ci=null,Ie=l,I.H=o,I.A=s,He===null&&(ft=null,Be=0,uf()),b}function cy(){for(;He!==null;)Cp(He)}function sy(e,t){var n=Ie;Ie|=2;var l=Dp(),o=Tp();ft!==e||Be!==t?(Bf=null,Uf=wt()+500,pr(e,t)):sr=qt(e,t);e:do try{if(et!==0&&He!==null){t=He;var s=Tn;t:switch(et){case 1:et=0,Tn=null,vr(e,t,s,1);break;case 2:case 9:if(jd(s)){et=0,Tn=null,Ap(t);break}t=function(){et!==2&&et!==9||ft!==e||(et=7),_a(e)},s.then(t,t);break e;case 3:et=7;break e;case 4:et=5;break e;case 7:jd(s)?(et=0,Tn=null,Ap(t)):(et=0,Tn=null,vr(e,t,s,7));break;case 5:var b=null;switch(He.tag){case 26:b=He.memoizedState;case 5:case 27:var C=He;if(b?sv(b):C.stateNode.complete){et=0,Tn=null;var z=C.sibling;if(z!==null)He=z;else{var Q=C.return;Q!==null?(He=Q,Gf(Q)):He=null}break t}}et=0,Tn=null,vr(e,t,s,5);break;case 6:et=0,Tn=null,vr(e,t,s,6);break;case 8:ns(),yt=6;break e;default:throw Error(f(462))}}dy();break}catch(ee){_p(e,ee)}while(!0);return Ba=ci=null,I.H=l,I.A=o,Ie=n,He!==null?0:(ft=null,Be=0,uf(),yt)}function dy(){for(;He!==null&&!zn();)Cp(He)}function Cp(e){var t=Fh(e.alternate,e,Fa);e.memoizedProps=e.pendingProps,t===null?Gf(e):He=t}function Ap(e){var t=e,n=t.alternate;switch(t.tag){case 15:case 0:t=Vh(n,t,t.pendingProps,t.type,void 0,Be);break;case 11:t=Vh(n,t,t.pendingProps,t.type.render,t.ref,Be);break;case 5:bc(t);default:$h(n,t),t=He=Td(t,Fa),t=Fh(n,t,Fa)}e.memoizedProps=e.pendingProps,t===null?Gf(e):He=t}function vr(e,t,n,l){Ba=ci=null,bc(t),lr=null,lu=0;var o=t.return;try{if(ey(e,o,t,n,Be)){yt=1,Mf(e,Gn(n,e.current)),He=null;return}}catch(s){if(o!==null)throw He=o,s;yt=1,Mf(e,Gn(n,e.current)),He=null;return}t.flags&32768?(Ve||l===1?e=!0:sr||(Be&536870912)!==0?e=!1:(Dl=e=!0,(l===2||l===9||l===3||l===6)&&(l=xn.current,l!==null&&l.tag===13&&(l.flags|=16384))),Ep(t,e)):Gf(t)}function Gf(e){var t=e;do{if((t.flags&32768)!==0){Ep(t,Dl);return}e=t.return;var n=ay(t.alternate,t,Fa);if(n!==null){He=n;return}if(t=t.sibling,t!==null){He=t;return}He=t=e}while(t!==null);yt===0&&(yt=5)}function Ep(e,t){do{var n=ly(e.alternate,e);if(n!==null){n.flags&=32767,He=n;return}if(n=e.return,n!==null&&(n.flags|=32768,n.subtreeFlags=0,n.deletions=null),!t&&(e=e.sibling,e!==null)){He=e;return}He=e=n}while(e!==null);yt=6,He=null}function Np(e,t,n,l,o,s,b,C,z){e.cancelPendingCommit=null;do Vf();while(Ht!==0);if((Ie&6)!==0)throw Error(f(327));if(t!==null){if(t===e.current)throw Error(f(177));if(s=t.lanes|t.childLanes,s|=Zo,Ju(e,n,s,b,C,z),e===ft&&(He=ft=null,Be=0),hr=t,Al=e,Wa=n,Pc=s,es=o,yp=l,(t.subtreeFlags&10256)!==0||(t.flags&10256)!==0?(e.callbackNode=null,e.callbackPriority=0,my(wi,function(){return Lp(),null})):(e.callbackNode=null,e.callbackPriority=0),l=(t.flags&13878)!==0,(t.subtreeFlags&13878)!==0||l){l=I.T,I.T=null,o=ue.p,ue.p=2,b=Ie,Ie|=4;try{iy(e,t,n)}finally{Ie=b,ue.p=o,I.T=l}}Ht=1,Mp(),wp(),Op()}}function Mp(){if(Ht===1){Ht=0;var e=Al,t=hr,n=(t.flags&13878)!==0;if((t.subtreeFlags&13878)!==0||n){n=I.T,I.T=null;var l=ue.p;ue.p=2;var o=Ie;Ie|=4;try{op(t,e);var s=vs,b=L(e.containerInfo),C=s.focusedElem,z=s.selectionRange;if(b!==C&&C&&C.ownerDocument&&E(C.ownerDocument.documentElement,C)){if(z!==null&&G(C)){var Q=z.start,ee=z.end;if(ee===void 0&&(ee=Q),"selectionStart"in C)C.selectionStart=Q,C.selectionEnd=Math.min(ee,C.value.length);else{var ae=C.ownerDocument||document,K=ae&&ae.defaultView||window;if(K.getSelection){var k=K.getSelection(),ce=C.textContent.length,_e=Math.min(z.start,ce),ut=z.end===void 0?_e:Math.min(z.end,ce);!k.extend&&_e>ut&&(b=ut,ut=_e,_e=b);var q=Pr(C,_e),H=Pr(C,ut);if(q&&H&&(k.rangeCount!==1||k.anchorNode!==q.node||k.anchorOffset!==q.offset||k.focusNode!==H.node||k.focusOffset!==H.offset)){var V=ae.createRange();V.setStart(q.node,q.offset),k.removeAllRanges(),_e>ut?(k.addRange(V),k.extend(H.node,H.offset)):(V.setEnd(H.node,H.offset),k.addRange(V))}}}}for(ae=[],k=C;k=k.parentNode;)k.nodeType===1&&ae.push({element:k,left:k.scrollLeft,top:k.scrollTop});for(typeof C.focus=="function"&&C.focus(),C=0;C<ae.length;C++){var ne=ae[C];ne.element.scrollLeft=ne.left,ne.element.scrollTop=ne.top}}to=!!ps,vs=ps=null}finally{Ie=o,ue.p=l,I.T=n}}e.current=t,Ht=2}}function wp(){if(Ht===2){Ht=0;var e=Al,t=hr,n=(t.flags&8772)!==0;if((t.subtreeFlags&8772)!==0||n){n=I.T,I.T=null;var l=ue.p;ue.p=2;var o=Ie;Ie|=4;try{lp(e,t.alternate,t)}finally{Ie=o,ue.p=l,I.T=n}}Ht=3}}function Op(){if(Ht===4||Ht===3){Ht=0,Vl();var e=Al,t=hr,n=Wa,l=yp;(t.subtreeFlags&10256)!==0||(t.flags&10256)!==0?Ht=5:(Ht=0,hr=Al=null,zp(e,e.pendingLanes));var o=e.pendingLanes;if(o===0&&(Cl=null),Rr(n),t=t.stateNode,gt&&typeof gt.onCommitFiberRoot=="function")try{gt.onCommitFiberRoot(nn,t,void 0,(t.current.flags&128)===128)}catch{}if(l!==null){t=I.T,o=ue.p,ue.p=2,I.T=null;try{for(var s=e.onRecoverableError,b=0;b<l.length;b++){var C=l[b];s(C.value,{componentStack:C.stack})}}finally{I.T=t,ue.p=o}}(Wa&3)!==0&&Vf(),_a(e),o=e.pendingLanes,(n&261930)!==0&&(o&42)!==0?e===ts?_u++:(_u=0,ts=e):_u=0,xu(0)}}function zp(e,t){(e.pooledCacheLanes&=t)===0&&(t=e.pooledCache,t!=null&&(e.pooledCache=null,nu(t)))}function Vf(){return Mp(),wp(),Op(),Lp()}function Lp(){if(Ht!==5)return!1;var e=Al,t=Pc;Pc=0;var n=Rr(Wa),l=I.T,o=ue.p;try{ue.p=32>n?32:n,I.T=null,n=es,es=null;var s=Al,b=Wa;if(Ht=0,hr=Al=null,Wa=0,(Ie&6)!==0)throw Error(f(331));var C=Ie;if(Ie|=4,pp(s.current),sp(s,s.current,b,n),Ie=C,xu(0,!1),gt&&typeof gt.onPostCommitFiberRoot=="function")try{gt.onPostCommitFiberRoot(nn,s)}catch{}return!0}finally{ue.p=o,I.T=l,zp(e,t)}}function Hp(e,t,n){t=Gn(n,t),t=Lc(e.stateNode,t,2),e=gl(e,t,2),e!==null&&(el(e,2),_a(e))}function tt(e,t,n){if(e.tag===3)Hp(e,e,n);else for(;t!==null;){if(t.tag===3){Hp(t,e,n);break}else if(t.tag===1){var l=t.stateNode;if(typeof t.type.getDerivedStateFromError=="function"||typeof l.componentDidCatch=="function"&&(Cl===null||!Cl.has(l))){e=Gn(n,e),n=jh(2),l=gl(t,n,2),l!==null&&(Rh(n,l,t,e),el(l,2),_a(l));break}}t=t.return}}function ls(e,t,n){var l=e.pingCache;if(l===null){l=e.pingCache=new fy;var o=new Set;l.set(t,o)}else o=l.get(t),o===void 0&&(o=new Set,l.set(t,o));o.has(n)||(Wc=!0,o.add(n),e=hy.bind(null,e,t,n),t.then(e,e))}function hy(e,t,n){var l=e.pingCache;l!==null&&l.delete(t),e.pingedLanes|=e.suspendedLanes&n,e.warmLanes&=~n,ft===e&&(Be&n)===n&&(yt===4||yt===3&&(Be&62914560)===Be&&300>wt()-qf?(Ie&2)===0&&pr(e,0):$c|=n,dr===Be&&(dr=0)),_a(e)}function jp(e,t){t===0&&(t=Li()),e=ui(e,t),e!==null&&(el(e,t),_a(e))}function py(e){var t=e.memoizedState,n=0;t!==null&&(n=t.retryLane),jp(e,n)}function vy(e,t){var n=0;switch(e.tag){case 31:case 13:var l=e.stateNode,o=e.memoizedState;o!==null&&(n=o.retryLane);break;case 19:l=e.stateNode;break;case 22:l=e.stateNode._retryCache;break;default:throw Error(f(314))}l!==null&&l.delete(t),jp(e,n)}function my(e,t){return zr(e,t)}var Qf=null,mr=null,is=!1,Zf=!1,rs=!1,Nl=0;function _a(e){e!==mr&&e.next===null&&(mr===null?Qf=mr=e:mr=mr.next=e),Zf=!0,is||(is=!0,by())}function xu(e,t){if(!rs&&Zf){rs=!0;do for(var n=!1,l=Qf;l!==null;){if(e!==0){var o=l.pendingLanes;if(o===0)var s=0;else{var b=l.suspendedLanes,C=l.pingedLanes;s=(1<<31-ln(42|e)+1)-1,s&=o&~(b&~C),s=s&201326741?s&201326741|1:s?s|2:0}s!==0&&(n=!0,Bp(l,s))}else s=Be,s=Zl(l,l===ft?s:0,l.cancelPendingCommit!==null||l.timeoutHandle!==-1),(s&3)===0||qt(l,s)||(n=!0,Bp(l,s));l=l.next}while(n);rs=!1}}function yy(){Rp()}function Rp(){Zf=is=!1;var e=0;Nl!==0&&Ny()&&(e=Nl);for(var t=wt(),n=null,l=Qf;l!==null;){var o=l.next,s=qp(l,t);s===0?(l.next=null,n===null?Qf=o:n.next=o,o===null&&(mr=n)):(n=l,(e!==0||(s&3)!==0)&&(Zf=!0)),l=o}Ht!==0&&Ht!==5||xu(e),Nl!==0&&(Nl=0)}function qp(e,t){for(var n=e.suspendedLanes,l=e.pingedLanes,o=e.expirationTimes,s=e.pendingLanes&-62914561;0<s;){var b=31-ln(s),C=1<<b,z=o[b];z===-1?((C&n)===0||(C&l)!==0)&&(o[b]=It(C,t)):z<=t&&(e.expiredLanes|=C),s&=~C}if(t=ft,n=Be,n=Zl(e,e===t?n:0,e.cancelPendingCommit!==null||e.timeoutHandle!==-1),l=e.callbackNode,n===0||e===t&&(et===2||et===9)||e.cancelPendingCommit!==null)return l!==null&&l!==null&&Lr(l),e.callbackNode=null,e.callbackPriority=0;if((n&3)===0||qt(e,n)){if(t=n&-n,t===e.callbackPriority)return t;switch(l!==null&&Lr(l),Rr(n)){case 2:case 8:n=Vu;break;case 32:n=wi;break;case 268435456:n=Hr;break;default:n=wi}return l=Up.bind(null,e),n=zr(n,l),e.callbackPriority=t,e.callbackNode=n,t}return l!==null&&l!==null&&Lr(l),e.callbackPriority=2,e.callbackNode=null,2}function Up(e,t){if(Ht!==0&&Ht!==5)return e.callbackNode=null,e.callbackPriority=0,null;var n=e.callbackNode;if(Vf()&&e.callbackNode!==n)return null;var l=Be;return l=Zl(e,e===ft?l:0,e.cancelPendingCommit!==null||e.timeoutHandle!==-1),l===0?null:(gp(e,l,t),qp(e,wt()),e.callbackNode!=null&&e.callbackNode===n?Up.bind(null,e):null)}function Bp(e,t){if(Vf())return null;gp(e,t,!0)}function by(){wy(function(){(Ie&6)!==0?zr(Gt,yy):Rp()})}function us(){if(Nl===0){var e=tr;e===0&&(e=zi,zi<<=1,(zi&261888)===0&&(zi=256)),Nl=e}return Nl}function Yp(e){return e==null||typeof e=="symbol"||typeof e=="boolean"?null:typeof e=="function"?e:fl(""+e)}function Xp(e,t){var n=t.ownerDocument.createElement("input");return n.name=t.name,n.value=t.value,e.id&&n.setAttribute("form",e.id),t.parentNode.insertBefore(n,t),e=new FormData(e),n.parentNode.removeChild(n),e}function gy(e,t,n,l,o){if(t==="submit"&&n&&n.stateNode===o){var s=Yp((o[Pt]||null).action),b=l.submitter;b&&(t=(t=b[Pt]||null)?Yp(t.formAction):b.getAttribute("formAction"),t!==null&&(s=t,b=null));var C=new Xi("action","action",null,l,o);e.push({event:C,listeners:[{instance:null,listener:function(){if(l.defaultPrevented){if(Nl!==0){var z=b?Xp(o,b):new FormData(o);Ec(n,{pending:!0,data:z,method:o.method,action:s},null,z)}}else typeof s=="function"&&(C.preventDefault(),z=b?Xp(o,b):new FormData(o),Ec(n,{pending:!0,data:z,method:o.method,action:s},s,z))},currentTarget:o}]})}}for(var fs=0;fs<Qo.length;fs++){var os=Qo[fs],Sy=os.toLowerCase(),_y=os[0].toUpperCase()+os.slice(1);ra(Sy,"on"+_y)}ra(Pe,"onAnimationEnd"),ra(Kt,"onAnimationIteration"),ra(Lt,"onAnimationStart"),ra("dblclick","onDoubleClick"),ra("focusin","onFocus"),ra("focusout","onBlur"),ra(pn,"onTransitionRun"),ra(Yn,"onTransitionStart"),ra(dl,"onTransitionCancel"),ra(Ut,"onTransitionEnd"),pa("onMouseEnter",["mouseout","mouseover"]),pa("onMouseLeave",["mouseout","mouseover"]),pa("onPointerEnter",["pointerout","pointerover"]),pa("onPointerLeave",["pointerout","pointerover"]),wa("onChange","change click focusin focusout input keydown keyup selectionchange".split(" ")),wa("onSelect","focusout contextmenu dragend focusin keydown keyup mousedown mouseup selectionchange".split(" ")),wa("onBeforeInput",["compositionend","keypress","textInput","paste"]),wa("onCompositionEnd","compositionend focusout keydown keypress keyup mousedown".split(" ")),wa("onCompositionStart","compositionstart focusout keydown keypress keyup mousedown".split(" ")),wa("onCompositionUpdate","compositionupdate focusout keydown keypress keyup mousedown".split(" "));var Du="abort canplay canplaythrough durationchange emptied encrypted ended error loadeddata loadedmetadata loadstart pause play playing progress ratechange resize seeked seeking stalled suspend timeupdate volumechange waiting".split(" "),xy=new Set("beforetoggle cancel close invalid load scroll scrollend toggle".split(" ").concat(Du));function Gp(e,t){t=(t&4)!==0;for(var n=0;n<e.length;n++){var l=e[n],o=l.event;l=l.listeners;e:{var s=void 0;if(t)for(var b=l.length-1;0<=b;b--){var C=l[b],z=C.instance,Q=C.currentTarget;if(C=C.listener,z!==s&&o.isPropagationStopped())break e;s=C,o.currentTarget=Q;try{s(o)}catch(ee){rf(ee)}o.currentTarget=null,s=z}else for(b=0;b<l.length;b++){if(C=l[b],z=C.instance,Q=C.currentTarget,C=C.listener,z!==s&&o.isPropagationStopped())break e;s=C,o.currentTarget=Q;try{s(o)}catch(ee){rf(ee)}o.currentTarget=null,s=z}}}}function je(e,t){var n=t[qr];n===void 0&&(n=t[qr]=new Set);var l=e+"__bubble";n.has(l)||(Vp(t,e,2,!1),n.add(l))}function cs(e,t,n){var l=0;t&&(l|=4),Vp(n,e,l,t)}var Kf="_reactListening"+Math.random().toString(36).slice(2);function ss(e){if(!e[Kf]){e[Kf]=!0,Ur.forEach(function(n){n!=="selectionchange"&&(xy.has(n)||cs(n,!1,e),cs(n,!0,e))});var t=e.nodeType===9?e:e.ownerDocument;t===null||t[Kf]||(t[Kf]=!0,cs("selectionchange",!1,t))}}function Vp(e,t,n,l){switch(bv(t)){case 2:var o=Wy;break;case 8:o=$y;break;default:o=As}n=o.bind(null,t,n,e),o=void 0,!Zr||t!=="touchstart"&&t!=="touchmove"&&t!=="wheel"||(o=!0),l?o!==void 0?e.addEventListener(t,n,{capture:!0,passive:o}):e.addEventListener(t,n,!0):o!==void 0?e.addEventListener(t,n,{passive:o}):e.addEventListener(t,n,!1)}function ds(e,t,n,l,o){var s=l;if((t&1)===0&&(t&2)===0&&l!==null)e:for(;;){if(l===null)return;var b=l.tag;if(b===3||b===4){var C=l.stateNode.containerInfo;if(C===o)break;if(b===4)for(b=l.return;b!==null;){var z=b.tag;if((z===3||z===4)&&b.stateNode.containerInfo===o)return;b=b.return}for(;C!==null;){if(b=Na(C),b===null)return;if(z=b.tag,z===5||z===6||z===26||z===27){l=s=b;continue e}C=C.parentNode}}l=l.return}af(function(){var Q=s,ee=Qr(n),ae=[];e:{var K=xd.get(e);if(K!==void 0){var k=Xi,ce=e;switch(e){case"keypress":if(ei(n)===0)break e;case"keydown":case"keyup":k=Ue;break;case"focusin":ce="focus",k=p;break;case"focusout":ce="blur",k=p;break;case"beforeblur":case"afterblur":k=p;break;case"click":if(n.button===2)break e;case"auxclick":case"dblclick":case"mousedown":case"mousemove":case"mouseup":case"mouseout":case"mouseover":case"contextmenu":k=lf;break;case"drag":case"dragend":case"dragenter":case"dragexit":case"dragleave":case"dragover":case"dragstart":case"drop":k=u;break;case"touchcancel":case"touchend":case"touchmove":case"touchstart":k=Hn;break;case Pe:case Kt:case Lt:k=S;break;case Ut:k=Le;break;case"scroll":case"scrollend":k=qo;break;case"wheel":k=Ge;break;case"copy":case"cut":case"paste":k=D;break;case"gotpointercapture":case"lostpointercapture":case"pointercancel":case"pointerdown":case"pointermove":case"pointerout":case"pointerover":case"pointerup":k=Ct;break;case"toggle":case"beforetoggle":k=ni}var _e=(t&4)!==0,ut=!_e&&(e==="scroll"||e==="scrollend"),q=_e?K!==null?K+"Capture":null:K;_e=[];for(var H=Q,V;H!==null;){var ne=H;if(V=ne.stateNode,ne=ne.tag,ne!==5&&ne!==26&&ne!==27||V===null||q===null||(ne=$l(H,q),ne!=null&&_e.push(Tu(H,ne,V))),ut)break;H=H.return}0<_e.length&&(K=new k(K,ce,null,n,ee),ae.push({event:K,listeners:_e}))}}if((t&7)===0){e:{if(K=e==="mouseover"||e==="pointerover",k=e==="mouseout"||e==="pointerout",K&&n!==Bi&&(ce=n.relatedTarget||n.fromElement)&&(Na(ce)||ce[Aa]))break e;if((k||K)&&(K=ee.window===ee?ee:(K=ee.ownerDocument)?K.defaultView||K.parentWindow:window,k?(ce=n.relatedTarget||n.toElement,k=Q,ce=ce?Na(ce):null,ce!==null&&(ut=h(ce),_e=ce.tag,ce!==ut||_e!==5&&_e!==27&&_e!==6)&&(ce=null)):(k=null,ce=Q),k!==ce)){if(_e=lf,ne="onMouseLeave",q="onMouseEnter",H="mouse",(e==="pointerout"||e==="pointerover")&&(_e=Ct,ne="onPointerLeave",q="onPointerEnter",H="pointer"),ut=k==null?K:ea(k),V=ce==null?K:ea(ce),K=new _e(ne,H+"leave",k,n,ee),K.target=ut,K.relatedTarget=V,ne=null,Na(ee)===Q&&(_e=new _e(q,H+"enter",ce,n,ee),_e.target=V,_e.relatedTarget=ut,ne=_e),ut=ne,k&&ce)t:{for(_e=Dy,q=k,H=ce,V=0,ne=q;ne;ne=_e(ne))V++;ne=0;for(var be=H;be;be=_e(be))ne++;for(;0<V-ne;)q=_e(q),V--;for(;0<ne-V;)H=_e(H),ne--;for(;V--;){if(q===H||H!==null&&q===H.alternate){_e=q;break t}q=_e(q),H=_e(H)}_e=null}else _e=null;k!==null&&Qp(ae,K,k,_e,!1),ce!==null&&ut!==null&&Qp(ae,ut,ce,_e,!0)}}e:{if(K=Q?ea(Q):window,k=K.nodeName&&K.nodeName.toLowerCase(),k==="select"||k==="input"&&K.type==="file")var ke=Fr;else if(Je(K))if(ya)ke=Ji;else{ke=$r;var de=Ki}else k=K.nodeName,!k||k.toLowerCase()!=="input"||K.type!=="checkbox"&&K.type!=="radio"?Q&&Vr(Q.elementType)&&(ke=Fr):ke=Vo;if(ke&&(ke=ke(e,Q))){ai(ae,ke,n,ee);break e}de&&de(e,K,Q),e==="focusout"&&Q&&K.type==="number"&&Q.memoizedProps.value!=null&&un(K,"number",K.value)}switch(de=Q?ea(Q):window,e){case"focusin":(Je(de)||de.contentEditable==="true")&&(P=de,ie=Q,fe=null);break;case"focusout":fe=ie=P=null;break;case"mousedown":se=!0;break;case"contextmenu":case"mouseup":case"dragend":se=!1,oe(ae,n,ee);break;case"selectionchange":if(J)break;case"keydown":case"keyup":oe(ae,n,ee)}var we;if(hn)e:{switch(e){case"compositionstart":var Ye="onCompositionStart";break e;case"compositionend":Ye="onCompositionEnd";break e;case"compositionupdate":Ye="onCompositionUpdate";break e}Ye=void 0}else en?Qi(e,n)&&(Ye="onCompositionEnd"):e==="keydown"&&n.keyCode===229&&(Ye="onCompositionStart");Ye&&(la&&n.locale!=="ko"&&(en||Ye!=="onCompositionStart"?Ye==="onCompositionEnd"&&en&&(we=Yi()):(ma=ee,Kr="value"in ma?ma.value:ma.textContent,en=!0)),de=Jf(Q,Ye),0<de.length&&(Ye=new w(Ye,e,null,n,ee),ae.push({event:Ye,listeners:de}),we?Ye.data=we:(we=qn(n),we!==null&&(Ye.data=we)))),(we=Rn?Bo(e,n):Yo(e,n))&&(Ye=Jf(Q,"onBeforeInput"),0<Ye.length&&(de=new w("onBeforeInput","beforeinput",null,n,ee),ae.push({event:de,listeners:Ye}),de.data=we)),gy(ae,e,Q,n,ee)}Gp(ae,t)})}function Tu(e,t,n){return{instance:e,listener:t,currentTarget:n}}function Jf(e,t){for(var n=t+"Capture",l=[];e!==null;){var o=e,s=o.stateNode;if(o=o.tag,o!==5&&o!==26&&o!==27||s===null||(o=$l(e,n),o!=null&&l.unshift(Tu(e,o,s)),o=$l(e,t),o!=null&&l.push(Tu(e,o,s))),e.tag===3)return l;e=e.return}return[]}function Dy(e){if(e===null)return null;do e=e.return;while(e&&e.tag!==5&&e.tag!==27);return e||null}function Qp(e,t,n,l,o){for(var s=t._reactName,b=[];n!==null&&n!==l;){var C=n,z=C.alternate,Q=C.stateNode;if(C=C.tag,z!==null&&z===l)break;C!==5&&C!==26&&C!==27||Q===null||(z=Q,o?(Q=$l(n,s),Q!=null&&b.unshift(Tu(n,Q,z))):o||(Q=$l(n,s),Q!=null&&b.push(Tu(n,Q,z)))),n=n.return}b.length!==0&&e.push({event:t,listeners:b})}var Ty=/\r\n?/g,Cy=/\u0000|\uFFFD/g;function Zp(e){return(typeof e=="string"?e:""+e).replace(Ty,`
`).replace(Cy,"")}function Kp(e,t){return t=Zp(t),Zp(e)===t}function rt(e,t,n,l,o,s){switch(n){case"children":typeof l=="string"?t==="body"||t==="textarea"&&l===""||ul(e,l):(typeof l=="number"||typeof l=="bigint")&&t!=="body"&&ul(e,""+l);break;case"className":Kl(e,"class",l);break;case"tabIndex":Kl(e,"tabindex",l);break;case"dir":case"role":case"viewBox":case"width":case"height":Kl(e,n,l);break;case"style":Ui(e,l,s);break;case"data":if(t!=="object"){Kl(e,"data",l);break}case"src":case"href":if(l===""&&(t!=="a"||n!=="href")){e.removeAttribute(n);break}if(l==null||typeof l=="function"||typeof l=="symbol"||typeof l=="boolean"){e.removeAttribute(n);break}l=fl(""+l),e.setAttribute(n,l);break;case"action":case"formAction":if(typeof l=="function"){e.setAttribute(n,"javascript:throw new Error('A React form was unexpectedly submitted. If you called form.submit() manually, consider using form.requestSubmit() instead. If you\\'re trying to use event.stopPropagation() in a submit event handler, consider also calling event.preventDefault().')");break}else typeof s=="function"&&(n==="formAction"?(t!=="input"&&rt(e,t,"name",o.name,o,null),rt(e,t,"formEncType",o.formEncType,o,null),rt(e,t,"formMethod",o.formMethod,o,null),rt(e,t,"formTarget",o.formTarget,o,null)):(rt(e,t,"encType",o.encType,o,null),rt(e,t,"method",o.method,o,null),rt(e,t,"target",o.target,o,null)));if(l==null||typeof l=="symbol"||typeof l=="boolean"){e.removeAttribute(n);break}l=fl(""+l),e.setAttribute(n,l);break;case"onClick":l!=null&&(e.onclick=Ln);break;case"onScroll":l!=null&&je("scroll",e);break;case"onScrollEnd":l!=null&&je("scrollend",e);break;case"dangerouslySetInnerHTML":if(l!=null){if(typeof l!="object"||!("__html"in l))throw Error(f(61));if(n=l.__html,n!=null){if(o.children!=null)throw Error(f(60));e.innerHTML=n}}break;case"multiple":e.multiple=l&&typeof l!="function"&&typeof l!="symbol";break;case"muted":e.muted=l&&typeof l!="function"&&typeof l!="symbol";break;case"suppressContentEditableWarning":case"suppressHydrationWarning":case"defaultValue":case"defaultChecked":case"innerHTML":case"ref":break;case"autoFocus":break;case"xlinkHref":if(l==null||typeof l=="function"||typeof l=="boolean"||typeof l=="symbol"){e.removeAttribute("xlink:href");break}n=fl(""+l),e.setAttributeNS("http://www.w3.org/1999/xlink","xlink:href",n);break;case"contentEditable":case"spellCheck":case"draggable":case"value":case"autoReverse":case"externalResourcesRequired":case"focusable":case"preserveAlpha":l!=null&&typeof l!="function"&&typeof l!="symbol"?e.setAttribute(n,""+l):e.removeAttribute(n);break;case"inert":case"allowFullScreen":case"async":case"autoPlay":case"controls":case"default":case"defer":case"disabled":case"disablePictureInPicture":case"disableRemotePlayback":case"formNoValidate":case"hidden":case"loop":case"noModule":case"noValidate":case"open":case"playsInline":case"readOnly":case"required":case"reversed":case"scoped":case"seamless":case"itemScope":l&&typeof l!="function"&&typeof l!="symbol"?e.setAttribute(n,""):e.removeAttribute(n);break;case"capture":case"download":l===!0?e.setAttribute(n,""):l!==!1&&l!=null&&typeof l!="function"&&typeof l!="symbol"?e.setAttribute(n,l):e.removeAttribute(n);break;case"cols":case"rows":case"size":case"span":l!=null&&typeof l!="function"&&typeof l!="symbol"&&!isNaN(l)&&1<=l?e.setAttribute(n,l):e.removeAttribute(n);break;case"rowSpan":case"start":l==null||typeof l=="function"||typeof l=="symbol"||isNaN(l)?e.removeAttribute(n):e.setAttribute(n,l);break;case"popover":je("beforetoggle",e),je("toggle",e),il(e,"popover",l);break;case"xlinkActuate":dt(e,"http://www.w3.org/1999/xlink","xlink:actuate",l);break;case"xlinkArcrole":dt(e,"http://www.w3.org/1999/xlink","xlink:arcrole",l);break;case"xlinkRole":dt(e,"http://www.w3.org/1999/xlink","xlink:role",l);break;case"xlinkShow":dt(e,"http://www.w3.org/1999/xlink","xlink:show",l);break;case"xlinkTitle":dt(e,"http://www.w3.org/1999/xlink","xlink:title",l);break;case"xlinkType":dt(e,"http://www.w3.org/1999/xlink","xlink:type",l);break;case"xmlBase":dt(e,"http://www.w3.org/XML/1998/namespace","xml:base",l);break;case"xmlLang":dt(e,"http://www.w3.org/XML/1998/namespace","xml:lang",l);break;case"xmlSpace":dt(e,"http://www.w3.org/XML/1998/namespace","xml:space",l);break;case"is":il(e,"is",l);break;case"innerText":case"textContent":break;default:(!(2<n.length)||n[0]!=="o"&&n[0]!=="O"||n[1]!=="n"&&n[1]!=="N")&&(n=Fl.get(n)||n,il(e,n,l))}}function hs(e,t,n,l,o,s){switch(n){case"style":Ui(e,l,s);break;case"dangerouslySetInnerHTML":if(l!=null){if(typeof l!="object"||!("__html"in l))throw Error(f(61));if(n=l.__html,n!=null){if(o.children!=null)throw Error(f(60));e.innerHTML=n}}break;case"children":typeof l=="string"?ul(e,l):(typeof l=="number"||typeof l=="bigint")&&ul(e,""+l);break;case"onScroll":l!=null&&je("scroll",e);break;case"onScrollEnd":l!=null&&je("scrollend",e);break;case"onClick":l!=null&&(e.onclick=Ln);break;case"suppressContentEditableWarning":case"suppressHydrationWarning":case"innerHTML":case"ref":break;case"innerText":case"textContent":break;default:if(!Br.hasOwnProperty(n))e:{if(n[0]==="o"&&n[1]==="n"&&(o=n.endsWith("Capture"),t=n.slice(2,o?n.length-7:void 0),s=e[Pt]||null,s=s!=null?s[n]:null,typeof s=="function"&&e.removeEventListener(t,s,o),typeof l=="function")){typeof s!="function"&&s!==null&&(n in e?e[n]=null:e.hasAttribute(n)&&e.removeAttribute(n)),e.addEventListener(t,l,o);break e}n in e?e[n]=l:l===!0?e.setAttribute(n,""):il(e,n,l)}}}function Wt(e,t,n){switch(t){case"div":case"span":case"svg":case"path":case"a":case"g":case"p":case"li":break;case"img":je("error",e),je("load",e);var l=!1,o=!1,s;for(s in n)if(n.hasOwnProperty(s)){var b=n[s];if(b!=null)switch(s){case"src":l=!0;break;case"srcSet":o=!0;break;case"children":case"dangerouslySetInnerHTML":throw Error(f(137,t));default:rt(e,t,s,b,n,null)}}o&&rt(e,t,"srcSet",n.srcSet,n,null),l&&rt(e,t,"src",n.src,n,null);return;case"input":je("invalid",e);var C=s=b=o=null,z=null,Q=null;for(l in n)if(n.hasOwnProperty(l)){var ee=n[l];if(ee!=null)switch(l){case"name":o=ee;break;case"type":b=ee;break;case"checked":z=ee;break;case"defaultChecked":Q=ee;break;case"value":s=ee;break;case"defaultValue":C=ee;break;case"children":case"dangerouslySetInnerHTML":if(ee!=null)throw Error(f(137,t));break;default:rt(e,t,l,ee,n,null)}}Pu(e,s,C,z,Q,b,o,!1);return;case"select":je("invalid",e),l=b=s=null;for(o in n)if(n.hasOwnProperty(o)&&(C=n[o],C!=null))switch(o){case"value":s=C;break;case"defaultValue":b=C;break;case"multiple":l=C;default:rt(e,t,o,C,n,null)}t=s,n=b,e.multiple=!!l,t!=null?Oa(e,!!l,t,!1):n!=null&&Oa(e,!!l,n,!0);return;case"textarea":je("invalid",e),s=o=l=null;for(b in n)if(n.hasOwnProperty(b)&&(C=n[b],C!=null))switch(b){case"value":l=C;break;case"defaultValue":o=C;break;case"children":s=C;break;case"dangerouslySetInnerHTML":if(C!=null)throw Error(f(91));break;default:rt(e,t,b,C,n,null)}ef(e,l,o,s);return;case"option":for(z in n)if(n.hasOwnProperty(z)&&(l=n[z],l!=null))switch(z){case"selected":e.selected=l&&typeof l!="function"&&typeof l!="symbol";break;default:rt(e,t,z,l,n,null)}return;case"dialog":je("beforetoggle",e),je("toggle",e),je("cancel",e),je("close",e);break;case"iframe":case"object":je("load",e);break;case"video":case"audio":for(l=0;l<Du.length;l++)je(Du[l],e);break;case"image":je("error",e),je("load",e);break;case"details":je("toggle",e);break;case"embed":case"source":case"link":je("error",e),je("load",e);case"area":case"base":case"br":case"col":case"hr":case"keygen":case"meta":case"param":case"track":case"wbr":case"menuitem":for(Q in n)if(n.hasOwnProperty(Q)&&(l=n[Q],l!=null))switch(Q){case"children":case"dangerouslySetInnerHTML":throw Error(f(137,t));default:rt(e,t,Q,l,n,null)}return;default:if(Vr(t)){for(ee in n)n.hasOwnProperty(ee)&&(l=n[ee],l!==void 0&&hs(e,t,ee,l,n,void 0));return}}for(C in n)n.hasOwnProperty(C)&&(l=n[C],l!=null&&rt(e,t,C,l,n,null))}function Ay(e,t,n,l){switch(t){case"div":case"span":case"svg":case"path":case"a":case"g":case"p":case"li":break;case"input":var o=null,s=null,b=null,C=null,z=null,Q=null,ee=null;for(k in n){var ae=n[k];if(n.hasOwnProperty(k)&&ae!=null)switch(k){case"checked":break;case"value":break;case"defaultValue":z=ae;default:l.hasOwnProperty(k)||rt(e,t,k,null,l,ae)}}for(var K in l){var k=l[K];if(ae=n[K],l.hasOwnProperty(K)&&(k!=null||ae!=null))switch(K){case"type":s=k;break;case"name":o=k;break;case"checked":Q=k;break;case"defaultChecked":ee=k;break;case"value":b=k;break;case"defaultValue":C=k;break;case"children":case"dangerouslySetInnerHTML":if(k!=null)throw Error(f(137,t));break;default:k!==ae&&rt(e,t,K,k,l,ae)}}Gr(e,b,C,z,Q,ee,s,o);return;case"select":k=b=C=K=null;for(s in n)if(z=n[s],n.hasOwnProperty(s)&&z!=null)switch(s){case"value":break;case"multiple":k=z;default:l.hasOwnProperty(s)||rt(e,t,s,null,l,z)}for(o in l)if(s=l[o],z=n[o],l.hasOwnProperty(o)&&(s!=null||z!=null))switch(o){case"value":K=s;break;case"defaultValue":C=s;break;case"multiple":b=s;default:s!==z&&rt(e,t,o,s,l,z)}t=C,n=b,l=k,K!=null?Oa(e,!!n,K,!1):!!l!=!!n&&(t!=null?Oa(e,!!n,t,!0):Oa(e,!!n,n?[]:"",!1));return;case"textarea":k=K=null;for(C in n)if(o=n[C],n.hasOwnProperty(C)&&o!=null&&!l.hasOwnProperty(C))switch(C){case"value":break;case"children":break;default:rt(e,t,C,null,l,o)}for(b in l)if(o=l[b],s=n[b],l.hasOwnProperty(b)&&(o!=null||s!=null))switch(b){case"value":K=o;break;case"defaultValue":k=o;break;case"children":break;case"dangerouslySetInnerHTML":if(o!=null)throw Error(f(91));break;default:o!==s&&rt(e,t,b,o,l,s)}rl(e,K,k);return;case"option":for(var ce in n)if(K=n[ce],n.hasOwnProperty(ce)&&K!=null&&!l.hasOwnProperty(ce))switch(ce){case"selected":e.selected=!1;break;default:rt(e,t,ce,null,l,K)}for(z in l)if(K=l[z],k=n[z],l.hasOwnProperty(z)&&K!==k&&(K!=null||k!=null))switch(z){case"selected":e.selected=K&&typeof K!="function"&&typeof K!="symbol";break;default:rt(e,t,z,K,l,k)}return;case"img":case"link":case"area":case"base":case"br":case"col":case"embed":case"hr":case"keygen":case"meta":case"param":case"source":case"track":case"wbr":case"menuitem":for(var _e in n)K=n[_e],n.hasOwnProperty(_e)&&K!=null&&!l.hasOwnProperty(_e)&&rt(e,t,_e,null,l,K);for(Q in l)if(K=l[Q],k=n[Q],l.hasOwnProperty(Q)&&K!==k&&(K!=null||k!=null))switch(Q){case"children":case"dangerouslySetInnerHTML":if(K!=null)throw Error(f(137,t));break;default:rt(e,t,Q,K,l,k)}return;default:if(Vr(t)){for(var ut in n)K=n[ut],n.hasOwnProperty(ut)&&K!==void 0&&!l.hasOwnProperty(ut)&&hs(e,t,ut,void 0,l,K);for(ee in l)K=l[ee],k=n[ee],!l.hasOwnProperty(ee)||K===k||K===void 0&&k===void 0||hs(e,t,ee,K,l,k);return}}for(var q in n)K=n[q],n.hasOwnProperty(q)&&K!=null&&!l.hasOwnProperty(q)&&rt(e,t,q,null,l,K);for(ae in l)K=l[ae],k=n[ae],!l.hasOwnProperty(ae)||K===k||K==null&&k==null||rt(e,t,ae,K,l,k)}function Jp(e){switch(e){case"css":case"script":case"font":case"img":case"image":case"input":case"link":return!0;default:return!1}}function Ey(){if(typeof performance.getEntriesByType=="function"){for(var e=0,t=0,n=performance.getEntriesByType("resource"),l=0;l<n.length;l++){var o=n[l],s=o.transferSize,b=o.initiatorType,C=o.duration;if(s&&C&&Jp(b)){for(b=0,C=o.responseEnd,l+=1;l<n.length;l++){var z=n[l],Q=z.startTime;if(Q>C)break;var ee=z.transferSize,ae=z.initiatorType;ee&&Jp(ae)&&(z=z.responseEnd,b+=ee*(z<C?1:(C-Q)/(z-Q)))}if(--l,t+=8*(s+b)/(o.duration/1e3),e++,10<e)break}}if(0<e)return t/e/1e6}return navigator.connection&&(e=navigator.connection.downlink,typeof e=="number")?e:5}var ps=null,vs=null;function kf(e){return e.nodeType===9?e:e.ownerDocument}function kp(e){switch(e){case"http://www.w3.org/2000/svg":return 1;case"http://www.w3.org/1998/Math/MathML":return 2;default:return 0}}function Fp(e,t){if(e===0)switch(t){case"svg":return 1;case"math":return 2;default:return 0}return e===1&&t==="foreignObject"?0:e}function ms(e,t){return e==="textarea"||e==="noscript"||typeof t.children=="string"||typeof t.children=="number"||typeof t.children=="bigint"||typeof t.dangerouslySetInnerHTML=="object"&&t.dangerouslySetInnerHTML!==null&&t.dangerouslySetInnerHTML.__html!=null}var ys=null;function Ny(){var e=window.event;return e&&e.type==="popstate"?e===ys?!1:(ys=e,!0):(ys=null,!1)}var Wp=typeof setTimeout=="function"?setTimeout:void 0,My=typeof clearTimeout=="function"?clearTimeout:void 0,$p=typeof Promise=="function"?Promise:void 0,wy=typeof queueMicrotask=="function"?queueMicrotask:typeof $p<"u"?function(e){return $p.resolve(null).then(e).catch(Oy)}:Wp;function Oy(e){setTimeout(function(){throw e})}function Ml(e){return e==="head"}function Ip(e,t){var n=t,l=0;do{var o=n.nextSibling;if(e.removeChild(n),o&&o.nodeType===8)if(n=o.data,n==="/$"||n==="/&"){if(l===0){e.removeChild(o),Sr(t);return}l--}else if(n==="$"||n==="$?"||n==="$~"||n==="$!"||n==="&")l++;else if(n==="html")Cu(e.ownerDocument.documentElement);else if(n==="head"){n=e.ownerDocument.head,Cu(n);for(var s=n.firstChild;s;){var b=s.nextSibling,C=s.nodeName;s[Ea]||C==="SCRIPT"||C==="STYLE"||C==="LINK"&&s.rel.toLowerCase()==="stylesheet"||n.removeChild(s),s=b}}else n==="body"&&Cu(e.ownerDocument.body);n=o}while(n);Sr(t)}function Pp(e,t){var n=e;e=0;do{var l=n.nextSibling;if(n.nodeType===1?t?(n._stashedDisplay=n.style.display,n.style.display="none"):(n.style.display=n._stashedDisplay||"",n.getAttribute("style")===""&&n.removeAttribute("style")):n.nodeType===3&&(t?(n._stashedText=n.nodeValue,n.nodeValue=""):n.nodeValue=n._stashedText||""),l&&l.nodeType===8)if(n=l.data,n==="/$"){if(e===0)break;e--}else n!=="$"&&n!=="$?"&&n!=="$~"&&n!=="$!"||e++;n=l}while(n)}function bs(e){var t=e.firstChild;for(t&&t.nodeType===10&&(t=t.nextSibling);t;){var n=t;switch(t=t.nextSibling,n.nodeName){case"HTML":case"HEAD":case"BODY":bs(n),al(n);continue;case"SCRIPT":case"STYLE":continue;case"LINK":if(n.rel.toLowerCase()==="stylesheet")continue}e.removeChild(n)}}function zy(e,t,n,l){for(;e.nodeType===1;){var o=n;if(e.nodeName.toLowerCase()!==t.toLowerCase()){if(!l&&(e.nodeName!=="INPUT"||e.type!=="hidden"))break}else if(l){if(!e[Ea])switch(t){case"meta":if(!e.hasAttribute("itemprop"))break;return e;case"link":if(s=e.getAttribute("rel"),s==="stylesheet"&&e.hasAttribute("data-precedence"))break;if(s!==o.rel||e.getAttribute("href")!==(o.href==null||o.href===""?null:o.href)||e.getAttribute("crossorigin")!==(o.crossOrigin==null?null:o.crossOrigin)||e.getAttribute("title")!==(o.title==null?null:o.title))break;return e;case"style":if(e.hasAttribute("data-precedence"))break;return e;case"script":if(s=e.getAttribute("src"),(s!==(o.src==null?null:o.src)||e.getAttribute("type")!==(o.type==null?null:o.type)||e.getAttribute("crossorigin")!==(o.crossOrigin==null?null:o.crossOrigin))&&s&&e.hasAttribute("async")&&!e.hasAttribute("itemprop"))break;return e;default:return e}}else if(t==="input"&&e.type==="hidden"){var s=o.name==null?null:""+o.name;if(o.type==="hidden"&&e.getAttribute("name")===s)return e}else return e;if(e=Jn(e.nextSibling),e===null)break}return null}function Ly(e,t,n){if(t==="")return null;for(;e.nodeType!==3;)if((e.nodeType!==1||e.nodeName!=="INPUT"||e.type!=="hidden")&&!n||(e=Jn(e.nextSibling),e===null))return null;return e}function ev(e,t){for(;e.nodeType!==8;)if((e.nodeType!==1||e.nodeName!=="INPUT"||e.type!=="hidden")&&!t||(e=Jn(e.nextSibling),e===null))return null;return e}function gs(e){return e.data==="$?"||e.data==="$~"}function Ss(e){return e.data==="$!"||e.data==="$?"&&e.ownerDocument.readyState!=="loading"}function Hy(e,t){var n=e.ownerDocument;if(e.data==="$~")e._reactRetry=t;else if(e.data!=="$?"||n.readyState!=="loading")t();else{var l=function(){t(),n.removeEventListener("DOMContentLoaded",l)};n.addEventListener("DOMContentLoaded",l),e._reactRetry=l}}function Jn(e){for(;e!=null;e=e.nextSibling){var t=e.nodeType;if(t===1||t===3)break;if(t===8){if(t=e.data,t==="$"||t==="$!"||t==="$?"||t==="$~"||t==="&"||t==="F!"||t==="F")break;if(t==="/$"||t==="/&")return null}}return e}var _s=null;function tv(e){e=e.nextSibling;for(var t=0;e;){if(e.nodeType===8){var n=e.data;if(n==="/$"||n==="/&"){if(t===0)return Jn(e.nextSibling);t--}else n!=="$"&&n!=="$!"&&n!=="$?"&&n!=="$~"&&n!=="&"||t++}e=e.nextSibling}return null}function nv(e){e=e.previousSibling;for(var t=0;e;){if(e.nodeType===8){var n=e.data;if(n==="$"||n==="$!"||n==="$?"||n==="$~"||n==="&"){if(t===0)return e;t--}else n!=="/$"&&n!=="/&"||t++}e=e.previousSibling}return null}function av(e,t,n){switch(t=kf(n),e){case"html":if(e=t.documentElement,!e)throw Error(f(452));return e;case"head":if(e=t.head,!e)throw Error(f(453));return e;case"body":if(e=t.body,!e)throw Error(f(454));return e;default:throw Error(f(451))}}function Cu(e){for(var t=e.attributes;t.length;)e.removeAttributeNode(t[0]);al(e)}var kn=new Map,lv=new Set;function Ff(e){return typeof e.getRootNode=="function"?e.getRootNode():e.nodeType===9?e:e.ownerDocument}var $a=ue.d;ue.d={f:jy,r:Ry,D:qy,C:Uy,L:By,m:Yy,X:Gy,S:Xy,M:Vy};function jy(){var e=$a.f(),t=Yf();return e||t}function Ry(e){var t=ll(e);t!==null&&t.tag===5&&t.type==="form"?_h(t):$a.r(e)}var yr=typeof document>"u"?null:document;function iv(e,t,n){var l=yr;if(l&&typeof t=="string"&&t){var o=rn(t);o='link[rel="'+e+'"][href="'+o+'"]',typeof n=="string"&&(o+='[crossorigin="'+n+'"]'),lv.has(o)||(lv.add(o),e={rel:e,crossOrigin:n,href:t},l.querySelector(o)===null&&(t=l.createElement("link"),Wt(t,"link",e),St(t),l.head.appendChild(t)))}}function qy(e){$a.D(e),iv("dns-prefetch",e,null)}function Uy(e,t){$a.C(e,t),iv("preconnect",e,t)}function By(e,t,n){$a.L(e,t,n);var l=yr;if(l&&e&&t){var o='link[rel="preload"][as="'+rn(t)+'"]';t==="image"&&n&&n.imageSrcSet?(o+='[imagesrcset="'+rn(n.imageSrcSet)+'"]',typeof n.imageSizes=="string"&&(o+='[imagesizes="'+rn(n.imageSizes)+'"]')):o+='[href="'+rn(e)+'"]';var s=o;switch(t){case"style":s=br(e);break;case"script":s=gr(e)}kn.has(s)||(e=N({rel:"preload",href:t==="image"&&n&&n.imageSrcSet?void 0:e,as:t},n),kn.set(s,e),l.querySelector(o)!==null||t==="style"&&l.querySelector(Au(s))||t==="script"&&l.querySelector(Eu(s))||(t=l.createElement("link"),Wt(t,"link",e),St(t),l.head.appendChild(t)))}}function Yy(e,t){$a.m(e,t);var n=yr;if(n&&e){var l=t&&typeof t.as=="string"?t.as:"script",o='link[rel="modulepreload"][as="'+rn(l)+'"][href="'+rn(e)+'"]',s=o;switch(l){case"audioworklet":case"paintworklet":case"serviceworker":case"sharedworker":case"worker":case"script":s=gr(e)}if(!kn.has(s)&&(e=N({rel:"modulepreload",href:e},t),kn.set(s,e),n.querySelector(o)===null)){switch(l){case"audioworklet":case"paintworklet":case"serviceworker":case"sharedworker":case"worker":case"script":if(n.querySelector(Eu(s)))return}l=n.createElement("link"),Wt(l,"link",e),St(l),n.head.appendChild(l)}}}function Xy(e,t,n){$a.S(e,t,n);var l=yr;if(l&&e){var o=Ma(l).hoistableStyles,s=br(e);t=t||"default";var b=o.get(s);if(!b){var C={loading:0,preload:null};if(b=l.querySelector(Au(s)))C.loading=5;else{e=N({rel:"stylesheet",href:e,"data-precedence":t},n),(n=kn.get(s))&&xs(e,n);var z=b=l.createElement("link");St(z),Wt(z,"link",e),z._p=new Promise(function(Q,ee){z.onload=Q,z.onerror=ee}),z.addEventListener("load",function(){C.loading|=1}),z.addEventListener("error",function(){C.loading|=2}),C.loading|=4,Wf(b,t,l)}b={type:"stylesheet",instance:b,count:1,state:C},o.set(s,b)}}}function Gy(e,t){$a.X(e,t);var n=yr;if(n&&e){var l=Ma(n).hoistableScripts,o=gr(e),s=l.get(o);s||(s=n.querySelector(Eu(o)),s||(e=N({src:e,async:!0},t),(t=kn.get(o))&&Ds(e,t),s=n.createElement("script"),St(s),Wt(s,"link",e),n.head.appendChild(s)),s={type:"script",instance:s,count:1,state:null},l.set(o,s))}}function Vy(e,t){$a.M(e,t);var n=yr;if(n&&e){var l=Ma(n).hoistableScripts,o=gr(e),s=l.get(o);s||(s=n.querySelector(Eu(o)),s||(e=N({src:e,async:!0,type:"module"},t),(t=kn.get(o))&&Ds(e,t),s=n.createElement("script"),St(s),Wt(s,"link",e),n.head.appendChild(s)),s={type:"script",instance:s,count:1,state:null},l.set(o,s))}}function rv(e,t,n,l){var o=(o=oa.current)?Ff(o):null;if(!o)throw Error(f(446));switch(e){case"meta":case"title":return null;case"style":return typeof n.precedence=="string"&&typeof n.href=="string"?(t=br(n.href),n=Ma(o).hoistableStyles,l=n.get(t),l||(l={type:"style",instance:null,count:0,state:null},n.set(t,l)),l):{type:"void",instance:null,count:0,state:null};case"link":if(n.rel==="stylesheet"&&typeof n.href=="string"&&typeof n.precedence=="string"){e=br(n.href);var s=Ma(o).hoistableStyles,b=s.get(e);if(b||(o=o.ownerDocument||o,b={type:"stylesheet",instance:null,count:0,state:{loading:0,preload:null}},s.set(e,b),(s=o.querySelector(Au(e)))&&!s._p&&(b.instance=s,b.state.loading=5),kn.has(e)||(n={rel:"preload",as:"style",href:n.href,crossOrigin:n.crossOrigin,integrity:n.integrity,media:n.media,hrefLang:n.hrefLang,referrerPolicy:n.referrerPolicy},kn.set(e,n),s||Qy(o,e,n,b.state))),t&&l===null)throw Error(f(528,""));return b}if(t&&l!==null)throw Error(f(529,""));return null;case"script":return t=n.async,n=n.src,typeof n=="string"&&t&&typeof t!="function"&&typeof t!="symbol"?(t=gr(n),n=Ma(o).hoistableScripts,l=n.get(t),l||(l={type:"script",instance:null,count:0,state:null},n.set(t,l)),l):{type:"void",instance:null,count:0,state:null};default:throw Error(f(444,e))}}function br(e){return'href="'+rn(e)+'"'}function Au(e){return'link[rel="stylesheet"]['+e+"]"}function uv(e){return N({},e,{"data-precedence":e.precedence,precedence:null})}function Qy(e,t,n,l){e.querySelector('link[rel="preload"][as="style"]['+t+"]")?l.loading=1:(t=e.createElement("link"),l.preload=t,t.addEventListener("load",function(){return l.loading|=1}),t.addEventListener("error",function(){return l.loading|=2}),Wt(t,"link",n),St(t),e.head.appendChild(t))}function gr(e){return'[src="'+rn(e)+'"]'}function Eu(e){return"script[async]"+e}function fv(e,t,n){if(t.count++,t.instance===null)switch(t.type){case"style":var l=e.querySelector('style[data-href~="'+rn(n.href)+'"]');if(l)return t.instance=l,St(l),l;var o=N({},n,{"data-href":n.href,"data-precedence":n.precedence,href:null,precedence:null});return l=(e.ownerDocument||e).createElement("style"),St(l),Wt(l,"style",o),Wf(l,n.precedence,e),t.instance=l;case"stylesheet":o=br(n.href);var s=e.querySelector(Au(o));if(s)return t.state.loading|=4,t.instance=s,St(s),s;l=uv(n),(o=kn.get(o))&&xs(l,o),s=(e.ownerDocument||e).createElement("link"),St(s);var b=s;return b._p=new Promise(function(C,z){b.onload=C,b.onerror=z}),Wt(s,"link",l),t.state.loading|=4,Wf(s,n.precedence,e),t.instance=s;case"script":return s=gr(n.src),(o=e.querySelector(Eu(s)))?(t.instance=o,St(o),o):(l=n,(o=kn.get(s))&&(l=N({},n),Ds(l,o)),e=e.ownerDocument||e,o=e.createElement("script"),St(o),Wt(o,"link",l),e.head.appendChild(o),t.instance=o);case"void":return null;default:throw Error(f(443,t.type))}else t.type==="stylesheet"&&(t.state.loading&4)===0&&(l=t.instance,t.state.loading|=4,Wf(l,n.precedence,e));return t.instance}function Wf(e,t,n){for(var l=n.querySelectorAll('link[rel="stylesheet"][data-precedence],style[data-precedence]'),o=l.length?l[l.length-1]:null,s=o,b=0;b<l.length;b++){var C=l[b];if(C.dataset.precedence===t)s=C;else if(s!==o)break}s?s.parentNode.insertBefore(e,s.nextSibling):(t=n.nodeType===9?n.head:n,t.insertBefore(e,t.firstChild))}function xs(e,t){e.crossOrigin==null&&(e.crossOrigin=t.crossOrigin),e.referrerPolicy==null&&(e.referrerPolicy=t.referrerPolicy),e.title==null&&(e.title=t.title)}function Ds(e,t){e.crossOrigin==null&&(e.crossOrigin=t.crossOrigin),e.referrerPolicy==null&&(e.referrerPolicy=t.referrerPolicy),e.integrity==null&&(e.integrity=t.integrity)}var $f=null;function ov(e,t,n){if($f===null){var l=new Map,o=$f=new Map;o.set(n,l)}else o=$f,l=o.get(n),l||(l=new Map,o.set(n,l));if(l.has(e))return l;for(l.set(e,null),n=n.getElementsByTagName(e),o=0;o<n.length;o++){var s=n[o];if(!(s[Ea]||s[Ot]||e==="link"&&s.getAttribute("rel")==="stylesheet")&&s.namespaceURI!=="http://www.w3.org/2000/svg"){var b=s.getAttribute(t)||"";b=e+b;var C=l.get(b);C?C.push(s):l.set(b,[s])}}return l}function cv(e,t,n){e=e.ownerDocument||e,e.head.insertBefore(n,t==="title"?e.querySelector("head > title"):null)}function Zy(e,t,n){if(n===1||t.itemProp!=null)return!1;switch(e){case"meta":case"title":return!0;case"style":if(typeof t.precedence!="string"||typeof t.href!="string"||t.href==="")break;return!0;case"link":if(typeof t.rel!="string"||typeof t.href!="string"||t.href===""||t.onLoad||t.onError)break;switch(t.rel){case"stylesheet":return e=t.disabled,typeof t.precedence=="string"&&e==null;default:return!0}case"script":if(t.async&&typeof t.async!="function"&&typeof t.async!="symbol"&&!t.onLoad&&!t.onError&&t.src&&typeof t.src=="string")return!0}return!1}function sv(e){return!(e.type==="stylesheet"&&(e.state.loading&3)===0)}function Ky(e,t,n,l){if(n.type==="stylesheet"&&(typeof l.media!="string"||matchMedia(l.media).matches!==!1)&&(n.state.loading&4)===0){if(n.instance===null){var o=br(l.href),s=t.querySelector(Au(o));if(s){t=s._p,t!==null&&typeof t=="object"&&typeof t.then=="function"&&(e.count++,e=If.bind(e),t.then(e,e)),n.state.loading|=4,n.instance=s,St(s);return}s=t.ownerDocument||t,l=uv(l),(o=kn.get(o))&&xs(l,o),s=s.createElement("link"),St(s);var b=s;b._p=new Promise(function(C,z){b.onload=C,b.onerror=z}),Wt(s,"link",l),n.instance=s}e.stylesheets===null&&(e.stylesheets=new Map),e.stylesheets.set(n,t),(t=n.state.preload)&&(n.state.loading&3)===0&&(e.count++,n=If.bind(e),t.addEventListener("load",n),t.addEventListener("error",n))}}var Ts=0;function Jy(e,t){return e.stylesheets&&e.count===0&&eo(e,e.stylesheets),0<e.count||0<e.imgCount?function(n){var l=setTimeout(function(){if(e.stylesheets&&eo(e,e.stylesheets),e.unsuspend){var s=e.unsuspend;e.unsuspend=null,s()}},6e4+t);0<e.imgBytes&&Ts===0&&(Ts=62500*Ey());var o=setTimeout(function(){if(e.waitingForImages=!1,e.count===0&&(e.stylesheets&&eo(e,e.stylesheets),e.unsuspend)){var s=e.unsuspend;e.unsuspend=null,s()}},(e.imgBytes>Ts?50:800)+t);return e.unsuspend=n,function(){e.unsuspend=null,clearTimeout(l),clearTimeout(o)}}:null}function If(){if(this.count--,this.count===0&&(this.imgCount===0||!this.waitingForImages)){if(this.stylesheets)eo(this,this.stylesheets);else if(this.unsuspend){var e=this.unsuspend;this.unsuspend=null,e()}}}var Pf=null;function eo(e,t){e.stylesheets=null,e.unsuspend!==null&&(e.count++,Pf=new Map,t.forEach(ky,e),Pf=null,If.call(e))}function ky(e,t){if(!(t.state.loading&4)){var n=Pf.get(e);if(n)var l=n.get(null);else{n=new Map,Pf.set(e,n);for(var o=e.querySelectorAll("link[data-precedence],style[data-precedence]"),s=0;s<o.length;s++){var b=o[s];(b.nodeName==="LINK"||b.getAttribute("media")!=="not all")&&(n.set(b.dataset.precedence,b),l=b)}l&&n.set(null,l)}o=t.instance,b=o.getAttribute("data-precedence"),s=n.get(b)||l,s===l&&n.set(null,o),n.set(b,o),this.count++,l=If.bind(this),o.addEventListener("load",l),o.addEventListener("error",l),s?s.parentNode.insertBefore(o,s.nextSibling):(e=e.nodeType===9?e.head:e,e.insertBefore(o,e.firstChild)),t.state.loading|=4}}var Nu={$$typeof:Se,Provider:null,Consumer:null,_currentValue:me,_currentValue2:me,_threadCount:0};function Fy(e,t,n,l,o,s,b,C,z){this.tag=1,this.containerInfo=e,this.pingCache=this.current=this.pendingChildren=null,this.timeoutHandle=-1,this.callbackNode=this.next=this.pendingContext=this.context=this.cancelPendingCommit=null,this.callbackPriority=0,this.expirationTimes=jr(-1),this.entangledLanes=this.shellSuspendCounter=this.errorRecoveryDisabledLanes=this.expiredLanes=this.warmLanes=this.pingedLanes=this.suspendedLanes=this.pendingLanes=0,this.entanglements=jr(0),this.hiddenUpdates=jr(null),this.identifierPrefix=l,this.onUncaughtError=o,this.onCaughtError=s,this.onRecoverableError=b,this.pooledCache=null,this.pooledCacheLanes=0,this.formState=z,this.incompleteTransitions=new Map}function dv(e,t,n,l,o,s,b,C,z,Q,ee,ae){return e=new Fy(e,t,n,b,z,Q,ee,ae,C),t=1,s===!0&&(t|=24),s=_n(3,null,null,t),e.current=s,s.stateNode=e,t=ac(),t.refCount++,e.pooledCache=t,t.refCount++,s.memoizedState={element:l,isDehydrated:n,cache:t},uc(s),e}function hv(e){return e?(e=Wi,e):Wi}function pv(e,t,n,l,o,s){o=hv(o),l.context===null?l.context=o:l.pendingContext=o,l=bl(t),l.payload={element:n},s=s===void 0?null:s,s!==null&&(l.callback=s),n=gl(e,l,t),n!==null&&(gn(n,e,t),ru(n,e,t))}function vv(e,t){if(e=e.memoizedState,e!==null&&e.dehydrated!==null){var n=e.retryLane;e.retryLane=n!==0&&n<t?n:t}}function Cs(e,t){vv(e,t),(e=e.alternate)&&vv(e,t)}function mv(e){if(e.tag===13||e.tag===31){var t=ui(e,67108864);t!==null&&gn(t,e,67108864),Cs(e,67108864)}}function yv(e){if(e.tag===13||e.tag===31){var t=An();t=tl(t);var n=ui(e,t);n!==null&&gn(n,e,t),Cs(e,t)}}var to=!0;function Wy(e,t,n,l){var o=I.T;I.T=null;var s=ue.p;try{ue.p=2,As(e,t,n,l)}finally{ue.p=s,I.T=o}}function $y(e,t,n,l){var o=I.T;I.T=null;var s=ue.p;try{ue.p=8,As(e,t,n,l)}finally{ue.p=s,I.T=o}}function As(e,t,n,l){if(to){var o=Es(l);if(o===null)ds(e,t,l,no,n),gv(e,l);else if(Py(o,e,t,n,l))l.stopPropagation();else if(gv(e,l),t&4&&-1<Iy.indexOf(e)){for(;o!==null;){var s=ll(o);if(s!==null)switch(s.tag){case 3:if(s=s.stateNode,s.current.memoizedState.isDehydrated){var b=ha(s.pendingLanes);if(b!==0){var C=s;for(C.pendingLanes|=2,C.entangledLanes|=2;b;){var z=1<<31-ln(b);C.entanglements[1]|=z,b&=~z}_a(s),(Ie&6)===0&&(Uf=wt()+500,xu(0))}}break;case 31:case 13:C=ui(s,2),C!==null&&gn(C,s,2),Yf(),Cs(s,2)}if(s=Es(l),s===null&&ds(e,t,l,no,n),s===o)break;o=s}o!==null&&l.stopPropagation()}else ds(e,t,l,null,n)}}function Es(e){return e=Qr(e),Ns(e)}var no=null;function Ns(e){if(no=null,e=Na(e),e!==null){var t=h(e);if(t===null)e=null;else{var n=t.tag;if(n===13){if(e=v(t),e!==null)return e;e=null}else if(n===31){if(e=g(t),e!==null)return e;e=null}else if(n===3){if(t.stateNode.current.memoizedState.isDehydrated)return t.tag===3?t.stateNode.containerInfo:null;e=null}else t!==e&&(e=null)}}return no=e,null}function bv(e){switch(e){case"beforetoggle":case"cancel":case"click":case"close":case"contextmenu":case"copy":case"cut":case"auxclick":case"dblclick":case"dragend":case"dragstart":case"drop":case"focusin":case"focusout":case"input":case"invalid":case"keydown":case"keypress":case"keyup":case"mousedown":case"mouseup":case"paste":case"pause":case"play":case"pointercancel":case"pointerdown":case"pointerup":case"ratechange":case"reset":case"resize":case"seeked":case"submit":case"toggle":case"touchcancel":case"touchend":case"touchstart":case"volumechange":case"change":case"selectionchange":case"textInput":case"compositionstart":case"compositionend":case"compositionupdate":case"beforeblur":case"afterblur":case"beforeinput":case"blur":case"fullscreenchange":case"focus":case"hashchange":case"popstate":case"select":case"selectstart":return 2;case"drag":case"dragenter":case"dragexit":case"dragleave":case"dragover":case"mousemove":case"mouseout":case"mouseover":case"pointermove":case"pointerout":case"pointerover":case"scroll":case"touchmove":case"wheel":case"mouseenter":case"mouseleave":case"pointerenter":case"pointerleave":return 8;case"message":switch(ve()){case Gt:return 2;case Vu:return 8;case wi:case Ho:return 32;case Hr:return 268435456;default:return 32}default:return 32}}var Ms=!1,wl=null,Ol=null,zl=null,Mu=new Map,wu=new Map,Ll=[],Iy="mousedown mouseup touchcancel touchend touchstart auxclick dblclick pointercancel pointerdown pointerup dragend dragstart drop compositionend compositionstart keydown keypress keyup input textInput copy cut paste click change contextmenu reset".split(" ");function gv(e,t){switch(e){case"focusin":case"focusout":wl=null;break;case"dragenter":case"dragleave":Ol=null;break;case"mouseover":case"mouseout":zl=null;break;case"pointerover":case"pointerout":Mu.delete(t.pointerId);break;case"gotpointercapture":case"lostpointercapture":wu.delete(t.pointerId)}}function Ou(e,t,n,l,o,s){return e===null||e.nativeEvent!==s?(e={blockedOn:t,domEventName:n,eventSystemFlags:l,nativeEvent:s,targetContainers:[o]},t!==null&&(t=ll(t),t!==null&&mv(t)),e):(e.eventSystemFlags|=l,t=e.targetContainers,o!==null&&t.indexOf(o)===-1&&t.push(o),e)}function Py(e,t,n,l,o){switch(t){case"focusin":return wl=Ou(wl,e,t,n,l,o),!0;case"dragenter":return Ol=Ou(Ol,e,t,n,l,o),!0;case"mouseover":return zl=Ou(zl,e,t,n,l,o),!0;case"pointerover":var s=o.pointerId;return Mu.set(s,Ou(Mu.get(s)||null,e,t,n,l,o)),!0;case"gotpointercapture":return s=o.pointerId,wu.set(s,Ou(wu.get(s)||null,e,t,n,l,o)),!0}return!1}function Sv(e){var t=Na(e.target);if(t!==null){var n=h(t);if(n!==null){if(t=n.tag,t===13){if(t=v(n),t!==null){e.blockedOn=t,Fu(e.priority,function(){yv(n)});return}}else if(t===31){if(t=g(n),t!==null){e.blockedOn=t,Fu(e.priority,function(){yv(n)});return}}else if(t===3&&n.stateNode.current.memoizedState.isDehydrated){e.blockedOn=n.tag===3?n.stateNode.containerInfo:null;return}}}e.blockedOn=null}function ao(e){if(e.blockedOn!==null)return!1;for(var t=e.targetContainers;0<t.length;){var n=Es(e.nativeEvent);if(n===null){n=e.nativeEvent;var l=new n.constructor(n.type,n);Bi=l,n.target.dispatchEvent(l),Bi=null}else return t=ll(n),t!==null&&mv(t),e.blockedOn=n,!1;t.shift()}return!0}function _v(e,t,n){ao(e)&&n.delete(t)}function e0(){Ms=!1,wl!==null&&ao(wl)&&(wl=null),Ol!==null&&ao(Ol)&&(Ol=null),zl!==null&&ao(zl)&&(zl=null),Mu.forEach(_v),wu.forEach(_v)}function lo(e,t){e.blockedOn===t&&(e.blockedOn=null,Ms||(Ms=!0,a.unstable_scheduleCallback(a.unstable_NormalPriority,e0)))}var io=null;function xv(e){io!==e&&(io=e,a.unstable_scheduleCallback(a.unstable_NormalPriority,function(){io===e&&(io=null);for(var t=0;t<e.length;t+=3){var n=e[t],l=e[t+1],o=e[t+2];if(typeof l!="function"){if(Ns(l||n)===null)continue;break}var s=ll(n);s!==null&&(e.splice(t,3),t-=3,Ec(s,{pending:!0,data:o,method:n.method,action:l},l,o))}}))}function Sr(e){function t(z){return lo(z,e)}wl!==null&&lo(wl,e),Ol!==null&&lo(Ol,e),zl!==null&&lo(zl,e),Mu.forEach(t),wu.forEach(t);for(var n=0;n<Ll.length;n++){var l=Ll[n];l.blockedOn===e&&(l.blockedOn=null)}for(;0<Ll.length&&(n=Ll[0],n.blockedOn===null);)Sv(n),n.blockedOn===null&&Ll.shift();if(n=(e.ownerDocument||e).$$reactFormReplay,n!=null)for(l=0;l<n.length;l+=3){var o=n[l],s=n[l+1],b=o[Pt]||null;if(typeof s=="function")b||xv(n);else if(b){var C=null;if(s&&s.hasAttribute("formAction")){if(o=s,b=s[Pt]||null)C=b.formAction;else if(Ns(o)!==null)continue}else C=b.action;typeof C=="function"?n[l+1]=C:(n.splice(l,3),l-=3),xv(n)}}}function Dv(){function e(s){s.canIntercept&&s.info==="react-transition"&&s.intercept({handler:function(){return new Promise(function(b){return o=b})},focusReset:"manual",scroll:"manual"})}function t(){o!==null&&(o(),o=null),l||setTimeout(n,20)}function n(){if(!l&&!navigation.transition){var s=navigation.currentEntry;s&&s.url!=null&&navigation.navigate(s.url,{state:s.getState(),info:"react-transition",history:"replace"})}}if(typeof navigation=="object"){var l=!1,o=null;return navigation.addEventListener("navigate",e),navigation.addEventListener("navigatesuccess",t),navigation.addEventListener("navigateerror",t),setTimeout(n,100),function(){l=!0,navigation.removeEventListener("navigate",e),navigation.removeEventListener("navigatesuccess",t),navigation.removeEventListener("navigateerror",t),o!==null&&(o(),o=null)}}}function ws(e){this._internalRoot=e}ro.prototype.render=ws.prototype.render=function(e){var t=this._internalRoot;if(t===null)throw Error(f(409));var n=t.current,l=An();pv(n,l,e,t,null,null)},ro.prototype.unmount=ws.prototype.unmount=function(){var e=this._internalRoot;if(e!==null){this._internalRoot=null;var t=e.containerInfo;pv(e.current,2,null,e,null,null),Yf(),t[Aa]=null}};function ro(e){this._internalRoot=e}ro.prototype.unstable_scheduleHydration=function(e){if(e){var t=ku();e={blockedOn:null,target:e,priority:t};for(var n=0;n<Ll.length&&t!==0&&t<Ll[n].priority;n++);Ll.splice(n,0,e),n===0&&Sv(e)}};var Tv=i.version;if(Tv!=="19.2.0")throw Error(f(527,Tv,"19.2.0"));ue.findDOMNode=function(e){var t=e._reactInternals;if(t===void 0)throw typeof e.render=="function"?Error(f(188)):(e=Object.keys(e).join(","),Error(f(268,e)));return e=T(t),e=e!==null?A(e):null,e=e===null?null:e.stateNode,e};var t0={bundleType:0,version:"19.2.0",rendererPackageName:"react-dom",currentDispatcherRef:I,reconcilerVersion:"19.2.0"};if(typeof __REACT_DEVTOOLS_GLOBAL_HOOK__<"u"){var uo=__REACT_DEVTOOLS_GLOBAL_HOOK__;if(!uo.isDisabled&&uo.supportsFiber)try{nn=uo.inject(t0),gt=uo}catch{}}return zu.createRoot=function(e,t){if(!d(e))throw Error(f(299));var n=!1,l="",o=Oh,s=zh,b=Lh;return t!=null&&(t.unstable_strictMode===!0&&(n=!0),t.identifierPrefix!==void 0&&(l=t.identifierPrefix),t.onUncaughtError!==void 0&&(o=t.onUncaughtError),t.onCaughtError!==void 0&&(s=t.onCaughtError),t.onRecoverableError!==void 0&&(b=t.onRecoverableError)),t=dv(e,1,!1,null,null,n,l,null,o,s,b,Dv),e[Aa]=t.current,ss(e),new ws(t)},zu.hydrateRoot=function(e,t,n){if(!d(e))throw Error(f(299));var l=!1,o="",s=Oh,b=zh,C=Lh,z=null;return n!=null&&(n.unstable_strictMode===!0&&(l=!0),n.identifierPrefix!==void 0&&(o=n.identifierPrefix),n.onUncaughtError!==void 0&&(s=n.onUncaughtError),n.onCaughtError!==void 0&&(b=n.onCaughtError),n.onRecoverableError!==void 0&&(C=n.onRecoverableError),n.formState!==void 0&&(z=n.formState)),t=dv(e,1,!0,t,n??null,l,o,z,s,b,C,Dv),t.context=hv(null),n=t.current,l=An(),l=tl(l),o=bl(l),o.callback=null,gl(n,o,l),n=l,t.current.lanes=n,el(t,n),_a(t),e[Aa]=t.current,ss(e),new ro(t)},zu.version="19.2.0",zu}var Mv;function o0(){if(Mv)return Os.exports;Mv=1;function a(){if(!(typeof __REACT_DEVTOOLS_GLOBAL_HOOK__>"u"||typeof __REACT_DEVTOOLS_GLOBAL_HOOK__.checkDCE!="function"))try{__REACT_DEVTOOLS_GLOBAL_HOOK__.checkDCE(a)}catch(i){console.error(i)}}return a(),Os.exports=f0(),Os.exports}var c0=o0(),s0=["autoFill","preAutoFill","buttons-action","buttons-processing","column-reorder","columns-reordered","childRow","column-sizing","column-visibility","destroy","draw","dt-error","info","init","length","options","order","page","preDraw","preInit","preXhr","processing","requestChild","search","stateLoaded","stateLoadParams","stateSaveParams","xhr","key","key-blur","key-focus","key-prefocus","key-refocus","key-return-submit","responsive-display","responsive-resize","rowgroup-datasrc","pre-row-reorder","row-reorder","row-reorder-canceled","row-reorder-changed","row-reordered","dtsb-inserted","deselect","select","select-blur","selectItems","selectStyle","user-select","stateRestore-change"];let co=null;const Pv=jl.forwardRef(function(i,r){const f=jl.useRef(null),d=jl.useRef(null),h=jl.useRef(i.options??{}),v=jl.useRef([]);return jl.useImperativeHandle(r,()=>({dt:()=>d.current})),i.data&&(h.current.data=i.data),i.ajax&&(h.current.ajax=i.ajax),i.columns&&(h.current.columns=i.columns),i.slots&&d0(v.current,h.current,i.slots),jl.useEffect(()=>{if(!co)throw new Error("DataTables library not set. See https://datatables.net/tn/23 for details.");if(f.current){const x=co.use("jq")(f.current);s0.forEach(T=>{const A="on"+T[0].toUpperCase()+T.slice(1).replace(/-[a-z]/g,N=>N[1].toUpperCase());i[A]&&x.on(T+".dt",i[A])}),d.current=new co(f.current,h.current)}return()=>{if(d.current){let g=v.current.slice();v.current.length=0,setTimeout(()=>{g.forEach(x=>{x.unmount()})},250),d.current.destroy(),d.current=null}}},[]),jl.useEffect(()=>{i.data&&d.current&&(d.current.clear(),d.current.rows.add(i.data).draw(!1))},[i.data]),Cv.jsx("div",{children:Cv.jsx("table",{ref:f,className:i.className??"",id:i.id??"",children:i.children??null})})});Pv.use=function(a){co=a};const Gb=Pv;function d0(a,i,r){i.columnDefs||(i.columnDefs=[]),Object.keys(r).forEach(f=>{let d=r[f];d&&(f.match(/^-?\d+$/)?i.columnDefs.unshift({target:parseInt(f),render:wv(a,d)}):i.columnDefs.unshift({target:f+":name",render:wv(a,d)}))})}function wv(a,i){return function(r,f,d,h){if(i.length===4){let v=i(r,f,d,h);return v.$$typeof?Gs(a,v):v}else if(i.length===3){let v=i(r,f,d,h);return v.$$typeof?Gs(a,v):v}return h0(a,()=>i(r,d))}}function h0(a,i){let r=i();return r.$$typeof?Gs(a,r):r}function Gs(a,i){let r=document.createElement("div"),f=c0.createRoot(r);return f.render(i),a.push(f),r}var so={exports:{}};/*!
 * jQuery JavaScript Library v3.7.1
 * https://jquery.com/
 *
 * Copyright OpenJS Foundation and other contributors
 * Released under the MIT license
 * https://jquery.org/license
 *
 * Date: 2023-08-28T13:37Z
 */var p0=so.exports,Ov;function v0(){return Ov||(Ov=1,(function(a){(function(i,r){a.exports=i.document?r(i,!0):function(f){if(!f.document)throw new Error("jQuery requires a window with a document");return r(f)}})(typeof window<"u"?window:p0,function(i,r){var f=[],d=Object.getPrototypeOf,h=f.slice,v=f.flat?function(u){return f.flat.call(u)}:function(u){return f.concat.apply([],u)},g=f.push,x=f.indexOf,T={},A=T.toString,N=T.hasOwnProperty,j=N.toString,X=j.call(Object),U={},B=function(c){return typeof c=="function"&&typeof c.nodeType!="number"&&typeof c.item!="function"},Y=function(c){return c!=null&&c===c.window},F=i.document,re={type:!0,src:!0,nonce:!0,noModule:!0};function Se(u,c,p){p=p||F;var m,S,_=p.createElement("script");if(_.text=u,c)for(m in re)S=c[m]||c.getAttribute&&c.getAttribute(m),S&&_.setAttribute(m,S);p.head.appendChild(_).parentNode.removeChild(_)}function Te(u){return u==null?u+"":typeof u=="object"||typeof u=="function"?T[A.call(u)]||"object":typeof u}var Oe="3.7.1",ge=/HTML$/i,y=function(u,c){return new y.fn.init(u,c)};y.fn=y.prototype={jquery:Oe,constructor:y,length:0,toArray:function(){return h.call(this)},get:function(u){return u==null?h.call(this):u<0?this[u+this.length]:this[u]},pushStack:function(u){var c=y.merge(this.constructor(),u);return c.prevObject=this,c},each:function(u){return y.each(this,u)},map:function(u){return this.pushStack(y.map(this,function(c,p){return u.call(c,p,c)}))},slice:function(){return this.pushStack(h.apply(this,arguments))},first:function(){return this.eq(0)},last:function(){return this.eq(-1)},even:function(){return this.pushStack(y.grep(this,function(u,c){return(c+1)%2}))},odd:function(){return this.pushStack(y.grep(this,function(u,c){return c%2}))},eq:function(u){var c=this.length,p=+u+(u<0?c:0);return this.pushStack(p>=0&&p<c?[this[p]]:[])},end:function(){return this.prevObject||this.constructor()},push:g,sort:f.sort,splice:f.splice},y.extend=y.fn.extend=function(){var u,c,p,m,S,_,D=arguments[0]||{},O=1,w=arguments.length,R=!1;for(typeof D=="boolean"&&(R=D,D=arguments[O]||{},O++),typeof D!="object"&&!B(D)&&(D={}),O===w&&(D=this,O--);O<w;O++)if((u=arguments[O])!=null)for(c in u)m=u[c],!(c==="__proto__"||D===m)&&(R&&m&&(y.isPlainObject(m)||(S=Array.isArray(m)))?(p=D[c],S&&!Array.isArray(p)?_=[]:!S&&!y.isPlainObject(p)?_={}:_=p,S=!1,D[c]=y.extend(R,_,m)):m!==void 0&&(D[c]=m));return D},y.extend({expando:"jQuery"+(Oe+Math.random()).replace(/\D/g,""),isReady:!0,error:function(u){throw new Error(u)},noop:function(){},isPlainObject:function(u){var c,p;return!u||A.call(u)!=="[object Object]"?!1:(c=d(u),c?(p=N.call(c,"constructor")&&c.constructor,typeof p=="function"&&j.call(p)===X):!0)},isEmptyObject:function(u){var c;for(c in u)return!1;return!0},globalEval:function(u,c,p){Se(u,{nonce:c&&c.nonce},p)},each:function(u,c){var p,m=0;if(Ze(u))for(p=u.length;m<p&&c.call(u[m],m,u[m])!==!1;m++);else for(m in u)if(c.call(u[m],m,u[m])===!1)break;return u},text:function(u){var c,p="",m=0,S=u.nodeType;if(!S)for(;c=u[m++];)p+=y.text(c);return S===1||S===11?u.textContent:S===9?u.documentElement.textContent:S===3||S===4?u.nodeValue:p},makeArray:function(u,c){var p=c||[];return u!=null&&(Ze(Object(u))?y.merge(p,typeof u=="string"?[u]:u):g.call(p,u)),p},inArray:function(u,c,p){return c==null?-1:x.call(c,u,p)},isXMLDoc:function(u){var c=u&&u.namespaceURI,p=u&&(u.ownerDocument||u).documentElement;return!ge.test(c||p&&p.nodeName||"HTML")},merge:function(u,c){for(var p=+c.length,m=0,S=u.length;m<p;m++)u[S++]=c[m];return u.length=S,u},grep:function(u,c,p){for(var m,S=[],_=0,D=u.length,O=!p;_<D;_++)m=!c(u[_],_),m!==O&&S.push(u[_]);return S},map:function(u,c,p){var m,S,_=0,D=[];if(Ze(u))for(m=u.length;_<m;_++)S=c(u[_],_,p),S!=null&&D.push(S);else for(_ in u)S=c(u[_],_,p),S!=null&&D.push(S);return v(D)},guid:1,support:U}),typeof Symbol=="function"&&(y.fn[Symbol.iterator]=f[Symbol.iterator]),y.each("Boolean Number String Function Array Date RegExp Object Error Symbol".split(" "),function(u,c){T["[object "+c+"]"]=c.toLowerCase()});function Ze(u){var c=!!u&&"length"in u&&u.length,p=Te(u);return B(u)||Y(u)?!1:p==="array"||c===0||typeof c=="number"&&c>0&&c-1 in u}function Ce(u,c){return u.nodeName&&u.nodeName.toLowerCase()===c.toLowerCase()}var Tt=f.pop,vt=f.sort,Xe=f.splice,Me="[\\x20\\t\\r\\n\\f]",$t=new RegExp("^"+Me+"+|((?:^|[^\\\\])(?:\\\\.)*)"+Me+"+$","g");y.contains=function(u,c){var p=c&&c.parentNode;return u===p||!!(p&&p.nodeType===1&&(u.contains?u.contains(p):u.compareDocumentPosition&&u.compareDocumentPosition(p)&16))};var $e=/([\0-\x1f\x7f]|^-?\d)|^-$|[^\x80-\uFFFF\w-]/g;function I(u,c){return c?u==="\0"?"�":u.slice(0,-1)+"\\"+u.charCodeAt(u.length-1).toString(16)+" ":"\\"+u}y.escapeSelector=function(u){return(u+"").replace($e,I)};var ue=F,me=g;(function(){var u,c,p,m,S,_=me,D,O,w,R,W,te=y.expando,Z=0,le=0,De=ai(),Ue=ai(),Ee=ai(),Ct=ai(),_t=function(E,L){return E===L&&(S=!0),0},Hn="checked|selected|async|autofocus|autoplay|controls|defer|disabled|hidden|ismap|loop|multiple|open|readonly|required|scoped",jn="(?:\\\\[\\da-fA-F]{1,6}"+Me+"?|\\\\[^\\r\\n\\f]|[\\w-]|[^\0-\\x7f])+",Le="\\["+Me+"*("+jn+")(?:"+Me+"*([*^$|!~]?=)"+Me+`*(?:'((?:\\\\.|[^\\\\'])*)'|"((?:\\\\.|[^\\\\"])*)"|(`+jn+"))|)"+Me+"*\\]",Ra=":("+jn+`)(?:\\((('((?:\\\\.|[^\\\\'])*)'|"((?:\\\\.|[^\\\\"])*)")|((?:\\\\.|[^\\\\()[\\]]|`+Le+")*)|.*)\\)|)",Ge=new RegExp(Me+"+","g"),ht=new RegExp("^"+Me+"*,"+Me+"*"),ni=new RegExp("^"+Me+"*([>+~]|"+Me+")"+Me+"*"),kr=new RegExp(Me+"|>"),hn=new RegExp(Ra),aa=new RegExp("^"+jn+"$"),Rn={ID:new RegExp("^#("+jn+")"),CLASS:new RegExp("^\\.("+jn+")"),TAG:new RegExp("^("+jn+"|[*])"),ATTR:new RegExp("^"+Le),PSEUDO:new RegExp("^"+Ra),CHILD:new RegExp("^:(only|first|last|nth|nth-last)-(child|of-type)(?:\\("+Me+"*(even|odd|(([+-]|)(\\d*)n|)"+Me+"*(?:([+-]|)"+Me+"*(\\d+)|))"+Me+"*\\)|)","i"),bool:new RegExp("^(?:"+Hn+")$","i"),needsContext:new RegExp("^"+Me+"*[>+~]|:(even|odd|eq|gt|lt|nth|first|last)(?:\\("+Me+"*((?:-\\d)?\\d*)"+Me+"*\\)|)(?=[^-]|$)","i")},la=/^(?:input|select|textarea|button)$/i,ia=/^h\d$/i,fn=/^(?:#([\w-]+)|(\w+)|\.([\w-]+))$/,Qi=/[+~]/,qn=new RegExp("\\\\[\\da-fA-F]{1,6}"+Me+"?|\\\\([^\\r\\n\\f])","g"),en=function(E,L){var G="0x"+E.slice(1)-65536;return L||(G<0?String.fromCharCode(G+65536):String.fromCharCode(G>>10|55296,G&1023|56320))},Bo=function(){Bn()},Yo=Ki(function(E){return E.disabled===!0&&Ce(E,"fieldset")},{dir:"parentNode",next:"legend"});function Xo(){try{return D.activeElement}catch{}}try{_.apply(f=h.call(ue.childNodes),ue.childNodes),f[ue.childNodes.length].nodeType}catch{_={apply:function(L,G){me.apply(L,h.call(G))},call:function(L){me.apply(L,h.call(arguments,1))}}}function Je(E,L,G,J){var P,ie,fe,se,oe,Ae,ye,xe=L&&L.ownerDocument,ze=L?L.nodeType:9;if(G=G||[],typeof E!="string"||!E||ze!==1&&ze!==9&&ze!==11)return G;if(!J&&(Bn(L),L=L||D,w)){if(ze!==11&&(oe=fn.exec(E)))if(P=oe[1]){if(ze===9)if(fe=L.getElementById(P)){if(fe.id===P)return _.call(G,fe),G}else return G;else if(xe&&(fe=xe.getElementById(P))&&Je.contains(L,fe)&&fe.id===P)return _.call(G,fe),G}else{if(oe[2])return _.apply(G,L.getElementsByTagName(E)),G;if((P=oe[3])&&L.getElementsByClassName)return _.apply(G,L.getElementsByClassName(P)),G}if(!Ct[E+" "]&&(!R||!R.test(E))){if(ye=E,xe=L,ze===1&&(kr.test(E)||ni.test(E))){for(xe=Qi.test(E)&&li(L.parentNode)||L,(xe!=L||!U.scope)&&((se=L.getAttribute("id"))?se=y.escapeSelector(se):L.setAttribute("id",se=te)),Ae=sl(E),ie=Ae.length;ie--;)Ae[ie]=(se?"#"+se:":scope")+" "+ii(Ae[ie]);ye=Ae.join(",")}try{return _.apply(G,xe.querySelectorAll(ye)),G}catch{Ct(E,!0)}finally{se===te&&L.removeAttribute("id")}}}return Pr(E.replace($t,"$1"),L,G,J)}function ai(){var E=[];function L(G,J){return E.push(G+" ")>c.cacheLength&&delete L[E.shift()],L[G+" "]=J}return L}function Qt(E){return E[te]=!0,E}function Un(E){var L=D.createElement("fieldset");try{return!!E(L)}catch{return!1}finally{L.parentNode&&L.parentNode.removeChild(L),L=null}}function Go(E){return function(L){return Ce(L,"input")&&L.type===E}}function Zi(E){return function(L){return(Ce(L,"input")||Ce(L,"button"))&&L.type===E}}function Fr(E){return function(L){return"form"in L?L.parentNode&&L.disabled===!1?"label"in L?"label"in L.parentNode?L.parentNode.disabled===E:L.disabled===E:L.isDisabled===E||L.isDisabled!==!E&&Yo(L)===E:L.disabled===E:"label"in L?L.disabled===E:!1}}function ya(E){return Qt(function(L){return L=+L,Qt(function(G,J){for(var P,ie=E([],G.length,L),fe=ie.length;fe--;)G[P=ie[fe]]&&(G[P]=!(J[P]=G[P]))})})}function li(E){return E&&typeof E.getElementsByTagName<"u"&&E}function Bn(E){var L,G=E?E.ownerDocument||E:ue;return G==D||G.nodeType!==9||!G.documentElement||(D=G,O=D.documentElement,w=!y.isXMLDoc(D),W=O.matches||O.webkitMatchesSelector||O.msMatchesSelector,O.msMatchesSelector&&ue!=D&&(L=D.defaultView)&&L.top!==L&&L.addEventListener("unload",Bo),U.getById=Un(function(J){return O.appendChild(J).id=y.expando,!D.getElementsByName||!D.getElementsByName(y.expando).length}),U.disconnectedMatch=Un(function(J){return W.call(J,"*")}),U.scope=Un(function(){return D.querySelectorAll(":scope")}),U.cssHas=Un(function(){try{return D.querySelector(":has(*,:jqfake)"),!1}catch{return!0}}),U.getById?(c.filter.ID=function(J){var P=J.replace(qn,en);return function(ie){return ie.getAttribute("id")===P}},c.find.ID=function(J,P){if(typeof P.getElementById<"u"&&w){var ie=P.getElementById(J);return ie?[ie]:[]}}):(c.filter.ID=function(J){var P=J.replace(qn,en);return function(ie){var fe=typeof ie.getAttributeNode<"u"&&ie.getAttributeNode("id");return fe&&fe.value===P}},c.find.ID=function(J,P){if(typeof P.getElementById<"u"&&w){var ie,fe,se,oe=P.getElementById(J);if(oe){if(ie=oe.getAttributeNode("id"),ie&&ie.value===J)return[oe];for(se=P.getElementsByName(J),fe=0;oe=se[fe++];)if(ie=oe.getAttributeNode("id"),ie&&ie.value===J)return[oe]}return[]}}),c.find.TAG=function(J,P){return typeof P.getElementsByTagName<"u"?P.getElementsByTagName(J):P.querySelectorAll(J)},c.find.CLASS=function(J,P){if(typeof P.getElementsByClassName<"u"&&w)return P.getElementsByClassName(J)},R=[],Un(function(J){var P;O.appendChild(J).innerHTML="<a id='"+te+"' href='' disabled='disabled'></a><select id='"+te+"-\r\\' disabled='disabled'><option selected=''></option></select>",J.querySelectorAll("[selected]").length||R.push("\\["+Me+"*(?:value|"+Hn+")"),J.querySelectorAll("[id~="+te+"-]").length||R.push("~="),J.querySelectorAll("a#"+te+"+*").length||R.push(".#.+[+~]"),J.querySelectorAll(":checked").length||R.push(":checked"),P=D.createElement("input"),P.setAttribute("type","hidden"),J.appendChild(P).setAttribute("name","D"),O.appendChild(J).disabled=!0,J.querySelectorAll(":disabled").length!==2&&R.push(":enabled",":disabled"),P=D.createElement("input"),P.setAttribute("name",""),J.appendChild(P),J.querySelectorAll("[name='']").length||R.push("\\["+Me+"*name"+Me+"*="+Me+`*(?:''|"")`)}),U.cssHas||R.push(":has"),R=R.length&&new RegExp(R.join("|")),_t=function(J,P){if(J===P)return S=!0,0;var ie=!J.compareDocumentPosition-!P.compareDocumentPosition;return ie||(ie=(J.ownerDocument||J)==(P.ownerDocument||P)?J.compareDocumentPosition(P):1,ie&1||!U.sortDetached&&P.compareDocumentPosition(J)===ie?J===D||J.ownerDocument==ue&&Je.contains(ue,J)?-1:P===D||P.ownerDocument==ue&&Je.contains(ue,P)?1:m?x.call(m,J)-x.call(m,P):0:ie&4?-1:1)}),D}Je.matches=function(E,L){return Je(E,null,null,L)},Je.matchesSelector=function(E,L){if(Bn(E),w&&!Ct[L+" "]&&(!R||!R.test(L)))try{var G=W.call(E,L);if(G||U.disconnectedMatch||E.document&&E.document.nodeType!==11)return G}catch{Ct(L,!0)}return Je(L,D,null,[E]).length>0},Je.contains=function(E,L){return(E.ownerDocument||E)!=D&&Bn(E),y.contains(E,L)},Je.attr=function(E,L){(E.ownerDocument||E)!=D&&Bn(E);var G=c.attrHandle[L.toLowerCase()],J=G&&N.call(c.attrHandle,L.toLowerCase())?G(E,L,!w):void 0;return J!==void 0?J:E.getAttribute(L)},Je.error=function(E){throw new Error("Syntax error, unrecognized expression: "+E)},y.uniqueSort=function(E){var L,G=[],J=0,P=0;if(S=!U.sortStable,m=!U.sortStable&&h.call(E,0),vt.call(E,_t),S){for(;L=E[P++];)L===E[P]&&(J=G.push(P));for(;J--;)Xe.call(E,G[J],1)}return m=null,E},y.fn.uniqueSort=function(){return this.pushStack(y.uniqueSort(h.apply(this)))},c=y.expr={cacheLength:50,createPseudo:Qt,match:Rn,attrHandle:{},find:{},relative:{">":{dir:"parentNode",first:!0}," ":{dir:"parentNode"},"+":{dir:"previousSibling",first:!0},"~":{dir:"previousSibling"}},preFilter:{ATTR:function(E){return E[1]=E[1].replace(qn,en),E[3]=(E[3]||E[4]||E[5]||"").replace(qn,en),E[2]==="~="&&(E[3]=" "+E[3]+" "),E.slice(0,4)},CHILD:function(E){return E[1]=E[1].toLowerCase(),E[1].slice(0,3)==="nth"?(E[3]||Je.error(E[0]),E[4]=+(E[4]?E[5]+(E[6]||1):2*(E[3]==="even"||E[3]==="odd")),E[5]=+(E[7]+E[8]||E[3]==="odd")):E[3]&&Je.error(E[0]),E},PSEUDO:function(E){var L,G=!E[6]&&E[2];return Rn.CHILD.test(E[0])?null:(E[3]?E[2]=E[4]||E[5]||"":G&&hn.test(G)&&(L=sl(G,!0))&&(L=G.indexOf(")",G.length-L)-G.length)&&(E[0]=E[0].slice(0,L),E[2]=G.slice(0,L)),E.slice(0,3))}},filter:{TAG:function(E){var L=E.replace(qn,en).toLowerCase();return E==="*"?function(){return!0}:function(G){return Ce(G,L)}},CLASS:function(E){var L=De[E+" "];return L||(L=new RegExp("(^|"+Me+")"+E+"("+Me+"|$)"))&&De(E,function(G){return L.test(typeof G.className=="string"&&G.className||typeof G.getAttribute<"u"&&G.getAttribute("class")||"")})},ATTR:function(E,L,G){return function(J){var P=Je.attr(J,E);return P==null?L==="!=":L?(P+="",L==="="?P===G:L==="!="?P!==G:L==="^="?G&&P.indexOf(G)===0:L==="*="?G&&P.indexOf(G)>-1:L==="$="?G&&P.slice(-G.length)===G:L==="~="?(" "+P.replace(Ge," ")+" ").indexOf(G)>-1:L==="|="?P===G||P.slice(0,G.length+1)===G+"-":!1):!0}},CHILD:function(E,L,G,J,P){var ie=E.slice(0,3)!=="nth",fe=E.slice(-4)!=="last",se=L==="of-type";return J===1&&P===0?function(oe){return!!oe.parentNode}:function(oe,Ae,ye){var xe,ze,pe,Pe,Kt,Lt=ie!==fe?"nextSibling":"previousSibling",pn=oe.parentNode,Yn=se&&oe.nodeName.toLowerCase(),dl=!ye&&!se,Ut=!1;if(pn){if(ie){for(;Lt;){for(pe=oe;pe=pe[Lt];)if(se?Ce(pe,Yn):pe.nodeType===1)return!1;Kt=Lt=E==="only"&&!Kt&&"nextSibling"}return!0}if(Kt=[fe?pn.firstChild:pn.lastChild],fe&&dl){for(ze=pn[te]||(pn[te]={}),xe=ze[E]||[],Pe=xe[0]===Z&&xe[1],Ut=Pe&&xe[2],pe=Pe&&pn.childNodes[Pe];pe=++Pe&&pe&&pe[Lt]||(Ut=Pe=0)||Kt.pop();)if(pe.nodeType===1&&++Ut&&pe===oe){ze[E]=[Z,Pe,Ut];break}}else if(dl&&(ze=oe[te]||(oe[te]={}),xe=ze[E]||[],Pe=xe[0]===Z&&xe[1],Ut=Pe),Ut===!1)for(;(pe=++Pe&&pe&&pe[Lt]||(Ut=Pe=0)||Kt.pop())&&!((se?Ce(pe,Yn):pe.nodeType===1)&&++Ut&&(dl&&(ze=pe[te]||(pe[te]={}),ze[E]=[Z,Ut]),pe===oe)););return Ut-=P,Ut===J||Ut%J===0&&Ut/J>=0}}},PSEUDO:function(E,L){var G,J=c.pseudos[E]||c.setFilters[E.toLowerCase()]||Je.error("unsupported pseudo: "+E);return J[te]?J(L):J.length>1?(G=[E,E,"",L],c.setFilters.hasOwnProperty(E.toLowerCase())?Qt(function(P,ie){for(var fe,se=J(P,L),oe=se.length;oe--;)fe=x.call(P,se[oe]),P[fe]=!(ie[fe]=se[oe])}):function(P){return J(P,0,G)}):J}},pseudos:{not:Qt(function(E){var L=[],G=[],J=ki(E.replace($t,"$1"));return J[te]?Qt(function(P,ie,fe,se){for(var oe,Ae=J(P,null,se,[]),ye=P.length;ye--;)(oe=Ae[ye])&&(P[ye]=!(ie[ye]=oe))}):function(P,ie,fe){return L[0]=P,J(L,null,fe,G),L[0]=null,!G.pop()}}),has:Qt(function(E){return function(L){return Je(E,L).length>0}}),contains:Qt(function(E){return E=E.replace(qn,en),function(L){return(L.textContent||y.text(L)).indexOf(E)>-1}}),lang:Qt(function(E){return aa.test(E||"")||Je.error("unsupported lang: "+E),E=E.replace(qn,en).toLowerCase(),function(L){var G;do if(G=w?L.lang:L.getAttribute("xml:lang")||L.getAttribute("lang"))return G=G.toLowerCase(),G===E||G.indexOf(E+"-")===0;while((L=L.parentNode)&&L.nodeType===1);return!1}}),target:function(E){var L=i.location&&i.location.hash;return L&&L.slice(1)===E.id},root:function(E){return E===O},focus:function(E){return E===Xo()&&D.hasFocus()&&!!(E.type||E.href||~E.tabIndex)},enabled:Fr(!1),disabled:Fr(!0),checked:function(E){return Ce(E,"input")&&!!E.checked||Ce(E,"option")&&!!E.selected},selected:function(E){return E.parentNode&&E.parentNode.selectedIndex,E.selected===!0},empty:function(E){for(E=E.firstChild;E;E=E.nextSibling)if(E.nodeType<6)return!1;return!0},parent:function(E){return!c.pseudos.empty(E)},header:function(E){return ia.test(E.nodeName)},input:function(E){return la.test(E.nodeName)},button:function(E){return Ce(E,"input")&&E.type==="button"||Ce(E,"button")},text:function(E){var L;return Ce(E,"input")&&E.type==="text"&&((L=E.getAttribute("type"))==null||L.toLowerCase()==="text")},first:ya(function(){return[0]}),last:ya(function(E,L){return[L-1]}),eq:ya(function(E,L,G){return[G<0?G+L:G]}),even:ya(function(E,L){for(var G=0;G<L;G+=2)E.push(G);return E}),odd:ya(function(E,L){for(var G=1;G<L;G+=2)E.push(G);return E}),lt:ya(function(E,L,G){var J;for(G<0?J=G+L:G>L?J=L:J=G;--J>=0;)E.push(J);return E}),gt:ya(function(E,L,G){for(var J=G<0?G+L:G;++J<L;)E.push(J);return E})}},c.pseudos.nth=c.pseudos.eq;for(u in{radio:!0,checkbox:!0,file:!0,password:!0,image:!0})c.pseudos[u]=Go(u);for(u in{submit:!0,reset:!0})c.pseudos[u]=Zi(u);function Wr(){}Wr.prototype=c.filters=c.pseudos,c.setFilters=new Wr;function sl(E,L){var G,J,P,ie,fe,se,oe,Ae=Ue[E+" "];if(Ae)return L?0:Ae.slice(0);for(fe=E,se=[],oe=c.preFilter;fe;){(!G||(J=ht.exec(fe)))&&(J&&(fe=fe.slice(J[0].length)||fe),se.push(P=[])),G=!1,(J=ni.exec(fe))&&(G=J.shift(),P.push({value:G,type:J[0].replace($t," ")}),fe=fe.slice(G.length));for(ie in c.filter)(J=Rn[ie].exec(fe))&&(!oe[ie]||(J=oe[ie](J)))&&(G=J.shift(),P.push({value:G,type:ie,matches:J}),fe=fe.slice(G.length));if(!G)break}return L?fe.length:fe?Je.error(E):Ue(E,se).slice(0)}function ii(E){for(var L=0,G=E.length,J="";L<G;L++)J+=E[L].value;return J}function Ki(E,L,G){var J=L.dir,P=L.next,ie=P||J,fe=G&&ie==="parentNode",se=le++;return L.first?function(oe,Ae,ye){for(;oe=oe[J];)if(oe.nodeType===1||fe)return E(oe,Ae,ye);return!1}:function(oe,Ae,ye){var xe,ze,pe=[Z,se];if(ye){for(;oe=oe[J];)if((oe.nodeType===1||fe)&&E(oe,Ae,ye))return!0}else for(;oe=oe[J];)if(oe.nodeType===1||fe)if(ze=oe[te]||(oe[te]={}),P&&Ce(oe,P))oe=oe[J]||oe;else{if((xe=ze[ie])&&xe[0]===Z&&xe[1]===se)return pe[2]=xe[2];if(ze[ie]=pe,pe[2]=E(oe,Ae,ye))return!0}return!1}}function $r(E){return E.length>1?function(L,G,J){for(var P=E.length;P--;)if(!E[P](L,G,J))return!1;return!0}:E[0]}function Vo(E,L,G){for(var J=0,P=L.length;J<P;J++)Je(E,L[J],G);return G}function Ji(E,L,G,J,P){for(var ie,fe=[],se=0,oe=E.length,Ae=L!=null;se<oe;se++)(ie=E[se])&&(!G||G(ie,J,P))&&(fe.push(ie),Ae&&L.push(se));return fe}function Ir(E,L,G,J,P,ie){return J&&!J[te]&&(J=Ir(J)),P&&!P[te]&&(P=Ir(P,ie)),Qt(function(fe,se,oe,Ae){var ye,xe,ze,pe,Pe=[],Kt=[],Lt=se.length,pn=fe||Vo(L||"*",oe.nodeType?[oe]:oe,[]),Yn=E&&(fe||!L)?Ji(pn,Pe,E,oe,Ae):pn;if(G?(pe=P||(fe?E:Lt||J)?[]:se,G(Yn,pe,oe,Ae)):pe=Yn,J)for(ye=Ji(pe,Kt),J(ye,[],oe,Ae),xe=ye.length;xe--;)(ze=ye[xe])&&(pe[Kt[xe]]=!(Yn[Kt[xe]]=ze));if(fe){if(P||E){if(P){for(ye=[],xe=pe.length;xe--;)(ze=pe[xe])&&ye.push(Yn[xe]=ze);P(null,pe=[],ye,Ae)}for(xe=pe.length;xe--;)(ze=pe[xe])&&(ye=P?x.call(fe,ze):Pe[xe])>-1&&(fe[ye]=!(se[ye]=ze))}}else pe=Ji(pe===se?pe.splice(Lt,pe.length):pe),P?P(null,se,pe,Ae):_.apply(se,pe)})}function Zt(E){for(var L,G,J,P=E.length,ie=c.relative[E[0].type],fe=ie||c.relative[" "],se=ie?1:0,oe=Ki(function(xe){return xe===L},fe,!0),Ae=Ki(function(xe){return x.call(L,xe)>-1},fe,!0),ye=[function(xe,ze,pe){var Pe=!ie&&(pe||ze!=p)||((L=ze).nodeType?oe(xe,ze,pe):Ae(xe,ze,pe));return L=null,Pe}];se<P;se++)if(G=c.relative[E[se].type])ye=[Ki($r(ye),G)];else{if(G=c.filter[E[se].type].apply(null,E[se].matches),G[te]){for(J=++se;J<P&&!c.relative[E[J].type];J++);return Ir(se>1&&$r(ye),se>1&&ii(E.slice(0,se-1).concat({value:E[se-2].type===" "?"*":""})).replace($t,"$1"),G,se<J&&Zt(E.slice(se,J)),J<P&&Zt(E=E.slice(J)),J<P&&ii(E))}ye.push(G)}return $r(ye)}function ri(E,L){var G=L.length>0,J=E.length>0,P=function(ie,fe,se,oe,Ae){var ye,xe,ze,pe=0,Pe="0",Kt=ie&&[],Lt=[],pn=p,Yn=ie||J&&c.find.TAG("*",Ae),dl=Z+=pn==null?1:Math.random()||.1,Ut=Yn.length;for(Ae&&(p=fe==D||fe||Ae);Pe!==Ut&&(ye=Yn[Pe])!=null;Pe++){if(J&&ye){for(xe=0,!fe&&ye.ownerDocument!=D&&(Bn(ye),se=!w);ze=E[xe++];)if(ze(ye,fe||D,se)){_.call(oe,ye);break}Ae&&(Z=dl)}G&&((ye=!ze&&ye)&&pe--,ie&&Kt.push(ye))}if(pe+=Pe,G&&Pe!==pe){for(xe=0;ze=L[xe++];)ze(Kt,Lt,fe,se);if(ie){if(pe>0)for(;Pe--;)Kt[Pe]||Lt[Pe]||(Lt[Pe]=Tt.call(oe));Lt=Ji(Lt)}_.apply(oe,Lt),Ae&&!ie&&Lt.length>0&&pe+L.length>1&&y.uniqueSort(oe)}return Ae&&(Z=dl,p=pn),Kt};return G?Qt(P):P}function ki(E,L){var G,J=[],P=[],ie=Ee[E+" "];if(!ie){for(L||(L=sl(E)),G=L.length;G--;)ie=Zt(L[G]),ie[te]?J.push(ie):P.push(ie);ie=Ee(E,ri(P,J)),ie.selector=E}return ie}function Pr(E,L,G,J){var P,ie,fe,se,oe,Ae=typeof E=="function"&&E,ye=!J&&sl(E=Ae.selector||E);if(G=G||[],ye.length===1){if(ie=ye[0]=ye[0].slice(0),ie.length>2&&(fe=ie[0]).type==="ID"&&L.nodeType===9&&w&&c.relative[ie[1].type]){if(L=(c.find.ID(fe.matches[0].replace(qn,en),L)||[])[0],L)Ae&&(L=L.parentNode);else return G;E=E.slice(ie.shift().value.length)}for(P=Rn.needsContext.test(E)?0:ie.length;P--&&(fe=ie[P],!c.relative[se=fe.type]);)if((oe=c.find[se])&&(J=oe(fe.matches[0].replace(qn,en),Qi.test(ie[0].type)&&li(L.parentNode)||L))){if(ie.splice(P,1),E=J.length&&ii(ie),!E)return _.apply(G,J),G;break}}return(Ae||ki(E,ye))(J,L,!w,G,!L||Qi.test(E)&&li(L.parentNode)||L),G}U.sortStable=te.split("").sort(_t).join("")===te,Bn(),U.sortDetached=Un(function(E){return E.compareDocumentPosition(D.createElement("fieldset"))&1}),y.find=Je,y.expr[":"]=y.expr.pseudos,y.unique=y.uniqueSort,Je.compile=ki,Je.select=Pr,Je.setDocument=Bn,Je.tokenize=sl,Je.escape=y.escapeSelector,Je.getText=y.text,Je.isXML=y.isXMLDoc,Je.selectors=y.expr,Je.support=y.support,Je.uniqueSort=y.uniqueSort})();var qe=function(u,c,p){for(var m=[],S=p!==void 0;(u=u[c])&&u.nodeType!==9;)if(u.nodeType===1){if(S&&y(u).is(p))break;m.push(u)}return m},lt=function(u,c){for(var p=[];u;u=u.nextSibling)u.nodeType===1&&u!==c&&p.push(u);return p},Rt=y.expr.match.needsContext,ot=/^<([a-z][^\/\0>:\x20\t\r\n\f]*)[\x20\t\r\n\f]*\/?>(?:<\/\1>|)$/i;function Qe(u,c,p){return B(c)?y.grep(u,function(m,S){return!!c.call(m,S,m)!==p}):c.nodeType?y.grep(u,function(m){return m===c!==p}):typeof c!="string"?y.grep(u,function(m){return x.call(c,m)>-1!==p}):y.filter(c,u,p)}y.filter=function(u,c,p){var m=c[0];return p&&(u=":not("+u+")"),c.length===1&&m.nodeType===1?y.find.matchesSelector(m,u)?[m]:[]:y.find.matches(u,y.grep(c,function(S){return S.nodeType===1}))},y.fn.extend({find:function(u){var c,p,m=this.length,S=this;if(typeof u!="string")return this.pushStack(y(u).filter(function(){for(c=0;c<m;c++)if(y.contains(S[c],this))return!0}));for(p=this.pushStack([]),c=0;c<m;c++)y.find(u,S[c],p);return m>1?y.uniqueSort(p):p},filter:function(u){return this.pushStack(Qe(this,u||[],!1))},not:function(u){return this.pushStack(Qe(this,u||[],!0))},is:function(u){return!!Qe(this,typeof u=="string"&&Rt.test(u)?y(u):u||[],!1).length}});var bt,wn=/^(?:\s*(<[\w\W]+>)[^>]*|#([\w-]+))$/,oa=y.fn.init=function(u,c,p){var m,S;if(!u)return this;if(p=p||bt,typeof u=="string")if(u[0]==="<"&&u[u.length-1]===">"&&u.length>=3?m=[null,u,null]:m=wn.exec(u),m&&(m[1]||!c))if(m[1]){if(c=c instanceof y?c[0]:c,y.merge(this,y.parseHTML(m[1],c&&c.nodeType?c.ownerDocument||c:F,!0)),ot.test(m[1])&&y.isPlainObject(c))for(m in c)B(this[m])?this[m](c[m]):this.attr(m,c[m]);return this}else return S=F.getElementById(m[2]),S&&(this[0]=S,this.length=1),this;else return!c||c.jquery?(c||p).find(u):this.constructor(c).find(u);else{if(u.nodeType)return this[0]=u,this.length=1,this;if(B(u))return p.ready!==void 0?p.ready(u):u(y)}return y.makeArray(u,this)};oa.prototype=y.fn,bt=y(F);var Ai=/^(?:parents|prev(?:Until|All))/,Ei={children:!0,contents:!0,next:!0,prev:!0};y.fn.extend({has:function(u){var c=y(u,this),p=c.length;return this.filter(function(){for(var m=0;m<p;m++)if(y.contains(this,c[m]))return!0})},closest:function(u,c){var p,m=0,S=this.length,_=[],D=typeof u!="string"&&y(u);if(!Rt.test(u)){for(;m<S;m++)for(p=this[m];p&&p!==c;p=p.parentNode)if(p.nodeType<11&&(D?D.index(p)>-1:p.nodeType===1&&y.find.matchesSelector(p,u))){_.push(p);break}}return this.pushStack(_.length>1?y.uniqueSort(_):_)},index:function(u){return u?typeof u=="string"?x.call(y(u),this[0]):x.call(this,u.jquery?u[0]:u):this[0]&&this[0].parentNode?this.first().prevAll().length:-1},add:function(u,c){return this.pushStack(y.uniqueSort(y.merge(this.get(),y(u,c))))},addBack:function(u){return this.add(u==null?this.prevObject:this.prevObject.filter(u))}});function Da(u,c){for(;(u=u[c])&&u.nodeType!==1;);return u}y.each({parent:function(u){var c=u.parentNode;return c&&c.nodeType!==11?c:null},parents:function(u){return qe(u,"parentNode")},parentsUntil:function(u,c,p){return qe(u,"parentNode",p)},next:function(u){return Da(u,"nextSibling")},prev:function(u){return Da(u,"previousSibling")},nextAll:function(u){return qe(u,"nextSibling")},prevAll:function(u){return qe(u,"previousSibling")},nextUntil:function(u,c,p){return qe(u,"nextSibling",p)},prevUntil:function(u,c,p){return qe(u,"previousSibling",p)},siblings:function(u){return lt((u.parentNode||{}).firstChild,u)},children:function(u){return lt(u.firstChild)},contents:function(u){return u.contentDocument!=null&&d(u.contentDocument)?u.contentDocument:(Ce(u,"template")&&(u=u.content||u),y.merge([],u.childNodes))}},function(u,c){y.fn[u]=function(p,m){var S=y.map(this,c,p);return u.slice(-5)!=="Until"&&(m=p),m&&typeof m=="string"&&(S=y.filter(m,S)),this.length>1&&(Ei[u]||y.uniqueSort(S),Ai.test(u)&&S.reverse()),this.pushStack(S)}});var dn=/[^\x20\t\r\n\f]+/g;function Ni(u){var c={};return y.each(u.match(dn)||[],function(p,m){c[m]=!0}),c}y.Callbacks=function(u){u=typeof u=="string"?Ni(u):y.extend({},u);var c,p,m,S,_=[],D=[],O=-1,w=function(){for(S=S||u.once,m=c=!0;D.length;O=-1)for(p=D.shift();++O<_.length;)_[O].apply(p[0],p[1])===!1&&u.stopOnFalse&&(O=_.length,p=!1);u.memory||(p=!1),c=!1,S&&(p?_=[]:_="")},R={add:function(){return _&&(p&&!c&&(O=_.length-1,D.push(p)),(function W(te){y.each(te,function(Z,le){B(le)?(!u.unique||!R.has(le))&&_.push(le):le&&le.length&&Te(le)!=="string"&&W(le)})})(arguments),p&&!c&&w()),this},remove:function(){return y.each(arguments,function(W,te){for(var Z;(Z=y.inArray(te,_,Z))>-1;)_.splice(Z,1),Z<=O&&O--}),this},has:function(W){return W?y.inArray(W,_)>-1:_.length>0},empty:function(){return _&&(_=[]),this},disable:function(){return S=D=[],_=p="",this},disabled:function(){return!_},lock:function(){return S=D=[],!p&&!c&&(_=p=""),this},locked:function(){return!!S},fireWith:function(W,te){return S||(te=te||[],te=[W,te.slice?te.slice():te],D.push(te),c||w()),this},fire:function(){return R.fireWith(this,arguments),this},fired:function(){return!!m}};return R};function ca(u){return u}function Xl(u){throw u}function sa(u,c,p,m){var S;try{u&&B(S=u.promise)?S.call(u).done(c).fail(p):u&&B(S=u.then)?S.call(u,c,p):c.apply(void 0,[u].slice(m))}catch(_){p.apply(void 0,[_])}}y.extend({Deferred:function(u){var c=[["notify","progress",y.Callbacks("memory"),y.Callbacks("memory"),2],["resolve","done",y.Callbacks("once memory"),y.Callbacks("once memory"),0,"resolved"],["reject","fail",y.Callbacks("once memory"),y.Callbacks("once memory"),1,"rejected"]],p="pending",m={state:function(){return p},always:function(){return S.done(arguments).fail(arguments),this},catch:function(_){return m.then(null,_)},pipe:function(){var _=arguments;return y.Deferred(function(D){y.each(c,function(O,w){var R=B(_[w[4]])&&_[w[4]];S[w[1]](function(){var W=R&&R.apply(this,arguments);W&&B(W.promise)?W.promise().progress(D.notify).done(D.resolve).fail(D.reject):D[w[0]+"With"](this,R?[W]:arguments)})}),_=null}).promise()},then:function(_,D,O){var w=0;function R(W,te,Z,le){return function(){var De=this,Ue=arguments,Ee=function(){var _t,Hn;if(!(W<w)){if(_t=Z.apply(De,Ue),_t===te.promise())throw new TypeError("Thenable self-resolution");Hn=_t&&(typeof _t=="object"||typeof _t=="function")&&_t.then,B(Hn)?le?Hn.call(_t,R(w,te,ca,le),R(w,te,Xl,le)):(w++,Hn.call(_t,R(w,te,ca,le),R(w,te,Xl,le),R(w,te,ca,te.notifyWith))):(Z!==ca&&(De=void 0,Ue=[_t]),(le||te.resolveWith)(De,Ue))}},Ct=le?Ee:function(){try{Ee()}catch(_t){y.Deferred.exceptionHook&&y.Deferred.exceptionHook(_t,Ct.error),W+1>=w&&(Z!==Xl&&(De=void 0,Ue=[_t]),te.rejectWith(De,Ue))}};W?Ct():(y.Deferred.getErrorHook?Ct.error=y.Deferred.getErrorHook():y.Deferred.getStackHook&&(Ct.error=y.Deferred.getStackHook()),i.setTimeout(Ct))}}return y.Deferred(function(W){c[0][3].add(R(0,W,B(O)?O:ca,W.notifyWith)),c[1][3].add(R(0,W,B(_)?_:ca)),c[2][3].add(R(0,W,B(D)?D:Xl))}).promise()},promise:function(_){return _!=null?y.extend(_,m):m}},S={};return y.each(c,function(_,D){var O=D[2],w=D[5];m[D[1]]=O.add,w&&O.add(function(){p=w},c[3-_][2].disable,c[3-_][3].disable,c[0][2].lock,c[0][3].lock),O.add(D[3].fire),S[D[0]]=function(){return S[D[0]+"With"](this===S?void 0:this,arguments),this},S[D[0]+"With"]=O.fireWith}),m.promise(S),u&&u.call(S,S),S},when:function(u){var c=arguments.length,p=c,m=Array(p),S=h.call(arguments),_=y.Deferred(),D=function(O){return function(w){m[O]=this,S[O]=arguments.length>1?h.call(arguments):w,--c||_.resolveWith(m,S)}};if(c<=1&&(sa(u,_.done(D(p)).resolve,_.reject,!c),_.state()==="pending"||B(S[p]&&S[p].then)))return _.then();for(;p--;)sa(S[p],D(p),_.reject);return _.promise()}});var wr=/^(Eval|Internal|Range|Reference|Syntax|Type|URI)Error$/;y.Deferred.exceptionHook=function(u,c){i.console&&i.console.warn&&u&&wr.test(u.name)&&i.console.warn("jQuery.Deferred exception: "+u.message,u.stack,c)},y.readyException=function(u){i.setTimeout(function(){throw u})};var Gl=y.Deferred();y.fn.ready=function(u){return Gl.then(u).catch(function(c){y.readyException(c)}),this},y.extend({isReady:!1,readyWait:1,ready:function(u){(u===!0?--y.readyWait:y.isReady)||(y.isReady=!0,!(u!==!0&&--y.readyWait>0)&&Gl.resolveWith(F,[y]))}}),y.ready.then=Gl.then;function Mi(){F.removeEventListener("DOMContentLoaded",Mi),i.removeEventListener("load",Mi),y.ready()}F.readyState==="complete"||F.readyState!=="loading"&&!F.documentElement.doScroll?i.setTimeout(y.ready):(F.addEventListener("DOMContentLoaded",Mi),i.addEventListener("load",Mi));var On=function(u,c,p,m,S,_,D){var O=0,w=u.length,R=p==null;if(Te(p)==="object"){S=!0;for(O in p)On(u,c,O,p[O],!0,_,D)}else if(m!==void 0&&(S=!0,B(m)||(D=!0),R&&(D?(c.call(u,m),c=null):(R=c,c=function(W,te,Z){return R.call(y(W),Z)})),c))for(;O<w;O++)c(u[O],p,D?m:m.call(u[O],O,c(u[O],p)));return S?u:R?c.call(u):w?c(u[0],p):_},Or=/^-ms-/,zr=/-([a-z])/g;function Lr(u,c){return c.toUpperCase()}function zn(u){return u.replace(Or,"ms-").replace(zr,Lr)}var Vl=function(u){return u.nodeType===1||u.nodeType===9||!+u.nodeType};function wt(){this.expando=y.expando+wt.uid++}wt.uid=1,wt.prototype={cache:function(u){var c=u[this.expando];return c||(c={},Vl(u)&&(u.nodeType?u[this.expando]=c:Object.defineProperty(u,this.expando,{value:c,configurable:!0}))),c},set:function(u,c,p){var m,S=this.cache(u);if(typeof c=="string")S[zn(c)]=p;else for(m in c)S[zn(m)]=c[m];return S},get:function(u,c){return c===void 0?this.cache(u):u[this.expando]&&u[this.expando][zn(c)]},access:function(u,c,p){return c===void 0||c&&typeof c=="string"&&p===void 0?this.get(u,c):(this.set(u,c,p),p!==void 0?p:c)},remove:function(u,c){var p,m=u[this.expando];if(m!==void 0){if(c!==void 0)for(Array.isArray(c)?c=c.map(zn):(c=zn(c),c=c in m?[c]:c.match(dn)||[]),p=c.length;p--;)delete m[c[p]];(c===void 0||y.isEmptyObject(m))&&(u.nodeType?u[this.expando]=void 0:delete u[this.expando])}},hasData:function(u){var c=u[this.expando];return c!==void 0&&!y.isEmptyObject(c)}};var ve=new wt,Gt=new wt,Vu=/^(?:\{[\w\W]*\}|\[[\w\W]*\])$/,wi=/[A-Z]/g;function Ho(u){return u==="true"?!0:u==="false"?!1:u==="null"?null:u===+u+""?+u:Vu.test(u)?JSON.parse(u):u}function Hr(u,c,p){var m;if(p===void 0&&u.nodeType===1)if(m="data-"+c.replace(wi,"-$&").toLowerCase(),p=u.getAttribute(m),typeof p=="string"){try{p=Ho(p)}catch{}Gt.set(u,c,p)}else p=void 0;return p}y.extend({hasData:function(u){return Gt.hasData(u)||ve.hasData(u)},data:function(u,c,p){return Gt.access(u,c,p)},removeData:function(u,c){Gt.remove(u,c)},_data:function(u,c,p){return ve.access(u,c,p)},_removeData:function(u,c){ve.remove(u,c)}}),y.fn.extend({data:function(u,c){var p,m,S,_=this[0],D=_&&_.attributes;if(u===void 0){if(this.length&&(S=Gt.get(_),_.nodeType===1&&!ve.get(_,"hasDataAttrs"))){for(p=D.length;p--;)D[p]&&(m=D[p].name,m.indexOf("data-")===0&&(m=zn(m.slice(5)),Hr(_,m,S[m])));ve.set(_,"hasDataAttrs",!0)}return S}return typeof u=="object"?this.each(function(){Gt.set(this,u)}):On(this,function(O){var w;if(_&&O===void 0)return w=Gt.get(_,u),w!==void 0||(w=Hr(_,u),w!==void 0)?w:void 0;this.each(function(){Gt.set(this,u,O)})},null,c,arguments.length>1,null,!0)},removeData:function(u){return this.each(function(){Gt.remove(this,u)})}}),y.extend({queue:function(u,c,p){var m;if(u)return c=(c||"fx")+"queue",m=ve.get(u,c),p&&(!m||Array.isArray(p)?m=ve.access(u,c,y.makeArray(p)):m.push(p)),m||[]},dequeue:function(u,c){c=c||"fx";var p=y.queue(u,c),m=p.length,S=p.shift(),_=y._queueHooks(u,c),D=function(){y.dequeue(u,c)};S==="inprogress"&&(S=p.shift(),m--),S&&(c==="fx"&&p.unshift("inprogress"),delete _.stop,S.call(u,D,_)),!m&&_&&_.empty.fire()},_queueHooks:function(u,c){var p=c+"queueHooks";return ve.get(u,p)||ve.access(u,p,{empty:y.Callbacks("once memory").add(function(){ve.remove(u,[c+"queue",p])})})}}),y.fn.extend({queue:function(u,c){var p=2;return typeof u!="string"&&(c=u,u="fx",p--),arguments.length<p?y.queue(this[0],u):c===void 0?this:this.each(function(){var m=y.queue(this,u,c);y._queueHooks(this,u),u==="fx"&&m[0]!=="inprogress"&&y.dequeue(this,u)})},dequeue:function(u){return this.each(function(){y.dequeue(this,u)})},clearQueue:function(u){return this.queue(u||"fx",[])},promise:function(u,c){var p,m=1,S=y.Deferred(),_=this,D=this.length,O=function(){--m||S.resolveWith(_,[_])};for(typeof u!="string"&&(c=u,u=void 0),u=u||"fx";D--;)p=ve.get(_[D],u+"queueHooks"),p&&p.empty&&(m++,p.empty.add(O));return O(),S.promise(c)}});var Qu=/[+-]?(?:\d*\.|)\d+(?:[eE][+-]?\d+|)/.source,Ql=new RegExp("^(?:([+-])=|)("+Qu+")([a-z%]*)$","i"),nn=["Top","Right","Bottom","Left"],gt=F.documentElement,an=function(u){return y.contains(u.ownerDocument,u)},ln={composed:!0};gt.getRootNode&&(an=function(u){return y.contains(u.ownerDocument,u)||u.getRootNode(ln)===u.ownerDocument});var Oi=function(u,c){return u=c||u,u.style.display==="none"||u.style.display===""&&an(u)&&y.css(u,"display")==="none"};function Zu(u,c,p,m){var S,_,D=20,O=m?function(){return m.cur()}:function(){return y.css(u,c,"")},w=O(),R=p&&p[3]||(y.cssNumber[c]?"":"px"),W=u.nodeType&&(y.cssNumber[c]||R!=="px"&&+w)&&Ql.exec(y.css(u,c));if(W&&W[3]!==R){for(w=w/2,R=R||W[3],W=+w||1;D--;)y.style(u,c,W+R),(1-_)*(1-(_=O()/w||.5))<=0&&(D=0),W=W/_;W=W*2,y.style(u,c,W+R),p=p||[]}return p&&(W=+W||+w||0,S=p[1]?W+(p[1]+1)*p[2]:+p[2],m&&(m.unit=R,m.start=W,m.end=S)),S}var Ku={};function zi(u){var c,p=u.ownerDocument,m=u.nodeName,S=Ku[m];return S||(c=p.body.appendChild(p.createElement(m)),S=y.css(c,"display"),c.parentNode.removeChild(c),S==="none"&&(S="block"),Ku[m]=S,S)}function In(u,c){for(var p,m,S=[],_=0,D=u.length;_<D;_++)m=u[_],m.style&&(p=m.style.display,c?(p==="none"&&(S[_]=ve.get(m,"display")||null,S[_]||(m.style.display="")),m.style.display===""&&Oi(m)&&(S[_]=zi(m))):p!=="none"&&(S[_]="none",ve.set(m,"display",p)));for(_=0;_<D;_++)S[_]!=null&&(u[_].style.display=S[_]);return u}y.fn.extend({show:function(){return In(this,!0)},hide:function(){return In(this)},toggle:function(u){return typeof u=="boolean"?u?this.show():this.hide():this.each(function(){Oi(this)?y(this).show():y(this).hide()})}});var da=/^(?:checkbox|radio)$/i,ha=/<([a-z][^\/\0>\x20\t\r\n\f]*)/i,Zl=/^$|^module$|\/(?:java|ecma)script/i;(function(){var u=F.createDocumentFragment(),c=u.appendChild(F.createElement("div")),p=F.createElement("input");p.setAttribute("type","radio"),p.setAttribute("checked","checked"),p.setAttribute("name","t"),c.appendChild(p),U.checkClone=c.cloneNode(!0).cloneNode(!0).lastChild.checked,c.innerHTML="<textarea>x</textarea>",U.noCloneChecked=!!c.cloneNode(!0).lastChild.defaultValue,c.innerHTML="<option></option>",U.option=!!c.lastChild})();var qt={thead:[1,"<table>","</table>"],col:[2,"<table><colgroup>","</colgroup></table>"],tr:[2,"<table><tbody>","</tbody></table>"],td:[3,"<table><tbody><tr>","</tr></tbody></table>"],_default:[0,"",""]};qt.tbody=qt.tfoot=qt.colgroup=qt.caption=qt.thead,qt.th=qt.td,U.option||(qt.optgroup=qt.option=[1,"<select multiple='multiple'>","</select>"]);function It(u,c){var p;return typeof u.getElementsByTagName<"u"?p=u.getElementsByTagName(c||"*"):typeof u.querySelectorAll<"u"?p=u.querySelectorAll(c||"*"):p=[],c===void 0||c&&Ce(u,c)?y.merge([u],p):p}function Li(u,c){for(var p=0,m=u.length;p<m;p++)ve.set(u[p],"globalEval",!c||ve.get(c[p],"globalEval"))}var jr=/<|&#?\w+;/;function el(u,c,p,m,S){for(var _,D,O,w,R,W,te=c.createDocumentFragment(),Z=[],le=0,De=u.length;le<De;le++)if(_=u[le],_||_===0)if(Te(_)==="object")y.merge(Z,_.nodeType?[_]:_);else if(!jr.test(_))Z.push(c.createTextNode(_));else{for(D=D||te.appendChild(c.createElement("div")),O=(ha.exec(_)||["",""])[1].toLowerCase(),w=qt[O]||qt._default,D.innerHTML=w[1]+y.htmlPrefilter(_)+w[2],W=w[0];W--;)D=D.lastChild;y.merge(Z,D.childNodes),D=te.firstChild,D.textContent=""}for(te.textContent="",le=0;_=Z[le++];){if(m&&y.inArray(_,m)>-1){S&&S.push(_);continue}if(R=an(_),D=It(te.appendChild(_),"script"),R&&Li(D),p)for(W=0;_=D[W++];)Zl.test(_.type||"")&&p.push(_)}return te}var Ju=/^([^.]*)(?:\.(.+)|)/;function Ta(){return!0}function Ca(){return!1}function Hi(u,c,p,m,S,_){var D,O;if(typeof c=="object"){typeof p!="string"&&(m=m||p,p=void 0);for(O in c)Hi(u,O,p,m,c[O],_);return u}if(m==null&&S==null?(S=p,m=p=void 0):S==null&&(typeof p=="string"?(S=m,m=void 0):(S=m,m=p,p=void 0)),S===!1)S=Ca;else if(!S)return u;return _===1&&(D=S,S=function(w){return y().off(w),D.apply(this,arguments)},S.guid=D.guid||(D.guid=y.guid++)),u.each(function(){y.event.add(this,c,S,m,p)})}y.event={global:{},add:function(u,c,p,m,S){var _,D,O,w,R,W,te,Z,le,De,Ue,Ee=ve.get(u);if(Vl(u))for(p.handler&&(_=p,p=_.handler,S=_.selector),S&&y.find.matchesSelector(gt,S),p.guid||(p.guid=y.guid++),(w=Ee.events)||(w=Ee.events=Object.create(null)),(D=Ee.handle)||(D=Ee.handle=function(Ct){return typeof y<"u"&&y.event.triggered!==Ct.type?y.event.dispatch.apply(u,arguments):void 0}),c=(c||"").match(dn)||[""],R=c.length;R--;)O=Ju.exec(c[R])||[],le=Ue=O[1],De=(O[2]||"").split(".").sort(),le&&(te=y.event.special[le]||{},le=(S?te.delegateType:te.bindType)||le,te=y.event.special[le]||{},W=y.extend({type:le,origType:Ue,data:m,handler:p,guid:p.guid,selector:S,needsContext:S&&y.expr.match.needsContext.test(S),namespace:De.join(".")},_),(Z=w[le])||(Z=w[le]=[],Z.delegateCount=0,(!te.setup||te.setup.call(u,m,De,D)===!1)&&u.addEventListener&&u.addEventListener(le,D)),te.add&&(te.add.call(u,W),W.handler.guid||(W.handler.guid=p.guid)),S?Z.splice(Z.delegateCount++,0,W):Z.push(W),y.event.global[le]=!0)},remove:function(u,c,p,m,S){var _,D,O,w,R,W,te,Z,le,De,Ue,Ee=ve.hasData(u)&&ve.get(u);if(!(!Ee||!(w=Ee.events))){for(c=(c||"").match(dn)||[""],R=c.length;R--;){if(O=Ju.exec(c[R])||[],le=Ue=O[1],De=(O[2]||"").split(".").sort(),!le){for(le in w)y.event.remove(u,le+c[R],p,m,!0);continue}for(te=y.event.special[le]||{},le=(m?te.delegateType:te.bindType)||le,Z=w[le]||[],O=O[2]&&new RegExp("(^|\\.)"+De.join("\\.(?:.*\\.|)")+"(\\.|$)"),D=_=Z.length;_--;)W=Z[_],(S||Ue===W.origType)&&(!p||p.guid===W.guid)&&(!O||O.test(W.namespace))&&(!m||m===W.selector||m==="**"&&W.selector)&&(Z.splice(_,1),W.selector&&Z.delegateCount--,te.remove&&te.remove.call(u,W));D&&!Z.length&&((!te.teardown||te.teardown.call(u,De,Ee.handle)===!1)&&y.removeEvent(u,le,Ee.handle),delete w[le])}y.isEmptyObject(w)&&ve.remove(u,"handle events")}},dispatch:function(u){var c,p,m,S,_,D,O=new Array(arguments.length),w=y.event.fix(u),R=(ve.get(this,"events")||Object.create(null))[w.type]||[],W=y.event.special[w.type]||{};for(O[0]=w,c=1;c<arguments.length;c++)O[c]=arguments[c];if(w.delegateTarget=this,!(W.preDispatch&&W.preDispatch.call(this,w)===!1)){for(D=y.event.handlers.call(this,w,R),c=0;(S=D[c++])&&!w.isPropagationStopped();)for(w.currentTarget=S.elem,p=0;(_=S.handlers[p++])&&!w.isImmediatePropagationStopped();)(!w.rnamespace||_.namespace===!1||w.rnamespace.test(_.namespace))&&(w.handleObj=_,w.data=_.data,m=((y.event.special[_.origType]||{}).handle||_.handler).apply(S.elem,O),m!==void 0&&(w.result=m)===!1&&(w.preventDefault(),w.stopPropagation()));return W.postDispatch&&W.postDispatch.call(this,w),w.result}},handlers:function(u,c){var p,m,S,_,D,O=[],w=c.delegateCount,R=u.target;if(w&&R.nodeType&&!(u.type==="click"&&u.button>=1)){for(;R!==this;R=R.parentNode||this)if(R.nodeType===1&&!(u.type==="click"&&R.disabled===!0)){for(_=[],D={},p=0;p<w;p++)m=c[p],S=m.selector+" ",D[S]===void 0&&(D[S]=m.needsContext?y(S,this).index(R)>-1:y.find(S,this,null,[R]).length),D[S]&&_.push(m);_.length&&O.push({elem:R,handlers:_})}}return R=this,w<c.length&&O.push({elem:R,handlers:c.slice(w)}),O},addProp:function(u,c){Object.defineProperty(y.Event.prototype,u,{enumerable:!0,configurable:!0,get:B(c)?function(){if(this.originalEvent)return c(this.originalEvent)}:function(){if(this.originalEvent)return this.originalEvent[u]},set:function(p){Object.defineProperty(this,u,{enumerable:!0,configurable:!0,writable:!0,value:p})}})},fix:function(u){return u[y.expando]?u:new y.Event(u)},special:{load:{noBubble:!0},click:{setup:function(u){var c=this||u;return da.test(c.type)&&c.click&&Ce(c,"input")&&tl(c,"click",!0),!1},trigger:function(u){var c=this||u;return da.test(c.type)&&c.click&&Ce(c,"input")&&tl(c,"click"),!0},_default:function(u){var c=u.target;return da.test(c.type)&&c.click&&Ce(c,"input")&&ve.get(c,"click")||Ce(c,"a")}},beforeunload:{postDispatch:function(u){u.result!==void 0&&u.originalEvent&&(u.originalEvent.returnValue=u.result)}}}};function tl(u,c,p){if(!p){ve.get(u,c)===void 0&&y.event.add(u,c,Ta);return}ve.set(u,c,!1),y.event.add(u,c,{namespace:!1,handler:function(m){var S,_=ve.get(this,c);if(m.isTrigger&1&&this[c]){if(_)(y.event.special[c]||{}).delegateType&&m.stopPropagation();else if(_=h.call(arguments),ve.set(this,c,_),this[c](),S=ve.get(this,c),ve.set(this,c,!1),_!==S)return m.stopImmediatePropagation(),m.preventDefault(),S}else _&&(ve.set(this,c,y.event.trigger(_[0],_.slice(1),this)),m.stopPropagation(),m.isImmediatePropagationStopped=Ta)}})}y.removeEvent=function(u,c,p){u.removeEventListener&&u.removeEventListener(c,p)},y.Event=function(u,c){if(!(this instanceof y.Event))return new y.Event(u,c);u&&u.type?(this.originalEvent=u,this.type=u.type,this.isDefaultPrevented=u.defaultPrevented||u.defaultPrevented===void 0&&u.returnValue===!1?Ta:Ca,this.target=u.target&&u.target.nodeType===3?u.target.parentNode:u.target,this.currentTarget=u.currentTarget,this.relatedTarget=u.relatedTarget):this.type=u,c&&y.extend(this,c),this.timeStamp=u&&u.timeStamp||Date.now(),this[y.expando]=!0},y.Event.prototype={constructor:y.Event,isDefaultPrevented:Ca,isPropagationStopped:Ca,isImmediatePropagationStopped:Ca,isSimulated:!1,preventDefault:function(){var u=this.originalEvent;this.isDefaultPrevented=Ta,u&&!this.isSimulated&&u.preventDefault()},stopPropagation:function(){var u=this.originalEvent;this.isPropagationStopped=Ta,u&&!this.isSimulated&&u.stopPropagation()},stopImmediatePropagation:function(){var u=this.originalEvent;this.isImmediatePropagationStopped=Ta,u&&!this.isSimulated&&u.stopImmediatePropagation(),this.stopPropagation()}},y.each({altKey:!0,bubbles:!0,cancelable:!0,changedTouches:!0,ctrlKey:!0,detail:!0,eventPhase:!0,metaKey:!0,pageX:!0,pageY:!0,shiftKey:!0,view:!0,char:!0,code:!0,charCode:!0,key:!0,keyCode:!0,button:!0,buttons:!0,clientX:!0,clientY:!0,offsetX:!0,offsetY:!0,pointerId:!0,pointerType:!0,screenX:!0,screenY:!0,targetTouches:!0,toElement:!0,touches:!0,which:!0},y.event.addProp),y.each({focus:"focusin",blur:"focusout"},function(u,c){function p(m){if(F.documentMode){var S=ve.get(this,"handle"),_=y.event.fix(m);_.type=m.type==="focusin"?"focus":"blur",_.isSimulated=!0,S(m),_.target===_.currentTarget&&S(_)}else y.event.simulate(c,m.target,y.event.fix(m))}y.event.special[u]={setup:function(){var m;if(tl(this,u,!0),F.documentMode)m=ve.get(this,c),m||this.addEventListener(c,p),ve.set(this,c,(m||0)+1);else return!1},trigger:function(){return tl(this,u),!0},teardown:function(){var m;if(F.documentMode)m=ve.get(this,c)-1,m?ve.set(this,c,m):(this.removeEventListener(c,p),ve.remove(this,c));else return!1},_default:function(m){return ve.get(m.target,u)},delegateType:c},y.event.special[c]={setup:function(){var m=this.ownerDocument||this.document||this,S=F.documentMode?this:m,_=ve.get(S,c);_||(F.documentMode?this.addEventListener(c,p):m.addEventListener(u,p,!0)),ve.set(S,c,(_||0)+1)},teardown:function(){var m=this.ownerDocument||this.document||this,S=F.documentMode?this:m,_=ve.get(S,c)-1;_?ve.set(S,c,_):(F.documentMode?this.removeEventListener(c,p):m.removeEventListener(u,p,!0),ve.remove(S,c))}}}),y.each({mouseenter:"mouseover",mouseleave:"mouseout",pointerenter:"pointerover",pointerleave:"pointerout"},function(u,c){y.event.special[u]={delegateType:c,bindType:c,handle:function(p){var m,S=this,_=p.relatedTarget,D=p.handleObj;return(!_||_!==S&&!y.contains(S,_))&&(p.type=D.origType,m=D.handler.apply(this,arguments),p.type=c),m}}}),y.fn.extend({on:function(u,c,p,m){return Hi(this,u,c,p,m)},one:function(u,c,p,m){return Hi(this,u,c,p,m,1)},off:function(u,c,p){var m,S;if(u&&u.preventDefault&&u.handleObj)return m=u.handleObj,y(u.delegateTarget).off(m.namespace?m.origType+"."+m.namespace:m.origType,m.selector,m.handler),this;if(typeof u=="object"){for(S in u)this.off(S,c,u[S]);return this}return(c===!1||typeof c=="function")&&(p=c,c=void 0),p===!1&&(p=Ca),this.each(function(){y.event.remove(this,u,p,c)})}});var Rr=/<script|<style|<link/i,ku=/checked\s*(?:[^=]|=\s*.checked.)/i,Fu=/^\s*<!\[CDATA\[|\]\]>\s*$/g;function Pn(u,c){return Ce(u,"table")&&Ce(c.nodeType!==11?c:c.firstChild,"tr")&&y(u).children("tbody")[0]||u}function Ot(u){return u.type=(u.getAttribute("type")!==null)+"/"+u.type,u}function Pt(u){return(u.type||"").slice(0,5)==="true/"?u.type=u.type.slice(5):u.removeAttribute("type"),u}function Aa(u,c){var p,m,S,_,D,O,w;if(c.nodeType===1){if(ve.hasData(u)&&(_=ve.get(u),w=_.events,w)){ve.remove(c,"handle events");for(S in w)for(p=0,m=w[S].length;p<m;p++)y.event.add(c,S,w[S][p])}Gt.hasData(u)&&(D=Gt.access(u),O=y.extend({},D),Gt.set(c,O))}}function qr(u,c){var p=c.nodeName.toLowerCase();p==="input"&&da.test(u.type)?c.checked=u.checked:(p==="input"||p==="textarea")&&(c.defaultValue=u.defaultValue)}function nl(u,c,p,m){c=v(c);var S,_,D,O,w,R,W=0,te=u.length,Z=te-1,le=c[0],De=B(le);if(De||te>1&&typeof le=="string"&&!U.checkClone&&ku.test(le))return u.each(function(Ue){var Ee=u.eq(Ue);De&&(c[0]=le.call(this,Ue,Ee.html())),nl(Ee,c,p,m)});if(te&&(S=el(c,u[0].ownerDocument,!1,u,m),_=S.firstChild,S.childNodes.length===1&&(S=_),_||m)){for(D=y.map(It(S,"script"),Ot),O=D.length;W<te;W++)w=S,W!==Z&&(w=y.clone(w,!0,!0),O&&y.merge(D,It(w,"script"))),p.call(u[W],w,W);if(O)for(R=D[D.length-1].ownerDocument,y.map(D,Pt),W=0;W<O;W++)w=D[W],Zl.test(w.type||"")&&!ve.access(w,"globalEval")&&y.contains(R,w)&&(w.src&&(w.type||"").toLowerCase()!=="module"?y._evalUrl&&!w.noModule&&y._evalUrl(w.src,{nonce:w.nonce||w.getAttribute("nonce")},R):Se(w.textContent.replace(Fu,""),w,R))}return u}function Wu(u,c,p){for(var m,S=c?y.filter(c,u):u,_=0;(m=S[_])!=null;_++)!p&&m.nodeType===1&&y.cleanData(It(m)),m.parentNode&&(p&&an(m)&&Li(It(m,"script")),m.parentNode.removeChild(m));return u}y.extend({htmlPrefilter:function(u){return u},clone:function(u,c,p){var m,S,_,D,O=u.cloneNode(!0),w=an(u);if(!U.noCloneChecked&&(u.nodeType===1||u.nodeType===11)&&!y.isXMLDoc(u))for(D=It(O),_=It(u),m=0,S=_.length;m<S;m++)qr(_[m],D[m]);if(c)if(p)for(_=_||It(u),D=D||It(O),m=0,S=_.length;m<S;m++)Aa(_[m],D[m]);else Aa(u,O);return D=It(O,"script"),D.length>0&&Li(D,!w&&It(u,"script")),O},cleanData:function(u){for(var c,p,m,S=y.event.special,_=0;(p=u[_])!==void 0;_++)if(Vl(p)){if(c=p[ve.expando]){if(c.events)for(m in c.events)S[m]?y.event.remove(p,m):y.removeEvent(p,m,c.handle);p[ve.expando]=void 0}p[Gt.expando]&&(p[Gt.expando]=void 0)}}}),y.fn.extend({detach:function(u){return Wu(this,u,!0)},remove:function(u){return Wu(this,u)},text:function(u){return On(this,function(c){return c===void 0?y.text(this):this.empty().each(function(){(this.nodeType===1||this.nodeType===11||this.nodeType===9)&&(this.textContent=c)})},null,u,arguments.length)},append:function(){return nl(this,arguments,function(u){if(this.nodeType===1||this.nodeType===11||this.nodeType===9){var c=Pn(this,u);c.appendChild(u)}})},prepend:function(){return nl(this,arguments,function(u){if(this.nodeType===1||this.nodeType===11||this.nodeType===9){var c=Pn(this,u);c.insertBefore(u,c.firstChild)}})},before:function(){return nl(this,arguments,function(u){this.parentNode&&this.parentNode.insertBefore(u,this)})},after:function(){return nl(this,arguments,function(u){this.parentNode&&this.parentNode.insertBefore(u,this.nextSibling)})},empty:function(){for(var u,c=0;(u=this[c])!=null;c++)u.nodeType===1&&(y.cleanData(It(u,!1)),u.textContent="");return this},clone:function(u,c){return u=u??!1,c=c??u,this.map(function(){return y.clone(this,u,c)})},html:function(u){return On(this,function(c){var p=this[0]||{},m=0,S=this.length;if(c===void 0&&p.nodeType===1)return p.innerHTML;if(typeof c=="string"&&!Rr.test(c)&&!qt[(ha.exec(c)||["",""])[1].toLowerCase()]){c=y.htmlPrefilter(c);try{for(;m<S;m++)p=this[m]||{},p.nodeType===1&&(y.cleanData(It(p,!1)),p.innerHTML=c);p=0}catch{}}p&&this.empty().append(c)},null,u,arguments.length)},replaceWith:function(){var u=[];return nl(this,arguments,function(c){var p=this.parentNode;y.inArray(this,u)<0&&(y.cleanData(It(this)),p&&p.replaceChild(c,this))},u)}}),y.each({appendTo:"append",prependTo:"prepend",insertBefore:"before",insertAfter:"after",replaceAll:"replaceWith"},function(u,c){y.fn[u]=function(p){for(var m,S=[],_=y(p),D=_.length-1,O=0;O<=D;O++)m=O===D?this:this.clone(!0),y(_[O])[c](m),g.apply(S,m.get());return this.pushStack(S)}});var ji=new RegExp("^("+Qu+")(?!px)[a-z%]+$","i"),Ea=/^--/,al=function(u){var c=u.ownerDocument.defaultView;return(!c||!c.opener)&&(c=i),c.getComputedStyle(u)},Na=function(u,c,p){var m,S,_={};for(S in c)_[S]=u.style[S],u.style[S]=c[S];m=p.call(u);for(S in c)u.style[S]=_[S];return m},ll=new RegExp(nn.join("|"),"i");(function(){function u(){if(R){w.style.cssText="position:absolute;left:-11111px;width:60px;margin-top:1px;padding:0;border:0",R.style.cssText="position:relative;display:block;box-sizing:border-box;overflow:scroll;margin:auto;border:1px;padding:1px;width:60%;top:1%",gt.appendChild(w).appendChild(R);var W=i.getComputedStyle(R);p=W.top!=="1%",O=c(W.marginLeft)===12,R.style.right="60%",_=c(W.right)===36,m=c(W.width)===36,R.style.position="absolute",S=c(R.offsetWidth/3)===12,gt.removeChild(w),R=null}}function c(W){return Math.round(parseFloat(W))}var p,m,S,_,D,O,w=F.createElement("div"),R=F.createElement("div");R.style&&(R.style.backgroundClip="content-box",R.cloneNode(!0).style.backgroundClip="",U.clearCloneStyle=R.style.backgroundClip==="content-box",y.extend(U,{boxSizingReliable:function(){return u(),m},pixelBoxStyles:function(){return u(),_},pixelPosition:function(){return u(),p},reliableMarginLeft:function(){return u(),O},scrollboxSize:function(){return u(),S},reliableTrDimensions:function(){var W,te,Z,le;return D==null&&(W=F.createElement("table"),te=F.createElement("tr"),Z=F.createElement("div"),W.style.cssText="position:absolute;left:-11111px;border-collapse:separate",te.style.cssText="box-sizing:content-box;border:1px solid",te.style.height="1px",Z.style.height="9px",Z.style.display="block",gt.appendChild(W).appendChild(te).appendChild(Z),le=i.getComputedStyle(te),D=parseInt(le.height,10)+parseInt(le.borderTopWidth,10)+parseInt(le.borderBottomWidth,10)===te.offsetHeight,gt.removeChild(W)),D}}))})();function ea(u,c,p){var m,S,_,D,O=Ea.test(c),w=u.style;return p=p||al(u),p&&(D=p.getPropertyValue(c)||p[c],O&&D&&(D=D.replace($t,"$1")||void 0),D===""&&!an(u)&&(D=y.style(u,c)),!U.pixelBoxStyles()&&ji.test(D)&&ll.test(c)&&(m=w.width,S=w.minWidth,_=w.maxWidth,w.minWidth=w.maxWidth=w.width=D,D=p.width,w.width=m,w.minWidth=S,w.maxWidth=_)),D!==void 0?D+"":D}function Ma(u,c){return{get:function(){if(u()){delete this.get;return}return(this.get=c).apply(this,arguments)}}}var St=["Webkit","Moz","ms"],Ur=F.createElement("div").style,Br={};function wa(u){for(var c=u[0].toUpperCase()+u.slice(1),p=St.length;p--;)if(u=St[p]+c,u in Ur)return u}function pa(u){var c=y.cssProps[u]||Br[u];return c||(u in Ur?u:Br[u]=wa(u)||u)}var jo=/^(none|table(?!-c[ea]).+)/,$u={position:"absolute",visibility:"hidden",display:"block"},Yr={letterSpacing:"0",fontWeight:"400"};function Iu(u,c,p){var m=Ql.exec(c);return m?Math.max(0,m[2]-(p||0))+(m[3]||"px"):c}function il(u,c,p,m,S,_){var D=c==="width"?1:0,O=0,w=0,R=0;if(p===(m?"border":"content"))return 0;for(;D<4;D+=2)p==="margin"&&(R+=y.css(u,p+nn[D],!0,S)),m?(p==="content"&&(w-=y.css(u,"padding"+nn[D],!0,S)),p!=="margin"&&(w-=y.css(u,"border"+nn[D]+"Width",!0,S))):(w+=y.css(u,"padding"+nn[D],!0,S),p!=="padding"?w+=y.css(u,"border"+nn[D]+"Width",!0,S):O+=y.css(u,"border"+nn[D]+"Width",!0,S));return!m&&_>=0&&(w+=Math.max(0,Math.ceil(u["offset"+c[0].toUpperCase()+c.slice(1)]-_-w-O-.5))||0),w+R}function Kl(u,c,p){var m=al(u),S=!U.boxSizingReliable()||p,_=S&&y.css(u,"boxSizing",!1,m)==="border-box",D=_,O=ea(u,c,m),w="offset"+c[0].toUpperCase()+c.slice(1);if(ji.test(O)){if(!p)return O;O="auto"}return(!U.boxSizingReliable()&&_||!U.reliableTrDimensions()&&Ce(u,"tr")||O==="auto"||!parseFloat(O)&&y.css(u,"display",!1,m)==="inline")&&u.getClientRects().length&&(_=y.css(u,"boxSizing",!1,m)==="border-box",D=w in u,D&&(O=u[w])),O=parseFloat(O)||0,O+il(u,c,p||(_?"border":"content"),D,m,O)+"px"}y.extend({cssHooks:{opacity:{get:function(u,c){if(c){var p=ea(u,"opacity");return p===""?"1":p}}}},cssNumber:{animationIterationCount:!0,aspectRatio:!0,borderImageSlice:!0,columnCount:!0,flexGrow:!0,flexShrink:!0,fontWeight:!0,gridArea:!0,gridColumn:!0,gridColumnEnd:!0,gridColumnStart:!0,gridRow:!0,gridRowEnd:!0,gridRowStart:!0,lineHeight:!0,opacity:!0,order:!0,orphans:!0,scale:!0,widows:!0,zIndex:!0,zoom:!0,fillOpacity:!0,floodOpacity:!0,stopOpacity:!0,strokeMiterlimit:!0,strokeOpacity:!0},cssProps:{},style:function(u,c,p,m){if(!(!u||u.nodeType===3||u.nodeType===8||!u.style)){var S,_,D,O=zn(c),w=Ea.test(c),R=u.style;if(w||(c=pa(O)),D=y.cssHooks[c]||y.cssHooks[O],p!==void 0){if(_=typeof p,_==="string"&&(S=Ql.exec(p))&&S[1]&&(p=Zu(u,c,S),_="number"),p==null||p!==p)return;_==="number"&&!w&&(p+=S&&S[3]||(y.cssNumber[O]?"":"px")),!U.clearCloneStyle&&p===""&&c.indexOf("background")===0&&(R[c]="inherit"),(!D||!("set"in D)||(p=D.set(u,p,m))!==void 0)&&(w?R.setProperty(c,p):R[c]=p)}else return D&&"get"in D&&(S=D.get(u,!1,m))!==void 0?S:R[c]}},css:function(u,c,p,m){var S,_,D,O=zn(c),w=Ea.test(c);return w||(c=pa(O)),D=y.cssHooks[c]||y.cssHooks[O],D&&"get"in D&&(S=D.get(u,!0,p)),S===void 0&&(S=ea(u,c,m)),S==="normal"&&c in Yr&&(S=Yr[c]),p===""||p?(_=parseFloat(S),p===!0||isFinite(_)?_||0:S):S}}),y.each(["height","width"],function(u,c){y.cssHooks[c]={get:function(p,m,S){if(m)return jo.test(y.css(p,"display"))&&(!p.getClientRects().length||!p.getBoundingClientRect().width)?Na(p,$u,function(){return Kl(p,c,S)}):Kl(p,c,S)},set:function(p,m,S){var _,D=al(p),O=!U.scrollboxSize()&&D.position==="absolute",w=O||S,R=w&&y.css(p,"boxSizing",!1,D)==="border-box",W=S?il(p,c,S,R,D):0;return R&&O&&(W-=Math.ceil(p["offset"+c[0].toUpperCase()+c.slice(1)]-parseFloat(D[c])-il(p,c,"border",!1,D)-.5)),W&&(_=Ql.exec(m))&&(_[3]||"px")!=="px"&&(p.style[c]=m,m=y.css(p,c)),Iu(p,m,W)}}}),y.cssHooks.marginLeft=Ma(U.reliableMarginLeft,function(u,c){if(c)return(parseFloat(ea(u,"marginLeft"))||u.getBoundingClientRect().left-Na(u,{marginLeft:0},function(){return u.getBoundingClientRect().left}))+"px"}),y.each({margin:"",padding:"",border:"Width"},function(u,c){y.cssHooks[u+c]={expand:function(p){for(var m=0,S={},_=typeof p=="string"?p.split(" "):[p];m<4;m++)S[u+nn[m]+c]=_[m]||_[m-2]||_[0];return S}},u!=="margin"&&(y.cssHooks[u+c].set=Iu)}),y.fn.extend({css:function(u,c){return On(this,function(p,m,S){var _,D,O={},w=0;if(Array.isArray(m)){for(_=al(p),D=m.length;w<D;w++)O[m[w]]=y.css(p,m[w],!1,_);return O}return S!==void 0?y.style(p,m,S):y.css(p,m)},u,c,arguments.length>1)}});function dt(u,c,p,m,S){return new dt.prototype.init(u,c,p,m,S)}y.Tween=dt,dt.prototype={constructor:dt,init:function(u,c,p,m,S,_){this.elem=u,this.prop=p,this.easing=S||y.easing._default,this.options=c,this.start=this.now=this.cur(),this.end=m,this.unit=_||(y.cssNumber[p]?"":"px")},cur:function(){var u=dt.propHooks[this.prop];return u&&u.get?u.get(this):dt.propHooks._default.get(this)},run:function(u){var c,p=dt.propHooks[this.prop];return this.options.duration?this.pos=c=y.easing[this.easing](u,this.options.duration*u,0,1,this.options.duration):this.pos=c=u,this.now=(this.end-this.start)*c+this.start,this.options.step&&this.options.step.call(this.elem,this.now,this),p&&p.set?p.set(this):dt.propHooks._default.set(this),this}},dt.prototype.init.prototype=dt.prototype,dt.propHooks={_default:{get:function(u){var c;return u.elem.nodeType!==1||u.elem[u.prop]!=null&&u.elem.style[u.prop]==null?u.elem[u.prop]:(c=y.css(u.elem,u.prop,""),!c||c==="auto"?0:c)},set:function(u){y.fx.step[u.prop]?y.fx.step[u.prop](u):u.elem.nodeType===1&&(y.cssHooks[u.prop]||u.elem.style[pa(u.prop)]!=null)?y.style(u.elem,u.prop,u.now+u.unit):u.elem[u.prop]=u.now}}},dt.propHooks.scrollTop=dt.propHooks.scrollLeft={set:function(u){u.elem.nodeType&&u.elem.parentNode&&(u.elem[u.prop]=u.now)}},y.easing={linear:function(u){return u},swing:function(u){return .5-Math.cos(u*Math.PI)/2},_default:"swing"},y.fx=dt.prototype.init,y.fx.step={};var zt,Jl,Ro=/^(?:toggle|show|hide)$/,Xr=/queueHooks$/;function Ri(){Jl&&(F.hidden===!1&&i.requestAnimationFrame?i.requestAnimationFrame(Ri):i.setTimeout(Ri,y.fx.interval),y.fx.tick())}function kl(){return i.setTimeout(function(){zt=void 0}),zt=Date.now()}function qi(u,c){var p,m=0,S={height:u};for(c=c?1:0;m<4;m+=2-c)p=nn[m],S["margin"+p]=S["padding"+p]=u;return c&&(S.opacity=S.width=u),S}function rn(u,c,p){for(var m,S=(un.tweeners[c]||[]).concat(un.tweeners["*"]),_=0,D=S.length;_<D;_++)if(m=S[_].call(p,c,u))return m}function Gr(u,c,p){var m,S,_,D,O,w,R,W,te="width"in c||"height"in c,Z=this,le={},De=u.style,Ue=u.nodeType&&Oi(u),Ee=ve.get(u,"fxshow");p.queue||(D=y._queueHooks(u,"fx"),D.unqueued==null&&(D.unqueued=0,O=D.empty.fire,D.empty.fire=function(){D.unqueued||O()}),D.unqueued++,Z.always(function(){Z.always(function(){D.unqueued--,y.queue(u,"fx").length||D.empty.fire()})}));for(m in c)if(S=c[m],Ro.test(S)){if(delete c[m],_=_||S==="toggle",S===(Ue?"hide":"show"))if(S==="show"&&Ee&&Ee[m]!==void 0)Ue=!0;else continue;le[m]=Ee&&Ee[m]||y.style(u,m)}if(w=!y.isEmptyObject(c),!(!w&&y.isEmptyObject(le))){te&&u.nodeType===1&&(p.overflow=[De.overflow,De.overflowX,De.overflowY],R=Ee&&Ee.display,R==null&&(R=ve.get(u,"display")),W=y.css(u,"display"),W==="none"&&(R?W=R:(In([u],!0),R=u.style.display||R,W=y.css(u,"display"),In([u]))),(W==="inline"||W==="inline-block"&&R!=null)&&y.css(u,"float")==="none"&&(w||(Z.done(function(){De.display=R}),R==null&&(W=De.display,R=W==="none"?"":W)),De.display="inline-block")),p.overflow&&(De.overflow="hidden",Z.always(function(){De.overflow=p.overflow[0],De.overflowX=p.overflow[1],De.overflowY=p.overflow[2]})),w=!1;for(m in le)w||(Ee?"hidden"in Ee&&(Ue=Ee.hidden):Ee=ve.access(u,"fxshow",{display:R}),_&&(Ee.hidden=!Ue),Ue&&In([u],!0),Z.done(function(){Ue||In([u]),ve.remove(u,"fxshow");for(m in le)y.style(u,m,le[m])})),w=rn(Ue?Ee[m]:0,m,Z),m in Ee||(Ee[m]=w.start,Ue&&(w.end=w.start,w.start=0))}}function Pu(u,c){var p,m,S,_,D;for(p in u)if(m=zn(p),S=c[m],_=u[p],Array.isArray(_)&&(S=_[1],_=u[p]=_[0]),p!==m&&(u[m]=_,delete u[p]),D=y.cssHooks[m],D&&"expand"in D){_=D.expand(_),delete u[m];for(p in _)p in u||(u[p]=_[p],c[p]=S)}else c[m]=S}function un(u,c,p){var m,S,_=0,D=un.prefilters.length,O=y.Deferred().always(function(){delete w.elem}),w=function(){if(S)return!1;for(var te=zt||kl(),Z=Math.max(0,R.startTime+R.duration-te),le=Z/R.duration||0,De=1-le,Ue=0,Ee=R.tweens.length;Ue<Ee;Ue++)R.tweens[Ue].run(De);return O.notifyWith(u,[R,De,Z]),De<1&&Ee?Z:(Ee||O.notifyWith(u,[R,1,0]),O.resolveWith(u,[R]),!1)},R=O.promise({elem:u,props:y.extend({},c),opts:y.extend(!0,{specialEasing:{},easing:y.easing._default},p),originalProperties:c,originalOptions:p,startTime:zt||kl(),duration:p.duration,tweens:[],createTween:function(te,Z){var le=y.Tween(u,R.opts,te,Z,R.opts.specialEasing[te]||R.opts.easing);return R.tweens.push(le),le},stop:function(te){var Z=0,le=te?R.tweens.length:0;if(S)return this;for(S=!0;Z<le;Z++)R.tweens[Z].run(1);return te?(O.notifyWith(u,[R,1,0]),O.resolveWith(u,[R,te])):O.rejectWith(u,[R,te]),this}}),W=R.props;for(Pu(W,R.opts.specialEasing);_<D;_++)if(m=un.prefilters[_].call(R,u,W,R.opts),m)return B(m.stop)&&(y._queueHooks(R.elem,R.opts.queue).stop=m.stop.bind(m)),m;return y.map(W,rn,R),B(R.opts.start)&&R.opts.start.call(u,R),R.progress(R.opts.progress).done(R.opts.done,R.opts.complete).fail(R.opts.fail).always(R.opts.always),y.fx.timer(y.extend(w,{elem:u,anim:R,queue:R.opts.queue})),R}y.Animation=y.extend(un,{tweeners:{"*":[function(u,c){var p=this.createTween(u,c);return Zu(p.elem,u,Ql.exec(c),p),p}]},tweener:function(u,c){B(u)?(c=u,u=["*"]):u=u.match(dn);for(var p,m=0,S=u.length;m<S;m++)p=u[m],un.tweeners[p]=un.tweeners[p]||[],un.tweeners[p].unshift(c)},prefilters:[Gr],prefilter:function(u,c){c?un.prefilters.unshift(u):un.prefilters.push(u)}}),y.speed=function(u,c,p){var m=u&&typeof u=="object"?y.extend({},u):{complete:p||!p&&c||B(u)&&u,duration:u,easing:p&&c||c&&!B(c)&&c};return y.fx.off?m.duration=0:typeof m.duration!="number"&&(m.duration in y.fx.speeds?m.duration=y.fx.speeds[m.duration]:m.duration=y.fx.speeds._default),(m.queue==null||m.queue===!0)&&(m.queue="fx"),m.old=m.complete,m.complete=function(){B(m.old)&&m.old.call(this),m.queue&&y.dequeue(this,m.queue)},m},y.fn.extend({fadeTo:function(u,c,p,m){return this.filter(Oi).css("opacity",0).show().end().animate({opacity:c},u,p,m)},animate:function(u,c,p,m){var S=y.isEmptyObject(u),_=y.speed(c,p,m),D=function(){var O=un(this,y.extend({},u),_);(S||ve.get(this,"finish"))&&O.stop(!0)};return D.finish=D,S||_.queue===!1?this.each(D):this.queue(_.queue,D)},stop:function(u,c,p){var m=function(S){var _=S.stop;delete S.stop,_(p)};return typeof u!="string"&&(p=c,c=u,u=void 0),c&&this.queue(u||"fx",[]),this.each(function(){var S=!0,_=u!=null&&u+"queueHooks",D=y.timers,O=ve.get(this);if(_)O[_]&&O[_].stop&&m(O[_]);else for(_ in O)O[_]&&O[_].stop&&Xr.test(_)&&m(O[_]);for(_=D.length;_--;)D[_].elem===this&&(u==null||D[_].queue===u)&&(D[_].anim.stop(p),S=!1,D.splice(_,1));(S||!p)&&y.dequeue(this,u)})},finish:function(u){return u!==!1&&(u=u||"fx"),this.each(function(){var c,p=ve.get(this),m=p[u+"queue"],S=p[u+"queueHooks"],_=y.timers,D=m?m.length:0;for(p.finish=!0,y.queue(this,u,[]),S&&S.stop&&S.stop.call(this,!0),c=_.length;c--;)_[c].elem===this&&_[c].queue===u&&(_[c].anim.stop(!0),_.splice(c,1));for(c=0;c<D;c++)m[c]&&m[c].finish&&m[c].finish.call(this);delete p.finish})}}),y.each(["toggle","show","hide"],function(u,c){var p=y.fn[c];y.fn[c]=function(m,S,_){return m==null||typeof m=="boolean"?p.apply(this,arguments):this.animate(qi(c,!0),m,S,_)}}),y.each({slideDown:qi("show"),slideUp:qi("hide"),slideToggle:qi("toggle"),fadeIn:{opacity:"show"},fadeOut:{opacity:"hide"},fadeToggle:{opacity:"toggle"}},function(u,c){y.fn[u]=function(p,m,S){return this.animate(c,p,m,S)}}),y.timers=[],y.fx.tick=function(){var u,c=0,p=y.timers;for(zt=Date.now();c<p.length;c++)u=p[c],!u()&&p[c]===u&&p.splice(c--,1);p.length||y.fx.stop(),zt=void 0},y.fx.timer=function(u){y.timers.push(u),y.fx.start()},y.fx.interval=13,y.fx.start=function(){Jl||(Jl=!0,Ri())},y.fx.stop=function(){Jl=null},y.fx.speeds={slow:600,fast:200,_default:400},y.fn.delay=function(u,c){return u=y.fx&&y.fx.speeds[u]||u,c=c||"fx",this.queue(c,function(p,m){var S=i.setTimeout(p,u);m.stop=function(){i.clearTimeout(S)}})},(function(){var u=F.createElement("input"),c=F.createElement("select"),p=c.appendChild(F.createElement("option"));u.type="checkbox",U.checkOn=u.value!=="",U.optSelected=p.selected,u=F.createElement("input"),u.value="t",u.type="radio",U.radioValue=u.value==="t"})();var Oa,rl=y.expr.attrHandle;y.fn.extend({attr:function(u,c){return On(this,y.attr,u,c,arguments.length>1)},removeAttr:function(u){return this.each(function(){y.removeAttr(this,u)})}}),y.extend({attr:function(u,c,p){var m,S,_=u.nodeType;if(!(_===3||_===8||_===2)){if(typeof u.getAttribute>"u")return y.prop(u,c,p);if((_!==1||!y.isXMLDoc(u))&&(S=y.attrHooks[c.toLowerCase()]||(y.expr.match.bool.test(c)?Oa:void 0)),p!==void 0){if(p===null){y.removeAttr(u,c);return}return S&&"set"in S&&(m=S.set(u,p,c))!==void 0?m:(u.setAttribute(c,p+""),p)}return S&&"get"in S&&(m=S.get(u,c))!==null?m:(m=y.find.attr(u,c),m??void 0)}},attrHooks:{type:{set:function(u,c){if(!U.radioValue&&c==="radio"&&Ce(u,"input")){var p=u.value;return u.setAttribute("type",c),p&&(u.value=p),c}}}},removeAttr:function(u,c){var p,m=0,S=c&&c.match(dn);if(S&&u.nodeType===1)for(;p=S[m++];)u.removeAttribute(p)}}),Oa={set:function(u,c,p){return c===!1?y.removeAttr(u,p):u.setAttribute(p,p),p}},y.each(y.expr.match.bool.source.match(/\w+/g),function(u,c){var p=rl[c]||y.find.attr;rl[c]=function(m,S,_){var D,O,w=S.toLowerCase();return _||(O=rl[w],rl[w]=D,D=p(m,S,_)!=null?w:null,rl[w]=O),D}});var ef=/^(?:input|select|textarea|button)$/i,ul=/^(?:a|area)$/i;y.fn.extend({prop:function(u,c){return On(this,y.prop,u,c,arguments.length>1)},removeProp:function(u){return this.each(function(){delete this[y.propFix[u]||u]})}}),y.extend({prop:function(u,c,p){var m,S,_=u.nodeType;if(!(_===3||_===8||_===2))return(_!==1||!y.isXMLDoc(u))&&(c=y.propFix[c]||c,S=y.propHooks[c]),p!==void 0?S&&"set"in S&&(m=S.set(u,p,c))!==void 0?m:u[c]=p:S&&"get"in S&&(m=S.get(u,c))!==null?m:u[c]},propHooks:{tabIndex:{get:function(u){var c=y.find.attr(u,"tabindex");return c?parseInt(c,10):ef.test(u.nodeName)||ul.test(u.nodeName)&&u.href?0:-1}}},propFix:{for:"htmlFor",class:"className"}}),U.optSelected||(y.propHooks.selected={get:function(u){var c=u.parentNode;return c&&c.parentNode&&c.parentNode.selectedIndex,null},set:function(u){var c=u.parentNode;c&&(c.selectedIndex,c.parentNode&&c.parentNode.selectedIndex)}}),y.each(["tabIndex","readOnly","maxLength","cellSpacing","cellPadding","rowSpan","colSpan","useMap","frameBorder","contentEditable"],function(){y.propFix[this.toLowerCase()]=this});function za(u){var c=u.match(dn)||[];return c.join(" ")}function va(u){return u.getAttribute&&u.getAttribute("class")||""}function Ui(u){return Array.isArray(u)?u:typeof u=="string"?u.match(dn)||[]:[]}y.fn.extend({addClass:function(u){var c,p,m,S,_,D;return B(u)?this.each(function(O){y(this).addClass(u.call(this,O,va(this)))}):(c=Ui(u),c.length?this.each(function(){if(m=va(this),p=this.nodeType===1&&" "+za(m)+" ",p){for(_=0;_<c.length;_++)S=c[_],p.indexOf(" "+S+" ")<0&&(p+=S+" ");D=za(p),m!==D&&this.setAttribute("class",D)}}):this)},removeClass:function(u){var c,p,m,S,_,D;return B(u)?this.each(function(O){y(this).removeClass(u.call(this,O,va(this)))}):arguments.length?(c=Ui(u),c.length?this.each(function(){if(m=va(this),p=this.nodeType===1&&" "+za(m)+" ",p){for(_=0;_<c.length;_++)for(S=c[_];p.indexOf(" "+S+" ")>-1;)p=p.replace(" "+S+" "," ");D=za(p),m!==D&&this.setAttribute("class",D)}}):this):this.attr("class","")},toggleClass:function(u,c){var p,m,S,_,D=typeof u,O=D==="string"||Array.isArray(u);return B(u)?this.each(function(w){y(this).toggleClass(u.call(this,w,va(this),c),c)}):typeof c=="boolean"&&O?c?this.addClass(u):this.removeClass(u):(p=Ui(u),this.each(function(){if(O)for(_=y(this),S=0;S<p.length;S++)m=p[S],_.hasClass(m)?_.removeClass(m):_.addClass(m);else(u===void 0||D==="boolean")&&(m=va(this),m&&ve.set(this,"__className__",m),this.setAttribute&&this.setAttribute("class",m||u===!1?"":ve.get(this,"__className__")||""))}))},hasClass:function(u){var c,p,m=0;for(c=" "+u+" ";p=this[m++];)if(p.nodeType===1&&(" "+za(va(p))+" ").indexOf(c)>-1)return!0;return!1}});var Vr=/\r/g;y.fn.extend({val:function(u){var c,p,m,S=this[0];return arguments.length?(m=B(u),this.each(function(_){var D;this.nodeType===1&&(m?D=u.call(this,_,y(this).val()):D=u,D==null?D="":typeof D=="number"?D+="":Array.isArray(D)&&(D=y.map(D,function(O){return O==null?"":O+""})),c=y.valHooks[this.type]||y.valHooks[this.nodeName.toLowerCase()],(!c||!("set"in c)||c.set(this,D,"value")===void 0)&&(this.value=D))})):S?(c=y.valHooks[S.type]||y.valHooks[S.nodeName.toLowerCase()],c&&"get"in c&&(p=c.get(S,"value"))!==void 0?p:(p=S.value,typeof p=="string"?p.replace(Vr,""):p??"")):void 0}}),y.extend({valHooks:{option:{get:function(u){var c=y.find.attr(u,"value");return c??za(y.text(u))}},select:{get:function(u){var c,p,m,S=u.options,_=u.selectedIndex,D=u.type==="select-one",O=D?null:[],w=D?_+1:S.length;for(_<0?m=w:m=D?_:0;m<w;m++)if(p=S[m],(p.selected||m===_)&&!p.disabled&&(!p.parentNode.disabled||!Ce(p.parentNode,"optgroup"))){if(c=y(p).val(),D)return c;O.push(c)}return O},set:function(u,c){for(var p,m,S=u.options,_=y.makeArray(c),D=S.length;D--;)m=S[D],(m.selected=y.inArray(y.valHooks.option.get(m),_)>-1)&&(p=!0);return p||(u.selectedIndex=-1),_}}}}),y.each(["radio","checkbox"],function(){y.valHooks[this]={set:function(u,c){if(Array.isArray(c))return u.checked=y.inArray(y(u).val(),c)>-1}},U.checkOn||(y.valHooks[this].get=function(u){return u.getAttribute("value")===null?"on":u.value})});var Fl=i.location,tf={guid:Date.now()},fl=/\?/;y.parseXML=function(u){var c,p;if(!u||typeof u!="string")return null;try{c=new i.DOMParser().parseFromString(u,"text/xml")}catch{}return p=c&&c.getElementsByTagName("parsererror")[0],(!c||p)&&y.error("Invalid XML: "+(p?y.map(p.childNodes,function(m){return m.textContent}).join(`
`):u)),c};var Ln=/^(?:focusinfocus|focusoutblur)$/,Bi=function(u){u.stopPropagation()};y.extend(y.event,{trigger:function(u,c,p,m){var S,_,D,O,w,R,W,te,Z=[p||F],le=N.call(u,"type")?u.type:u,De=N.call(u,"namespace")?u.namespace.split("."):[];if(_=te=D=p=p||F,!(p.nodeType===3||p.nodeType===8)&&!Ln.test(le+y.event.triggered)&&(le.indexOf(".")>-1&&(De=le.split("."),le=De.shift(),De.sort()),w=le.indexOf(":")<0&&"on"+le,u=u[y.expando]?u:new y.Event(le,typeof u=="object"&&u),u.isTrigger=m?2:3,u.namespace=De.join("."),u.rnamespace=u.namespace?new RegExp("(^|\\.)"+De.join("\\.(?:.*\\.|)")+"(\\.|$)"):null,u.result=void 0,u.target||(u.target=p),c=c==null?[u]:y.makeArray(c,[u]),W=y.event.special[le]||{},!(!m&&W.trigger&&W.trigger.apply(p,c)===!1))){if(!m&&!W.noBubble&&!Y(p)){for(O=W.delegateType||le,Ln.test(O+le)||(_=_.parentNode);_;_=_.parentNode)Z.push(_),D=_;D===(p.ownerDocument||F)&&Z.push(D.defaultView||D.parentWindow||i)}for(S=0;(_=Z[S++])&&!u.isPropagationStopped();)te=_,u.type=S>1?O:W.bindType||le,R=(ve.get(_,"events")||Object.create(null))[u.type]&&ve.get(_,"handle"),R&&R.apply(_,c),R=w&&_[w],R&&R.apply&&Vl(_)&&(u.result=R.apply(_,c),u.result===!1&&u.preventDefault());return u.type=le,!m&&!u.isDefaultPrevented()&&(!W._default||W._default.apply(Z.pop(),c)===!1)&&Vl(p)&&w&&B(p[le])&&!Y(p)&&(D=p[w],D&&(p[w]=null),y.event.triggered=le,u.isPropagationStopped()&&te.addEventListener(le,Bi),p[le](),u.isPropagationStopped()&&te.removeEventListener(le,Bi),y.event.triggered=void 0,D&&(p[w]=D)),u.result}},simulate:function(u,c,p){var m=y.extend(new y.Event,p,{type:u,isSimulated:!0});y.event.trigger(m,null,c)}}),y.fn.extend({trigger:function(u,c){return this.each(function(){y.event.trigger(u,c,this)})},triggerHandler:function(u,c){var p=this[0];if(p)return y.event.trigger(u,c,p,!0)}});var Qr=/\[\]$/,La=/\r?\n/g,ol=/^(?:submit|button|image|reset|file)$/i,nf=/^(?:input|select|textarea|keygen)/i;function Wl(u,c,p,m){var S;if(Array.isArray(c))y.each(c,function(_,D){p||Qr.test(u)?m(u,D):Wl(u+"["+(typeof D=="object"&&D!=null?_:"")+"]",D,p,m)});else if(!p&&Te(c)==="object")for(S in c)Wl(u+"["+S+"]",c[S],p,m);else m(u,c)}y.param=function(u,c){var p,m=[],S=function(_,D){var O=B(D)?D():D;m[m.length]=encodeURIComponent(_)+"="+encodeURIComponent(O??"")};if(u==null)return"";if(Array.isArray(u)||u.jquery&&!y.isPlainObject(u))y.each(u,function(){S(this.name,this.value)});else for(p in u)Wl(p,u[p],c,S);return m.join("&")},y.fn.extend({serialize:function(){return y.param(this.serializeArray())},serializeArray:function(){return this.map(function(){var u=y.prop(this,"elements");return u?y.makeArray(u):this}).filter(function(){var u=this.type;return this.name&&!y(this).is(":disabled")&&nf.test(this.nodeName)&&!ol.test(u)&&(this.checked||!da.test(u))}).map(function(u,c){var p=y(this).val();return p==null?null:Array.isArray(p)?y.map(p,function(m){return{name:c.name,value:m.replace(La,`\r
`)}}):{name:c.name,value:p.replace(La,`\r
`)}}).get()}});var af=/%20/g,$l=/#.*$/,ta=/([?&])_=[^&]*/,Zr=/^(.*?):[ \t]*([^\r\n]*)$/mg,Il=/^(?:about|app|app-storage|.+-extension|file|res|widget):$/,ma=/^(?:GET|HEAD)$/,Kr=/^\/\//,Pl={},Yi={},ei="*/".concat("*"),cl=F.createElement("a");cl.href=Fl.href;function Jr(u){return function(c,p){typeof c!="string"&&(p=c,c="*");var m,S=0,_=c.toLowerCase().match(dn)||[];if(B(p))for(;m=_[S++];)m[0]==="+"?(m=m.slice(1)||"*",(u[m]=u[m]||[]).unshift(p)):(u[m]=u[m]||[]).push(p)}}function Vt(u,c,p,m){var S={},_=u===Yi;function D(O){var w;return S[O]=!0,y.each(u[O]||[],function(R,W){var te=W(c,p,m);if(typeof te=="string"&&!_&&!S[te])return c.dataTypes.unshift(te),D(te),!1;if(_)return!(w=te)}),w}return D(c.dataTypes[0])||!S["*"]&&D("*")}function na(u,c){var p,m,S=y.ajaxSettings.flatOptions||{};for(p in c)c[p]!==void 0&&((S[p]?u:m||(m={}))[p]=c[p]);return m&&y.extend(!0,u,m),u}function Xi(u,c,p){for(var m,S,_,D,O=u.contents,w=u.dataTypes;w[0]==="*";)w.shift(),m===void 0&&(m=u.mimeType||c.getResponseHeader("Content-Type"));if(m){for(S in O)if(O[S]&&O[S].test(m)){w.unshift(S);break}}if(w[0]in p)_=w[0];else{for(S in p){if(!w[0]||u.converters[S+" "+w[0]]){_=S;break}D||(D=S)}_=_||D}if(_)return _!==w[0]&&w.unshift(_),p[_]}function ti(u,c,p,m){var S,_,D,O,w,R={},W=u.dataTypes.slice();if(W[1])for(D in u.converters)R[D.toLowerCase()]=u.converters[D];for(_=W.shift();_;)if(u.responseFields[_]&&(p[u.responseFields[_]]=c),!w&&m&&u.dataFilter&&(c=u.dataFilter(c,u.dataType)),w=_,_=W.shift(),_){if(_==="*")_=w;else if(w!=="*"&&w!==_){if(D=R[w+" "+_]||R["* "+_],!D){for(S in R)if(O=S.split(" "),O[1]===_&&(D=R[w+" "+O[0]]||R["* "+O[0]],D)){D===!0?D=R[S]:R[S]!==!0&&(_=O[0],W.unshift(O[1]));break}}if(D!==!0)if(D&&u.throws)c=D(c);else try{c=D(c)}catch(te){return{state:"parsererror",error:D?te:"No conversion from "+w+" to "+_}}}}return{state:"success",data:c}}y.extend({active:0,lastModified:{},etag:{},ajaxSettings:{url:Fl.href,type:"GET",isLocal:Il.test(Fl.protocol),global:!0,processData:!0,async:!0,contentType:"application/x-www-form-urlencoded; charset=UTF-8",accepts:{"*":ei,text:"text/plain",html:"text/html",xml:"application/xml, text/xml",json:"application/json, text/javascript"},contents:{xml:/\bxml\b/,html:/\bhtml/,json:/\bjson\b/},responseFields:{xml:"responseXML",text:"responseText",json:"responseJSON"},converters:{"* text":String,"text html":!0,"text json":JSON.parse,"text xml":y.parseXML},flatOptions:{url:!0,context:!0}},ajaxSetup:function(u,c){return c?na(na(u,y.ajaxSettings),c):na(y.ajaxSettings,u)},ajaxPrefilter:Jr(Pl),ajaxTransport:Jr(Yi),ajax:function(u,c){typeof u=="object"&&(c=u,u=void 0),c=c||{};var p,m,S,_,D,O,w,R,W,te,Z=y.ajaxSetup({},c),le=Z.context||Z,De=Z.context&&(le.nodeType||le.jquery)?y(le):y.event,Ue=y.Deferred(),Ee=y.Callbacks("once memory"),Ct=Z.statusCode||{},_t={},Hn={},jn="canceled",Le={readyState:0,getResponseHeader:function(Ge){var ht;if(w){if(!_)for(_={};ht=Zr.exec(S);)_[ht[1].toLowerCase()+" "]=(_[ht[1].toLowerCase()+" "]||[]).concat(ht[2]);ht=_[Ge.toLowerCase()+" "]}return ht==null?null:ht.join(", ")},getAllResponseHeaders:function(){return w?S:null},setRequestHeader:function(Ge,ht){return w==null&&(Ge=Hn[Ge.toLowerCase()]=Hn[Ge.toLowerCase()]||Ge,_t[Ge]=ht),this},overrideMimeType:function(Ge){return w==null&&(Z.mimeType=Ge),this},statusCode:function(Ge){var ht;if(Ge)if(w)Le.always(Ge[Le.status]);else for(ht in Ge)Ct[ht]=[Ct[ht],Ge[ht]];return this},abort:function(Ge){var ht=Ge||jn;return p&&p.abort(ht),Ra(0,ht),this}};if(Ue.promise(Le),Z.url=((u||Z.url||Fl.href)+"").replace(Kr,Fl.protocol+"//"),Z.type=c.method||c.type||Z.method||Z.type,Z.dataTypes=(Z.dataType||"*").toLowerCase().match(dn)||[""],Z.crossDomain==null){O=F.createElement("a");try{O.href=Z.url,O.href=O.href,Z.crossDomain=cl.protocol+"//"+cl.host!=O.protocol+"//"+O.host}catch{Z.crossDomain=!0}}if(Z.data&&Z.processData&&typeof Z.data!="string"&&(Z.data=y.param(Z.data,Z.traditional)),Vt(Pl,Z,c,Le),w)return Le;R=y.event&&Z.global,R&&y.active++===0&&y.event.trigger("ajaxStart"),Z.type=Z.type.toUpperCase(),Z.hasContent=!ma.test(Z.type),m=Z.url.replace($l,""),Z.hasContent?Z.data&&Z.processData&&(Z.contentType||"").indexOf("application/x-www-form-urlencoded")===0&&(Z.data=Z.data.replace(af,"+")):(te=Z.url.slice(m.length),Z.data&&(Z.processData||typeof Z.data=="string")&&(m+=(fl.test(m)?"&":"?")+Z.data,delete Z.data),Z.cache===!1&&(m=m.replace(ta,"$1"),te=(fl.test(m)?"&":"?")+"_="+tf.guid+++te),Z.url=m+te),Z.ifModified&&(y.lastModified[m]&&Le.setRequestHeader("If-Modified-Since",y.lastModified[m]),y.etag[m]&&Le.setRequestHeader("If-None-Match",y.etag[m])),(Z.data&&Z.hasContent&&Z.contentType!==!1||c.contentType)&&Le.setRequestHeader("Content-Type",Z.contentType),Le.setRequestHeader("Accept",Z.dataTypes[0]&&Z.accepts[Z.dataTypes[0]]?Z.accepts[Z.dataTypes[0]]+(Z.dataTypes[0]!=="*"?", "+ei+"; q=0.01":""):Z.accepts["*"]);for(W in Z.headers)Le.setRequestHeader(W,Z.headers[W]);if(Z.beforeSend&&(Z.beforeSend.call(le,Le,Z)===!1||w))return Le.abort();if(jn="abort",Ee.add(Z.complete),Le.done(Z.success),Le.fail(Z.error),p=Vt(Yi,Z,c,Le),!p)Ra(-1,"No Transport");else{if(Le.readyState=1,R&&De.trigger("ajaxSend",[Le,Z]),w)return Le;Z.async&&Z.timeout>0&&(D=i.setTimeout(function(){Le.abort("timeout")},Z.timeout));try{w=!1,p.send(_t,Ra)}catch(Ge){if(w)throw Ge;Ra(-1,Ge)}}function Ra(Ge,ht,ni,kr){var hn,aa,Rn,la,ia,fn=ht;w||(w=!0,D&&i.clearTimeout(D),p=void 0,S=kr||"",Le.readyState=Ge>0?4:0,hn=Ge>=200&&Ge<300||Ge===304,ni&&(la=Xi(Z,Le,ni)),!hn&&y.inArray("script",Z.dataTypes)>-1&&y.inArray("json",Z.dataTypes)<0&&(Z.converters["text script"]=function(){}),la=ti(Z,la,Le,hn),hn?(Z.ifModified&&(ia=Le.getResponseHeader("Last-Modified"),ia&&(y.lastModified[m]=ia),ia=Le.getResponseHeader("etag"),ia&&(y.etag[m]=ia)),Ge===204||Z.type==="HEAD"?fn="nocontent":Ge===304?fn="notmodified":(fn=la.state,aa=la.data,Rn=la.error,hn=!Rn)):(Rn=fn,(Ge||!fn)&&(fn="error",Ge<0&&(Ge=0))),Le.status=Ge,Le.statusText=(ht||fn)+"",hn?Ue.resolveWith(le,[aa,fn,Le]):Ue.rejectWith(le,[Le,fn,Rn]),Le.statusCode(Ct),Ct=void 0,R&&De.trigger(hn?"ajaxSuccess":"ajaxError",[Le,Z,hn?aa:Rn]),Ee.fireWith(le,[Le,fn]),R&&(De.trigger("ajaxComplete",[Le,Z]),--y.active||y.event.trigger("ajaxStop")))}return Le},getJSON:function(u,c,p){return y.get(u,c,p,"json")},getScript:function(u,c){return y.get(u,void 0,c,"script")}}),y.each(["get","post"],function(u,c){y[c]=function(p,m,S,_){return B(m)&&(_=_||S,S=m,m=void 0),y.ajax(y.extend({url:p,type:c,dataType:_,data:m,success:S},y.isPlainObject(p)&&p))}}),y.ajaxPrefilter(function(u){var c;for(c in u.headers)c.toLowerCase()==="content-type"&&(u.contentType=u.headers[c]||"")}),y._evalUrl=function(u,c,p){return y.ajax({url:u,type:"GET",dataType:"script",cache:!0,async:!1,global:!1,converters:{"text script":function(){}},dataFilter:function(m){y.globalEval(m,c,p)}})},y.fn.extend({wrapAll:function(u){var c;return this[0]&&(B(u)&&(u=u.call(this[0])),c=y(u,this[0].ownerDocument).eq(0).clone(!0),this[0].parentNode&&c.insertBefore(this[0]),c.map(function(){for(var p=this;p.firstElementChild;)p=p.firstElementChild;return p}).append(this)),this},wrapInner:function(u){return B(u)?this.each(function(c){y(this).wrapInner(u.call(this,c))}):this.each(function(){var c=y(this),p=c.contents();p.length?p.wrapAll(u):c.append(u)})},wrap:function(u){var c=B(u);return this.each(function(p){y(this).wrapAll(c?u.call(this,p):u)})},unwrap:function(u){return this.parent(u).not("body").each(function(){y(this).replaceWith(this.childNodes)}),this}}),y.expr.pseudos.hidden=function(u){return!y.expr.pseudos.visible(u)},y.expr.pseudos.visible=function(u){return!!(u.offsetWidth||u.offsetHeight||u.getClientRects().length)},y.ajaxSettings.xhr=function(){try{return new i.XMLHttpRequest}catch{}};var qo={0:200,1223:204},Ha=y.ajaxSettings.xhr();U.cors=!!Ha&&"withCredentials"in Ha,U.ajax=Ha=!!Ha,y.ajaxTransport(function(u){var c,p;if(U.cors||Ha&&!u.crossDomain)return{send:function(m,S){var _,D=u.xhr();if(D.open(u.type,u.url,u.async,u.username,u.password),u.xhrFields)for(_ in u.xhrFields)D[_]=u.xhrFields[_];u.mimeType&&D.overrideMimeType&&D.overrideMimeType(u.mimeType),!u.crossDomain&&!m["X-Requested-With"]&&(m["X-Requested-With"]="XMLHttpRequest");for(_ in m)D.setRequestHeader(_,m[_]);c=function(O){return function(){c&&(c=p=D.onload=D.onerror=D.onabort=D.ontimeout=D.onreadystatechange=null,O==="abort"?D.abort():O==="error"?typeof D.status!="number"?S(0,"error"):S(D.status,D.statusText):S(qo[D.status]||D.status,D.statusText,(D.responseType||"text")!=="text"||typeof D.responseText!="string"?{binary:D.response}:{text:D.responseText},D.getAllResponseHeaders()))}},D.onload=c(),p=D.onerror=D.ontimeout=c("error"),D.onabort!==void 0?D.onabort=p:D.onreadystatechange=function(){D.readyState===4&&i.setTimeout(function(){c&&p()})},c=c("abort");try{D.send(u.hasContent&&u.data||null)}catch(O){if(c)throw O}},abort:function(){c&&c()}}}),y.ajaxPrefilter(function(u){u.crossDomain&&(u.contents.script=!1)}),y.ajaxSetup({accepts:{script:"text/javascript, application/javascript, application/ecmascript, application/x-ecmascript"},contents:{script:/\b(?:java|ecma)script\b/},converters:{"text script":function(u){return y.globalEval(u),u}}}),y.ajaxPrefilter("script",function(u){u.cache===void 0&&(u.cache=!1),u.crossDomain&&(u.type="GET")}),y.ajaxTransport("script",function(u){if(u.crossDomain||u.scriptAttrs){var c,p;return{send:function(m,S){c=y("<script>").attr(u.scriptAttrs||{}).prop({charset:u.scriptCharset,src:u.url}).on("load error",p=function(_){c.remove(),p=null,_&&S(_.type==="error"?404:200,_.type)}),F.head.appendChild(c[0])},abort:function(){p&&p()}}}});var Gi=[],ja=/(=)\?(?=&|$)|\?\?/;y.ajaxSetup({jsonp:"callback",jsonpCallback:function(){var u=Gi.pop()||y.expando+"_"+tf.guid++;return this[u]=!0,u}}),y.ajaxPrefilter("json jsonp",function(u,c,p){var m,S,_,D=u.jsonp!==!1&&(ja.test(u.url)?"url":typeof u.data=="string"&&(u.contentType||"").indexOf("application/x-www-form-urlencoded")===0&&ja.test(u.data)&&"data");if(D||u.dataTypes[0]==="jsonp")return m=u.jsonpCallback=B(u.jsonpCallback)?u.jsonpCallback():u.jsonpCallback,D?u[D]=u[D].replace(ja,"$1"+m):u.jsonp!==!1&&(u.url+=(fl.test(u.url)?"&":"?")+u.jsonp+"="+m),u.converters["script json"]=function(){return _||y.error(m+" was not called"),_[0]},u.dataTypes[0]="json",S=i[m],i[m]=function(){_=arguments},p.always(function(){S===void 0?y(i).removeProp(m):i[m]=S,u[m]&&(u.jsonpCallback=c.jsonpCallback,Gi.push(m)),_&&B(S)&&S(_[0]),_=S=void 0}),"script"}),U.createHTMLDocument=(function(){var u=F.implementation.createHTMLDocument("").body;return u.innerHTML="<form></form><form></form>",u.childNodes.length===2})(),y.parseHTML=function(u,c,p){if(typeof u!="string")return[];typeof c=="boolean"&&(p=c,c=!1);var m,S,_;return c||(U.createHTMLDocument?(c=F.implementation.createHTMLDocument(""),m=c.createElement("base"),m.href=F.location.href,c.head.appendChild(m)):c=F),S=ot.exec(u),_=!p&&[],S?[c.createElement(S[1])]:(S=el([u],c,_),_&&_.length&&y(_).remove(),y.merge([],S.childNodes))},y.fn.load=function(u,c,p){var m,S,_,D=this,O=u.indexOf(" ");return O>-1&&(m=za(u.slice(O)),u=u.slice(0,O)),B(c)?(p=c,c=void 0):c&&typeof c=="object"&&(S="POST"),D.length>0&&y.ajax({url:u,type:S||"GET",dataType:"html",data:c}).done(function(w){_=arguments,D.html(m?y("<div>").append(y.parseHTML(w)).find(m):w)}).always(p&&function(w,R){D.each(function(){p.apply(this,_||[w.responseText,R,w])})}),this},y.expr.pseudos.animated=function(u){return y.grep(y.timers,function(c){return u===c.elem}).length},y.offset={setOffset:function(u,c,p){var m,S,_,D,O,w,R,W=y.css(u,"position"),te=y(u),Z={};W==="static"&&(u.style.position="relative"),O=te.offset(),_=y.css(u,"top"),w=y.css(u,"left"),R=(W==="absolute"||W==="fixed")&&(_+w).indexOf("auto")>-1,R?(m=te.position(),D=m.top,S=m.left):(D=parseFloat(_)||0,S=parseFloat(w)||0),B(c)&&(c=c.call(u,p,y.extend({},O))),c.top!=null&&(Z.top=c.top-O.top+D),c.left!=null&&(Z.left=c.left-O.left+S),"using"in c?c.using.call(u,Z):te.css(Z)}},y.fn.extend({offset:function(u){if(arguments.length)return u===void 0?this:this.each(function(S){y.offset.setOffset(this,u,S)});var c,p,m=this[0];if(m)return m.getClientRects().length?(c=m.getBoundingClientRect(),p=m.ownerDocument.defaultView,{top:c.top+p.pageYOffset,left:c.left+p.pageXOffset}):{top:0,left:0}},position:function(){if(this[0]){var u,c,p,m=this[0],S={top:0,left:0};if(y.css(m,"position")==="fixed")c=m.getBoundingClientRect();else{for(c=this.offset(),p=m.ownerDocument,u=m.offsetParent||p.documentElement;u&&(u===p.body||u===p.documentElement)&&y.css(u,"position")==="static";)u=u.parentNode;u&&u!==m&&u.nodeType===1&&(S=y(u).offset(),S.top+=y.css(u,"borderTopWidth",!0),S.left+=y.css(u,"borderLeftWidth",!0))}return{top:c.top-S.top-y.css(m,"marginTop",!0),left:c.left-S.left-y.css(m,"marginLeft",!0)}}},offsetParent:function(){return this.map(function(){for(var u=this.offsetParent;u&&y.css(u,"position")==="static";)u=u.offsetParent;return u||gt})}}),y.each({scrollLeft:"pageXOffset",scrollTop:"pageYOffset"},function(u,c){var p=c==="pageYOffset";y.fn[u]=function(m){return On(this,function(S,_,D){var O;if(Y(S)?O=S:S.nodeType===9&&(O=S.defaultView),D===void 0)return O?O[c]:S[_];O?O.scrollTo(p?O.pageXOffset:D,p?D:O.pageYOffset):S[_]=D},u,m,arguments.length)}}),y.each(["top","left"],function(u,c){y.cssHooks[c]=Ma(U.pixelPosition,function(p,m){if(m)return m=ea(p,c),ji.test(m)?y(p).position()[c]+"px":m})}),y.each({Height:"height",Width:"width"},function(u,c){y.each({padding:"inner"+u,content:c,"":"outer"+u},function(p,m){y.fn[m]=function(S,_){var D=arguments.length&&(p||typeof S!="boolean"),O=p||(S===!0||_===!0?"margin":"border");return On(this,function(w,R,W){var te;return Y(w)?m.indexOf("outer")===0?w["inner"+u]:w.document.documentElement["client"+u]:w.nodeType===9?(te=w.documentElement,Math.max(w.body["scroll"+u],te["scroll"+u],w.body["offset"+u],te["offset"+u],te["client"+u])):W===void 0?y.css(w,R,O):y.style(w,R,W,O)},c,D?S:void 0,D)}})}),y.each(["ajaxStart","ajaxStop","ajaxComplete","ajaxError","ajaxSuccess","ajaxSend"],function(u,c){y.fn[c]=function(p){return this.on(c,p)}}),y.fn.extend({bind:function(u,c,p){return this.on(u,null,c,p)},unbind:function(u,c){return this.off(u,null,c)},delegate:function(u,c,p,m){return this.on(c,u,p,m)},undelegate:function(u,c,p){return arguments.length===1?this.off(u,"**"):this.off(c,u||"**",p)},hover:function(u,c){return this.on("mouseenter",u).on("mouseleave",c||u)}}),y.each("blur focus focusin focusout resize scroll click dblclick mousedown mouseup mousemove mouseover mouseout mouseenter mouseleave change select submit keydown keypress keyup contextmenu".split(" "),function(u,c){y.fn[c]=function(p,m){return arguments.length>0?this.on(c,null,p,m):this.trigger(c)}});var Vi=/^[\s\uFEFF\xA0]+|([^\s\uFEFF\xA0])[\s\uFEFF\xA0]+$/g;y.proxy=function(u,c){var p,m,S;if(typeof c=="string"&&(p=u[c],c=u,u=p),!!B(u))return m=h.call(arguments,2),S=function(){return u.apply(c||this,m.concat(h.call(arguments)))},S.guid=u.guid=u.guid||y.guid++,S},y.holdReady=function(u){u?y.readyWait++:y.ready(!0)},y.isArray=Array.isArray,y.parseJSON=JSON.parse,y.nodeName=Ce,y.isFunction=B,y.isWindow=Y,y.camelCase=zn,y.type=Te,y.now=Date.now,y.isNumeric=function(u){var c=y.type(u);return(c==="number"||c==="string")&&!isNaN(u-parseFloat(u))},y.trim=function(u){return u==null?"":(u+"").replace(Vi,"$1")};var lf=i.jQuery,Uo=i.$;return y.noConflict=function(u){return i.$===y&&(i.$=Uo),u&&i.jQuery===y&&(i.jQuery=lf),y},typeof r>"u"&&(i.jQuery=i.$=y),y})})(so)),so.exports}var m0=v0();const nd=i0(m0);/*! DataTables 2.3.4
 * © SpryMedia Ltd - datatables.net/license
 */var M=nd,$=function(a,i){if($.factory(a,i))return $;if(this instanceof $)return M(a).DataTable(i);i=a;var r=this,f=i===void 0,d=this.length;return f&&(i={}),this.api=function(){return new Re(this)},this.each(function(){var h={},v=d>1?Xv(h,i,!0):i,g=0,x,T=this.getAttribute("id"),A=$.defaults,N=M(this);if(this.nodeName.toLowerCase()!="table"){Wn(null,0,"Non-table node initialisation ("+this.nodeName+")",2);return}v.on&&v.on.options&&Gv(N,"options",v.on.options),N.trigger("options.dt",v),Lv(A),lm(A.column),ql(A,A,!0),ql(A.column,A.column,!0),ql(A,M.extend(v,_m(N.data())),!0);var j=$.settings;for(g=0,x=j.length;g<x;g++){var X=j[g];if(X.nTable==this||X.nTHead&&X.nTHead.parentNode==this||X.nTFoot&&X.nTFoot.parentNode==this){var U=v.bRetrieve!==void 0?v.bRetrieve:A.bRetrieve,B=v.bDestroy!==void 0?v.bDestroy:A.bDestroy;if(f||U)return X.oInstance;if(B){new $.Api(X).destroy();break}else{Wn(X,0,"Cannot reinitialise DataTable",3);return}}if(X.sTableId==this.id){j.splice(g,1);break}}(T===null||T==="")&&(T="DataTables_Table_"+$.ext._unique++,this.id=T),N.children("colgroup").remove();var Y=M.extend(!0,{},$.models.oSettings,{sDestroyWidth:N[0].style.width,sInstance:T,sTableId:T,colgroup:M("<colgroup>").prependTo(this),fastData:function($e,I,ue){return $n(Y,$e,I,ue)}});Y.nTable=this,Y.oInit=v,j.push(Y),Y.api=new Re(Y),Y.oInstance=r.length===1?r:N.dataTable(),Lv(v),v.aLengthMenu&&!v.iDisplayLength&&(v.iDisplayLength=Array.isArray(v.aLengthMenu[0])?v.aLengthMenu[0][0]:M.isPlainObject(v.aLengthMenu[0])?v.aLengthMenu[0].value:v.aLengthMenu[0]),v=Xv(M.extend(!0,{},A),v),Bl(Y.oFeatures,v,["bPaginate","bLengthChange","bFilter","bSort","bSortMulti","bInfo","bProcessing","bAutoWidth","bSortClasses","bServerSide","bDeferRender"]),Bl(Y,v,["ajax","fnFormatNumber","sServerMethod","aaSorting","aaSortingFixed","aLengthMenu","sPaginationType","iStateDuration","bSortCellsTop","iTabIndex","sDom","fnStateLoadCallback","fnStateSaveCallback","renderer","searchDelay","rowId","caption","layout","orderDescReverse","orderIndicators","orderHandler","titleRow","typeDetect",["iCookieDuration","iStateDuration"],["oSearch","oPreviousSearch"],["aoSearchCols","aoPreSearchCols"],["iDisplayLength","_iDisplayLength"]]),Bl(Y.oScroll,v,[["sScrollX","sX"],["sScrollXInner","sXInner"],["sScrollY","sY"],["bScrollCollapse","bCollapse"]]),Bl(Y.oLanguage,v,"fnInfoCallback"),Fn(Y,"aoDrawCallback",v.fnDrawCallback),Fn(Y,"aoStateSaveParams",v.fnStateSaveParams),Fn(Y,"aoStateLoadParams",v.fnStateLoadParams),Fn(Y,"aoStateLoaded",v.fnStateLoaded),Fn(Y,"aoRowCallback",v.fnRowCallback),Fn(Y,"aoRowCreatedCallback",v.fnCreatedRow),Fn(Y,"aoHeaderCallback",v.fnHeaderCallback),Fn(Y,"aoFooterCallback",v.fnFooterCallback),Fn(Y,"aoInitComplete",v.fnInitComplete),Fn(Y,"aoPreDrawCallback",v.fnPreDrawCallback),Y.rowIdFn=Cr(v.rowId),v.on&&Object.keys(v.on).forEach(function($e){Gv(N,$e,v.on[$e])}),x0(Y);var F=Y.oClasses;M.extend(F,$.ext.classes,v.oClasses),N.addClass(F.table),Y.oFeatures.bPaginate||(v.iDisplayStart=0),Y.iInitDisplayStart===void 0&&(Y.iInitDisplayStart=v.iDisplayStart,Y._iDisplayStart=v.iDisplayStart);var re=v.iDeferLoading;if(re!==null){Y.deferLoading=!0;var Se=Array.isArray(re);Y._iRecordsDisplay=Se?re[0]:re,Y._iRecordsTotal=Se?re[1]:re}var Te=[],Oe=this.getElementsByTagName("thead"),ge=sm(Y,Oe[0]);if(v.aoColumns)Te=v.aoColumns;else if(ge.length)for(g=0,x=ge[0].length;g<x;g++)Te.push(null);for(g=0,x=Te.length;g<x;g++)im(Y);C0(Y,v.aoColumnDefs,Te,ge,function($e,I){Vs(Y,$e,I)});var y=N.children("tbody").find("tr:first-child").eq(0);if(y.length){var Ze=function($e,I){return $e.getAttribute("data-"+I)!==null?I:null};M(y[0]).children("th, td").each(function($e,I){var ue=Y.aoColumns[$e];if(ue||Wn(Y,0,"Incorrect column count",18),ue.mData===$e){var me=Ze(I,"sort")||Ze(I,"order"),qe=Ze(I,"filter")||Ze(I,"search");(me!==null||qe!==null)&&(ue.mData={_:$e+".display",sort:me!==null?$e+".@data-"+me:void 0,type:me!==null?$e+".@data-"+me:void 0,filter:qe!==null?$e+".@data-"+qe:void 0},ue._isArrayHost=!0,Vs(Y,$e))}})}Fn(Y,"aoDrawCallback",No);var Ce=Y.oFeatures;if(v.bStateSave&&(Ce.bStateSave=!0),v.aaSorting===void 0){var Tt=Y.aaSorting;for(g=0,x=Tt.length;g<x;g++)Tt[g][1]=Y.aoColumns[g].asSorting[0]}Ks(Y),Fn(Y,"aoDrawCallback",function(){(Y.bSorted||Mn(Y)==="ssp"||Ce.bDeferRender)&&Ks(Y)});var vt=N.children("caption");Y.caption&&(vt.length===0&&(vt=M("<caption/>").appendTo(N)),vt.html(Y.caption)),vt.length&&(vt[0]._captionSide=vt.css("caption-side"),Y.captionNode=vt[0]),Oe.length===0&&(Oe=M("<thead/>").appendTo(N)),Y.nTHead=Oe[0];var Xe=N.children("tbody");Xe.length===0&&(Xe=M("<tbody/>").insertAfter(Oe)),Y.nTBody=Xe[0];var Me=N.children("tfoot");Me.length===0&&(Me=M("<tfoot/>").appendTo(N)),Y.nTFoot=Me[0],Y.aiDisplay=Y.aiDisplayMaster.slice(),Y.bInitialised=!0;var $t=Y.oLanguage;M.extend(!0,$t,v.oLanguage),$t.sUrl?M.ajax({dataType:"json",url:$t.sUrl,success:function($e){ql(A.oLanguage,$e),M.extend(!0,$t,$e,Y.oInit.oLanguage),We(Y,null,"i18n",[Y],!0),po(Y)},error:function(){Wn(Y,0,"i18n file loading error",21),po(Y)}}):(We(Y,null,"i18n",[Y],!0),po(Y))}),r=null,this};$.ext=Yt={builder:"-source-",buttons:{},ccContent:{},classes:{},errMode:"alert",escape:{attributes:!1},feature:[],features:{},search:[],selector:{cell:[],column:[],row:[]},legacy:{ajax:null},pager:{},renderer:{pageButton:{},header:{}},order:{},type:{className:{},detect:[],render:{},search:{},order:{}},_unique:0,fnVersionCheck:$.fnVersionCheck,iApiIndex:0,sVersion:$.version};M.extend(Yt,{afnFiltering:Yt.search,aTypes:Yt.type.detect,ofnSearch:Yt.type.search,oSort:Yt.type.order,afnSortData:Yt.order,aoFeatures:Yt.feature,oStdClasses:Yt.classes,oPagination:Yt.pager});M.extend($.ext.classes,{container:"dt-container",empty:{row:"dt-empty"},info:{container:"dt-info"},layout:{row:"dt-layout-row",cell:"dt-layout-cell",tableRow:"dt-layout-table",tableCell:"",start:"dt-layout-start",end:"dt-layout-end",full:"dt-layout-full"},length:{container:"dt-length",select:"dt-input"},order:{canAsc:"dt-orderable-asc",canDesc:"dt-orderable-desc",isAsc:"dt-ordering-asc",isDesc:"dt-ordering-desc",none:"dt-orderable-none",position:"sorting_"},processing:{container:"dt-processing"},scrolling:{body:"dt-scroll-body",container:"dt-scroll",footer:{self:"dt-scroll-foot",inner:"dt-scroll-footInner"},header:{self:"dt-scroll-head",inner:"dt-scroll-headInner"}},search:{container:"dt-search",input:"dt-input"},table:"dataTable",tbody:{cell:"",row:""},thead:{cell:"",row:""},tfoot:{cell:"",row:""},paging:{active:"current",button:"dt-paging-button",container:"dt-paging",disabled:"disabled",nav:""}});var Yt,Re,he,at,Hs={},y0=/[\r\n\u2028]/g,ad=/<([^>]*>)/g,b0=Math.pow(2,28),zv=/^\d{2,4}[./-]\d{1,2}[./-]\d{1,2}([T ]{1}\d{1,2}[:.]\d{2}([.:]\d{2})?)?$/,g0=new RegExp("(\\"+["/",".","*","+","?","|","(",")","[","]","{","}","\\","$","^","-"].join("|\\")+")","g"),ld=/['\u00A0,$£€¥%\u2009\u202F\u20BD\u20a9\u20BArfkɃΞ]/gi,En=function(a){return!a||a===!0||a==="-"},em=function(a){var i=parseInt(a,10);return!isNaN(i)&&isFinite(a)?i:null},tm=function(a,i){return Hs[i]||(Hs[i]=new RegExp(Zs(i),"g")),typeof a=="string"&&i!=="."?a.replace(/\./g,"").replace(Hs[i],"."):a},Ru=function(a,i,r,f){var d=typeof a,h=d==="string";return d==="number"||d==="bigint"||f&&En(a)?!0:(i&&h&&(a=tm(a,i)),r&&h&&(a=a.replace(ld,"")),!isNaN(parseFloat(a))&&isFinite(a))},S0=function(a){return En(a)||typeof a=="string"},vo=function(a,i,r,f){if(f&&En(a))return!0;if(typeof a=="string"&&a.match(/<(input|select)/i))return null;var d=S0(a);return d&&Ru(Pa(a),i,r,f)?!0:null},tn=function(a,i,r){var f=[],d=0,h=a.length;if(r!==void 0)for(;d<h;d++)a[d]&&a[d][i]&&f.push(a[d][i][r]);else for(;d<h;d++)a[d]&&f.push(a[d][i]);return f},Yu=function(a,i,r,f){var d=[],h=0,v=i.length;if(f!==void 0)for(;h<v;h++)a[i[h]]&&a[i[h]][r]&&d.push(a[i[h]][r][f]);else for(;h<v;h++)a[i[h]]&&d.push(a[i[h]][r]);return d},Rl=function(a,i){var r=[],f;i===void 0?(i=0,f=a):(f=i,i=a);for(var d=i;d<f;d++)r.push(d);return r},nm=function(a){for(var i=[],r=0,f=a.length;r<f;r++)a[r]&&i.push(a[r]);return i},Pa=function(a){if(!a||typeof a!="string")return a;if(a.length>b0)throw new Error("Exceeded max str len");var i;a=a.replace(ad,"");do i=a,a=a.replace(/<script/i,"");while(a!==i);return i},xi=function(a){return Array.isArray(a)&&(a=a.join(",")),typeof a=="string"?a.replace(/&/g,"&amp;").replace(/</g,"&lt;").replace(/>/g,"&gt;").replace(/"/g,"&quot;"):a},mo=function(a,i){if(typeof a!="string")return a;var r=a.normalize?a.normalize("NFD"):a;return r.length!==a.length?(i===!0?a+" ":"")+r.replace(/[\u0300-\u036f]/g,""):r},_0=function(a){if(a.length<2)return!0;for(var i=a.slice().sort(),r=i[0],f=1,d=i.length;f<d;f++){if(i[f]===r)return!1;r=i[f]}return!0},Er=function(a){if(Array.from&&Set)return Array.from(new Set(a));if(_0(a))return a.slice();var i=[],r,f,d=a.length,h,v=0;e:for(f=0;f<d;f++){for(r=a[f],h=0;h<v;h++)if(i[h]===r)continue e;i.push(r),v++}return i},am=function(a,i){if(Array.isArray(i))for(var r=0;r<i.length;r++)am(a,i[r]);else a.push(i);return a};function Dr(a,i){i&&i.split(" ").forEach(function(r){r&&a.classList.add(r)})}$.util={diacritics:function(a,i){var r=typeof a;if(r!=="function")return mo(a,i);mo=a},debounce:function(a,i){var r;return function(){var f=this,d=arguments;clearTimeout(r),r=setTimeout(function(){a.apply(f,d)},i||250)}},throttle:function(a,i){var r=i!==void 0?i:200,f,d;return function(){var h=this,v=+new Date,g=arguments;f&&v<f+r?(clearTimeout(d),d=setTimeout(function(){f=void 0,a.apply(h,g)},r)):(f=v,a.apply(h,g))}},escapeRegex:function(a){return a.replace(g0,"\\$1")},set:function(a){if(M.isPlainObject(a))return $.util.set(a._);if(a===null)return function(){};if(typeof a=="function")return function(r,f,d){a(r,"set",f,d)};if(typeof a=="string"&&(a.indexOf(".")!==-1||a.indexOf("[")!==-1||a.indexOf("(")!==-1)){var i=function(r,f,d){for(var h=jv(d),v,g=h[h.length-1],x,T,A,N,j=0,X=h.length-1;j<X;j++){if(h[j]==="__proto__"||h[j]==="constructor")throw new Error("Cannot set prototype values");if(x=h[j].match(Lu),T=h[j].match(_r),x){if(h[j]=h[j].replace(Lu,""),r[h[j]]=[],v=h.slice(),v.splice(0,j+1),N=v.join("."),Array.isArray(f))for(var U=0,B=f.length;U<B;U++)A={},i(A,f[U],N),r[h[j]].push(A);else r[h[j]]=f;return}else T&&(h[j]=h[j].replace(_r,""),r=r[h[j]](f));(r[h[j]]===null||r[h[j]]===void 0)&&(r[h[j]]={}),r=r[h[j]]}g.match(_r)?r=r[g.replace(_r,"")](f):r[g.replace(Lu,"")]=f};return function(r,f){return i(r,f,a)}}else return function(r,f){r[a]=f}},get:function(a){if(M.isPlainObject(a)){var i={};return M.each(a,function(f,d){d&&(i[f]=$.util.get(d))}),function(f,d,h,v){var g=i[d]||i._;return g!==void 0?g(f,d,h,v):f}}else{if(a===null)return function(f){return f};if(typeof a=="function")return function(f,d,h,v){return a(f,d,h,v)};if(typeof a=="string"&&(a.indexOf(".")!==-1||a.indexOf("[")!==-1||a.indexOf("(")!==-1)){var r=function(f,d,h){var v,g,x,T;if(h!=="")for(var A=jv(h),N=0,j=A.length;N<j;N++){if(v=A[N].match(Lu),g=A[N].match(_r),v){if(A[N]=A[N].replace(Lu,""),A[N]!==""&&(f=f[A[N]]),x=[],A.splice(0,N+1),T=A.join("."),Array.isArray(f))for(var X=0,U=f.length;X<U;X++)x.push(r(f[X],d,T));var B=v[0].substring(1,v[0].length-1);f=B===""?x:x.join(B);break}else if(g){A[N]=A[N].replace(_r,""),f=f[A[N]]();continue}if(f===null||f[A[N]]===null)return null;if(f===void 0||f[A[N]]===void 0)return;f=f[A[N]]}return f};return function(f,d){return r(f,d,a)}}else return function(f){return f[a]}}},stripHtml:function(a){var i=typeof a;if(i==="function"){Pa=a;return}else if(i==="string")return Pa(a);return a},escapeHtml:function(a){var i=typeof a;if(i==="function"){xi=a;return}else if(i==="string"||Array.isArray(a))return xi(a);return a},unique:Er};function xo(a){var i="a aa ai ao as b fn i m o s ",r,f,d={};M.each(a,function(h){r=h.match(/^([^A-Z]+?)([A-Z])/),r&&i.indexOf(r[1]+" ")!==-1&&(f=h.replace(r[0],r[2].toLowerCase()),d[f]=h,r[1]==="o"&&xo(a[h]))}),a._hungarianMap=d}function ql(a,i,r){a._hungarianMap||xo(a);var f;M.each(i,function(d){f=a._hungarianMap[d],f!==void 0&&(r||i[f]===void 0)&&(f.charAt(0)==="o"?(i[f]||(i[f]={}),M.extend(!0,i[f],i[d]),ql(a[f],i[f],r)):i[f]=i[d])})}var Sn=function(a,i,r){a[i]!==void 0&&(a[r]=a[i])};function Lv(a){Sn(a,"ordering","bSort"),Sn(a,"orderMulti","bSortMulti"),Sn(a,"orderClasses","bSortClasses"),Sn(a,"orderCellsTop","bSortCellsTop"),Sn(a,"order","aaSorting"),Sn(a,"orderFixed","aaSortingFixed"),Sn(a,"paging","bPaginate"),Sn(a,"pagingType","sPaginationType"),Sn(a,"pageLength","iDisplayLength"),Sn(a,"searching","bFilter"),typeof a.sScrollX=="boolean"&&(a.sScrollX=a.sScrollX?"100%":""),typeof a.scrollX=="boolean"&&(a.scrollX=a.scrollX?"100%":""),typeof a.bSort=="object"?(a.orderIndicators=a.bSort.indicators!==void 0?a.bSort.indicators:!0,a.orderHandler=a.bSort.handler!==void 0?a.bSort.handler:!0,a.bSort=!0):a.bSort===!1?(a.orderIndicators=!1,a.orderHandler=!1):a.bSort===!0&&(a.orderIndicators=!0,a.orderHandler=!0),typeof a.bSortCellsTop=="boolean"&&(a.titleRow=a.bSortCellsTop);var i=a.aoSearchCols;if(i)for(var r=0,f=i.length;r<f;r++)i[r]&&ql($.models.oSearch,i[r]);a.serverSide&&!a.searchDelay&&(a.searchDelay=400)}function lm(a){Sn(a,"orderable","bSortable"),Sn(a,"orderData","aDataSort"),Sn(a,"orderSequence","asSorting"),Sn(a,"orderDataType","sortDataType");var i=a.aDataSort;typeof i=="number"&&!Array.isArray(i)&&(a.aDataSort=[i])}function x0(a){if(!$.__browser){var i={};$.__browser=i;var r=M("<div/>").css({position:"fixed",top:0,left:-1*window.pageXOffset,height:1,width:1,overflow:"hidden"}).append(M("<div/>").css({position:"absolute",top:1,left:1,width:100,overflow:"scroll"}).append(M("<div/>").css({width:"100%",height:10}))).appendTo("body"),f=r.children(),d=f.children();i.barWidth=f[0].offsetWidth-f[0].clientWidth,i.bScrollbarLeft=Math.round(d.offset().left)!==1,r.remove()}M.extend(a.oBrowser,$.__browser),a.oScroll.iBarWidth=$.__browser.barWidth}function im(a){var i=$.defaults.column,r=a.aoColumns.length,f=M.extend({},$.models.oColumn,i,{aDataSort:i.aDataSort?i.aDataSort:[r],mData:i.mData?i.mData:r,idx:r,searchFixed:{},colEl:M("<col>").attr("data-dt-column",r)});a.aoColumns.push(f);var d=a.aoPreSearchCols;d[r]=M.extend({},$.models.oSearch,d[r])}function Vs(a,i,r){var f=a.aoColumns[i];if(r!=null){lm(r),ql($.defaults.column,r,!0),r.mDataProp!==void 0&&!r.mData&&(r.mData=r.mDataProp),r.sType&&(f._sManualType=r.sType),r.className&&!r.sClass&&(r.sClass=r.className);var d=f.sClass;M.extend(f,r),Bl(f,r,"sWidth","sWidthOrig"),d!==f.sClass&&(f.sClass=d+" "+f.sClass),r.iDataSort!==void 0&&(f.aDataSort=[r.iDataSort]),Bl(f,r,"aDataSort")}var h=f.mData,v=Cr(h);if(f.mRender&&Array.isArray(f.mRender)){var g=f.mRender.slice(),x=g.shift();f.mRender=$.render[x].apply(window,g)}f._render=f.mRender?Cr(f.mRender):null;var T=function(A){return typeof A=="string"&&A.indexOf("@")!==-1};f._bAttrSrc=M.isPlainObject(h)&&(T(h.sort)||T(h.type)||T(h.filter)),f._setter=null,f.fnGetData=function(A,N,j){var X=v(A,N,void 0,j);return f._render&&N?f._render(X,N,A,j):X},f.fnSetData=function(A,N,j){return Si(h)(A,N,j)},typeof h!="number"&&!f._isArrayHost&&(a._rowReadObject=!0),a.oFeatures.bSort||(f.bSortable=!1)}function Do(a){Y0(a),D0(a);var i=a.oScroll;(i.sY!==""||i.sX!=="")&&vm(a),We(a,null,"column-sizing",[a])}function D0(a){for(var i=a.aoColumns,r=0;r<i.length;r++){var f=rm(a,[r],!1);i[r].colEl.css("width",f),a.oScroll.sX&&i[r].colEl.css("min-width",f)}}function id(a,i){var r=rd(a,"bVisible");return typeof r[i]=="number"?r[i]:null}function Xu(a,i){var r=rd(a,"bVisible"),f=r.indexOf(i);return f!==-1?f:null}function To(a){var i=a.aoHeader,r=a.aoColumns,f=0;if(i.length)for(var d=0,h=i[0].length;d<h;d++)r[d].bVisible&&M(i[0][d].cell).css("display")!=="none"&&f++;return f}function rd(a,i){var r=[];return a.aoColumns.map(function(f,d){f[i]&&r.push(d)}),r}function js(a,i){return i===!0?a._name:i}function Co(a){var i=a.aoColumns,r=a.aoData,f=$.ext.type.detect,d,h,v,g,x,T,A,N,j;for(d=0,h=i.length;d<h;d++){if(A=i[d],j=[],!A.sType&&A._sManualType)A.sType=A._sManualType;else if(!A.sType){if(!a.typeDetect)return;for(v=0,g=f.length;v<g;v++){var X=f[v],U=X.oneOf,B=X.allOf||X,Y=X.init,F=!1;if(N=null,Y&&(N=js(X,Y(a,A,d)),N)){A.sType=N;break}for(x=0,T=r.length;x<T&&!(r[x]&&(j[x]===void 0&&(j[x]=$n(a,x,d,"type")),U&&!F&&(F=js(X,U(j[x],a))),N=js(X,B(j[x],a)),!N&&v!==f.length-3||N==="html"&&!En(j[x])));x++);if(U&&F&&N||!U&&N){A.sType=N;break}}A.sType||(A.sType="string")}var re=Yt.type.className[A.sType];re&&(Hv(a.aoHeader,d,re),Hv(a.aoFooter,d,re));var Se=Yt.type.render[A.sType];Se&&!A._render&&(A._render=$.util.get(Se),T0(a,d))}}function T0(a,i){for(var r=a.aoData,f=0;f<r.length;f++)if(r[f].nTr){var d=$n(a,f,i,"display");r[f].displayData[i]=d,yo(r[f].anCells[i],d)}}function Hv(a,i,r){a.forEach(function(f){f[i]&&f[i].unique&&Dr(f[i].cell,r)})}function C0(a,i,r,f,d){var h,v,g,x,T,A,N,j=a.aoColumns;if(r)for(h=0,v=r.length;h<v;h++)r[h]&&r[h].name&&(j[h].sName=r[h].name);if(i)for(h=i.length-1;h>=0;h--){N=i[h];var X=N.target!==void 0?N.target:N.targets!==void 0?N.targets:N.aTargets;for(Array.isArray(X)||(X=[X]),g=0,x=X.length;g<x;g++){var U=X[g];if(typeof U=="number"&&U>=0){for(;j.length<=U;)im(a);d(U,N)}else if(typeof U=="number"&&U<0)d(j.length+U,N);else if(typeof U=="string")for(T=0,A=j.length;T<A;T++)U==="_all"?d(T,N):U.indexOf(":name")!==-1?j[T].sName===U.replace(":name","")&&d(T,N):f.forEach(function(B){if(B[T]){var Y=M(B[T].cell);U.match(/^[a-z][\w-]*$/i)&&(U="."+U),Y.is(U)&&d(T,N)}})}}if(r)for(h=0,v=r.length;h<v;h++)d(h,r[h])}function rm(a,i,r,f){Array.isArray(i)||(i=ud(i));for(var d=0,h,v=a.aoColumns,g=0,x=i.length;g<x;g++){var T=v[i[g]],A=r?T.sWidthOrig:T.sWidth;if(T.bVisible!==!1){if(A==null)return null;if(typeof A=="number")h="px",d+=A;else{var N=A.match(/([\d\.]+)([^\d]*)/);N&&(d+=N[1]*1,h=N.length===3?N[2]:"px")}}}return d+h}function ud(a){var i=M(a).closest("[data-dt-column]").attr("data-dt-column");return i?i.split(",").map(function(r){return r*1}):[]}function Di(a,i,r,f){var d=a.aoData.length,h=M.extend(!0,{},$.models.oRow,{src:r?"dom":"data",idx:d});h._aData=i,a.aoData.push(h);for(var v=a.aoColumns,g=0,x=v.length;g<x;g++)v[g].sType=null;a.aiDisplayMaster.push(d);var T=a.rowIdFn(i);return T!==void 0&&(a.aIds[T]=h),(r||!a.oFeatures.bDeferRender)&&fm(a,d,r,f),d}function fd(a,i){var r;return i instanceof M||(i=M(i)),i.map(function(f,d){return r=um(a,d),Di(a,r.data,d,r.cells)})}function $n(a,i,r,f){f==="search"?f="filter":f==="order"&&(f="sort");var d=a.aoData[i];if(d){var h=a.iDraw,v=a.aoColumns[r],g=d._aData,x=v.sDefaultContent,T=v.fnGetData(g,f,{settings:a,row:i,col:r});if(f!=="display"&&T&&typeof T=="object"&&T.nodeName&&(T=T.innerHTML),T===void 0)return a.iDrawError!=h&&x===null&&(Wn(a,0,"Requested unknown parameter "+(typeof v.mData=="function"?"{function}":"'"+v.mData+"'")+" for row "+i+", column "+r,4),a.iDrawError=h),x;if((T===g||T===null)&&x!==null&&f!==void 0)T=x;else if(typeof T=="function")return T.call(g);if(T===null&&f==="display")return"";if(f==="filter"){var A=$.ext.type.search;A[v.sType]&&(T=A[v.sType](T))}return T}}function A0(a,i,r,f){var d=a.aoColumns[r],h=a.aoData[i]._aData;d.fnSetData(h,f,{settings:a,row:i,col:r})}function yo(a,i){i&&typeof i=="object"&&i.nodeName?M(a).empty().append(i):a.innerHTML=i}var Lu=/\[.*?\]$/,_r=/\(\)$/;function jv(a){var i=a.match(/(\\.|[^.])+/g)||[""];return i.map(function(r){return r.replace(/\\\./g,".")})}var Cr=$.util.get,Si=$.util.set;function Rv(a){return tn(a.aoData,"_aData")}function od(a){a.aoData.length=0,a.aiDisplayMaster.length=0,a.aiDisplay.length=0,a.aIds={}}function Ao(a,i,r,f){var d=a.aoData[i],h,v;if(d._aSortData=null,d._aFilterData=null,d.displayData=null,r==="dom"||(!r||r==="auto")&&d.src==="dom")d._aData=um(a,d,f,f===void 0?void 0:d._aData).data;else{var g=d.anCells,x=cd(a,i);if(g)if(f!==void 0)yo(g[f],x[f]);else for(h=0,v=g.length;h<v;h++)yo(g[h],x[h])}var T=a.aoColumns;if(f!==void 0)T[f].sType=null,T[f].maxLenString=null;else{for(h=0,v=T.length;h<v;h++)T[h].sType=null,T[h].maxLenString=null;om(a,d)}}function um(a,i,r,f){var d=[],h=i.firstChild,v,g,x=0,T,A=a.aoColumns,N=a._rowReadObject;f=f!==void 0?f:N?{}:[];var j=function(re,Se){if(typeof re=="string"){var Te=re.indexOf("@");if(Te!==-1){var Oe=re.substring(Te+1),ge=Si(re);ge(f,Se.getAttribute(Oe))}}},X=function(re){if(r===void 0||r===x)if(g=A[x],T=re.innerHTML.trim(),g&&g._bAttrSrc){var Se=Si(g.mData._);Se(f,T),j(g.mData.sort,re),j(g.mData.type,re),j(g.mData.filter,re)}else N?(g._setter||(g._setter=Si(g.mData)),g._setter(f,T)):f[x]=T;x++};if(h)for(;h;)v=h.nodeName.toUpperCase(),(v=="TD"||v=="TH")&&(X(h),d.push(h)),h=h.nextSibling;else{d=i.anCells;for(var U=0,B=d.length;U<B;U++)X(d[U])}var Y=i.firstChild?i:i.nTr;if(Y){var F=Y.getAttribute("id");F&&Si(a.rowId)(f,F)}return{data:f,cells:d}}function cd(a,i){var r=a.aoData[i],f=a.aoColumns;if(!r.displayData){r.displayData=[];for(var d=0,h=f.length;d<h;d++)r.displayData.push($n(a,i,d,"display"))}return r.displayData}function fm(a,i,r,f){var d=a.aoData[i],h=d._aData,v=[],g,x,T,A,N,j,X=a.oClasses.tbody.row;if(d.nTr===null){for(g=r||document.createElement("tr"),d.nTr=g,d.anCells=v,Dr(g,X),g._DT_RowIndex=i,om(a,d),A=0,N=a.aoColumns.length;A<N;A++){T=a.aoColumns[A],j=!(r&&f[A]),x=j?document.createElement(T.sCellType):f[A],x||Wn(a,0,"Incorrect column count",18),x._DT_CellIndex={row:i,column:A},v.push(x);var U=cd(a,i);(j||(T.mRender||T.mData!==A)&&(!M.isPlainObject(T.mData)||T.mData._!==A+".display"))&&yo(x,U[A]),Dr(x,T.sClass),T.bVisible&&j?g.appendChild(x):!T.bVisible&&!j&&x.parentNode.removeChild(x),T.fnCreatedCell&&T.fnCreatedCell.call(a.oInstance,x,$n(a,i,A),h,i,A)}We(a,"aoRowCreatedCallback","row-created",[g,h,i,v])}else Dr(d.nTr,X)}function om(a,i){var r=i.nTr,f=i._aData;if(r){var d=a.rowIdFn(f);if(d&&(r.id=d),f.DT_RowClass){var h=f.DT_RowClass.split(" ");i.__rowc=i.__rowc?Er(i.__rowc.concat(h)):h,M(r).removeClass(i.__rowc.join(" ")).addClass(f.DT_RowClass)}f.DT_RowAttr&&M(r).attr(f.DT_RowAttr),f.DT_RowData&&M(r).data(f.DT_RowData)}}function qv(a,i){var r=a.oClasses,f=a.aoColumns,d,h,v,g=i==="header"?a.nTHead:a.nTFoot,x=i==="header"?"sTitle":i;if(g){if((i==="header"||tn(a.aoColumns,x).join(""))&&(v=M("tr",g),v.length||(v=M("<tr/>").appendTo(g)),v.length===1)){var T=0;for(M("td, th",v).each(function(){T+=this.colSpan}),d=T,h=f.length;d<h;d++)M("<th/>").html(f[d][x]||"").appendTo(v)}var A=sm(a,g,!0);i==="header"?(a.aoHeader=A,M("tr",g).addClass(r.thead.row)):(a.aoFooter=A,M("tr",g).addClass(r.tfoot.row)),M(g).children("tr").children("th, td").each(function(){So(a,i)(a,M(this),r)})}}function cm(a,i,r){var f,d,h,v=[],g=[],x=a.aoColumns,T=x.length,A,N;if(i){for(r||(r=Rl(T).filter(function(U){return x[U].bVisible})),f=0;f<i.length;f++)v[f]=i[f].slice().filter(function(U,B){return r.includes(B)}),g.push([]);for(f=0;f<v.length;f++)for(d=0;d<v[f].length;d++)if(A=1,N=1,g[f][d]===void 0){for(h=v[f][d].cell;v[f+A]!==void 0&&v[f][d].cell==v[f+A][d].cell;)g[f+A][d]=null,A++;for(;v[f][d+N]!==void 0&&v[f][d].cell==v[f][d+N].cell;){for(var j=0;j<A;j++)g[f+j][d+N]=null;N++}var X=M("span.dt-column-title",h);g[f][d]={cell:h,colspan:N,rowspan:A,title:X.length?X.html():M(h).html()}}return g}}function bo(a,i){for(var r=cm(a,i),f,d,h=0;h<i.length;h++){if(f=i[h].row,f)for(;d=f.firstChild;)f.removeChild(d);for(var v=0;v<r[h].length;v++){var g=r[h][v];g&&M(g.cell).appendTo(f).attr("rowspan",g.rowspan).attr("colspan",g.colspan)}}}function Nr(a,i){O0(a);var r=We(a,"aoPreDrawCallback","preDraw",[a]);if(r.indexOf(!1)!==-1){Nn(a,!1);return}var f=[],d=0,h=Mn(a)=="ssp",v=a.aiDisplay,g=a._iDisplayStart,x=a.fnDisplayEnd(),T=a.aoColumns,A=M(a.nTBody);if(a.bDrawing=!0,a.deferLoading)a.deferLoading=!1,a.iDraw++,Nn(a,!1);else if(!h)a.iDraw++;else if(!a.bDestroying&&!i){a.iDraw===0&&A.empty().append(Uv(a)),z0(a);return}if(v.length!==0)for(var N=h?0:g,j=h?a.aoData.length:x,X=N;X<j;X++){var U=v[X],B=a.aoData[U];B.nTr===null&&fm(a,U);for(var Y=B.nTr,F=0;F<T.length;F++){var re=T[F],Se=B.anCells[F];Dr(Se,Yt.type.className[re.sType]),Dr(Se,a.oClasses.tbody.cell)}We(a,"aoRowCallback",null,[Y,B._aData,d,X,U]),f.push(Y),d++}else f[0]=Uv(a);We(a,"aoHeaderCallback","header",[M(a.nTHead).children("tr")[0],Rv(a),g,x,v]),We(a,"aoFooterCallback","footer",[M(a.nTFoot).children("tr")[0],Rv(a),g,x,v]),A[0].replaceChildren?A[0].replaceChildren.apply(A[0],f):(A.children().detach(),A.append(M(f))),M(a.nTableWrapper).toggleClass("dt-empty-footer",M("tr",a.nTFoot).length===0),We(a,"aoDrawCallback","draw",[a],!0),a.bSorted=!1,a.bFiltered=!1,a.bDrawing=!1}function Ar(a,i,r){var f=a.oFeatures,d=f.bSort,h=f.bFilter;(r===void 0||r===!0)&&(Co(a),d&&hd(a),h?Uu(a,a.oPreviousSearch):a.aiDisplay=a.aiDisplayMaster.slice()),i!==!0&&(a._iDisplayStart=0),a._drawHold=i,Nr(a),a.api.one("draw",function(){a._drawHold=!1})}function Uv(a){var i=a.oLanguage,r=i.sZeroRecords,f=Mn(a);return(f==="ssp"||f==="ajax")&&!a.json?r=i.sLoadingRecords:i.sEmptyTable&&a.fnRecordsTotal()===0&&(r=i.sEmptyTable),M("<tr/>").append(M("<td />",{colSpan:To(a),class:a.oClasses.empty.row}).html(r))[0]}function Qs(a,i,r){if(Array.isArray(r)){for(var f=0;f<r.length;f++)Qs(a,i,r[f]);return}var d=a[i];M.isPlainObject(r)?r.features?(r.rowId&&(a.id=r.rowId),r.rowClass&&(a.className=r.rowClass),d.id=r.id,d.className=r.className,Qs(a,i,r.features)):Object.keys(r).map(function(h){d.contents.push({feature:h,opts:r[h]})}):d.contents.push(r)}function E0(a,i,r){for(var f,d=0;d<a.length;d++)if(f=a[d],f.rowNum===i&&(r==="full"&&f.full||(r==="start"||r==="end")&&(f.start||f.end)))return f[r]||(f[r]={contents:[]}),f;return f={rowNum:i},f[r]={contents:[]},a.push(f),f}function Bv(a,i,r){var f=[];M.each(i,function(h,v){if(v!==null){var g=h.match(/^([a-z]+)([0-9]*)([A-Za-z]*)$/),x=g[2]?g[2]*1:0,T=g[3]?g[3].toLowerCase():"full";if(g[1]===r){var A=E0(f,x,T);Qs(A,T,v)}}}),f.sort(function(h,v){var g=h.rowNum,x=v.rowNum;if(g===x){var T=h.full&&!v.full?-1:1;return r==="bottom"?T*-1:T}return x-g}),r==="bottom"&&f.reverse();for(var d=0;d<f.length;d++)delete f[d].rowNum,N0(a,f[d]);return f}function N0(a,i){var r=function(d,h){return Yt.features[d]||Wn(a,0,"Unknown feature: "+d),Yt.features[d].apply(this,[a,h])},f=function(d){if(i[d])for(var h=i[d].contents,v=0,g=h.length;v<g;v++)if(h[v]){if(typeof h[v]=="string")h[v]=r(h[v],null);else if(M.isPlainObject(h[v]))h[v]=r(h[v].feature,h[v].opts);else if(typeof h[v].node=="function")h[v]=h[v].node(a);else if(typeof h[v]=="function"){var x=h[v](a);h[v]=typeof x.node=="function"?x.node():x}}else continue};f("start"),f("end"),f("full")}function M0(a){var i=a.oClasses,r=M(a.nTable),f=M("<div/>").attr({id:a.sTableId+"_wrapper",class:i.container}).insertBefore(r);if(a.nTableWrapper=f[0],a.sDom)w0(a,a.sDom,f);else{var d=Bv(a,a.layout,"top"),h=Bv(a,a.layout,"bottom"),v=So(a,"layout");d.forEach(function(g){v(a,f,g)}),v(a,f,{full:{table:!0,contents:[pm(a)]}}),h.forEach(function(g){v(a,f,g)})}B0(a)}function w0(a,i,r){for(var f=i.match(/(".*?")|('.*?')|./g),d,h,v,g,x,T=0;T<f.length;T++){if(d=null,h=f[T],h=="<"){if(v=M("<div/>"),g=f[T+1],g[0]=="'"||g[0]=='"'){x=g.replace(/['"]/g,"");var A="",N;if(x.indexOf(".")!=-1){var j=x.split(".");A=j[0],N=j[1]}else x[0]=="#"?A=x:N=x;v.attr("id",A.substring(1)).addClass(N),T++}r.append(v),r=v}else h==">"?r=r.parent():h=="t"?d=pm(a):$.ext.feature.forEach(function(X){h==X.cFeature&&(d=X.fnInit(a))});d&&r.append(d)}}function sm(a,i,r){var f=a.aoColumns,d=M(i).children("tr"),h,v,g,x,T,A,N,j,X,U,B=a.titleRow,Y=i&&i.nodeName.toLowerCase()==="thead",F=[],re,Se=function(Tt,vt,Xe){for(var Me=Tt[vt];Me[Xe];)Xe++;return Xe};for(g=0,A=d.length;g<A;g++)F.push([]);for(g=0,A=d.length;g<A;g++)for(h=d[g],j=0,v=h.firstChild;v;){if(v.nodeName.toUpperCase()=="TD"||v.nodeName.toUpperCase()=="TH"){var Te=[],Oe=M(v);if(X=v.getAttribute("colspan")*1,U=v.getAttribute("rowspan")*1,X=!X||X===0||X===1?1:X,U=!U||U===0||U===1?1:U,N=Se(F,g,j),re=X===1,r){if(re){Vs(a,N,_m(Oe.data()));var ge=f[N],y=v.getAttribute("width")||null,Ze=v.style.width.match(/width:\s*(\d+[pxem%]+)/);Ze&&(y=Ze[1]),ge.sWidthOrig=ge.sWidth||y,Y?(ge.sTitle!==null&&!ge.autoTitle&&(B===!0&&g===0||B===!1&&g===d.length-1||B===g||B===null)&&(v.innerHTML=ge.sTitle),!ge.sTitle&&re&&(ge.sTitle=Pa(v.innerHTML),ge.autoTitle=!0)):ge.footer&&(v.innerHTML=ge.footer),ge.ariaTitle||(ge.ariaTitle=Oe.attr("aria-label")||ge.sTitle),ge.className&&Oe.addClass(ge.className)}M("span.dt-column-title",v).length===0&&M("<span>").addClass("dt-column-title").append(v.childNodes).appendTo(v),a.orderIndicators&&Y&&Oe.filter(":not([data-dt-order=disable])").length!==0&&Oe.parent(":not([data-dt-order=disable])").length!==0&&M("span.dt-column-order",v).length===0&&M("<span>").addClass("dt-column-order").appendTo(v);var Ce=Y?"header":"footer";M("span.dt-column-"+Ce,v).length===0&&M("<div>").addClass("dt-column-"+Ce).append(v.childNodes).appendTo(v)}for(T=0;T<X;T++){for(x=0;x<U;x++)F[g+x][N+T]={cell:v,unique:re},F[g+x].row=h;Te.push(N+T)}v.setAttribute("data-dt-column",Er(Te).join(","))}v=v.nextSibling}return F}function O0(a){var i=Mn(a)=="ssp",r=a.iInitDisplayStart;r!==void 0&&r!==-1&&(a._iDisplayStart=i?r:r>=a.fnRecordsDisplay()?0:r,a.iInitDisplayStart=-1)}function sd(a,i,r){var f,d=a.ajax,h=a.oInstance,v=function(A){var N=a.jqXHR?a.jqXHR.status:null;(A===null||typeof N=="number"&&N==204)&&(A={},qu(a,A,[]));var j=A.error||A.sError;if(j&&Wn(a,0,j),A.d&&typeof A.d=="string")try{A=JSON.parse(A.d)}catch{}a.json=A,We(a,null,"xhr",[a,A,a.jqXHR],!0),r(A)};if(M.isPlainObject(d)&&d.data){f=d.data;var g=typeof f=="function"?f(i,a):f;i=typeof f=="function"&&g?g:M.extend(!0,i,g),delete d.data}var x={url:typeof d=="string"?d:"",data:i,success:v,dataType:"json",cache:!1,type:a.sServerMethod,error:function(A,N){var j=We(a,null,"xhr",[a,null,a.jqXHR],!0);j.indexOf(!0)===-1&&(N=="parsererror"?Wn(a,0,"Invalid JSON response",1):A.readyState===4&&Wn(a,0,"Ajax error",7)),Nn(a,!1)}};if(M.isPlainObject(d)&&M.extend(x,d),a.oAjaxData=i,We(a,null,"preXhr",[a,i,x],!0),x.submitAs==="json"&&typeof i=="object"&&(x.data=JSON.stringify(i)),typeof d=="function")a.jqXHR=d.call(h,i,v,a);else if(d.url===""){var T={};qu(a,T,[]),v(T)}else a.jqXHR=M.ajax(x);f&&(d.data=f)}function z0(a){a.iDraw++,Nn(a,!0),sd(a,L0(a),function(i){H0(a,i)})}function L0(a){var i=a.aoColumns,r=a.oFeatures,f=a.oPreviousSearch,d=a.aoPreSearchCols,h=function(v,g){return typeof i[v][g]=="function"?"function":i[v][g]};return{draw:a.iDraw,columns:i.map(function(v,g){return{data:h(g,"mData"),name:v.sName,searchable:v.bSearchable,orderable:v.bSortable,search:{value:d[g].search,regex:d[g].regex,fixed:Object.keys(v.searchFixed).map(function(x){return{name:x,term:typeof v.searchFixed[x]!="function"?v.searchFixed[x].toString():"function"}})}}}),order:Eo(a).map(function(v){return{column:v.col,dir:v.dir,name:h(v.col,"sName")}}),start:a._iDisplayStart,length:r.bPaginate?a._iDisplayLength:-1,search:{value:f.search,regex:f.regex,fixed:Object.keys(a.searchFixed).map(function(v){return{name:v,term:typeof a.searchFixed[v]!="function"?a.searchFixed[v].toString():"function"}})}}}function H0(a,i){var r=qu(a,i),f=Rs(a,"draw",i),d=Rs(a,"recordsTotal",i),h=Rs(a,"recordsFiltered",i);if(f!==void 0){if(f*1<a.iDraw)return;a.iDraw=f*1}r||(r=[]),od(a),a._iRecordsTotal=parseInt(d,10),a._iRecordsDisplay=parseInt(h,10);for(var v=0,g=r.length;v<g;v++)Di(a,r[v]);a.aiDisplay=a.aiDisplayMaster.slice(),Co(a),Nr(a,!0),go(a),Nn(a,!1)}function qu(a,i,r){var f="data";if(M.isPlainObject(a.ajax)&&a.ajax.dataSrc!==void 0){var d=a.ajax.dataSrc;typeof d=="string"||typeof d=="function"?f=d:d.data!==void 0&&(f=d.data)}if(!r)return f==="data"?i.aaData||i[f]:f!==""?Cr(f)(i):i;Si(f)(i,r)}function Rs(a,i,r){var f=M.isPlainObject(a.ajax)?a.ajax.dataSrc:null;if(f&&f[i])return Cr(f[i])(r);var d="";return i==="draw"?d="sEcho":i==="recordsTotal"?d="iTotalRecords":i==="recordsFiltered"&&(d="iTotalDisplayRecords"),r[d]!==void 0?r[d]:r[i]}function Uu(a,i){var r=a.aoPreSearchCols;if(Mn(a)!="ssp"){U0(a),a.aiDisplay=a.aiDisplayMaster.slice(),fo(a.aiDisplay,a,i.search,i),M.each(a.searchFixed,function(h,v){fo(a.aiDisplay,a,v,{})});for(var f=0;f<r.length;f++){var d=r[f];fo(a.aiDisplay,a,d.search,d,f),M.each(a.aoColumns[f].searchFixed,function(h,v){fo(a.aiDisplay,a,v,{},f)})}j0(a)}a.bFiltered=!0,We(a,null,"search",[a])}function j0(a){for(var i=$.ext.search,r=a.aiDisplay,f,d,h=0,v=i.length;h<v;h++){for(var g=[],x=0,T=r.length;x<T;x++)d=r[x],f=a.aoData[d],i[h](a,f._aFilterData,d,f._aData,x)&&g.push(d);r.length=0,Mo(r,g)}}function fo(a,i,r,f,d){if(r!==""){var h=0,v=[],g=typeof r=="function"?r:null,x=r instanceof RegExp?r:g?null:R0(r,f);for(h=0;h<a.length;h++){var T=i.aoData[a[h]],A=d===void 0?T._sFilterRow:T._aFilterData[d];(g&&g(A,T._aData,a[h],d)||x&&x.test(A))&&v.push(a[h])}for(a.length=v.length,h=0;h<v.length;h++)a[h]=v[h]}}function R0(a,i){var r=[],f=M.extend({},{boundary:!1,caseInsensitive:!0,exact:!1,regex:!1,smart:!0},i);if(typeof a!="string"&&(a=a.toString()),a=mo(a),f.exact)return new RegExp("^"+Zs(a)+"$",f.caseInsensitive?"i":"");if(a=f.regex?a:Zs(a),f.smart){var d=a.match(/!?["\u201C][^"\u201D]+["\u201D]|[^ ]+/g)||[""],h=d.map(function(x){var T=!1,A;return x.charAt(0)==="!"&&(T=!0,x=x.substring(1)),x.charAt(0)==='"'?(A=x.match(/^"(.*)"$/),x=A?A[1]:x):x.charAt(0)==="“"&&(A=x.match(/^\u201C(.*)\u201D$/),x=A?A[1]:x),T&&(x.length>1&&r.push("(?!"+x+")"),x=""),x.replace(/"/g,"")}),v=r.length?r.join(""):"",g=f.boundary?"\\b":"";a="^(?=.*?"+g+h.join(")(?=.*?"+g)+")("+v+".)*$"}return new RegExp(a,f.caseInsensitive?"i":"")}var Zs=$.util.escapeRegex,ho=M("<div>")[0],q0=ho.textContent!==void 0;function U0(a){for(var i=a.aoColumns,r=a.aoData,f,d,h,v,g,x,T=!1,A=0;A<r.length;A++)if(r[A]&&(x=r[A],!x._aFilterData)){for(v=[],d=0,h=i.length;d<h;d++)f=i[d],f.bSearchable?(g=$n(a,A,d,"filter"),g===null&&(g=""),typeof g!="string"&&g.toString&&(g=g.toString())):g="",g.indexOf&&g.indexOf("&")!==-1&&(ho.innerHTML=g,g=q0?ho.textContent:ho.innerText),g.replace&&(g=g.replace(/[\r\n\u2028]/g,"")),v.push(g);x._aFilterData=v,x._sFilterRow=v.join("  "),T=!0}return T}function po(a){var i,r=a.oInit,f=a.deferLoading,d=Mn(a);if(!a.bInitialised){setTimeout(function(){po(a)},200);return}qv(a,"header"),qv(a,"footer"),Z0(a,r,function(){bo(a,a.aoHeader),bo(a,a.aoFooter);var h=a.iInitDisplayStart;if(r.aaData)for(i=0;i<r.aaData.length;i++)Di(a,r.aaData[i]);else(f||d=="dom")&&fd(a,M(a.nTBody).children("tr"));a.aiDisplay=a.aiDisplayMaster.slice(),M0(a),G0(a),mm(a),Nn(a,!0),We(a,null,"preInit",[a],!0),Ar(a),(d!="ssp"||f)&&(d=="ajax"?sd(a,{},function(v){var g=qu(a,v);for(i=0;i<g.length;i++)Di(a,g[i]);a.iInitDisplayStart=h,Ar(a),Nn(a,!1),go(a)}):(go(a),Nn(a,!1)))})}function go(a){if(!a._bInitComplete){var i=[a,a.json];a._bInitComplete=!0,Do(a),We(a,null,"plugin-init",i,!0),We(a,"aoInitComplete","init",i,!0)}}function dm(a,i){var r=parseInt(i,10);a._iDisplayLength=r,Sm(a),We(a,null,"length",[a,r])}function dd(a,i,r){var f=a._iDisplayStart,d=a._iDisplayLength,h=a.fnRecordsDisplay();if(h===0||d===-1)f=0;else if(typeof i=="number")f=i*d,f>h&&(f=0);else if(i=="first")f=0;else if(i=="previous")f=d>=0?f-d:0,f<0&&(f=0);else if(i=="next")f+d<h&&(f+=d);else if(i=="last")f=Math.floor((h-1)/d)*d;else{if(i==="ellipsis")return;Wn(a,0,"Unknown paging action: "+i,5)}var v=a._iDisplayStart!==f;return a._iDisplayStart=f,We(a,null,v?"page":"page-nc",[a]),v&&r&&Nr(a),v}function B0(a){var i=a.nTable,r=a.oScroll.sX!==""||a.oScroll.sY!=="";if(a.oFeatures.bProcessing){var f=M("<div/>",{id:a.sTableId+"_processing",class:a.oClasses.processing.container,role:"status"}).html(a.oLanguage.sProcessing).append("<div><div></div><div></div><div></div><div></div></div>");r?f.prependTo(M("div.dt-scroll",a.nTableWrapper)):f.insertBefore(i),M(i).on("processing.dt.DT",function(d,h,v){f.css("display",v?"block":"none")})}}function Nn(a,i){a.bDrawing&&i===!1||We(a,null,"processing",[a,i])}function hm(a,i,r){i?(Nn(a,!0),setTimeout(function(){r(),Nn(a,!1)},0)):r()}function pm(a){var i=M(a.nTable),r=a.oScroll;if(r.sX===""&&r.sY==="")return a.nTable;var f=r.sX,d=r.sY,h=a.oClasses.scrolling,v=a.captionNode,g=v?v._captionSide:null,x=M(i[0].cloneNode(!1)),T=M(i[0].cloneNode(!1)),A=i.children("tfoot"),N="<div/>",j=function(re){return re?Ul(re):null};A.length||(A=null);var X=M(N,{class:h.container}).append(M(N,{class:h.header.self}).css({overflow:"hidden",position:"relative",border:0,width:f?j(f):"100%"}).append(M(N,{class:h.header.inner}).css({"box-sizing":"content-box",width:r.sXInner||"100%"}).append(x.removeAttr("id").css("margin-left",0).append(g==="top"?v:null).append(i.children("thead"))))).append(M(N,{class:h.body}).css({position:"relative",overflow:"auto",width:j(f)}).append(i));A&&X.append(M(N,{class:h.footer.self}).css({overflow:"hidden",border:0,width:f?j(f):"100%"}).append(M(N,{class:h.footer.inner}).append(T.removeAttr("id").css("margin-left",0).append(g==="bottom"?v:null).append(i.children("tfoot")))));var U=X.children(),B=U[0],Y=U[1],F=A?U[2]:null;return M(Y).on("scroll.DT",function(){var re=this.scrollLeft;B.scrollLeft=re,A&&(F.scrollLeft=re)}),M("th, td",B).on("focus",function(){var re=B.scrollLeft;Y.scrollLeft=re,A&&(Y.scrollLeft=re)}),M(Y).css("max-height",d),r.bCollapse||M(Y).css("height",d),a.nScrollHead=B,a.nScrollBody=Y,a.nScrollFoot=F,a.aoDrawCallback.push(vm),X[0]}function vm(a){var i=a.oScroll,r=i.iBarWidth,f=M(a.nScrollHead),d=f.children("div"),h=d.children("table"),v=a.nScrollBody,g=M(v),x=M(a.nScrollFoot),T=x.children("div"),A=T.children("table"),N=M(a.nTHead),j=M(a.nTable),X=a.nTFoot&&M("th, td",a.nTFoot).length?M(a.nTFoot):null,U=a.oBrowser,B,Y,F=v.scrollHeight>v.clientHeight;if(a.scrollBarVis!==F&&a.scrollBarVis!==void 0){a.scrollBarVis=F,Do(a);return}else a.scrollBarVis=F;if(j.children("thead, tfoot").remove(),B=N.clone().prependTo(j),B.find("th, td").removeAttr("tabindex"),B.find("[id]").removeAttr("id"),X&&(Y=X.clone().prependTo(j),Y.find("[id]").removeAttr("id")),a.aiDisplay.length){var re=null,Se=Mn(a)!=="ssp"?a._iDisplayStart:0;for(y=Se;y<Se+a.aiDisplay.length;y++){var Te=a.aiDisplay[y],Oe=a.aoData[Te].nTr;if(Oe){re=Oe;break}}if(re)for(var ge=M(re).children("th, td").map(function(Me){return{idx:id(a,Me),width:M(this).outerWidth()}}),y=0;y<ge.length;y++){var Ze=a.aoColumns[ge[y].idx].colEl[0],Ce=Ze.style.width.replace("px","");Ce!==ge[y].width&&(Ze.style.width=ge[y].width+"px",i.sX&&(Ze.style.minWidth=ge[y].width+"px"))}}h.find("colgroup").remove(),h.append(a.colgroup.clone()),X&&(A.find("colgroup").remove(),A.append(a.colgroup.clone())),M("th, td",B).each(function(){M(this.childNodes).wrapAll('<div class="dt-scroll-sizing">')}),X&&M("th, td",Y).each(function(){M(this.childNodes).wrapAll('<div class="dt-scroll-sizing">')});var Tt=Math.floor(j.height())>v.clientHeight||g.css("overflow-y")=="scroll",vt="padding"+(U.bScrollbarLeft?"Left":"Right"),Xe=j.outerWidth();h.css("width",Ul(Xe)),d.css("width",Ul(Xe)).css(vt,Tt?r+"px":"0px"),X&&(A.css("width",Ul(Xe)),T.css("width",Ul(Xe)).css(vt,Tt?r+"px":"0px")),j.children("colgroup").prependTo(j),g.trigger("scroll"),(a.bSorted||a.bFiltered)&&!a._drawHold&&(v.scrollTop=0)}function Y0(a){if(a.oFeatures.bAutoWidth){var i=a.nTable,r=a.aoColumns,f=a.oScroll,d=f.sY,h=f.sX,v=f.sXInner,g=rd(a,"bVisible"),x=i.getAttribute("width"),T=i.parentNode,A,N,j,X=i.style.width,U=Yv(a);if(U===a.containerWidth)return!1;a.containerWidth=U,!X&&!x&&(i.style.width="100%",X="100%"),X&&X.indexOf("%")!==-1&&(x=X),We(a,null,"column-calc",{visible:g},!1);var B=M(i.cloneNode()).css("visibility","hidden").removeAttr("id");B.append("<tbody/>");var Y=M("<tr/>").appendTo(B.find("tbody"));for(B.append(M(a.nTHead).clone()).append(M(a.nTFoot).clone()),B.find("tfoot th, tfoot td").css("width",""),B.find("thead th, thead td").each(function(){var Xe=rm(a,this,!0);Xe?(this.style.width=Xe,h&&(this.style.minWidth=Xe,M(this).append(M("<div/>").css({width:Xe,margin:0,padding:0,border:0,height:1})))):this.style.width=""}),A=0;A<g.length;A++){j=g[A],N=r[j];var F=X0(a,j),re=Yt.type.className[N.sType],Se=F+N.sContentPadding,Te=F.indexOf("<")===-1?document.createTextNode(Se):Se;M("<td/>").addClass(re).addClass(N.sClass).append(Te).appendTo(Y)}M("[name]",B).removeAttr("name");var Oe=M("<div/>").css(h||d?{position:"absolute",top:0,left:0,height:1,right:0,overflow:"hidden"}:{}).append(B).appendTo(T);h&&v?B.width(v):h?(B.css("width","auto"),B.removeAttr("width"),B.outerWidth()<T.clientWidth&&x&&B.outerWidth(T.clientWidth)):d?B.outerWidth(T.clientWidth):x&&B.outerWidth(x);var ge=0,y=B.find("tbody tr").eq(0).children();for(A=0;A<g.length;A++){var Ze=y[A].getBoundingClientRect().width;ge+=Ze,r[g[A]].sWidth=Ul(Ze)}if(i.style.width=Ul(ge),Oe.remove(),x&&(i.style.width=Ul(x)),(x||h)&&!a._reszEvt){var Ce=$.util.throttle(function(){var Xe=Yv(a);!a.bDestroying&&Xe!==0&&Do(a)});if(window.ResizeObserver){var Tt=M(a.nTableWrapper).is(":visible"),vt=M("<div>").css({width:"100%",height:0}).addClass("dt-autosize").appendTo(a.nTableWrapper);a.resizeObserver=new ResizeObserver(function(Xe){Tt?Tt=!1:Ce()}),a.resizeObserver.observe(vt[0])}else M(window).on("resize.DT-"+a.sInstance,Ce);a._reszEvt=!0}}}function Yv(a){return M(a.nTableWrapper).is(":visible")?M(a.nTableWrapper).width():0}function X0(a,i){var r=a.aoColumns[i];if(!r.maxLenString){for(var f,d="",h=-1,v=0,g=a.aiDisplayMaster.length;v<g;v++){var x=a.aiDisplayMaster[v],T=cd(a,x)[i],A=T&&typeof T=="object"&&T.nodeType?T.innerHTML:T+"";A=A.replace(/id=".*?"/g,"").replace(/name=".*?"/g,""),f=Pa(A).replace(/&nbsp;/g," "),f.length>h&&(d=A,h=f.length)}r.maxLenString=d}return r.maxLenString}function Ul(a){return a===null?"0px":typeof a=="number"?a<0?"0px":a+"px":a.match(/\d$/)?a+"px":a}function mm(a){var i=a.aoColumns;for(a.colgroup.empty(),jt=0;jt<i.length;jt++)i[jt].bVisible&&a.colgroup.append(i[jt].colEl)}function G0(a){var i=a.nTHead,r=i.querySelectorAll("tr"),f=a.titleRow,d=':not([data-dt-order="disable"]):not([data-dt-order="icon-only"])';f===!0?i=r[0]:f===!1?i=r[r.length-1]:f!==null&&(i=r[f]),a.orderHandler&&ym(a,i,i===a.nTHead?"tr"+d+" th"+d+", tr"+d+" td"+d:"th"+d+", td"+d);var h=[];_i(a,h,a.aaSorting),a.aaSorting=h}function ym(a,i,r,f,d){gm(i,r,function(h){var v=!1,g=f===void 0?ud(h.target):typeof f=="function"?f():Array.isArray(f)?f:[f];if(g.length){for(var x=0,T=g.length;x<T;x++){var A=V0(a,g[x],x,h.shiftKey);if(A!==!1&&(v=!0),a.aaSorting.length===1&&a.aaSorting[0][1]==="")break}v&&hm(a,!0,function(){hd(a),bm(a,a.aiDisplay),Ar(a,!1,!1),d&&d()})}})}function bm(a,i){if(!(i.length<2)){var r=a.aiDisplayMaster,f={},d={},h;for(h=0;h<r.length;h++)f[r[h]]=h;for(h=0;h<i.length;h++)d[i[h]]=f[i[h]];i.sort(function(v,g){return d[v]-d[g]})}}function _i(a,i,r){var f=function(h){if(M.isPlainObject(h)){if(h.idx!==void 0)i.push([h.idx,h.dir]);else if(h.name){var v=tn(a.aoColumns,"sName"),g=v.indexOf(h.name);g!==-1&&i.push([g,h.dir])}}else i.push(h)};if(M.isPlainObject(r))f(r);else if(r.length&&typeof r[0]=="number")f(r);else if(r.length)for(var d=0;d<r.length;d++)f(r[d])}function Eo(a){var i,r,f,d=[],h=$.ext.type.order,v=a.aoColumns,g,x,T,A,N=a.aaSortingFixed,j=M.isPlainObject(N),X=[];if(!a.oFeatures.bSort)return d;for(Array.isArray(N)&&_i(a,X,N),j&&N.pre&&_i(a,X,N.pre),_i(a,X,a.aaSorting),j&&N.post&&_i(a,X,N.post),i=0;i<X.length;i++)if(A=X[i][0],v[A])for(g=v[A].aDataSort,r=0,f=g.length;r<f;r++)x=g[r],T=v[x].sType||"string",X[i]._idx===void 0&&(X[i]._idx=v[x].asSorting.indexOf(X[i][1])),X[i][1]&&d.push({src:A,col:x,dir:X[i][1],index:X[i]._idx,type:T,formatter:h[T+"-pre"],sorter:h[T+"-"+X[i][1]]});return d}function hd(a,i,r){var f,d,h=[],v=$.ext.type.order,g=a.aoData,x,T=a.aiDisplayMaster,A;if(Co(a),i!==void 0){var N=a.aoColumns[i];A=[{src:i,col:i,dir:r,index:0,type:N.sType,formatter:v[N.sType+"-pre"],sorter:v[N.sType+"-"+r]}],T=T.slice()}else A=Eo(a);for(f=0,d=A.length;f<d;f++)x=A[f],Q0(a,x.col);if(Mn(a)!="ssp"&&A.length!==0){for(f=0,d=T.length;f<d;f++)h[f]=f;A.length&&A[0].dir==="desc"&&a.orderDescReverse&&h.reverse(),T.sort(function(j,X){var U,B,Y,F,re,Se=A.length,Te=g[j]._aSortData,Oe=g[X]._aSortData;for(Y=0;Y<Se;Y++)if(re=A[Y],U=Te[re.col],B=Oe[re.col],re.sorter){if(F=re.sorter(U,B),F!==0)return F}else if(F=U<B?-1:U>B?1:0,F!==0)return re.dir==="asc"?F:-F;return U=h[j],B=h[X],U<B?-1:U>B?1:0})}else A.length===0&&T.sort(function(j,X){return j<X?-1:j>X?1:0});return i===void 0&&(a.bSorted=!0,a.sortDetails=A,We(a,null,"order",[a,A])),T}function V0(a,i,r,f){var d=a.aoColumns[i],h=a.aaSorting,v=d.asSorting,g,x=function(A,N){var j=A._idx;return j===void 0&&(j=v.indexOf(A[1])),j+1<v.length?j+1:N?null:0};if(!d.bSortable)return!1;if(typeof h[0]=="number"&&(h=a.aaSorting=[h]),(f||r)&&a.oFeatures.bSortMulti){var T=tn(h,"0").indexOf(i);T!==-1?(g=x(h[T],!0),g===null&&h.length===1&&(g=0),g===null||v[g]===""?h.splice(T,1):(h[T][1]=v[g],h[T]._idx=g)):f?(h.push([i,v[0],0]),h[h.length-1]._idx=0):(h.push([i,h[0][1],0]),h[h.length-1]._idx=0)}else h.length&&h[0][0]==i?(g=x(h[0]),h.length=1,h[0][1]=v[g],h[0]._idx=g):(h.length=0,h.push([i,v[0]]),h[0]._idx=0)}function Ks(a){var i=a.aLastSort,r=a.oClasses.order.position,f=Eo(a),d=a.oFeatures,h,v,g;if(d.bSort&&d.bSortClasses){for(h=0,v=i.length;h<v;h++)g=i[h].src,M(tn(a.aoData,"anCells",g)).removeClass(r+(h<2?h+1:3));for(h=0,v=f.length;h<v;h++)g=f[h].src,M(tn(a.aoData,"anCells",g)).addClass(r+(h<2?h+1:3))}a.aLastSort=f}function Q0(a,i){var r=a.aoColumns[i],f=$.ext.order[r.sSortDataType],d;f&&(d=f.call(a.oInstance,a,i,Xu(a,i)));for(var h,v,g=$.ext.type.order[r.sType+"-pre"],x=a.aoData,T=0;T<x.length;T++)x[T]&&(h=x[T],h._aSortData||(h._aSortData=[]),(!h._aSortData[i]||f)&&(v=f?d[T]:$n(a,T,i,"sort"),h._aSortData[i]=g?g(v,a):v))}function No(a){if(!a._bLoadingState){var i=[];_i(a,i,a.aaSorting);var r=a.aoColumns,f={time:+new Date,start:a._iDisplayStart,length:a._iDisplayLength,order:i.map(function(d){return r[d[0]]&&r[d[0]].sName?[r[d[0]].sName,d[1]]:d.slice()}),search:M.extend({},a.oPreviousSearch),columns:a.aoColumns.map(function(d,h){return{name:d.sName,visible:d.bVisible,search:M.extend({},a.aoPreSearchCols[h])}})};a.oSavedState=f,We(a,"aoStateSaveParams","stateSaveParams",[a,f]),a.oFeatures.bStateSave&&!a.bDestroying&&a.fnStateSaveCallback.call(a.oInstance,a,f)}}function Z0(a,i,r){if(!a.oFeatures.bStateSave){r();return}var f=function(h){Js(a,h,r)},d=a.fnStateLoadCallback.call(a.oInstance,a,f);return d!==void 0&&Js(a,d,r),!0}function Js(a,i,r){var f,d,h=a.aoColumns,v=tn(a.aoColumns,"sName");a._bLoadingState=!0;var g=a._bInitComplete?new $.Api(a):null;if(!i||!i.time){a._bLoadingState=!1,r();return}var x=a.iStateDuration;if(x>0&&i.time<+new Date-x*1e3){a._bLoadingState=!1,r();return}var T=We(a,"aoStateLoadParams","stateLoadParams",[a,i]);if(T.indexOf(!1)!==-1){a._bLoadingState=!1,r();return}if(a.oLoadedState=M.extend(!0,{},i),We(a,null,"stateLoadInit",[a,i],!0),i.length!==void 0&&(g?g.page.len(i.length):a._iDisplayLength=i.length),i.start!==void 0&&(g===null?(a._iDisplayStart=i.start,a.iInitDisplayStart=i.start):dd(a,i.start/a._iDisplayLength)),i.order!==void 0&&(a.aaSorting=[],M.each(i.order,function(U,B){var Y=[B[0],B[1]];if(typeof B[0]=="string"){var F=v.indexOf(B[0]);if(F<0)return;Y[0]=F}else if(Y[0]>=h.length)return;a.aaSorting.push(Y)})),i.search!==void 0&&M.extend(a.oPreviousSearch,i.search),i.columns){var A=i.columns,N=tn(i.columns,"name");if(N.join("").length&&N.join("")!==v.join(""))for(A=[],f=0;f<v.length;f++)if(v[f]!=""){var j=N.indexOf(v[f]);j>=0?A.push(i.columns[j]):A.push({})}else A.push({});if(A.length===h.length){for(f=0,d=A.length;f<d;f++){var X=A[f];X.visible!==void 0&&(g?g.column(f).visible(X.visible,!1):h[f].bVisible=X.visible),X.search!==void 0&&M.extend(a.aoPreSearchCols[f],X.search)}g&&g.one("draw",function(){g.columns.adjust()})}}a._bLoadingState=!1,We(a,"aoStateLoaded","stateLoaded",[a,i]),r()}function Wn(a,i,r,f){r="DataTables warning: "+(a?"table id="+a.sTableId+" - ":"")+r,f&&(r+=". For more information about this error, please see https://datatables.net/tn/"+f);{var d=$.ext,h=d.sErrMode||d.errMode;if(a&&We(a,null,"dt-error",[a,f,r],!0),h=="alert")alert(r);else{if(h=="throw")throw new Error(r);typeof h=="function"&&h(a,f,r)}}}function Bl(a,i,r,f){if(Array.isArray(r)){M.each(r,function(d,h){Array.isArray(h)?Bl(a,i,h[0],h[1]):Bl(a,i,h)});return}f===void 0&&(f=r),i[r]!==void 0&&(a[f]=i[r])}function Xv(a,i,r){var f;for(var d in i)Object.prototype.hasOwnProperty.call(i,d)&&(f=i[d],M.isPlainObject(f)?(M.isPlainObject(a[d])||(a[d]={}),M.extend(!0,a[d],f)):r&&d!=="data"&&d!=="aaData"&&Array.isArray(f)?a[d]=f.slice():a[d]=f);return a}function gm(a,i,r){M(a).on("click.DT",i,function(f){r(f)}).on("keypress.DT",i,function(f){f.which===13&&(f.preventDefault(),r(f))}).on("selectstart.DT",i,function(){return!1})}function Fn(a,i,r){r&&a[i].push(r)}function We(a,i,r,f,d){var h=[];if(i&&(h=a[i].slice().reverse().map(function(x){return x.apply(a.oInstance,f)})),r!==null){var v=M.Event(r+".dt"),g=M(a.nTable);v.dt=a.api,g[d?"trigger":"triggerHandler"](v,f),d&&g.parents("body").length===0&&M("body").trigger(v,f),h.push(v.result)}return h}function Sm(a){var i=a._iDisplayStart,r=a.fnDisplayEnd(),f=a._iDisplayLength;i>=r&&(i=r-f),i-=i%f,(f===-1||i<0)&&(i=0),a._iDisplayStart=i}function So(a,i){var r=a.renderer,f=$.ext.renderer[i];return M.isPlainObject(r)&&r[i]?f[r[i]]||f._:typeof r=="string"&&f[r]||f._}function Mn(a){return a.oFeatures.bServerSide?"ssp":a.ajax?"ajax":"dom"}function pd(a,i,r){var f=a.fnFormatNumber,d=a._iDisplayStart+1,h=a._iDisplayLength,v=a.fnRecordsDisplay(),g=a.fnRecordsTotal(),x=h===-1;return i.replace(/_START_/g,f.call(a,d)).replace(/_END_/g,f.call(a,a.fnDisplayEnd())).replace(/_MAX_/g,f.call(a,g)).replace(/_TOTAL_/g,f.call(a,v)).replace(/_PAGE_/g,f.call(a,x?1:Math.ceil(d/h))).replace(/_PAGES_/g,f.call(a,x?1:Math.ceil(v/h))).replace(/_ENTRIES_/g,a.api.i18n("entries","",r)).replace(/_ENTRIES-MAX_/g,a.api.i18n("entries","",g)).replace(/_ENTRIES-TOTAL_/g,a.api.i18n("entries","",v))}function Mo(a,i){if(i)if(i.length<1e4)a.push.apply(a,i);else for(jt=0;jt<i.length;jt++)a.push(i[jt])}function Gv(a,i,r){for(Array.isArray(r)||(r=[r]),jt=0;jt<r.length;jt++)a.on(i+".dt",r[jt])}function _m(a){return $.ext.escape.attributes&&M.each(a,function(i,r){a[i]=xi(r)}),a}var vd=[],cn=Array.prototype,K0=function(a){var i,r,f=$.settings,d=tn(f,"nTable");if(a){if(a.nTable&&a.oFeatures)return[a];if(a.nodeName&&a.nodeName.toLowerCase()==="table")return i=d.indexOf(a),i!==-1?[f[i]]:null;if(a&&typeof a.settings=="function")return a.settings().toArray();typeof a=="string"?r=M(a).get():a instanceof M&&(r=a.get())}else return[];if(r)return f.filter(function(h,v){return r.includes(d[v])})};Re=function(a,i){if(!(this instanceof Re))return new Re(a,i);var r,f=[],d=function(h){var v=K0(h);v&&f.push.apply(f,v)};if(Array.isArray(a))for(r=0;r<a.length;r++)d(a[r]);else d(a);this.context=f.length>1?Er(f):f,Mo(this,i),this.selector={rows:null,cols:null,opts:null},Re.extend(this,this,vd)};$.Api=Re;M.extend(Re.prototype,{any:function(){return this.count()!==0},context:[],count:function(){return this.flatten().length},each:function(a){for(var i=0,r=this.length;i<r;i++)a.call(this,this[i],i,this);return this},eq:function(a){var i=this.context;return i.length>a?new Re(i[a],this[a]):null},filter:function(a){var i=cn.filter.call(this,a,this);return new Re(this.context,i)},flatten:function(){var a=[];return new Re(this.context,a.concat.apply(a,this.toArray()))},get:function(a){return this[a]},join:cn.join,includes:function(a){return this.indexOf(a)!==-1},indexOf:cn.indexOf,iterator:function(a,i,r,f){var d=[],h,v,g,x,T,A=this.context,N,j,X,U=this.selector;for(typeof a=="string"&&(f=r,r=i,i=a,a=!1),v=0,g=A.length;v<g;v++){var B=new Re(A[v]);if(i==="table")h=r.call(B,A[v],v),h!==void 0&&d.push(h);else if(i==="columns"||i==="rows")h=r.call(B,A[v],this[v],v),h!==void 0&&d.push(h);else if(i==="every"||i==="column"||i==="column-rows"||i==="row"||i==="cell")for(j=this[v],i==="column-rows"&&(N=wo(A[v],U.opts)),x=0,T=j.length;x<T;x++)X=j[x],i==="cell"?h=r.call(B,A[v],X.row,X.column,v,x):h=r.call(B,A[v],X,v,x,N),h!==void 0&&d.push(h)}if(d.length||f){var Y=new Re(A,a?d.concat.apply([],d):d),F=Y.selector;return F.rows=U.rows,F.cols=U.cols,F.opts=U.opts,Y}return this},lastIndexOf:cn.lastIndexOf,length:0,map:function(a){var i=cn.map.call(this,a,this);return new Re(this.context,i)},pluck:function(a){var i=$.util.get(a);return this.map(function(r){return i(r)})},pop:cn.pop,push:cn.push,reduce:cn.reduce,reduceRight:cn.reduceRight,reverse:cn.reverse,selector:null,shift:cn.shift,slice:function(){return new Re(this.context,this)},sort:cn.sort,splice:cn.splice,toArray:function(){return cn.slice.call(this)},to$:function(){return M(this)},toJQuery:function(){return M(this)},unique:function(){return new Re(this.context,Er(this.toArray()))},unshift:cn.unshift});function J0(a,i,r){return function(){var f=i.apply(a||this,arguments);return Re.extend(f,f,r.methodExt),f}}function k0(a,i){for(var r=0,f=a.length;r<f;r++)if(a[r].name===i)return a[r];return null}window.__apiStruct=vd;Re.extend=function(a,i,r){if(!(!r.length||!i||!(i instanceof Re)&&!i.__dt_wrapper)){var f,d,h;for(f=0,d=r.length;f<d;f++)h=r[f],h.name!=="__proto__"&&(i[h.name]=h.type==="function"?J0(a,h.val,h):h.type==="object"?{}:h.val,i[h.name].__dt_wrapper=!0,Re.extend(a,i[h.name],h.propExt))}};Re.register=he=function(a,i){if(Array.isArray(a)){for(var r=0,f=a.length;r<f;r++)Re.register(a[r],i);return}var d,h,v=a.split("."),g=vd,x,T;for(d=0,h=v.length;d<h;d++){T=v[d].indexOf("()")!==-1,x=T?v[d].replace("()",""):v[d];var A=k0(g,x);A||(A={name:x,val:{},methodExt:[],propExt:[],type:"object"},g.push(A)),d===h-1?(A.val=i,A.type=typeof i=="function"?"function":M.isPlainObject(i)?"object":"other"):g=T?A.methodExt:A.propExt}};Re.registerPlural=at=function(a,i,r){Re.register(a,r),Re.register(i,function(){var f=r.apply(this,arguments);return f===this?this:f instanceof Re?f.length?Array.isArray(f[0])?new Re(f.context,f[0]):f[0]:void 0:f})};var xm=function(a,i){if(Array.isArray(a)){var r=[];return a.forEach(function(d){var h=xm(d,i);Mo(r,h)}),r.filter(function(d){return d})}if(typeof a=="number")return[i[a]];var f=i.map(function(d){return d.nTable});return M(f).filter(a).map(function(){var d=f.indexOf(this);return i[d]}).toArray()};he("tables()",function(a){return a!=null?new Re(xm(a,this.context)):this});he("table()",function(a){var i=this.tables(a),r=i.context;return r.length?new Re(r[0]):i});[["nodes","node","nTable"],["body","body","nTBody"],["header","header","nTHead"],["footer","footer","nTFoot"]].forEach(function(a){at("tables()."+a[0]+"()","table()."+a[1]+"()",function(){return this.iterator("table",function(i){return i[a[2]]},1)})});[["header","aoHeader"],["footer","aoFooter"]].forEach(function(a){he("table()."+a[0]+".structure()",function(i){var r=this.columns(i).indexes().flatten().toArray(),f=this.context[0],d=cm(f,f[a[1]],r),h=r.slice().sort(function(v,g){return v-g});return d.map(function(v){return r.map(function(g){return v[h.indexOf(g)]})})})});at("tables().containers()","table().container()",function(){return this.iterator("table",function(a){return a.nTableWrapper},1)});he("tables().every()",function(a){var i=this;return this.iterator("table",function(r,f){a.call(i.table(f),f)})});he("caption()",function(a,i){var r=this.context;if(a===void 0){var f=r[0].captionNode;return f&&r.length?f.innerHTML:null}return this.iterator("table",function(d){var h=M(d.nTable),v=M(d.captionNode),g=M(d.nTableWrapper);if(v.length||(v=M("<caption/>").html(a),d.captionNode=v[0],i||(h.prepend(v),i=v.css("caption-side"))),v.html(a),i&&(v.css("caption-side",i),v[0]._captionSide=i),g.find("div.dataTables_scroll").length){var x=i==="top"?"Head":"Foot";g.find("div.dataTables_scroll"+x+" table").prepend(v)}else h.prepend(v)},1)});he("caption.node()",function(){var a=this.context;return a.length?a[0].captionNode:null});he("draw()",function(a){return this.iterator("table",function(i){a==="page"?Nr(i):(typeof a=="string"&&(a=a!=="full-hold"),Ar(i,a===!1))})});he("page()",function(a){return a===void 0?this.page.info().page:this.iterator("table",function(i){dd(i,a)})});he("page.info()",function(){if(this.context.length!==0){var a=this.context[0],i=a._iDisplayStart,r=a.oFeatures.bPaginate?a._iDisplayLength:-1,f=a.fnRecordsDisplay(),d=r===-1;return{page:d?0:Math.floor(i/r),pages:d?1:Math.ceil(f/r),start:i,end:a.fnDisplayEnd(),length:r,recordsTotal:a.fnRecordsTotal(),recordsDisplay:f,serverSide:Mn(a)==="ssp"}}});he("page.len()",function(a){return a===void 0?this.context.length!==0?this.context[0]._iDisplayLength:void 0:this.iterator("table",function(i){dm(i,a)})});var Dm=function(a,i,r){if(r){var f=new Re(a);f.one("draw",function(){r(f.ajax.json())})}if(Mn(a)=="ssp")Ar(a,i);else{Nn(a,!0);var d=a.jqXHR;d&&d.readyState!==4&&d.abort(),sd(a,{},function(h){od(a);for(var v=qu(a,h),g=0,x=v.length;g<x;g++)Di(a,v[g]);Ar(a,i),go(a),Nn(a,!1)})}};he("ajax.json()",function(){var a=this.context;if(a.length>0)return a[0].json});he("ajax.params()",function(){var a=this.context;if(a.length>0)return a[0].oAjaxData});he("ajax.reload()",function(a,i){return this.iterator("table",function(r){Dm(r,i===!1,a)})});he("ajax.url()",function(a){var i=this.context;return a===void 0?i.length===0?void 0:(i=i[0],M.isPlainObject(i.ajax)?i.ajax.url:i.ajax):this.iterator("table",function(r){M.isPlainObject(r.ajax)?r.ajax.url=a:r.ajax=a})});he("ajax.url().load()",function(a,i){return this.iterator("table",function(r){Dm(r,i===!1,a)})});var md=function(a,i,r,f,d){var h=[],v,g,x,T=typeof i;for((!i||T==="string"||T==="function"||i.length===void 0)&&(i=[i]),g=0,x=i.length;g<x;g++)v=r(typeof i[g]=="string"?i[g].trim():i[g]),v=v.filter(function(N){return N!=null}),v&&v.length&&(h=h.concat(v));var A=Yt.selector[a];if(A.length)for(g=0,x=A.length;g<x;g++)h=A[g](f,d,h);return Er(h)},yd=function(a){return a||(a={}),a.filter&&a.search===void 0&&(a.search=a.filter),M.extend({columnOrder:"implied",search:"none",order:"current",page:"all"},a)},bd=function(a){var i=new Re(a.context[0]);return a.length&&i.push(a[0]),i.selector=a.selector,i.length&&i[0].length>1&&i[0].splice(1),i},wo=function(a,i){var r,f,d,h=[],v=a.aiDisplay,g=a.aiDisplayMaster,x=i.search,T=i.order,A=i.page;if(Mn(a)=="ssp")return x==="removed"?[]:Rl(0,g.length);if(A=="current")for(r=a._iDisplayStart,f=a.fnDisplayEnd();r<f;r++)h.push(v[r]);else if(T=="current"||T=="applied"){if(x=="none")h=g.slice();else if(x=="applied")h=v.slice();else if(x=="removed"){var N={};for(r=0,f=v.length;r<f;r++)N[v[r]]=null;g.forEach(function(X){Object.prototype.hasOwnProperty.call(N,X)||h.push(X)})}}else if(T=="index"||T=="original")for(r=0,f=a.aoData.length;r<f;r++)a.aoData[r]&&(x=="none"?h.push(r):(d=v.indexOf(r),(d===-1&&x=="removed"||d>=0&&x=="applied")&&h.push(r)));else if(typeof T=="number"){var j=hd(a,T,"asc");if(x==="none")h=j;else for(r=0;r<j.length;r++)d=v.indexOf(j[r]),(d===-1&&x=="removed"||d>=0&&x=="applied")&&h.push(j[r])}return h},F0=function(a,i,r){var f,d=function(v){var g=em(v),x=a.aoData;if(g!==null&&!r)return[g];if(f||(f=wo(a,r)),g!==null&&f.indexOf(g)!==-1)return[g];if(v==null||v==="")return f;if(typeof v=="function")return f.map(function(U){var B=x[U];return v(U,B._aData,B.nTr)?U:null});if(v.nodeName){var T=v._DT_RowIndex,A=v._DT_CellIndex;if(T!==void 0)return x[T]&&x[T].nTr===v?[T]:[];if(A)return x[A.row]&&x[A.row].nTr===v.parentNode?[A.row]:[];var N=M(v).closest("*[data-dt-row]");return N.length?[N.data("dt-row")]:[]}if(typeof v=="string"&&v.charAt(0)==="#"){var j=a.aIds[v.replace(/^#/,"")];if(j!==void 0)return[j.idx]}var X=nm(Yu(a.aoData,f,"nTr"));return M(X).filter(v).map(function(){return this._DT_RowIndex}).toArray()},h=md("row",i,d,a,r);return(r.order==="current"||r.order==="applied")&&bm(a,h),h};he("rows()",function(a,i){a===void 0?a="":M.isPlainObject(a)&&(i=a,a=""),i=yd(i);var r=this.iterator("table",function(f){return F0(f,a,i)},1);return r.selector.rows=a,r.selector.opts=i,r});he("rows().nodes()",function(){return this.iterator("row",function(a,i){return a.aoData[i].nTr||void 0},1)});he("rows().data()",function(){return this.iterator(!0,"rows",function(a,i){return Yu(a.aoData,i,"_aData")},1)});at("rows().cache()","row().cache()",function(a){return this.iterator("row",function(i,r){var f=i.aoData[r];return a==="search"?f._aFilterData:f._aSortData},1)});at("rows().invalidate()","row().invalidate()",function(a){return this.iterator("row",function(i,r){Ao(i,r,a)})});at("rows().indexes()","row().index()",function(){return this.iterator("row",function(a,i){return i},1)});at("rows().ids()","row().id()",function(a){for(var i=[],r=this.context,f=0,d=r.length;f<d;f++)for(var h=0,v=this[f].length;h<v;h++){var g=r[f].rowIdFn(r[f].aoData[this[f][h]]._aData);i.push((a===!0?"#":"")+g)}return new Re(r,i)});at("rows().remove()","row().remove()",function(){return this.iterator("row",function(a,i){var r=a.aoData,f=r[i],d=a.aiDisplayMaster.indexOf(i);d!==-1&&a.aiDisplayMaster.splice(d,1),a._iRecordsDisplay>0&&a._iRecordsDisplay--,Sm(a);var h=a.rowIdFn(f._aData);h!==void 0&&delete a.aIds[h],r[i]=null}),this});he("rows.add()",function(a){var i=this.iterator("table",function(f){var d,h,v,g=[];for(h=0,v=a.length;h<v;h++)d=a[h],d.nodeName&&d.nodeName.toUpperCase()==="TR"?g.push(fd(f,d)[0]):g.push(Di(f,d));return g},1),r=this.rows(-1);return r.pop(),Mo(r,i),r});he("row()",function(a,i){return bd(this.rows(a,i))});he("row().data()",function(a){var i=this.context;if(a===void 0)return i.length&&this.length&&this[0].length?i[0].aoData[this[0]]._aData:void 0;var r=i[0].aoData[this[0]];return r._aData=a,Array.isArray(a)&&r.nTr&&r.nTr.id&&Si(i[0].rowId)(a,r.nTr.id),Ao(i[0],this[0],"data"),this});he("row().node()",function(){var a=this.context;if(a.length&&this.length&&this[0].length){var i=a[0].aoData[this[0]];if(i&&i.nTr)return i.nTr}return null});he("row.add()",function(a){a instanceof M&&a.length&&(a=a[0]);var i=this.iterator("table",function(r){return a.nodeName&&a.nodeName.toUpperCase()==="TR"?fd(r,a)[0]:Di(r,a)});return this.row(i[0])});M(document).on("plugin-init.dt",function(a,i){var r=new Re(i);r.on("stateSaveParams.DT",function(f,d,h){for(var v=d.rowIdFn,g=d.aiDisplayMaster,x=[],T=0;T<g.length;T++){var A=g[T],N=d.aoData[A];N._detailsShow&&x.push("#"+v(N._aData))}h.childRows=x}),r.on("stateLoaded.DT",function(f,d,h){Vv(r,h)}),Vv(r,r.state.loaded())});var Vv=function(a,i){i&&i.childRows&&a.rows(i.childRows.map(function(r){return r.replace(/([^:\\]*(?:\\.[^:\\]*)*):/g,"$1\\:")})).every(function(){We(a.settings()[0],null,"requestChild",[this])})},W0=function(a,i,r,f){var d=[],h=function(v,g){if(Array.isArray(v)||v instanceof M){for(var x=0,T=v.length;x<T;x++)h(v[x],g);return}if(v.nodeName&&v.nodeName.toLowerCase()==="tr")v.setAttribute("data-dt-row",i.idx),d.push(v);else{var A=M("<tr><td></td></tr>").attr("data-dt-row",i.idx).addClass(g);M("td",A).addClass(g).html(v)[0].colSpan=To(a),d.push(A[0])}};h(r,f),i._details&&i._details.detach(),i._details=M(d),i._detailsShow&&i._details.insertAfter(i.nTr)},Tm=$.util.throttle(function(a){No(a[0])},500),gd=function(a,i){var r=a.context;if(r.length){var f=r[0].aoData[i!==void 0?i:a[0]];f&&f._details&&(f._details.detach(),f._detailsShow=void 0,f._details=void 0,M(f.nTr).removeClass("dt-hasChild"),Tm(r))}},Cm=function(a,i){var r=a.context;if(r.length&&a.length){var f=r[0].aoData[a[0]];f._details&&(f._detailsShow=i,i?(f._details.insertAfter(f.nTr),M(f.nTr).addClass("dt-hasChild")):(f._details.detach(),M(f.nTr).removeClass("dt-hasChild")),We(r[0],null,"childRow",[i,a.row(a[0])]),$0(r[0]),Tm(r))}},$0=function(a){var i=new Re(a),r=".dt.DT_details",f="draw"+r,d="column-sizing"+r,h="destroy"+r,v=a.aoData;i.off(f+" "+d+" "+h),tn(v,"_details").length>0&&(i.on(f,function(g,x){a===x&&i.rows({page:"current"}).eq(0).each(function(T){var A=v[T];A._detailsShow&&A._details.insertAfter(A.nTr)})}),i.on(d,function(g,x){if(a===x)for(var T,A=To(x),N=0,j=v.length;N<j;N++)T=v[N],T&&T._details&&T._details.each(function(){var X=M(this).children("td");X.length==1&&X.attr("colspan",A)})}),i.on(h,function(g,x){if(a===x)for(var T=0,A=v.length;T<A;T++)v[T]&&v[T]._details&&gd(i,T)}))},I0="",Gu=I0+"row().child",Oo=Gu+"()";he(Oo,function(a,i){var r=this.context;return a===void 0?r.length&&this.length&&r[0].aoData[this[0]]?r[0].aoData[this[0]]._details:void 0:(a===!0?this.child.show():a===!1?gd(this):r.length&&this.length&&W0(r[0],r[0].aoData[this[0]],a,i),this)});he([Gu+".show()",Oo+".show()"],function(){return Cm(this,!0),this});he([Gu+".hide()",Oo+".hide()"],function(){return Cm(this,!1),this});he([Gu+".remove()",Oo+".remove()"],function(){return gd(this),this});he(Gu+".isShown()",function(){var a=this.context;return a.length&&this.length&&a[0].aoData[this[0]]&&a[0].aoData[this[0]]._detailsShow||!1});var P0=/^([^:]+)?:(name|title|visIdx|visible)$/,Sd=function(a,i,r,f,d,h){for(var v=[],g=0,x=d.length;g<x;g++)v.push($n(a,d[g],i,h));return v},Am=function(a,i,r){var f=a.aoHeader,d=a.titleRow,h=null;if(r!==void 0)h=r;else if(d===!0)h=0;else if(d===!1)h=f.length-1;else if(d!==null)h=d;else{for(var v=0;v<f.length;v++)f[v][i].unique&&M("span.dt-column-title",f[v][i].cell).text()&&(h=v);h===null&&(h=0)}return f[h][i].cell},eb=function(a){for(var i=[],r=0;r<a.length;r++)for(var f=0;f<a[r].length;f++){var d=a[r][f].cell;i.includes(d)||i.push(d)}return i},tb=function(a,i,r){var f=a.aoColumns,d,h,v=eb(a.aoHeader),g=function(T){var A=em(T);if(T==="")return Rl(f.length);if(A!==null)return[A>=0?A:f.length+A];if(typeof T=="function"){var N=wo(a,r);return f.map(function(F,re){return T(re,Sd(a,re,0,0,N),Am(a,re))?re:null})}var j=typeof T=="string"?T.match(P0):"";if(j)switch(j[2]){case"visIdx":case"visible":if(j[1]&&j[1].match(/^\d+$/)){var X=parseInt(j[1],10);if(X<0){var U=f.map(function(F,re){return F.bVisible?re:null});return[U[U.length+X]]}return[id(a,X)]}return f.map(function(F,re){return F.bVisible?j[1]?M(v[re]).filter(j[1]).length>0?re:null:re:null});case"name":return d||(d=tn(f,"sName")),d.map(function(F,re){return F===j[1]?re:null});case"title":return h||(h=tn(f,"sTitle")),h.map(function(F,re){return F===j[1]?re:null});default:return[]}if(T.nodeName&&T._DT_CellIndex)return[T._DT_CellIndex.column];var B=M(v).filter(T).map(function(){return ud(this)}).toArray().sort(function(F,re){return F-re});if(B.length||!T.nodeName)return B;var Y=M(T).closest("*[data-dt-column]");return Y.length?[Y.data("dt-column")]:[]},x=md("column",i,g,a,r);return r.columnOrder&&r.columnOrder==="index"?x.sort(function(T,A){return T-A}):x},nb=function(a,i,r){var f=a.aoColumns,d=f[i],h=a.aoData,v,g,x,T;if(r===void 0)return d.bVisible;if(d.bVisible===r)return!1;if(r){var A=tn(f,"bVisible").indexOf(!0,i+1);for(g=0,x=h.length;g<x;g++)h[g]&&(T=h[g].nTr,v=h[g].anCells,T&&T.insertBefore(v[i],v[A]||null))}else M(tn(a.aoData,"anCells",i)).detach();return d.bVisible=r,mm(a),!0};he("columns()",function(a,i){a===void 0?a="":M.isPlainObject(a)&&(i=a,a=""),i=yd(i);var r=this.iterator("table",function(f){return tb(f,a,i)},1);return r.selector.cols=a,r.selector.opts=i,r});at("columns().header()","column().header()",function(a){return this.iterator("column",function(i,r){return Am(i,r,a)},1)});at("columns().footer()","column().footer()",function(a){return this.iterator("column",function(i,r){var f=i.aoFooter;return f.length?i.aoFooter[a!==void 0?a:0][r].cell:null},1)});at("columns().data()","column().data()",function(){return this.iterator("column-rows",Sd,1)});at("columns().render()","column().render()",function(a){return this.iterator("column-rows",function(i,r,f,d,h){return Sd(i,r,f,d,h,a)},1)});at("columns().dataSrc()","column().dataSrc()",function(){return this.iterator("column",function(a,i){return a.aoColumns[i].mData},1)});at("columns().cache()","column().cache()",function(a){return this.iterator("column-rows",function(i,r,f,d,h){return Yu(i.aoData,h,a==="search"?"_aFilterData":"_aSortData",r)},1)});at("columns().init()","column().init()",function(){return this.iterator("column",function(a,i){return a.aoColumns[i]},1)});at("columns().names()","column().name()",function(){return this.iterator("column",function(a,i){return a.aoColumns[i].sName},1)});at("columns().nodes()","column().nodes()",function(){return this.iterator("column-rows",function(a,i,r,f,d){return Yu(a.aoData,d,"anCells",i)},1)});at("columns().titles()","column().title()",function(a,i){return this.iterator("column",function(r,f){typeof a=="number"&&(i=a,a=void 0);var d=M("span.dt-column-title",this.column(f).header(i));return a!==void 0?(d.html(a),this):d.html()},1)});at("columns().types()","column().type()",function(){return this.iterator("column",function(a,i){var r=a.aoColumns[i].sType;return r||Co(a),r},1)});at("columns().visible()","column().visible()",function(a,i){var r=this,f=[],d=this.iterator("column",function(h,v){if(a===void 0)return h.aoColumns[v].bVisible;nb(h,v,a)&&f.push(v)});return a!==void 0&&this.iterator("table",function(h){bo(h,h.aoHeader),bo(h,h.aoFooter),h.aiDisplay.length||M(h.nTBody).find("td[colspan]").attr("colspan",To(h)),No(h),r.iterator("column",function(v,g){f.includes(g)&&We(v,null,"column-visibility",[v,g,a,i])}),f.length&&(i===void 0||i)&&r.columns.adjust()}),d});at("columns().widths()","column().width()",function(){var a=this.columns(":visible").count(),i=M("<tr>").html("<td>"+Array(a).join("</td><td>")+"</td>");M(this.table().body()).append(i);var r=i.children().map(function(){return M(this).outerWidth()});return i.remove(),this.iterator("column",function(f,d){var h=Xu(f,d);return h!==null?r[h]:0},1)});at("columns().indexes()","column().index()",function(a){return this.iterator("column",function(i,r){return a==="visible"?Xu(i,r):r},1)});he("columns.adjust()",function(){return this.iterator("table",function(a){a.containerWidth=-1,Do(a)},1)});he("column.index()",function(a,i){if(this.context.length!==0){var r=this.context[0];if(a==="fromVisible"||a==="toData")return id(r,i);if(a==="fromData"||a==="toVisible")return Xu(r,i)}});he("column()",function(a,i){return bd(this.columns(a,i))});var ab=function(a,i,r){var f=a.aoData,d=wo(a,r),h=nm(Yu(f,d,"anCells")),v=M(am([],h)),g,x=a.aoColumns.length,T,A,N,j,X,U,B=function(Y){var F=typeof Y=="function";if(Y==null||F){for(T=[],A=0,N=d.length;A<N;A++)for(g=d[A],j=0;j<x;j++)X={row:g,column:j},F?(U=f[g],Y(X,$n(a,g,j),U.anCells?U.anCells[j]:null)&&T.push(X)):T.push(X);return T}if(M.isPlainObject(Y))return Y.column!==void 0&&Y.row!==void 0&&d.indexOf(Y.row)!==-1?[Y]:[];var re=v.filter(Y).map(function(Se,Te){return{row:Te._DT_CellIndex.row,column:Te._DT_CellIndex.column}}).toArray();return re.length||!Y.nodeName?re:(U=M(Y).closest("*[data-dt-row]"),U.length?[{row:U.data("dt-row"),column:U.data("dt-column")}]:[])};return md("cell",i,B,a,r)};he("cells()",function(a,i,r){if(M.isPlainObject(a)&&(a.row===void 0?(r=a,a=null):(r=i,i=null)),M.isPlainObject(i)&&(r=i,i=null),i==null)return this.iterator("table",function(j){return ab(j,a,yd(r))});var f=r?{page:r.page,order:r.order,search:r.search}:{},d=this.columns(i,f),h=this.rows(a,f),v,g,x,T,A=this.iterator("table",function(j,X){var U=[];for(v=0,g=h[X].length;v<g;v++)for(x=0,T=d[X].length;x<T;x++)U.push({row:h[X][v],column:d[X][x]});return U},1),N=r&&r.selected?this.cells(A,r):A;return M.extend(N.selector,{cols:i,rows:a,opts:r}),N});at("cells().nodes()","cell().node()",function(){return this.iterator("cell",function(a,i,r){var f=a.aoData[i];return f&&f.anCells?f.anCells[r]:void 0},1)});he("cells().data()",function(){return this.iterator("cell",function(a,i,r){return $n(a,i,r)},1)});at("cells().cache()","cell().cache()",function(a){return a=a==="search"?"_aFilterData":"_aSortData",this.iterator("cell",function(i,r,f){return i.aoData[r][a][f]},1)});at("cells().render()","cell().render()",function(a){return this.iterator("cell",function(i,r,f){return $n(i,r,f,a)},1)});at("cells().indexes()","cell().index()",function(){return this.iterator("cell",function(a,i,r){return{row:i,column:r,columnVisible:Xu(a,r)}},1)});at("cells().invalidate()","cell().invalidate()",function(a){return this.iterator("cell",function(i,r,f){Ao(i,r,a,f)})});he("cell()",function(a,i,r){return bd(this.cells(a,i,r))});he("cell().data()",function(a){var i=this.context,r=this[0];return a===void 0?i.length&&r.length?$n(i[0],r[0].row,r[0].column):void 0:(A0(i[0],r[0].row,r[0].column,a),Ao(i[0],r[0].row,"data",r[0].column),this)});he("order()",function(a,i){var r=this.context,f=Array.prototype.slice.call(arguments);return a===void 0?r.length!==0?r[0].aaSorting:void 0:(typeof a=="number"?a=[[a,i]]:f.length>1&&(a=f),this.iterator("table",function(d){var h=[];_i(d,h,a),d.aaSorting=h}))});he("order.listener()",function(a,i,r){return this.iterator("table",function(f){ym(f,a,{},i,r)})});he("order.fixed()",function(a){if(!a){var i=this.context,r=i.length?i[0].aaSortingFixed:void 0;return Array.isArray(r)?{pre:r}:r}return this.iterator("table",function(f){f.aaSortingFixed=M.extend(!0,{},a)})});he(["columns().order()","column().order()"],function(a){var i=this;return a?this.iterator("table",function(r,f){r.aaSorting=i[f].map(function(d){return[d,a]})}):this.iterator("column",function(r,f){for(var d=Eo(r),h=0,v=d.length;h<v;h++)if(d[h].col===f)return d[h].dir;return null},1)});at("columns().orderable()","column().orderable()",function(a){return this.iterator("column",function(i,r){var f=i.aoColumns[r];return a?f.asSorting:f.bSortable},1)});he("processing()",function(a){return this.iterator("table",function(i){Nn(i,a)})});he("search()",function(a,i,r,f){var d=this.context;return a===void 0?d.length!==0?d[0].oPreviousSearch.search:void 0:this.iterator("table",function(h){h.oFeatures.bFilter&&(typeof i=="object"?Uu(h,M.extend(h.oPreviousSearch,i,{search:a})):Uu(h,M.extend(h.oPreviousSearch,{search:a,regex:i===null?!1:i,smart:r===null?!0:r,caseInsensitive:f===null?!0:f})))})});he("search.fixed()",function(a,i){var r=this.iterator(!0,"table",function(f){var d=f.searchFixed;if(a){if(i===void 0)return d[a];i===null?delete d[a]:d[a]=i}else return Object.keys(d);return this});return a!==void 0&&i===void 0?r[0]:r});at("columns().search()","column().search()",function(a,i,r,f){return this.iterator("column",function(d,h){var v=d.aoPreSearchCols;if(a===void 0)return v[h].search;d.oFeatures.bFilter&&(typeof i=="object"?M.extend(v[h],i,{search:a}):M.extend(v[h],{search:a,regex:i===null?!1:i,smart:r===null?!0:r,caseInsensitive:f===null?!0:f}),Uu(d,d.oPreviousSearch))})});he(["columns().search.fixed()","column().search.fixed()"],function(a,i){var r=this.iterator(!0,"column",function(f,d){var h=f.aoColumns[d].searchFixed;if(a){if(i===void 0)return h[a]||null;i===null?delete h[a]:h[a]=i}else return Object.keys(h);return this});return a!==void 0&&i===void 0?r[0]:r});he("state()",function(a,i){if(!a)return this.context.length?this.context[0].oSavedState:null;var r=M.extend(!0,{},a);return this.iterator("table",function(f){i!==!1&&(r.time=+new Date+100),Js(f,r,function(){})})});he("state.clear()",function(){return this.iterator("table",function(a){a.fnStateSaveCallback.call(a.oInstance,a,{})})});he("state.loaded()",function(){return this.context.length?this.context[0].oLoadedState:null});he("state.save()",function(){return this.iterator("table",function(a){No(a)})});var Qv,Zv;$.use=function(a,i){var r=typeof a=="string"?i:a,f=typeof i=="string"?i:a;if(r===void 0&&typeof f=="string")switch(f){case"lib":case"jq":return M;case"win":return window;case"datetime":return $.DateTime;case"luxon":return Yl;case"moment":return Ti;case"bootstrap":return Qv||window.bootstrap;case"foundation":return Zv||window.Foundation;default:return null}f==="lib"||f==="jq"||r&&r.fn&&r.fn.jquery?M=r:f==="win"||r&&r.document?(window=r,document=r.document):f==="datetime"||r&&r.type==="DateTime"?$.DateTime=r:f==="luxon"||r&&r.FixedOffsetZone?Yl=r:f==="moment"||r&&r.isMoment?Ti=r:f==="bootstrap"||r&&r.Modal&&r.Modal.NAME==="modal"?Qv=r:(f==="foundation"||r&&r.Reveal)&&(Zv=r)};$.factory=function(a,i){var r=!1;return a&&a.document&&(window=a,document=a.document),i&&i.fn&&i.fn.jquery&&(M=i,r=!0),r};$.versionCheck=function(a,i){for(var r=i?i.split("."):$.version.split("."),f=a.split("."),d,h,v=0,g=f.length;v<g;v++)if(d=parseInt(r[v],10)||0,h=parseInt(f[v],10)||0,d!==h)return d>h;return!0};$.isDataTable=function(a){var i=M(a).get(0),r=!1;return a instanceof $.Api?!0:(M.each($.settings,function(f,d){var h=d.nScrollHead?M("table",d.nScrollHead)[0]:null,v=d.nScrollFoot?M("table",d.nScrollFoot)[0]:null;(d.nTable===i||h===i||v===i)&&(r=!0)}),r)};$.tables=function(a){var i=!1;M.isPlainObject(a)&&(i=a.api,a=a.visible);var r=$.settings.filter(function(f){return!!(!a||a&&M(f.nTable).is(":visible"))}).map(function(f){return f.nTable});return i?new Re(r):r};$.camelToHungarian=ql;he("$()",function(a,i){var r=this.rows(i).nodes(),f=M(r);return M([].concat(f.filter(a).toArray(),f.find(a).toArray()))});M.each(["on","one","off"],function(a,i){he(i+"()",function(){var r=Array.prototype.slice.call(arguments);r[0]=r[0].split(/\s/).map(function(d){return d.match(/\.dt\b/)?d:d+".dt"}).join(" ");var f=M(this.tables().nodes());return f[i].apply(f,r),this})});he("clear()",function(){return this.iterator("table",function(a){od(a)})});he("error()",function(a){return this.iterator("table",function(i){Wn(i,0,a)})});he("settings()",function(){return new Re(this.context,this.context)});he("init()",function(){var a=this.context;return a.length?a[0].oInit:null});he("data()",function(){return this.iterator("table",function(a){return tn(a.aoData,"_aData")}).flatten()});he("trigger()",function(a,i,r){return this.iterator("table",function(f){return We(f,null,a,i,r)}).flatten()});he("ready()",function(a){var i=this.context;return a?this.tables().every(function(){var r=this;this.context[0]._bInitComplete?a.call(r):this.on("init.dt.DT",function(){a.call(r)})}):i.length?i[0]._bInitComplete||!1:null});he("destroy()",function(a){return a=a||!1,this.iterator("table",function(i){var r=i.oClasses,f=i.nTable,d=i.nTBody,h=i.nTHead,v=i.nTFoot,g=M(f),x=M(d),T=M(i.nTableWrapper),A=i.aoData.map(function(Y){return Y?Y.nTr:null}),N=r.order;i.bDestroying=!0,We(i,"aoDestroyCallback","destroy",[i],!0),a||new Re(i).columns().visible(!0),i.resizeObserver&&i.resizeObserver.disconnect(),T.off(".DT").find(":not(tbody *)").off(".DT"),M(window).off(".DT-"+i.sInstance),f!=h.parentNode&&(g.children("thead").detach(),g.append(h)),v&&f!=v.parentNode&&(g.children("tfoot").detach(),g.append(v)),Kv(h,"header"),Kv(v,"footer"),i.colgroup.remove(),i.aaSorting=[],i.aaSortingFixed=[],Ks(i),M(g).find("th, td").removeClass(M.map($.ext.type.className,function(Y){return Y}).join(" ")),M("th, td",h).removeClass(N.none+" "+N.canAsc+" "+N.canDesc+" "+N.isAsc+" "+N.isDesc).css("width","").removeAttr("aria-sort"),x.children().detach(),x.append(A);var j=i.nTableWrapper.parentNode,X=i.nTableWrapper.nextSibling,U=a?"remove":"detach";g[U](),T[U](),!a&&j&&(j.insertBefore(f,X),g.css("width",i.sDestroyWidth).removeClass(r.table));var B=$.settings.indexOf(i);B!==-1&&$.settings.splice(B,1)})});M.each(["column","row","cell"],function(a,i){he(i+"s().every()",function(r){var f=this.selector.opts,d=this,h,v=0;return this.iterator("every",function(g,x,T){h=d[i](x,f),i==="cell"?r.call(h,h[0][0].row,h[0][0].column,T,v):r.call(h,x,T,v),v++})})});he("i18n()",function(a,i,r){var f=this.context[0],d=Cr(a)(f.oLanguage);return d===void 0&&(d=i),M.isPlainObject(d)&&(d=r!==void 0&&d[r]!==void 0?d[r]:r===!1?d:d._),typeof d=="string"?d.replace("%d",r):d});function Kv(a,i){M(a).find("span.dt-column-order").remove(),M(a).find("span.dt-column-title").each(function(){var r=M(this).html();M(this).parent().parent().append(r),M(this).remove()}),M(a).find("div.dt-column-"+i).remove(),M("th, td",a).removeAttr("data-dt-column")}$.version="2.3.4";$.settings=[];$.models={};$.models.oSearch={caseInsensitive:!0,search:"",regex:!1,smart:!0,return:!1};$.models.oRow={nTr:null,anCells:null,_aData:[],_aSortData:null,_aFilterData:null,_sFilterRow:null,src:null,idx:-1,displayData:null};$.models.oColumn={idx:null,aDataSort:null,asSorting:null,bSearchable:null,bSortable:null,bVisible:null,_sManualType:null,_bAttrSrc:!1,fnCreatedCell:null,fnGetData:null,fnSetData:null,mData:null,mRender:null,sClass:null,sContentPadding:null,sDefaultContent:null,sName:null,sSortDataType:"std",sSortingClass:null,sTitle:null,sType:null,sWidth:null,sWidthOrig:null,maxLenString:null,searchFixed:null};$.defaults={aaData:null,aaSorting:[[0,"asc"]],aaSortingFixed:[],ajax:null,aLengthMenu:[10,25,50,100],aoColumns:null,aoColumnDefs:null,aoSearchCols:[],bAutoWidth:!0,bDeferRender:!0,bDestroy:!1,bFilter:!0,bInfo:!0,bLengthChange:!0,bPaginate:!0,bProcessing:!1,bRetrieve:!1,bScrollCollapse:!1,bServerSide:!1,bSort:!0,bSortMulti:!0,bSortCellsTop:null,titleRow:null,bSortClasses:!0,bStateSave:!1,fnCreatedRow:null,fnDrawCallback:null,fnFooterCallback:null,fnFormatNumber:function(a){return a.toString().replace(/\B(?=(\d{3})+(?!\d))/g,this.oLanguage.sThousands)},fnHeaderCallback:null,fnInfoCallback:null,fnInitComplete:null,fnPreDrawCallback:null,fnRowCallback:null,fnStateLoadCallback:function(a){try{return JSON.parse((a.iStateDuration===-1?sessionStorage:localStorage).getItem("DataTables_"+a.sInstance+"_"+location.pathname))}catch{return{}}},fnStateLoadParams:null,fnStateLoaded:null,fnStateSaveCallback:function(a,i){try{(a.iStateDuration===-1?sessionStorage:localStorage).setItem("DataTables_"+a.sInstance+"_"+location.pathname,JSON.stringify(i))}catch{}},fnStateSaveParams:null,iStateDuration:7200,iDisplayLength:10,iDisplayStart:0,iTabIndex:0,oClasses:{},oLanguage:{oAria:{orderable:": Activate to sort",orderableReverse:": Activate to invert sorting",orderableRemove:": Activate to remove sorting",paginate:{first:"First",last:"Last",next:"Next",previous:"Previous",number:""}},oPaginate:{sFirst:"«",sLast:"»",sNext:"›",sPrevious:"‹"},entries:{_:"entries",1:"entry"},lengthLabels:{"-1":"All"},sEmptyTable:"No data available in table",sInfo:"Showing _START_ to _END_ of _TOTAL_ _ENTRIES-TOTAL_",sInfoEmpty:"Showing 0 to 0 of 0 _ENTRIES-TOTAL_",sInfoFiltered:"(filtered from _MAX_ total _ENTRIES-MAX_)",sInfoPostFix:"",sDecimal:"",sThousands:",",sLengthMenu:"_MENU_ _ENTRIES_ per page",sLoadingRecords:"Loading...",sProcessing:"",sSearch:"Search:",sSearchPlaceholder:"",sUrl:"",sZeroRecords:"No matching records found"},orderDescReverse:!0,oSearch:M.extend({},$.models.oSearch),layout:{topStart:"pageLength",topEnd:"search",bottomStart:"info",bottomEnd:"paging"},sDom:null,searchDelay:null,sPaginationType:"",sScrollX:"",sScrollXInner:"",sScrollY:"",sServerMethod:"GET",renderer:null,rowId:"DT_RowId",caption:null,iDeferLoading:null,on:null};xo($.defaults);$.defaults.column={aDataSort:null,iDataSort:-1,ariaTitle:"",asSorting:["asc","desc",""],bSearchable:!0,bSortable:!0,bVisible:!0,fnCreatedCell:null,mData:null,mRender:null,sCellType:"td",sClass:"",sContentPadding:"",sDefaultContent:null,sName:"",sSortDataType:"std",sTitle:null,sType:null,sWidth:null};xo($.defaults.column);$.models.oSettings={oFeatures:{bAutoWidth:null,bDeferRender:null,bFilter:null,bInfo:!0,bLengthChange:!0,bPaginate:null,bProcessing:null,bServerSide:null,bSort:null,bSortMulti:null,bSortClasses:null,bStateSave:null},oScroll:{bCollapse:null,iBarWidth:0,sX:null,sXInner:null,sY:null},oLanguage:{fnInfoCallback:null},oBrowser:{bScrollbarLeft:!1,barWidth:0},ajax:null,aanFeatures:[],aoData:[],aiDisplay:[],aiDisplayMaster:[],aIds:{},aoColumns:[],aoHeader:[],aoFooter:[],oPreviousSearch:{},searchFixed:{},aoPreSearchCols:[],aaSorting:null,aaSortingFixed:[],sDestroyWidth:0,aoRowCallback:[],aoHeaderCallback:[],aoFooterCallback:[],aoDrawCallback:[],aoRowCreatedCallback:[],aoPreDrawCallback:[],aoInitComplete:[],aoStateSaveParams:[],aoStateLoadParams:[],aoStateLoaded:[],sTableId:"",nTable:null,nTHead:null,nTFoot:null,nTBody:null,nTableWrapper:null,bInitialised:!1,aoOpenRows:[],sDom:null,searchDelay:null,sPaginationType:"two_button",pagingControls:0,iStateDuration:0,aoStateSave:[],aoStateLoad:[],oSavedState:null,oLoadedState:null,bAjaxDataGet:!0,jqXHR:null,json:void 0,oAjaxData:void 0,sServerMethod:null,fnFormatNumber:null,aLengthMenu:null,iDraw:0,bDrawing:!1,iDrawError:-1,_iDisplayLength:10,_iDisplayStart:0,_iRecordsTotal:0,_iRecordsDisplay:0,oClasses:{},bFiltered:!1,bSorted:!1,bSortCellsTop:null,oInit:null,aoDestroyCallback:[],fnRecordsTotal:function(){return Mn(this)=="ssp"?this._iRecordsTotal*1:this.aiDisplayMaster.length},fnRecordsDisplay:function(){return Mn(this)=="ssp"?this._iRecordsDisplay*1:this.aiDisplay.length},fnDisplayEnd:function(){var a=this._iDisplayLength,i=this._iDisplayStart,r=i+a,f=this.aiDisplay.length,d=this.oFeatures,h=d.bPaginate;return d.bServerSide?h===!1||a===-1?i+f:Math.min(i+a,this._iRecordsDisplay):!h||r>f||a===-1?f:r},oInstance:null,sInstance:null,iTabIndex:0,nScrollHead:null,nScrollFoot:null,aLastSort:[],oPlugins:{},rowIdFn:null,rowId:null,caption:"",captionNode:null,colgroup:null,deferLoading:null,typeDetect:!0,resizeObserver:null,containerWidth:-1,orderDescReverse:null,orderIndicators:!0,orderHandler:!0,titleRow:null};var lb=$.ext.pager;M.extend(lb,{simple:function(){return["previous","next"]},full:function(){return["first","previous","next","last"]},numbers:function(){return["numbers"]},simple_numbers:function(){return["previous","numbers","next"]},full_numbers:function(){return["first","previous","numbers","next","last"]},first_last:function(){return["first","last"]},first_last_numbers:function(){return["first","numbers","last"]},_numbers:wm,numbers_length:7});M.extend(!0,$.ext.renderer,{pagingButton:{_:function(a,i,r,f,d){var h=a.oClasses.paging,v=[h.button],g;return f&&v.push(h.active),d&&v.push(h.disabled),i==="ellipsis"?g=M('<span class="ellipsis"></span>').html(r)[0]:g=M("<button>",{class:v.join(" "),role:"link",type:"button"}).html(r),{display:g,clicker:g}}},pagingContainer:{_:function(a,i){return i}}});var Mr=function(a,i){return function(r){return En(r)||typeof r!="string"||(r=r.replace(y0," "),a&&(r=Pa(r)),r=mo(r,!1)),r}};function Jv(a,i,r,f,d){return Ti?a[i](d):Yl?a[r](d):f?a[f](d):a}var kv=!1,Yl,Ti;function ib(){window.luxon&&!Yl&&(Yl=window.luxon),window.moment&&!Ti&&(Ti=window.moment)}function _o(a,i,r){var f;if(ib(),Ti){if(f=Ti.utc(a,i,r,!0),!f.isValid())return null}else if(Yl){if(f=i&&typeof a=="string"?Yl.DateTime.fromFormat(a,i):Yl.DateTime.fromISO(a),!f.isValid)return null;f=f.setLocale(r)}else i?(kv||alert("DataTables warning: Formatted date without Moment.js or Luxon - https://datatables.net/tn/17"),kv=!0):f=new Date(a);return f}function qs(a){return function(i,r,f,d){arguments.length===0?(f="en",r=null,i=null):arguments.length===1?(f="en",r=i,i=null):arguments.length===2&&(f=r,r=i,i=null);var h="datetime"+(r?"-"+r:"");return $.ext.type.order[h+"-pre"]||$.type(h,{detect:function(v){return v===h?h:!1},order:{pre:function(v){return v.valueOf()}},className:"dt-right"}),function(v,g){if(v==null)if(d==="--now"){var x=new Date;v=new Date(Date.UTC(x.getFullYear(),x.getMonth(),x.getDate(),x.getHours(),x.getMinutes(),x.getSeconds()))}else v="";if(g==="type")return h;if(v==="")return g!=="sort"?"":_o("0000-01-01 00:00:00",null,f);if(r!==null&&i===r&&g!=="sort"&&g!=="type"&&!(v instanceof Date))return v;var T=_o(v,i,f);if(T===null)return v;if(g==="sort")return T;var A=r===null?Jv(T,"toDate","toJSDate","")[a](navigator.language,{timeZone:"UTC"}):Jv(T,"format","toFormat","toISOString",r);return g==="display"?xi(A):A}}}var Em=",",Nm=".";if(window.Intl!==void 0)try{for(var Hu=new Intl.NumberFormat().formatToParts(100000.1),jt=0;jt<Hu.length;jt++)Hu[jt].type==="group"?Em=Hu[jt].value:Hu[jt].type==="decimal"&&(Nm=Hu[jt].value)}catch{}$.datetime=function(a,i){var r="datetime-"+a;i||(i="en"),$.ext.type.order[r]||$.type(r,{detect:function(f){var d=_o(f,a,i);return f===""||d?r:!1},order:{pre:function(f){return _o(f,a,i)||0}},className:"dt-right"})};$.render={date:qs("toLocaleDateString"),datetime:qs("toLocaleString"),time:qs("toLocaleTimeString"),number:function(a,i,r,f,d){return a==null&&(a=Em),i==null&&(i=Nm),{display:function(h){if(typeof h!="number"&&typeof h!="string"||h===""||h===null)return h;var v=h<0?"-":"",g=parseFloat(h),x=Math.abs(g);if(x>=1e11||x<1e-4&&x!==0){var T=g.toExponential(r).split(/e\+?/);return T[0]+" x 10<sup>"+T[1]+"</sup>"}if(isNaN(g))return xi(h);g=g.toFixed(r),h=Math.abs(g);var A=parseInt(h,10),N=r?i+(h-A).toFixed(r).substring(2):"";return A===0&&parseFloat(N)===0&&(v=""),v+(f||"")+A.toString().replace(/\B(?=(\d{3})+(?!\d))/g,a)+N+(d||"")}}},text:function(){return{display:xi,filter:xi}}};var sn=$.ext.type;$.type=function(a,i,r){if(!i)return{className:sn.className[a],detect:sn.detect.find(function(v){return v._name===a}),order:{pre:sn.order[a+"-pre"],asc:sn.order[a+"-asc"],desc:sn.order[a+"-desc"]},render:sn.render[a],search:sn.search[a]};var f=function(v,g){sn[v][a]=g},d=function(v){Object.defineProperty(v,"_name",{value:a});var g=sn.detect.findIndex(function(x){return x._name===a});g===-1?sn.detect.unshift(v):sn.detect.splice(g,1,v)},h=function(v){sn.order[a+"-pre"]=v.pre,sn.order[a+"-asc"]=v.asc,sn.order[a+"-desc"]=v.desc};r===void 0&&(r=i,i=null),i==="className"?f("className",r):i==="detect"?d(r):i==="order"?h(r):i==="render"?f("render",r):i==="search"?f("search",r):i||(r.className&&f("className",r.className),r.detect!==void 0&&d(r.detect),r.order&&h(r.order),r.render!==void 0&&f("render",r.render),r.search!==void 0&&f("search",r.search))};$.types=function(){return sn.detect.map(function(a){return a._name})};var ks=function(a,i){return a=a!=null?a.toString().toLowerCase():"",i=i!=null?i.toString().toLowerCase():"",a.localeCompare(i,navigator.languages[0]||navigator.language,{numeric:!0,ignorePunctuation:!0})},Fv=function(a,i){return a=Pa(a),i=Pa(i),ks(a,i)};$.type("string",{detect:function(){return"string"},order:{pre:function(a){return En(a)&&typeof a!="boolean"?"":typeof a=="string"?a.toLowerCase():a.toString?a.toString():""}},search:Mr(!1)});$.type("string-utf8",{detect:{allOf:function(a){return!0},oneOf:function(a){return!En(a)&&navigator.languages&&typeof a=="string"&&a.match(/[^\x00-\x7F]/)}},order:{asc:ks,desc:function(a,i){return ks(a,i)*-1}},search:Mr(!1)});$.type("html",{detect:{allOf:function(a){return En(a)||typeof a=="string"&&a.indexOf("<")!==-1},oneOf:function(a){return!En(a)&&typeof a=="string"&&a.indexOf("<")!==-1}},order:{pre:function(a){return En(a)?"":a.replace?Pa(a).trim().toLowerCase():a+""}},search:Mr(!0)});$.type("html-utf8",{detect:{allOf:function(a){return En(a)||typeof a=="string"&&a.indexOf("<")!==-1},oneOf:function(a){return navigator.languages&&!En(a)&&typeof a=="string"&&a.indexOf("<")!==-1&&typeof a=="string"&&a.match(/[^\x00-\x7F]/)}},order:{asc:Fv,desc:function(a,i){return Fv(a,i)*-1}},search:Mr(!0)});$.type("date",{className:"dt-type-date",detect:{allOf:function(a){if(a&&!(a instanceof Date)&&!zv.test(a))return null;var i=Date.parse(a);return i!==null&&!isNaN(i)||En(a)},oneOf:function(a){return a instanceof Date||typeof a=="string"&&zv.test(a)}},order:{pre:function(a){var i=Date.parse(a);return isNaN(i)?-1/0:i}}});$.type("html-num-fmt",{className:"dt-type-numeric",detect:{allOf:function(a,i){var r=i.oLanguage.sDecimal;return vo(a,r,!0,!1)},oneOf:function(a,i){var r=i.oLanguage.sDecimal;return vo(a,r,!0,!1)}},order:{pre:function(a,i){var r=i.oLanguage.sDecimal;return zo(a,r,ad,ld)}},search:Mr(!0)});$.type("html-num",{className:"dt-type-numeric",detect:{allOf:function(a,i){var r=i.oLanguage.sDecimal;return vo(a,r,!1,!0)},oneOf:function(a,i){var r=i.oLanguage.sDecimal;return vo(a,r,!1,!1)}},order:{pre:function(a,i){var r=i.oLanguage.sDecimal;return zo(a,r,ad)}},search:Mr(!0)});$.type("num-fmt",{className:"dt-type-numeric",detect:{allOf:function(a,i){var r=i.oLanguage.sDecimal;return Ru(a,r,!0,!0)},oneOf:function(a,i){var r=i.oLanguage.sDecimal;return Ru(a,r,!0,!1)}},order:{pre:function(a,i){var r=i.oLanguage.sDecimal;return zo(a,r,ld)}}});$.type("num",{className:"dt-type-numeric",detect:{allOf:function(a,i){var r=i.oLanguage.sDecimal;return Ru(a,r,!1,!0)},oneOf:function(a,i){var r=i.oLanguage.sDecimal;return Ru(a,r,!1,!1)}},order:{pre:function(a,i){var r=i.oLanguage.sDecimal;return zo(a,r)}}});var zo=function(a,i,r,f){if(a!==0&&(!a||a==="-"))return-1/0;var d=typeof a;return d==="number"||d==="bigint"?a:(i&&(a=tm(a,i)),a.replace&&(r&&(a=a.replace(r,"")),f&&(a=a.replace(f,""))),a*1)};M.extend(!0,$.ext.renderer,{footer:{_:function(a,i,r){i.addClass(r.tfoot.cell)}},header:{_:function(a,i,r){i.addClass(r.thead.cell),a.oFeatures.bSort||i.addClass(r.order.none);var f=a.titleRow,d=i.closest("thead").find("tr"),h=i.parent().index();i.attr("data-dt-order")==="disable"||i.parent().attr("data-dt-order")==="disable"||f===!0&&h!==0||f===!1&&h!==d.length-1||typeof f=="number"&&h!==f||M(a.nTable).on("order.dt.DT column-visibility.dt.DT",function(v,g,x){if(a===g){var T=g.sortDetails;if(T){var A=tn(T,"col");if(!(v.type==="column-visibility"&&!A.includes(x))){var N,j=r.order,X=g.api.columns(i),U=a.aoColumns[X.flatten()[0]],B=X.orderable().includes(!0),Y="",F=X.indexes(),re=X.orderable(!0).flatten(),Se=a.iTabIndex,Te=g.orderHandler&&B;i.removeClass(j.isAsc+" "+j.isDesc).toggleClass(j.none,!B).toggleClass(j.canAsc,Te&&re.includes("asc")).toggleClass(j.canDesc,Te&&re.includes("desc"));var Oe=!0;for(N=0;N<F.length;N++)A.includes(F[N])||(Oe=!1);if(Oe){var ge=X.order();i.addClass(ge.includes("asc")?j.isAsc:""+ge.includes("desc")?j.isDesc:"")}var y=-1;for(N=0;N<A.length;N++)if(a.aoColumns[A[N]].bVisible){y=A[N];break}if(F[0]==y){var Ze=T[0],Ce=U.asSorting;i.attr("aria-sort",Ze.dir==="asc"?"ascending":"descending"),Y=Ce[Ze.index+1]?"Reverse":"Remove"}else i.removeAttr("aria-sort");if(B){var Tt=i.find(".dt-column-order");Tt.attr("role","button").attr("aria-label",B?U.ariaTitle+g.api.i18n("oAria.orderable"+Y):U.ariaTitle),Se!==-1&&Tt.attr("tabindex",Se)}}}}})}},layout:{_:function(a,i,r){var f=a.oClasses.layout,d=M("<div/>").attr("id",r.id||null).addClass(r.className||f.row).appendTo(i);$.ext.renderer.layout._forLayoutRow(r,function(h,v){if(!(h==="id"||h==="className")){var g="";v.table&&(d.addClass(f.tableRow),g+=f.tableCell+" "),h==="start"?g+=f.start:h==="end"?g+=f.end:g+=f.full,M("<div/>").attr({id:v.id||null,class:v.className?v.className:f.cell+" "+g}).append(v.contents).appendTo(d)}})},_forLayoutRow:function(a,i){var r=function(f){switch(f){case"":return 0;case"start":return 1;case"end":return 2;default:return 3}};Object.keys(a).sort(function(f,d){return r(f)-r(d)}).forEach(function(f){i(f,a[f])})}}});$.feature={};$.feature.register=function(a,i,r){$.ext.features[a]=i,r&&Yt.feature.push({cFeature:r,fnInit:i})};function oo(a,i,r){r&&(a[i]=r)}$.feature.register("div",function(a,i){var r=M("<div>")[0];return i&&(oo(r,"className",i.className),oo(r,"id",i.id),oo(r,"innerHTML",i.html),oo(r,"textContent",i.text)),r});$.feature.register("info",function(a,i){if(!a.oFeatures.bInfo)return null;var r=a.oLanguage,f=a.sTableId,d=M("<div/>",{class:a.oClasses.info.container});return i=M.extend({callback:r.fnInfoCallback,empty:r.sInfoEmpty,postfix:r.sInfoPostFix,search:r.sInfoFiltered,text:r.sInfo},i),a.aoDrawCallback.push(function(h){rb(h,i,d)}),a._infoEl||(d.attr({"aria-live":"polite",id:f+"_info",role:"status"}),M(a.nTable).attr("aria-describedby",f+"_info"),a._infoEl=d),d},"i");function rb(a,i,r){var f=a._iDisplayStart+1,d=a.fnDisplayEnd(),h=a.fnRecordsTotal(),v=a.fnRecordsDisplay(),g=v?i.text:i.empty;v!==h&&(g+=" "+i.search),g+=i.postfix,g=pd(a,g),i.callback&&(g=i.callback.call(a.oInstance,a,f,d,h,v,g)),r.html(g),We(a,null,"info",[a,r[0],g])}var Us=0;$.feature.register("search",function(a,i){if(!a.oFeatures.bFilter)return null;var r=a.oClasses.search,f=a.sTableId,d=a.oLanguage,h=a.oPreviousSearch,v='<input type="search" class="'+r.input+'"/>';i=M.extend({placeholder:d.sSearchPlaceholder,processing:!1,text:d.sSearch},i),i.text.indexOf("_INPUT_")===-1&&(i.text+="_INPUT_"),i.text=pd(a,i.text);var g=i.text.match(/_INPUT_$/),x=i.text.match(/^_INPUT_/),T=i.text.replace(/_INPUT_/,""),A="<label>"+i.text+"</label>";x?A="_INPUT_<label>"+T+"</label>":g&&(A="<label>"+T+"</label>_INPUT_");var N=M("<div>").addClass(r.container).append(A.replace(/_INPUT_/,v));N.find("label").attr("for","dt-search-"+Us),N.find("input").attr("id","dt-search-"+Us),Us++;var j=function(B){var Y=this.value;h.return&&B.key!=="Enter"||Y!=h.search&&hm(a,i.processing,function(){h.search=Y,Uu(a,h),a._iDisplayStart=0,Nr(a)})},X=a.searchDelay!==null?a.searchDelay:0,U=M("input",N).val(h.search).attr("placeholder",i.placeholder).on("keyup.DT search.DT input.DT paste.DT cut.DT",X?$.util.debounce(j,X):j).on("mouseup.DT",function(B){setTimeout(function(){j.call(U[0],B)},10)}).on("keypress.DT",function(B){if(B.keyCode==13)return!1}).attr("aria-controls",f);return M(a.nTable).on("search.dt.DT",function(B,Y){a===Y&&U[0]!==document.activeElement&&U.val(typeof h.search!="function"?h.search:"")}),N},"f");$.feature.register("paging",function(a,i){if(!a.oFeatures.bPaginate)return null;i=M.extend({buttons:$.ext.pager.numbers_length,type:a.sPaginationType,boundaryNumbers:!0,firstLast:!0,previousNext:!0,numbers:!0},i);var r=M("<div/>").addClass(a.oClasses.paging.container+(i.type?" paging_"+i.type:"")).append(M("<nav>").attr("aria-label","pagination").addClass(a.oClasses.paging.nav)),f=function(){Mm(a,r.children(),i)};return a.aoDrawCallback.push(f),M(a.nTable).on("column-sizing.dt.DT",f),r},"p");function ub(a){var i=[];return a.numbers&&i.push("numbers"),a.previousNext&&(i.unshift("previous"),i.push("next")),a.firstLast&&(i.unshift("first"),i.push("last")),i}function Mm(a,i,r){if(a._bInitComplete){var f=r.type?$.ext.pager[r.type]:ub,d=a.oLanguage.oAria.paginate||{},h=a._iDisplayStart,v=a._iDisplayLength,g=a.fnRecordsDisplay(),x=v===-1,T=x?0:Math.ceil(h/v),A=x?1:Math.ceil(g/v),N=[],j=[],X=f(r).map(function(ge){return ge==="numbers"?wm(T,A,r.buttons,r.boundaryNumbers):ge});N=N.concat.apply(N,X);for(var U=0;U<N.length;U++){var B=N[U],Y=fb(a,B,T,A),F=So(a,"pagingButton")(a,B,Y.display,Y.active,Y.disabled),re=typeof B=="string"?d[B]:d.number?d.number+(B+1):null;M(F.clicker).attr({"aria-controls":a.sTableId,"aria-disabled":Y.disabled?"true":null,"aria-current":Y.active?"page":null,"aria-label":re,"data-dt-idx":B,tabIndex:Y.disabled?-1:a.iTabIndex&&F.clicker[0].nodeName.toLowerCase()!=="span"?a.iTabIndex:null}),typeof B!="number"&&M(F.clicker).addClass(B),gm(F.clicker,{action:B},function(ge){ge.preventDefault(),dd(a,ge.data.action,!0)}),j.push(F.display)}var Se=So(a,"pagingContainer")(a,j),Te=i.find(document.activeElement).data("dt-idx");if(i.empty().append(Se),Te!==void 0&&i.find("[data-dt-idx="+Te+"]").trigger("focus"),j.length){var Oe=M(j[0]).outerHeight();r.buttons>1&&Oe>0&&M(i).height()>=Oe*2-10&&Mm(a,i,M.extend({},r,{buttons:r.buttons-2}))}}}function fb(a,i,r,f){var d=a.oLanguage.oPaginate,h={display:"",active:!1,disabled:!1};switch(i){case"ellipsis":h.display="&#x2026;";break;case"first":h.display=d.sFirst,r===0&&(h.disabled=!0);break;case"previous":h.display=d.sPrevious,r===0&&(h.disabled=!0);break;case"next":h.display=d.sNext,(f===0||r===f-1)&&(h.disabled=!0);break;case"last":h.display=d.sLast,(f===0||r===f-1)&&(h.disabled=!0);break;default:typeof i=="number"&&(h.display=a.fnFormatNumber(i+1),r===i&&(h.active=!0));break}return h}function wm(a,i,r,f){var d=[],h=Math.floor(r/2),v=f?2:1,g=f?1:0;return i<=r?d=Rl(0,i):r===1?d=[a]:r===3?a<=1?d=[0,1,"ellipsis"]:a>=i-2?(d=Rl(i-2,i),d.unshift("ellipsis")):d=["ellipsis",a,"ellipsis"]:a<=h?(d=Rl(0,r-v),d.push("ellipsis"),f&&d.push(i-1)):a>=i-1-h?(d=Rl(i-(r-v),i),d.unshift("ellipsis"),f&&d.unshift(0)):(d=Rl(a-h+v,a+h-g),d.push("ellipsis"),d.unshift("ellipsis"),f&&(d.push(i-1),d.unshift(0))),d}var Bs=0;$.feature.register("pageLength",function(a,i){var r=a.oFeatures;if(!r.bPaginate||!r.bLengthChange)return null;i=M.extend({menu:a.aLengthMenu,text:a.oLanguage.sLengthMenu},i);var f=a.oClasses.length,d=a.sTableId,h=i.menu,v=[],g=[],x;if(Array.isArray(h[0]))v=h[0],g=h[1];else for(x=0;x<h.length;x++)M.isPlainObject(h[x])?(v.push(h[x].value),g.push(h[x].label)):(v.push(h[x]),g.push(h[x]));var T=i.text.match(/_MENU_$/),A=i.text.match(/^_MENU_/),N=i.text.replace(/_MENU_/,""),j="<label>"+i.text+"</label>";A?j="_MENU_<label>"+N+"</label>":T&&(j="<label>"+N+"</label>_MENU_");var X="tmp-"+ +new Date,U=M("<div/>").addClass(f.container).append(j.replace("_MENU_",'<span id="'+X+'"></span>')),B=[];Array.prototype.slice.call(U.find("label")[0].childNodes).forEach(function(Se){Se.nodeType===Node.TEXT_NODE&&B.push({el:Se,text:Se.textContent})});var Y=function(Se){B.forEach(function(Te){Te.el.textContent=pd(a,Te.text,Se)})},F=M("<select/>",{"aria-controls":d,class:f.select});for(x=0;x<v.length;x++){var re=a.api.i18n("lengthLabels."+v[x],null);re===null&&(re=typeof g[x]=="number"?a.fnFormatNumber(g[x]):g[x]),F[0][x]=new Option(re,v[x])}return U.find("label").attr("for","dt-length-"+Bs),F.attr("id","dt-length-"+Bs),Bs++,U.find("#"+X).replaceWith(F),M("select",U).val(a._iDisplayLength).on("change.DT",function(){dm(a,M(this).val()),Nr(a)}),M(a.nTable).on("length.dt.DT",function(Se,Te,Oe){a===Te&&(M("select",U).val(Oe),Y(Oe))}),Y(a._iDisplayLength),U},"l");M.fn.dataTable=$;$.$=M;M.fn.dataTableSettings=$.settings;M.fn.dataTableExt=$.ext;M.fn.DataTable=function(a){return M(this).dataTable(a).api()};M.each($,function(a,i){M.fn.DataTable[a]=i});/*! DataTables Bootstrap 5 integration
 * © SpryMedia Ltd - datatables.net/license
 */let Bu=nd;Bu.extend(!0,$.defaults,{renderer:"bootstrap"});Bu.extend(!0,$.ext.classes,{container:"dt-container dt-bootstrap5",search:{input:"form-control form-control-sm"},length:{select:"form-select form-select-sm"},processing:{container:"dt-processing card"},layout:{row:"row mt-2 justify-content-between",cell:"d-md-flex justify-content-between align-items-center",tableCell:"col-12",start:"dt-layout-start col-md-auto me-auto",end:"dt-layout-end col-md-auto ms-auto",full:"dt-layout-full col-md"}});$.ext.renderer.pagingButton.bootstrap=function(a,i,r,f,d){var h=["dt-paging-button","page-item"];f&&h.push("active"),d&&h.push("disabled");var v=Bu("<li>").addClass(h.join(" ")),g=Bu("<button>",{class:"page-link",role:"link",type:"button"}).html(r).appendTo(v);return{display:v,clicker:g}};$.ext.renderer.pagingContainer.bootstrap=function(a,i){return Bu("<ul/>").addClass("pagination").append(i)};/*! ColumnControl 1.1.1
 * Copyright (c) SpryMedia Ltd - datatables.net/license
 *
 * SVG icons: ISC License
 * Copyright (c) for portions of Lucide are held by Cole Bemis 2013-2022 as part of Feather (MIT).
 * All other copyright (c) for Lucide are held by Lucide Contributors 2022.
 */let Om=nd;function nt(a,i,r,f){i===void 0&&(i=[]),r===void 0&&(r=null),f===void 0&&(f=[]);var d=document.createElement(a);return zm(d,i),r&&(d.innerHTML=r),f.forEach(function(h){d.appendChild(h)}),d}function zm(a,i){i&&(Array.isArray(i)||(i=[i]),i.forEach(function(r){a&&r&&a.classList.add(r)}))}function Ke(a){return'<svg xmlns="http://www.w3.org/2000/svg" width="16" height="16" viewBox="0 0 24 24" fill="none" stroke="currentColor" stroke-width="2" stroke-linecap="round" stroke-linejoin="round">'+a+"</svg>"}var Ia={chevronRight:Ke('<path d="m9 18 6-6-6-6"/>'),columns:Ke('<rect width="18" height="18" x="3" y="3" rx="2"/><path d="M9 3v18"/><path d="M15 3v18"/>'),contains:Ke('<path d="M10 3h4v18h-4z"/><path d="M18 8h3v9h-3"/><path d="M6 17H3V8h3"/>'),empty:Ke('<circle cx="12" cy="12" r="10"/>'),ends:Ke('<path d="M21 3h-4v18h4z"/><path d="M13 8H3v9h10"/>'),equal:Ke('<line x1="5" x2="19" y1="9" y2="9"/><line x1="5" x2="19" y1="15" y2="15"/>'),greater:Ke('<path d="m9 18 6-6-6-6"/>'),greaterOrEqual:Ke('<path d="m9 16 6-6-6-6"/><path d="m9 21 6-6"/>'),groupAdd:Ke('<path d="M6 21v-7.5m-3.549 3.75H9.75"/><rect width="13.5" height="7.5" x="3" y="3" rx="1.5"/><rect width="7.5" height="7.5" x="13.5" y="13.5" fill="currentColor" rx="1.5"/>'),groupClear:Ke('<rect width="13.5" height="7.5" x="3" y="3" rx="1.5"/><rect width="7.5" height="7.5" x="13.5" y="13.5" rx="1.5"/>'),groupTop:Ke('<rect width="13.5" height="7.5" x="3" y="3" fill="currentColor" rx="1.5"/><rect width="7.5" height="7.5" x="13.5" y="13.5" rx="1.5"/>'),groupRemove:Ke('<path d="M2.451 17.25H9.75"/><rect width="13.5" height="7.5" x="3" y="3" rx="1.5"/><rect width="7.5" height="7.5" x="13.5" y="13.5" rx="1.5"/>'),less:Ke('<path d="m15 18-6-6 6-6"/>'),lessOrEqual:Ke('<path d="m15 16-6-6 6-6"/><path d="m15 21-6-6"/>'),menu:Ke('<line x1="4" x2="20" y1="12" y2="12"/><line x1="4" x2="20" y1="6" y2="6"/><line x1="4" x2="20" y1="18" y2="18"/>'),move:Ke('<line x1="12" x2="12" y1="3" y2="21"/><polyline points="8 8 4 12 8 16"/><polyline points="16 16 20 12 16 8"/>'),moveLeft:Ke('<path d="m9 6-6 6 6 6"/><path d="M3 12h14"/><path d="M21 19V5"/>'),moveRight:Ke('<path d="M3 5v14"/><path d="M21 12H7"/><path d="m15 18 6-6-6-6"/>'),notContains:Ke('<path d="M15 4 9 20"/><path d="M3 8h18v9H3z"/>'),notEmpty:Ke('<circle cx="12" cy="12" r="10"/><line x1="9" x2="15" y1="15" y2="9"/>'),notEqual:Ke('<path d="M5 9h14"/><path d="M5 15h14"/><path d="M15 5 9 19"/>'),orderAddAsc:Ke('<path d="M17 21v-8"/><path d="M3 4h6"/><path d="M3 8h9"/><path d="M3 12h10"/><path d="M13 17h8"/>'),orderAddDesc:Ke('<path d="M17 21v-8"/><path d="M3 4h12"/><path d="M3 8h9"/><path d="M3 12h6"/><path d="M13 17h8"/>'),orderAsc:Ke('<path d="m3 8 4-4 4 4"/><path d="M7 4v16"/><path d="M11 12h4"/><path d="M11 16h7"/><path d="M11 20h10"/>'),orderClear:Ke('<path d="m21 21-8-8"/><path d="M3 4h12"/><path d="M3 8h9"/><path d="M3 12h6"/><path d="m13 21 8-8"/>'),orderDesc:Ke('<path d="m3 16 4 4 4-4"/><path d="M7 20V4"/><path d="M11 4h10"/><path d="M11 8h7"/><path d="M11 12h4"/>'),orderRemove:Ke('<path d="M3 4h12"/><path d="M3 8h9"/><path d="M3 12h6"/><path d="M13 17h8"/>'),orderNone:Ke('<path d="m3 8 4-4 4 4"/><path d="m11 16-4 4-4-4"/><path d="M7 4v16"/><path d="M15 8h6"/><path d="M15 16h6"/><path d="M13 12h8"/>'),search:Ke('<circle cx="11" cy="11" r="8"/><path d="m21 21-4.3-4.3"/>'),searchClear:Ke('<path d="m13.5 8.5-5 5"/><path d="m8.5 8.5 5 5"/><circle cx="11" cy="11" r="8"/><path d="m21 21-4.3-4.3"/>'),starts:Ke('<path d="M3 3h4v18H3z"/><path d="M11 8h10v9H11"/>'),tick:Ke('<path d="M20 6 9 17l-5-5"/>'),x:Ke('<path d="M18 6 6 18"/><path d="m6 6 12 12"/>')};function ob(a){a===void 0&&(a=null),document.querySelectorAll("div.dtcc-dropdown").forEach(function(i){(a===null||!i.contains(a.target))&&(i._close(),a._closed||(a._closed=[]),a._closed.push(i))})}function Lm(a,i){return i.closest("div.dtfh-floatingparent")||a.table().container()}function Hm(a,i,r){var f=r.closest("div.dt-column-header"),d=Lm(i,r),h=getComputedStyle(f),v=a.offsetWidth,g=cb(d,r),x,T;T=g.top+r.offsetHeight,h.flexDirection==="row-reverse"?x=g.left:x=g.left-v+r.offsetWidth;var A=d.offsetWidth;x+v>A&&(x-=x+v-A),x<0&&(x=0),a.style.top=T+"px",a.style.left=x+"px"}function Wv(a,i,r){var f=Lm(i,r.element());a._shown=!0,f.append(a),Hm(a,i,r.element()),r.element().setAttribute("aria-expanded","true");var d=function(h){if(!a._shown){document.body.removeEventListener("click",d);return}if(!(h.target===a||a.contains(h.target))){var v=document.querySelector("div.dt-datetime");v&&(h.target===v||v.contains(h.target))||(a._close(),document.body.removeEventListener("click",d))}};return document.body.addEventListener("click",d),d}function cb(a,i){for(var r=0,f=0;i&&i!==a&&i!==document.body;)r+=i.offsetTop,f+=i.offsetLeft,i.scrollTop&&(f-=i.scrollTop),i.scrollLeft&&(f-=i.scrollLeft),i=i.offsetParent;return{top:r,left:f}}function sb(a,i){return function(r){if(a._shown){var f=Array.from(a.querySelectorAll("a, button, input, select")),d=document.activeElement;if(r.key==="Escape"){a._close(),i.focus();return}else if(r.key!=="Tab"||f.length===0)return;f.includes(d)?r.shiftKey?d===f[0]&&(f[f.length-1].focus(),r.preventDefault()):d===f[f.length-1]&&(f[0].focus(),r.preventDefault()):(f[0].focus(),r.preventDefault())}}}var Fs={classes:{container:"dtcc-dropdown",liner:"dtcc-dropdown-liner"},defaults:{className:"dropdown",content:[],icon:"menu",text:"More..."},init:function(a){var i=this.dt(),r=nt("div",Fs.classes.container,"",[nt("div",Fs.classes.liner)]);r._shown=!1,r._close=function(){r.remove(),r._shown=!1,d.element().setAttribute("aria-expanded","false")},r.setAttribute("role","dialog"),r.setAttribute("aria-label",i.i18n("columnControl.dropdown",a.text)),i.on("fixedheader-mode",function(){r._shown&&Wv(r,i,a._parents?a._parents[0]:d)});var f=r.childNodes[0],d=new Xt(i,this).text(i.i18n("columnControl.dropdown",a.text)).icon(a.icon).className(a.className).dropdownDisplay(f).handler(function(T){if(!(T._closed&&T._closed.includes(r))){Wv(r,i,a._parents?a._parents[0]:d);var A=r.querySelector("input, a, button");A&&T.type==="keypress"&&A.focus()}});d.element().setAttribute("aria-haspopup","dialog"),d.element().setAttribute("aria-expanded","false");for(var h=0;h<a.content.length;h++){var v=this.resolve(a.content[h]);v.config._parents||(v.config._parents=[]),v.config._parents.push(d);var g=v.plugin.init.call(this,v.config);f.appendChild(g)}a._parents&&a._parents.length&&d.extra("chevronRight"),i.on("columns-reordered",function(){Hm(r,i,d.element())});var x=sb(r,d.element());return document.body.addEventListener("keydown",x),i.on("destroy",function(){document.body.removeEventListener("keydown",x)}),d.element()}},db=0,Xt=(function(){function a(i,r){this._s={active:!1,activeList:[],buttonClick:null,dt:null,enabled:!0,host:null,label:"",namespace:"",value:null},this._s.dt=i,this._s.host=r,this._dom={button:nt("button",a.classes.container),dropdownDisplay:null,extra:nt("span","dtcc-button-extra"),icon:nt("span","dtcc-button-icon"),state:nt("span","dtcc-button-state"),text:nt("span","dtcc-button-text")},this._dom.button.setAttribute("type","button"),this._dom.button.append(this._dom.icon),this._dom.button.append(this._dom.text),this._dom.button.append(this._dom.state),this._dom.button.append(this._dom.extra),this.enable(!0)}return a.prototype.active=function(i){return i===void 0?this._s.active:(this._s.active=i,this._checkActive(),this)},a.prototype.activeList=function(i,r){return this._s.activeList[i]=r,this._checkActive(),this},a.prototype.checkDisplay=function(){for(var i=0,r=this._dom.dropdownDisplay.childNodes,f=0;f<r.length;f++)r[f].style.display!=="none"&&i++;return i===0&&(this._dom.button.style.display="none"),this},a.prototype.className=function(i){return this._dom.button.classList.add("dtcc-button_"+i),this},a.prototype.destroy=function(){this._s.buttonClick&&(this._dom.button.removeEventListener("click",this._s.buttonClick),this._dom.button.removeEventListener("keypress",this._s.buttonClick)),this._s.host.destroyRemove(this)},a.prototype.dropdownDisplay=function(i){return this._dom.dropdownDisplay=i,this},a.prototype.element=function(){return this._dom.button},a.prototype.enable=function(i){return i===void 0?this._s.enabled:(this._dom.button.classList.toggle("dtcc-button_disabled",!i),this._s.enabled=i,this)},a.prototype.extra=function(i){return this._dom.extra.innerHTML=i?Ia[i]:"",this},a.prototype.handler=function(i){var r=this,f=function(d){ob(d),d.stopPropagation(),d.preventDefault(),r._s.enabled&&i(d)};return this._s.buttonClick=f,this._s.namespace="dtcc-"+db++,this._dom.button.addEventListener("click",f),this._dom.button.addEventListener("keypress",f),this._s.host.destroyAdd(this),this},a.prototype.icon=function(i){return this._dom.icon.innerHTML=i?Ia[i]:"",this},a.prototype.text=function(i){return i===void 0?this._s.label:(this._dom.text.innerHTML=i,this._s.label=i,this._dom.button.setAttribute("aria-label",i),this)},a.prototype.value=function(i){return i===void 0?this._s.value:(this._s.value=i,this)},a.prototype._checkActive=function(){return this._s.active===!0||Object.values(this._s.activeList).includes(!0)?(this._dom.state.innerHTML=Ia.tick,this._dom.button.classList.add("dtcc-button_active")):(this._dom.state.innerHTML="",this._dom.button.classList.remove("dtcc-button_active")),this},a.classes={container:"dtcc-button"},a})(),_d=(function(){function a(i,r,f){var d=this;this._s={buttons:[],dt:null,handler:function(){},host:null,search:""},this._s.dt=i,this._s.host=r,this._dom={buttons:nt("div","dtcc-list-buttons"),container:nt("div",a.classes.container),controls:nt("div","dtcc-list-controls"),empty:nt("div","dtcc-list-empty",i.i18n("columnControl.list.empty","No options")),title:nt("div","dtcc-list-title"),selectAll:nt("button","dtcc-list-selectAll",i.i18n("columnControl.list.all","Select all")),selectAllCount:nt("span"),selectNone:nt("button","dtcc-list-selectNone",i.i18n("columnControl.list.none","Deselect")),selectNoneCount:nt("span"),search:nt("input",a.classes.input)};var h=this._dom;h.search.setAttribute("type","text"),h.container.append(h.title),h.container.append(h.controls),h.container.append(h.empty),h.container.append(h.buttons),f.select&&(h.controls.append(h.selectAll),h.controls.append(h.selectNone),h.selectAll.append(h.selectAllCount),h.selectNone.append(h.selectNoneCount),h.selectAll.setAttribute("type","button"),h.selectNone.setAttribute("type","button"));var v=function(){d._s.search=h.search.value,d._redraw()},g=function(T){d.selectAll(),d._s.handler(T,null,d._s.buttons,!0),d._updateCount()},x=function(T){d.selectNone(),d._s.handler(T,null,d._s.buttons,!0),d._updateCount()};f.search&&(h.controls.append(h.search),h.search.setAttribute("placeholder",i.i18n("columnControl.list.search","Search...")),h.search.addEventListener("input",v)),h.selectAll.addEventListener("click",g),h.selectNone.addEventListener("click",x),i.on("destroy",function(){h.selectAll.removeEventListener("click",g),h.selectNone.removeEventListener("click",x),h.search.removeEventListener("input",v)})}return a.prototype.add=function(i,r){var f=this;Array.isArray(i)||(i=[i]);for(var d=function(x){var T=i[x],A=new Xt(h._s.dt,h._s.host).active(T.active||!1).handler(function(N){f._s.handler(N,A,f._s.buttons,!0),f._updateCount()}).icon(T.icon||"").text(T.label!==""?T.label:h._s.dt.i18n("columnControl.list.empty","Empty")).value(T.value);T.label===""&&A.className("empty"),h._s.buttons.push(A)},h=this,v=0;v<i.length;v++)d(v);var g=this._s.buttons.length;return(r===!0||r===void 0)&&(this._dom.selectAllCount.innerHTML=g?"("+g+")":"",this._redraw()),this},a.prototype.button=function(i){for(var r=this._s.buttons,f=0;f<r.length;f++)if(r[f].value()===i)return r[f];return null},a.prototype.clear=function(){for(var i=0;i<this._s.buttons.length;i++)this._s.buttons[i].destroy();return this._dom.buttons.replaceChildren(),this._s.buttons.length=0,this},a.prototype.element=function(){return this._dom.container},a.prototype.handler=function(i){return this._s.handler=i,this},a.prototype.searchListener=function(i){var r=this;return i.on("cc-search-clear",function(f,d){d===r._s.host.idx()&&(r.selectNone(),r._s.handler(f,null,r._s.buttons,!1),r._s.search="",r._dom.search.value="",r._redraw(),r._updateCount())}),this},a.prototype.selectAll=function(){for(var i=0;i<this._s.buttons.length;i++)this._s.buttons[i].active(!0);return this},a.prototype.selectNone=function(){for(var i=0;i<this._s.buttons.length;i++)this._s.buttons[i].active(!1);return this},a.prototype.title=function(i){return this._dom.title.innerHTML=i,this},a.prototype.values=function(i){var r,f=[],d=this._s.buttons;if(i!==void 0){for(r=0;r<d.length;r++)i.includes(d[r].value())&&d[r].active(!0);return this._updateCount(),this}for(r=0;r<d.length;r++)d[r].active()&&f.push(d[r].value());return f},a.prototype._updateCount=function(){var i=this.values().length;this._dom.selectNoneCount.innerHTML=i?"("+i+")":""},a.prototype._redraw=function(){var i=this._s.buttons,r=this._dom.buttons,f=this._s.search.toLowerCase();r.replaceChildren();for(var d=0;d<i.length;d++){var h=i[d];(!f||h.text().toLowerCase().includes(f))&&r.appendChild(h.element())}this._dom.empty.style.display=i.length===0?"block":"none",r.style.display=i.length>0?"block":"none"},a.classes={container:"dtcc-list",input:"dtcc-list-search"},a})(),hb={defaults:{className:"colVis",columns:"",search:!1,select:!1,title:"Column visibility"},init:function(a){var i=this.dt(),r=new _d(i,this,{search:a.search,select:a.select}).title(i.i18n("columnControl.colVis",a.title)).handler(function(h,v,g){v&&v.active(!v.active()),f(g)}),f=function(h){for(var v=0;v<h.length;v++){var g=h[v],x=g.value(),T=i.column(x);g.active()&&!T.visible()?T.visible(!0):!g.active()&&T.visible()&&T.visible(!1)}},d=function(){var h=i.columns(a.columns);h.every(function(){r.add({active:this.visible(),label:this.title(),value:this.index()})})};return d(),i.on("column-visibility",function(h,v,g,x){var T=r.button(g);T&&T.active(x)}),i.on("columns-reordered",function(h,v){r.clear(),d()}),r.element()}},pb={defaults:{className:"colVis",columns:"",search:!1,select:!1,text:"Column visibility",title:"Column visibility"},extend:function(a){var i=this.dt();return{extend:"dropdown",icon:"columns",text:i.i18n("columnControl.colVisDropdown",a.text),content:[Object.assign(a,{extend:"colVis"})]}}},vb={defaults:{className:"reorder",icon:"move",text:"Reorder columns"},init:function(a){var i=this,r=this.dt(),f=new Xt(r,this).text(r.i18n("columnControl.reorder",a.text)).icon(a.icon).className(a.className);return this.idx()===0&&f.enable(!1),r.on("columns-reordered",function(d,h){f.enable(i.idx()>0)}),r.init().colReorder||new $.ColReorder(r,{}),f.element()}},mb={defaults:{className:"reorderLeft",icon:"moveLeft",text:"Move column left"},init:function(a){var i=this,r=this.dt(),f=new Xt(r,this).text(r.i18n("columnControl.reorderLeft",a.text)).icon(a.icon).className(a.className).handler(function(){var d=i.idx();d>0&&r.colReorder.move(d,d-1)});return this.idx()===0&&f.enable(!1),r.on("columns-reordered",function(d,h){f.enable(i.idx()>0)}),f.element()}},yb={defaults:{className:"reorderRight",icon:"moveRight",text:"Move column right"},init:function(a){var i=this,r=this.dt(),f=new Xt(r,this).text(r.i18n("columnControl.reorderRight",a.text)).icon(a.icon).className(a.className).handler(function(){var d=i.idx();d<r.columns().count()-1&&r.colReorder.move(d,d+1)});return this.idx()===r.columns().count()-1&&f.enable(!1),r.on("columns-reordered",function(d,h){f.enable(i.idx()<r.columns().count()-1)}),f.element()}},bb={defaults:{className:"order",iconAsc:"orderAsc",iconDesc:"orderDesc",iconNone:"orderNone",statusOnly:!1,text:"Toggle ordering"},init:function(a){var i=this,r=this.dt(),f=new Xt(r,this).text(r.i18n("columnControl.order",a.text)).icon("orderAsc").className(a.className);return a.statusOnly||r.order.listener(f.element(),$.versionCheck("2.3.2")?function(){return[i.idx()]}:this.idx(),function(){}),r.on("order",function(d,h,v){var g=v.find(function(x){return x.col===i.idx()});g?g.dir==="asc"?f.active(!0).icon(a.iconAsc):g.dir==="desc"&&f.active(!0).icon(a.iconDesc):f.active(!1).icon(a.iconNone)}),f.element()}},gb={defaults:{className:"orderAddAsc",icon:"orderAddAsc",text:"Add Sort Ascending"},init:function(a){var i=this,r=this.dt(),f=new Xt(r,this).text(r.i18n("columnControl.orderAddAsc",a.text)).icon(a.icon).className(a.className).handler(function(){var d=r.order();d.push([i.idx(),"asc"]),r.draw()});return r.on("order",function(d,h,v){var g=v.some(function(x){return x.col===i.idx()});f.enable(!g)}),f.element()}},Sb={defaults:{className:"orderAddDesc",icon:"orderAddDesc",text:"Add Sort Descending"},init:function(a){var i=this,r=this.dt(),f=new Xt(r,this).text(r.i18n("columnControl.orderAddDesc",a.text)).icon(a.icon).className(a.className).handler(function(){var d=r.order();d.push([i.idx(),"desc"]),r.draw()});return r.on("order",function(d,h,v){var g=v.some(function(x){return x.col===i.idx()});f.enable(!g)}),f.element()}},_b={defaults:{className:"orderAsc",icon:"orderAsc",text:"Sort Ascending"},init:function(a){var i=this,r=this.dt(),f=new Xt(r,this).text(r.i18n("columnControl.orderAsc",a.text)).icon(a.icon).className(a.className).handler(function(){i.dt().order([{idx:i.idx(),dir:"asc"}]).draw()});return r.on("order",function(d,h,v){var g=v.some(function(x){return x.col===i.idx()&&x.dir==="asc"});f.active(g)}),f.element()}},xb={defaults:{className:"orderClear",icon:"orderClear",text:"Clear sort"},init:function(a){var i=this.dt(),r=new Xt(i,this).text(i.i18n("columnControl.orderClear",a.text)).icon(a.icon).className(a.className).handler(function(){i.order([]).draw()});return i.on("order",function(f,d,h){r.enable(h.length>0)}),i.order().length===0&&r.enable(!1),r.element()}},Db={defaults:{className:"orderDesc",icon:"orderDesc",text:"Sort Descending"},init:function(a){var i=this,r=this.dt(),f=new Xt(r,this).text(r.i18n("columnControl.orderDesc",a.text)).icon(a.icon).className(a.className).handler(function(){i.dt().order([{idx:i.idx(),dir:"desc"}]).draw()});return r.on("order",function(d,h,v){var g=v.some(function(x){return x.col===i.idx()&&x.dir==="desc"});f.active(g)}),f.element()}},Tb={defaults:{className:"orderRemove",icon:"orderRemove",text:"Remove from sort"},init:function(a){var i=this,r=this.dt(),f=new Xt(r,this).text(r.i18n("columnControl.orderRemove",a.text)).icon(a.icon).className(a.className).handler(function(){var d=r.order(),h=d.findIndex(function(v){return v[0]===i.idx()});d.splice(h,1),r.order(d).draw()});return r.on("order",function(d,h,v){var g=v.some(function(x){return x.col===i.idx()});f.enable(g)}),f.enable(!1),f.element()}},Cb={defaults:{className:"order",iconAsc:"orderAsc",iconDesc:"orderDesc",iconNone:"orderNone",statusOnly:!0,text:"Sort status"},extend:function(a){return Object.assign(a,{extend:"order"})}};function jm(a,i){var r=Ci(a),f=r.indexOf(i);return f===-1&&(r.push(i),a.rowGroup().dataSrc(r)),r}function Ci(a){var i=a.rowGroup().dataSrc();return Array.isArray(i)?i:[i]}function Rm(a){a.rowGroup().dataSrc([])}function qm(a,i){var r=Ci(a),f=r.indexOf(i);return f!==-1&&(r.splice(f,1),a.rowGroup().dataSrc(r)),r}var Ab={defaults:{className:"rowGroup",icon:"groupTop",order:!0,text:"Group rows"},init:function(a){var i=this,r=this.dt(),f=new Xt(r,this).text(r.i18n("columnControl.rowGroup",a.text)).icon(a.icon).className(a.className).handler(function(){var d=r.column(i.idx()).dataSrc();f.active()?qm(r,d):(Rm(r),jm(r,d),a.order!==!1&&r.order([i.idx(),"asc"])),r.draw()});return r.on("rowgroup-datasrc",function(){var d=Ci(r),h=r.column(i.idx()).dataSrc();f.active(d.includes(h))}),f.element()}},Eb={defaults:{className:"rowGroupAdd",icon:"groupAdd",order:!0,text:"Add to grouping"},init:function(a){var i=this,r=this.dt(),f=new Xt(r,this).text(r.i18n("columnControl.rowGroup",a.text)).icon(a.icon).className(a.className).handler(function(){var d=r.column(i.idx()).dataSrc();f.enable()&&jm(r,d),r.draw()});return r.on("rowgroup-datasrc",function(){var d=Ci(r),h=r.column(i.idx()).dataSrc();f.enable(!d.includes(h))}),f.element()}},Nb={defaults:{className:"rowGroupClear",icon:"groupClear",text:"Clear all grouping"},init:function(a){var i=this.dt(),r=new Xt(i,this).text(i.i18n("columnControl.rowGroup",a.text)).icon(a.icon).className(a.className).handler(function(){Rm(i),i.draw()});return i.on("rowgroup-datasrc",function(){r.enable(Ci(i).length>0)}),r.enable(Ci(i).length>0),r.element()}},Mb={defaults:{className:"rowGroupRemove",icon:"groupRemove",order:!0,text:"Remove from grouping"},init:function(a){var i=this,r=this.dt(),f=new Xt(r,this).text(r.i18n("columnControl.rowGroup",a.text)).icon(a.icon).className(a.className).handler(function(){var d=r.column(i.idx()).dataSrc();f.enable()&&(qm(r,d),r.draw())});return r.on("rowgroup-datasrc",function(){var d=Ci(r),h=r.column(i.idx()).dataSrc();f.enable(d.includes(h))}),f.enable(!1),f.element()}},Lo=(function(){function a(i,r){var f=this;this._type="text",this._sspTransform=null,this._sspData={},this._dt=i,this._idx=r,this._dom={clear:nt("span","dtcc-search-clear",Ia.x),container:nt("div",a.classes.container),typeIcon:nt("div","dtcc-search-type-icon"),searchIcon:nt("div","dtcc-search-icon",Ia.search),input:nt("input",a.classes.input),inputs:nt("div"),select:nt("select",a.classes.select),title:nt("div","dtcc-search-title")};var d=this._dom,h=r;d.input.setAttribute("type","text"),d.container.append(d.title,d.inputs),d.inputs.append(d.typeIcon,d.select,d.searchIcon,d.clear,d.input);var v=function(){f.runSearch()},g=function(){d.typeIcon.innerHTML=Ia[d.select.value],f.runSearch()},x=function(){f.clear()};d.input.addEventListener("input",v),d.select.addEventListener("input",g),d.clear.addEventListener("click",x),i.on("destroy",function(){d.input.removeEventListener("input",v),d.select.removeEventListener("input",g),d.clear.removeEventListener("click",x)}),i.on("stateSaveParams.DT",function(T,A,N){N.columnControl||(N.columnControl={}),N.columnControl[f._idx]||(N.columnControl[f._idx]={}),N.columnControl[f._idx].searchInput={logic:d.select.value,type:f._type,value:d.input.value}}),i.on("stateLoaded.DT",function(T,A,N){f._stateLoad(N)}),i.on("columns-reordered.DT",function(T,A){f._idx=i.colReorder.transpose(h,"fromOriginal")}),i.on("cc-search-clear.DT",function(T,A){A===f._idx&&(f._loadingState=!0,f.clear(),f._loadingState=!1)}),i.page.info().serverSide&&i.on("preXhr.DT",function(T,A,N){if(!(!N.columns||!N.columns[f._idx])){N.columns[f._idx].columnControl||(N.columns[f._idx].columnControl={});var j=f._dom.input.value;f._sspTransform&&(j=f._sspTransform(j)),N.columns[f._idx].columnControl.search=Object.assign({value:j,logic:f._dom.select.value,type:f._type},f._sspData)}})}return a.prototype.addClass=function(i){return this._dom.container.classList.add(i),this},a.prototype.clear=function(){return this.set(this._dom.select.children[0].getAttribute("value"),""),this},a.prototype.clearable=function(i){return i||this._dom.clear.remove(),this},a.prototype.element=function(){return this._dom.container},a.prototype.input=function(){return this._dom.input},a.prototype.options=function(i){for(var r=this._dom.select,f=0;f<i.length;f++)r.add(new Option(i[f].label,i[f].value));return this._dom.typeIcon.innerHTML=Ia[i[0].value],this},a.prototype.placeholder=function(i){if(i){var r=this._dt.column(this._idx).title();this._dom.input.placeholder=i.replace("[title]",r)}return this},a.prototype.runSearch=function(){var i=this._dom,r=i.select.value==="empty"||i.select.value==="notEmpty"||i.input.value!=="";i.container.classList.toggle("dtcc-search_active",r),this._search&&(this._lastValue!==i.input.value||this._lastType!==i.select.value)&&(this._search(i.select.value,i.input.value,this._loadingState),this._lastValue=i.input.value,this._lastType=i.select.value)},a.prototype.search=function(i){return this._search=i,this._stateLoad(this._dt.state.loaded()),this},a.prototype.set=function(i,r){var f=this._dom;return f.input.value=r,f.select.value=i,f.typeIcon.innerHTML=Ia[f.select.value],this.runSearch(),this},a.prototype.sspTransform=function(i){return this._sspTransform=i,this},a.prototype.sspData=function(i){return this._sspData=i,this},a.prototype.title=function(i){if(i){var r=this._dt.column(this._idx).title();this._dom.title.innerHTML=i.replace("[title]",r)}return this},a.prototype.titleAttr=function(i){if(i){var r=this._dt.column(this._idx).title();this._dom.input.title=i.replace("[title]",r)}return this},a.prototype.type=function(i){return this._type=i,this},a.prototype._stateLoad=function(i){var r,f,d=this._dom,h=this._idx,v=(f=(r=i?.columnControl)===null||r===void 0?void 0:r[h])===null||f===void 0?void 0:f.searchInput;v&&(this._loadingState=!0,d.select.value=v.logic,d.input.value=v.value,d.select.dispatchEvent(new Event("input")),this._loadingState=!1)},a.classes={container:["dtcc-content","dtcc-search"],input:"",select:""},a})(),Ws={defaults:{clear:!0,format:"",mask:"",placeholder:"",title:"",titleAttr:""},init:function(a){var i=this,r=!1,f=$.use("moment"),d=$.use("luxon"),h=this.dt(),v="columnControl.search.datetime.",g="",x="",T,A=new Lo(h,this.idx()).type("date").addClass("dtcc-searchDateTime").sspTransform(function(N){return Ob(N,g,f,d)}).sspData({mask:a.mask}).clearable(a.clear).placeholder(a.placeholder).title(a.title).titleAttr(a.titleAttr).options([{label:h.i18n(v+"equal","Equals"),value:"equal"},{label:h.i18n(v+"notEqual","Does not equal"),value:"notEqual"},{label:h.i18n(v+"greater","After"),value:"greater"},{label:h.i18n(v+"less","Before"),value:"less"},{label:h.i18n(v+"empty","Empty"),value:"empty"},{label:h.i18n(v+"notEmpty","Not empty"),value:"notEmpty"}]).search(function(N,j,X){if(h.page.info().serverSide){X||h.draw();return}var U=a.mask,B=h.column(i.idx()),Y=j===""?"":ju(T&&r?T.val():j.trim(),g,f,d,U);if(N==="empty")B.search.fixed("dtcc",function(F){return!F});else if(N==="notEmpty")B.search.fixed("dtcc",function(F){return!!F});else{if(B.search.fixed("dtcc")===""&&Y==="")return;Y?N==="equal"?B.search.fixed("dtcc",function(F){return ju(F,x,f,d,U)==Y}):N==="notEqual"?B.search.fixed("dtcc",function(F){return ju(F,x,f,d,U)!=Y}):N==="greater"?B.search.fixed("dtcc",function(F){return ju(F,x,f,d,U)>Y}):N==="less"&&B.search.fixed("dtcc",function(F){return ju(F,x,f,d,U)<Y}):B.search.fixed("dtcc","")}a._parents&&a._parents.forEach(function(F){return F.activeList(i.unique(),!!B.search.fixed("dtcc"))}),X||B.draw()});return h.ready(function(){var N=$.use("datetime");x=wb(h,i.idx()),g=a.format?a.format:x,N&&(T=new N(A.input(),{format:g,i18n:h.settings()[0].oLanguage.datetime,onChange:function(){r=!0,A.runSearch(),r=!1}}))}),A.element()}};function wb(a,i){var r=a.column(i).type();if(r)if(r==="datetime"){var f=a.settings()[0].aoColumns[i].mRender,d=f("1999-08-07T23:05:04Z","display"),h=f("1999-08-07T03:05:04Z","display"),v=h.includes("03");if($.use("moment"))return d.replace("23",v?"HH":"H").replace("11",v?"hh":"h").replace("05","mm").replace("04","ss").replace("PM","A").replace("pm","a").replace("07","DD").replace("7","D").replace("08","MM").replace("8","M").replace("1999","YYYY").replace("99","YY");if($.use("luxon"))return d.replace("23",v?"HH":"H").replace("11",v?"hh":"h").replace("05","mm").replace("04","ss").replace("PM","a").replace("07","dd").replace("7","d").replace("08","MM").replace("8","M").replace("1999","yyyy").replace("99","yy");if(d.includes("23")&&d.includes("1999"))return"YYYY-MM-DD hh:mm:ss";if(d.includes("23"))return"hh:mm:ss"}else{if(r.includes("datetime-"))return r.replace(/datetime-/g,"");if(r.includes("moment"))return r.replace(/moment-/g,"");if(r.includes("luxon"))return r.replace(/luxon-/g,"")}else return"YYYY-MM-DD";return"YYYY-MM-DD"}function ju(a,i,r,f,d){var h;return a===""?"":(a instanceof Date?h=a:i!=="YYYY-MM-DD"&&(r||f)?h=new Date(r?r(a,i).unix()*1e3:f.DateTime.fromFormat(a,i).toMillis()):h=new Date(a.replace(/\//g,"-")),d&&(d.includes("YYYY")||h.setFullYear(1970),d.includes("MM")||h.setUTCMonth(0),d.includes("DD")||h.setUTCDate(1),d.includes("hh")||h.setUTCHours(0),d.includes("mm")||h.setUTCMinutes(0),d.includes("ss")||h.setUTCSeconds(0),d.includes("sss")||h.setUTCMilliseconds(0)),h.getTime())}function Ob(a,i,r,f){return a===""?"":i!=="YYYY-MM-DD"&&r?r.utc(a,i).toISOString():i!=="YYYY-MM-DD"&&f?f.DateTime.fromFormat(a,i).toISO():(a=a.replace(/\//g,"-"),a)}function $s(a,i){var r=a.values();a.clear();for(var f=0;f<i.length;f++)typeof i[f]=="object"?a.add({active:!1,label:i[f].label,value:i[f].value},f===i.length-1):a.add({active:!1,label:i[f],value:i[f]},f===i.length-1);r.length&&a.values(r)}function $v(a,i){var r,f,d=(f=(r=i?.columnControl)===null||r===void 0?void 0:r[a])===null||f===void 0?void 0:f.searchList;if(d)return d}function Um(a,i){var r,f=(r=a.ajax.json())===null||r===void 0?void 0:r.columnControl,d=a.column(i),h=d.name(),v=d.dataSrc();return f&&f[h]?f[h]:f&&typeof v=="string"&&f[v]?f[v]:f&&f[i]?f[i]:null}function Ys(a,i,r,f,d){var h,v=(h=a.ajax.json())===null||h===void 0?void 0:h.columnControl,g=[],x=Um(a,r);if(x)g=x;else if(v&&i.ajaxOnly){i.hidable&&(f.element().style.display="none",i._parents&&i._parents.forEach(function(B){return B.checkDisplay()}));return}else if(!a.page.info().serverSide)for(var T={},A=a.rows({order:r}).indexes().toArray(),N=a.settings()[0],j=0;j<A.length;j++){var X=N.fastData(A[j],r,"filter"),U=X!=null?X.toString():"";T[U]||(T[U]=!0,g.push({label:N.fastData(A[j],r,i.orthogonal),value:U}))}$s(f,g),d&&f.values(d)}var Is={defaults:{ajaxOnly:!0,className:"searchList",hidable:!0,options:null,orthogonal:"display",search:!0,select:!0,title:""},init:function(a){var i=this,r=null,f=this.dt(),d=function(v){if(!f.page.info().serverSide){var g=f.column(i.idx());if(v)v.length===0?g.search.fixed("dtcc-list",""):g.search.fixed("dtcc-list",function(x){return v.includes(x)});else return;a._parents&&a._parents.forEach(function(x){return x.activeList(i.unique(),!!v.length)})}},h=new _d(f,this,{search:a.search,select:a.select}).searchListener(f).title(f.i18n("columnControl.searchList",a.title).replace("[title]",f.column(this.idx()).title())).handler(function(v,g,x,T){g&&g.active(!g.active()),d(h.values()),T&&f.draw()});return a.options?$s(h,a.options):(f.ready(function(){Ys(f,a,i.idx(),h,r)}),f.on("xhr",function(v,g,x){f.one("draw",function(){Ys(f,a,i.idx(),h,r)})})),f.page.info().serverSide&&f.on("preXhr.DT",function(v,g,x){!x.columns||!x.columns[i.idx()]||(x.columns[i.idx()].columnControl||(x.columns[i.idx()].columnControl={}),x.columns[i.idx()].columnControl.list=Object.assign({},h.values()))}),f.on("stateLoaded",function(v,g,x){var T=$v(i.idx(),x);T&&(h.values(T),d(T))}),f.on("stateSaveParams",function(v,g,x){var T=i.idx();x.columnControl||(x.columnControl={}),x.columnControl[T]||(x.columnControl[T]={}),x.columnControl[T].searchList=f.ready()?h.values():r}),f.settings()[0].aoColumns[this.idx()].columnControlSearchList=function(v){v==="refresh"?Ys(f,a,i.idx(),h,null):$s(h,v)},r=$v(this.idx(),f.state.loaded()),d(r),h.element()}},Ps={defaults:{clear:!0,placeholder:"",title:"",titleAttr:""},init:function(a){var i=this,r=this.dt(),f="columnControl.search.number.",d=new Lo(r,this.idx()).type("num").addClass("dtcc-searchNumber").clearable(a.clear).placeholder(a.placeholder).title(a.title).titleAttr(a.titleAttr).options([{label:r.i18n(f+"equal","Equals"),value:"equal"},{label:r.i18n(f+"notEqual","Does not equal"),value:"notEqual"},{label:r.i18n(f+"greater","Greater than"),value:"greater"},{label:r.i18n(f+"greaterOrEqual","Greater or equal"),value:"greaterOrEqual"},{label:r.i18n(f+"less","Less than"),value:"less"},{label:r.i18n(f+"lessOrEqual","Less or equal"),value:"lessOrEqual"},{label:r.i18n(f+"empty","Empty"),value:"empty"},{label:r.i18n(f+"notEmpty","Not empty"),value:"notEmpty"}]).search(function(h,v,g){if(r.page.info().serverSide){g||r.draw();return}var x=r.column(i.idx());if(h==="empty")x.search.fixed("dtcc",function(T){return!T});else if(h==="notEmpty")x.search.fixed("dtcc",function(T){return!!T});else{if(x.search.fixed("dtcc")===""&&v==="")return;v===""?x.search.fixed("dtcc",""):h==="equal"?x.search.fixed("dtcc",function(T){return xr(T)==v}):h==="notEqual"?x.search.fixed("dtcc",function(T){return xr(T)!=v}):h==="greater"?x.search.fixed("dtcc",function(T){return xr(T)>v}):h==="greaterOrEqual"?x.search.fixed("dtcc",function(T){return xr(T)>=v}):h==="less"?x.search.fixed("dtcc",function(T){return xr(T)<v}):h==="lessOrEqual"&&x.search.fixed("dtcc",function(T){return xr(T)<=v})}a._parents&&a._parents.forEach(function(T){return T.activeList(i.unique(),!!x.search.fixed("dtcc"))}),g||x.draw()});return d.input().setAttribute("inputmode","numeric"),d.input().setAttribute("pattern","[0-9]*"),d.element()}},zb=/<([^>]*>)/g,Lb=/['\u00A0,$£€¥%\u2009\u202F\u20BD\u20a9\u20BArfkɃΞ]/gi;function xr(a){if(a!==0&&(!a||a==="-"))return-1/0;var i=typeof a;return i==="number"||i==="bigint"?a:(a.replace&&(a=a.replace(zb,"").replace(Lb,"")),a*1)}var ed={defaults:{clear:!0,placeholder:"",title:"",titleAttr:""},init:function(a){var i=this,r=this.dt(),f="columnControl.search.text.",d=new Lo(r,this.idx()).addClass("dtcc-searchText").clearable(a.clear).placeholder(a.placeholder).title(a.title).titleAttr(a.titleAttr).options([{label:r.i18n(f+"contains","Contains"),value:"contains"},{label:r.i18n(f+"notContains","Does not contain"),value:"notContains"},{label:r.i18n(f+"equal","Equals"),value:"equal"},{label:r.i18n(f+"notEqual","Does not equal"),value:"notEqual"},{label:r.i18n(f+"starts","Starts"),value:"starts"},{label:r.i18n(f+"ends","Ends"),value:"ends"},{label:r.i18n(f+"empty","Empty"),value:"empty"},{label:r.i18n(f+"notEmpty","Not empty"),value:"notEmpty"}]).search(function(h,v,g){if(r.page.info().serverSide){g||r.draw();return}var x=r.column(i.idx());if(v=v.toLowerCase(),h==="empty")x.search.fixed("dtcc",function(T){return!T});else if(h==="notEmpty")x.search.fixed("dtcc",function(T){return!!T});else{if(x.search.fixed("dtcc")===""&&v==="")return;v===""?x.search.fixed("dtcc",""):h==="equal"?x.search.fixed("dtcc",function(T){return T.toLowerCase()==v}):h==="notEqual"?x.search.fixed("dtcc",function(T){return T.toLowerCase()!=v}):h==="contains"?x.search.fixed("dtcc",v):h==="notContains"?x.search.fixed("dtcc",function(T){return!T.toLowerCase().includes(v)}):h==="starts"?x.search.fixed("dtcc",function(T){return T.toLowerCase().startsWith(v)}):h==="ends"&&x.search.fixed("dtcc",function(T){return T.toLowerCase().endsWith(v)})}a._parents&&a._parents.forEach(function(T){return T.activeList(i.unique(),!!x.search.fixed("dtcc"))}),g||x.draw()});return d.element()}},Hb={defaults:{allowSearchList:!1},init:function(a){var i=this,r,f,d,h=this.dt(),v=this.idx(),g,x=(d=(f=(r=h.state.loaded())===null||r===void 0?void 0:r.columnControl)===null||f===void 0?void 0:f[v])===null||d===void 0?void 0:d.searchInput,T=function(A){var N=Um(h,v);return a.allowSearchList&&N?Is.init.call(i,Object.assign({},Is.defaults,a)):A==="date"||A.startsWith("datetime")?Ws.init.call(i,Object.assign({},Ws.defaults,a)):A.includes("num")?Ps.init.call(i,Object.assign({},Ps.defaults,a)):ed.init.call(i,Object.assign({},ed.defaults,a))};return x?g=T(x.type):(g=document.createElement("div"),h.ready(function(){var A=h.column(v),N=T(A.type());g.replaceWith(N)})),g}},jb={defaults:{className:"searchClear",icon:"searchClear",text:"Clear Search"},init:function(a){var i=this,r=this.dt(),f=new Xt(r,this).text(r.i18n("columnControl.searchClear",a.text)).icon(a.icon).className(a.className).handler(function(){r.column(i.idx()).columnControl.searchClear().draw()}).enable(!1);return r.on("draw",function(){var d=r.column(i.idx()).search.fixed("dtcc"),h=r.column(i.idx()).search.fixed("dtcc-list");f.enable(!!(d||h))}),f.element()}},Rb={defaults:{ajaxOnly:!0,allowSearchList:!0,className:"searchDropdown",clear:!0,columns:"",hidable:!0,options:null,orthogonal:"display",placeholder:"",search:!0,select:!0,text:"Search",title:"",titleAttr:""},extend:function(a){var i=this.dt();return{extend:"dropdown",icon:"search",text:i.i18n("columnControl.searchDropdown",a.text),content:[Object.assign(a,{extend:"search"})]}}},qb={defaults:{className:"dtcc-spacer",text:""},init:function(a){var i=this.dt(),r=nt("div",a.className,i.i18n("columnControl.spacer",a.text));return r.setAttribute("role","separator"),r}},Ub={defaults:{className:"dtcc-title",text:null},init:function(a){var i=this.dt(),r=i.column(this.idx()).title(),f=a.text===null?"[title]":a.text,d=nt("div",a.className,f.replace("[title]",r));return d}},Bb={colVis:hb,colVisDropdown:pb,dropdown:Fs,reorder:vb,reorderLeft:mb,reorderRight:yb,rowGroup:Ab,rowGroupAdd:Eb,rowGroupClear:Nb,rowGroupRemove:Mb,order:bb,orderAddAsc:gb,orderAddDesc:Sb,orderAsc:_b,orderClear:xb,orderDesc:Db,orderRemove:Tb,orderStatus:Cb,search:Hb,searchClear:jb,searchDropdown:Rb,searchDateTime:Ws,searchList:Is,searchNumber:Ps,searchText:ed,spacer:qb,title:Ub},xa=(function(){function a(i,r,f){var d=this;this._dom={target:null,wrapper:null},this._c={},this._s={columnIdx:null,unique:null,toDestroy:[]},this._dt=i,this._s.columnIdx=r,this._s.unique=Math.random();var h=r;Object.assign(this._c,a.defaults,f),this._dom.target=this._target(),f.className&&zm(this._dom.target.closest("tr"),f.className),this._c.content&&(i.on("columns-reordered",function(v,g){d._s.columnIdx=i.colReorder.transpose(h,"fromOriginal")}),this._dom.wrapper=document.createElement("span"),this._dom.wrapper.classList.add("dtcc"),this._dom.target.appendChild(this._dom.wrapper),this._c.content.forEach(function(v){var g=d.resolve(v),x=g.plugin,T=g.config,A=x.init.call(d,T);d._dom.wrapper.appendChild(A)}),i.on("destroy",function(){d._s.toDestroy.slice().forEach(function(v){v.destroy()}),d._dom.wrapper.remove()}))}return a.prototype.destroyAdd=function(i){this._s.toDestroy.push(i)},a.prototype.destroyRemove=function(i){var r=this._s.toDestroy.indexOf(i);r!==-1&&this._s.toDestroy.splice(r,1)},a.prototype.dt=function(){return this._dt},a.prototype.idx=function(){return this._s.columnIdx},a.prototype.resolve=function(i){var r=null,f=null,d=null;if(typeof i=="string"?(d=i,r=a.content[d],f=Object.assign({},r?.defaults)):Array.isArray(i)?(d="dropdown",r=a.content[d],f=Object.assign({},r?.defaults,{content:i})):i.extend&&(d=i.extend,r=a.content[d],f=Object.assign({},r?.defaults,i)),!r)throw new Error("Unknown ColumnControl content type: "+d);if(r.extend){var h=r.extend.call(this,f);return this.resolve(h)}return{config:f,type:d,plugin:r}},a.prototype.unique=function(){return this._s.unique},a.prototype._target=function(){var i=this._c.target,r=this._dt.column(this._s.columnIdx),f,d="header";if(typeof i=="number")f=r.header(i);else{var h=i.split(":"),v=h[0]!=="tfoot",g=h[1]?parseInt(h[1]):0;v?f=r.header(g):(f=r.footer(g),d="footer")}return f.querySelector("div.dt-column-"+d)},a.Button=Xt,a.CheckList=_d,a.SearchInput=Lo,a.content=Bb,a.defaults={className:"",content:null,target:0},a.icons=Ia,a.version="1.1.1",a})();$.ColumnControl=xa;Om(document).on("i18n.dt",function(a,i){if(a.namespace==="dt"){var r=new $.Api(i),f=r.table().header(),d=i.oInit.columnControl,h=xa.defaults,v=[],g={};f.querySelectorAll("tr").length<=1&&i.titleRow===null&&(i.titleRow=0),Tr(v,d),xa.defaults.content&&Tr(v,h),r.columns().every(function(T){var A=this.init().columnControl;Tr(v,A)});for(var x=0;x<v.length;x++)Yb(g,v[x],r)}});Om(document).on("preInit.dt",function(a,i){if(a.namespace==="dt"){var r=new $.Api(i),f=i.oInit.columnControl,d=xa.defaults,h=[];Tr(h,f),xa.defaults.content&&Tr(h,d),r.columns().every(function(v){for(var g=this.init().columnControl,x=Tr(h.slice(),g),T=0;T<x.length;T++){var A=Xs(x[T],g),N=Xs(x[T],f),j=Xs(x[T],d);(j||N||A)&&new xa(r,this.index(),Object.assign({},j||{},N||{},A||{}))}})}});function Bm(){var a=this;return this.iterator("column",function(i,r){a.trigger("cc-search-clear",[r])})}$.Api.registerPlural("columns().columnControl.searchClear()","column().columnControl.searchClear()",Bm);$.Api.registerPlural("columns().ccSearchClear()","column().ccSearchClear()",Bm);$.Api.registerPlural("columns().columnControl.searchList()","column().columnControl.searchList()",function(a){return this.iterator("column",function(i,r){var f=i.aoColumns[r].columnControlSearchList;f&&f(a)})});$.ext.buttons.ccSearchClear={text:function(a){return a.i18n("columnControl.buttons.searchClear","Clear search")},init:function(a,i,r){var f=this;a.on("draw.DT",function(){var d=!1,h=!!a.search();h||a.columns().every(function(){(this.search.fixed("dtcc")||this.search.fixed("dtcc-list"))&&(d=!0)}),f.enable(h||d)}),this.enable(!1)},action:function(a,i,r,f){i.search(""),i.columns().columnControl.searchClear(),i.draw()}};function Yb(a,i,r){if(!a[i]){var f=!0,d=0;if(typeof i=="number")d=i;else{var h=i.split(":");h[0]==="tfoot"&&(f=!1),h[1]&&(d=parseInt(h[1]))}var v=f?r.table().header():r.table().footer();if(!v.querySelectorAll("tr")[d]){var g=r.columns().count(),x=nt("tr");x.setAttribute("data-dt-order","disable");for(var T=0;T<g;T++)x.appendChild(nt("td"));v.appendChild(x)}a[i]=!0}}function Xs(a,i){var r=xa.defaults.target,f;if(Iv(i)){if(r===a)return{target:r,content:i}}else if(Array.isArray(i))for(var d=0;d<i.length;d++){var h=i[d];if(Iv(h)){if(r===a)return{target:r,content:h}}else if(td(h)){if(f=h.target!==void 0?h.target:r,a===f)return h}else if(a===r)return{target:r,content:i}}else if(typeof i=="object"){if(td(i)){if(f=i.target!==void 0?i.target:r,a===f)return i}else if(a===r)return{target:r,content:i}}}function Tr(a,i){function r(f){a.includes(f)||a.push(f)}return Array.isArray(i)?i.length===0?r(xa.defaults.target):i.forEach(function(f){r(typeof f=="object"&&f.target!==void 0?f.target:xa.defaults.target)}):typeof i=="object"&&r(i.target!==void 0?i.target:xa.defaults.target),a}function td(a){return typeof a=="object"&&a.target!==void 0}function Iv(a){var i=!1;if(!Array.isArray(a))return!1;for(var r=0;r<a.length;r++)if(td(a[r])){i=!0;break}return!i}/*! Bootstrap 5 styling wrapper for ColumnControl
 * © SpryMedia Ltd - datatables.net/license
 */$.ColumnControl.content.dropdown.classes.container=["dtcc-dropdown","dropdown-menu","show"];$.ColumnControl.CheckList.classes.input=["dtcc-list-search","form-control","form-control-sm"];$.ColumnControl.SearchInput.classes.input=["form-control","form-control-sm"];$.ColumnControl.SearchInput.classes.select=["form-select","form-select-sm"];export{$ as D,Gb as E,c0 as c};
//# sourceMappingURL=@datatables-libs-DD3NUtWT.js.map
